#!/usr/bin/perl

use FindBin;
use lib $FindBin::Bin;
use f5waflib;

use Data::Dumper;
use JSON qw(decode_json);
use Getopt::Long;

require F5::BigipVersionUtils;
require F5::ASMConfig::Handler;

my %generate_json_opts;

GetOptions('rst' => \$generate_json_opts{generate_rst},
           'md' =>\$generate_json_opts{generate_md},
          );

my $client = F5::ASMConfig::Handler->new();
my $session = F5::ASMConfig::Session->new();

my $rv;
eval {
    $rv = $client->generate_json_schema($session, undef, \%generate_json_opts);

    open(my $fh, '>', 'policy.json');
    print $fh JSON->new->ascii->canonical->pretty->encode($rv->{json_rv});
    close $fh;

    if ($generate_json_opts{generate_md}) {
        open(my $fh, '>', 'policy.md');
        print $fh $rv->{md_rv};
        close $fh;
    }

    if ($generate_json_opts{generate_rst}) {
        open(my $fh, '>', 'policy.rst');
        print $fh $rv->{rst_rv};
        close $fh;

        for my $collection (keys %{$rv->{collections}}) {
            open(my $fh, '>', "$collection.rst");
            print $fh $rv->{collections}->{$collection};
            close $fh;
        }
    }
};
if ($@) {
    print "Error generating JSON Schema\n";
    print Dumper($@);
    exit;
}
