#!/usr/bin/perl
use strict;
use warnings;
no warnings qw(redefine);

use FindBin;
use lib $FindBin::Bin;
use f5waflib;

use Getopt::Long;
use File::Basename;
use File::Spec;
use JSON;
use F5::BigipUtils;
use F5::ASMConfig::EasyClient;
use F5::ASMConfig::GuaranteedClient;

my $action = '';
my $policy_id = 0;
my $filename = '';
my $user_name = 0;
my $export_file_version = 6;
my $force = 0;
my $enforce_strictness = 1;
my $json_output = 0;
my $overwrite = 0;
my $reset_sigs = 0;
my $file_format = 'binary';
my $proposed_policy_name = '';
my $minimal = 0;
my $skip_mcp = 0;

my $action_export = 'export';
my $action_import = 'import';

GetOptions(
   'action|a=s'         => \$action,
   'filename|f=s'       => \$filename,
   'policy_id|p=s'      => \$policy_id,
   'user_name|u=s'      => \$user_name,
   'ver=i'              => \$export_file_version,
   'force!'             => \$force,
   'strictness!'        => \$enforce_strictness,
   'json!'              => \$json_output,
   'reset_sigs!'        => \$reset_sigs,
   'overwrite!'         => \$overwrite,
   'format=s'           => \$file_format,
   'minimal!'           => \$minimal,
   'proposed_policy_name|n=s' => \$proposed_policy_name,
   'skip_mcp!'         => \$skip_mcp,
);

my $script_name = basename($0);

die usage() if  ($action ne $action_export && $action ne $action_import) ||
                ($action eq $action_export && $policy_id == 0 && $filename eq '')           ||
                ($action eq $action_import && $filename eq '');

my $abs_filename = File::Spec->rel2abs($filename);
my $client = F5::ASMConfig::GuaranteedClient::get_client((username => $user_name || 'import_export_policy'),
                                                          gui_user_name => $user_name,
                                                        );

my $result;
eval {
    if ($action eq 'import') {
        my $call_flags = F5::ASMConfig::CallFlags->new(skip_mcp => $skip_mcp);
        my $import_export_properties = F5::ASMConfig::ImportExportProperties->new(
                                                                                filename            => $abs_filename,
                                                                                force_policy_id     => ($overwrite ? $policy_id : undef),
                                                                                enforce_strictness => $enforce_strictness,
                                                                                policy_name => $proposed_policy_name,
                                                                               );
        if (F5::BigipUtils::is_docker()) {
            $import_export_properties->{flg_encoding_selected} = 1;
            $import_export_properties->{encoding} = 'utf-8';
        }
        $result = $client->import_policy($client->{session},
    				         $call_flags,
                                         $import_export_properties);
    }
    elsif ($action eq 'export') {
        $result = $client->export_policy($client->{session},
    				     F5::ASMConfig::CallFlags->new,
    				     F5::ASMConfig::Policy->new(policy_id => $policy_id),
    				     F5::ASMConfig::ImportExportProperties->new(
    										filename => $abs_filename, 
    										file_format => $file_format,
    										minimal_export => $minimal,
    									       ));
    }
};

if ($@) {
  die $@->{error_message} if ref $@ eq 'F5::ASMConfig::ASMConfigException';
  die $@;
}

if ($json_output) {
    print to_json($result, { allow_blessed => 1, convert_blessed => 1 });
}
else {
  print join(" ", map($_->{msg}, @{$result->{msgs}})), "\n";
}

if ($result->{completed_successfully}) {
    exit;
}

exit 1;


sub usage
{
    return <<EOF;
Policy Import/Export utility was not invoked correctly.
Usage: $script_name options

  Required:
    -a, --action=import|export
    -f, --filename=/full/path/to/file

  If action=export:
    -p, --policy_id=#
EOF
}

sub UNIVERSAL::TO_JSON { return { %{ shift() } }; }
