#!/usr/bin/perl

use strict;
use warnings;

use FindBin;
use lib $FindBin::Bin;
use f5waflib;

use F5::SetActive;
use Getopt::Long;

my %params;
my $help;

GetOptions(
    'u|username=s'          => \$params{username},
    'p|policy_id=i'         => \$params{policy_id},
    'i|account_id=i'        => \$params{account_id},
    'h|hsl_profile_id=i'    => \$params{hsl_profile_id},
    'o|hsl_profile_option=s'    => \$params{hsl_profile_options},
    'c|upate_core_only|update_core_only'     => \$params{update_core_only},
    'g|update_logger_cfg!'   => \$params{update_logger_cfg},
    'dump_bd_config_tables!'    => \$params{dump_bd_config_tables},
    'd|invalidate_hsl_data_profiles=s@'    => \$params{invalidate_hsl_data_profiles},
    'history!'              => \$params{do_save_policy_history},
    'force_full_sync'       => \$params{force_full_sync},
    'use_daemon!'               => \$params{use_daemon},
    'update_ramcache!'               => \$params{update_ramcache},
    'ignore_if_policy_unmodified!'    => \$params{ignore_if_policy_unmodified},
    'ignore_if_policy_not_active!'    => \$params{ignore_if_policy_not_active},
    'force_failover!'         => \$params{force_failover},
    'help'                  => \$help,
);

if ($help) {
    print usage();
    exit 0;
}

$params{send_sync} = 1;

my $rv = F5::SetActive::set_active(%params,
                               expanded_return_value => 1,
                              );

if ($rv->{completed_successfully}) {
  print '0';
}
else {
  print $rv->{error_message};
}

sub usage {
    return qq{
USAGE:
    $0 <mandatory arguments>

Mandatory arguments:

    --policy_id|p='policy id'
        The id of the policy. Use it to set the policy active.
    --account_id|i='account id'
        The id of the account.
    --hsl_profile_id|h='hsl profile id'
        Sets the specified hsl profile active
    --hsl_profile_option|o='hsl profile option'
        Used together with hsl_profile_id. Specifies whether to delete the hsl profile
    --invalidate_hsl_data_profiles|d='invalidate_hsl_data_profiles'
        When used the RamCache for the data profiles and the WA Cache for their policies will be invalidated
    --upate_core_only|c='upate core only'
        Used only by itself, without any other argunemts. Informs the bd on changes.
    --force_full_sync
        (For cluster only, do a full sync even when applying policy)
        
Optional arguments:

    --username='username'
    --update_logger_cfg|g='update_logger_cfg'
        send to bd table code CONFIG_TYPE_UPDATE_LOGGER_CFG
    --dump_bd_config_tables='dump_bd_config_tables'
        send to bd table code DUMP_BD_CONFIG_TABLES

    [ --use_daemon / --nouse_daemon ]
        If --nouse_daemon is specified, set active will be run directly in this process (rather than contacting the daemon).

Optionally, using --help will issue this help message.
    \n};
}

