#ifndef _DEBUG_H_

#define _DEBUG_H_

#define MAX_DEBUG_BUF_SIZE           1024

//#define DYNAMIC_DEBUG
//#define PRECOMPILED_DEBUG_ONLY
#define SHOW_FILE_AND_LINE printf(" (%s, %d) ", __FILE__, __LINE__);

// portable sized sizeof() for debug prints
#define DEBUG_SIZEOF(X) ((unsigned long)(sizeof(X)))

#ifndef MIN
#define MIN(a, b) (((a)<(b))?(a):(b))
#endif

//colors
#define DEBUG_ORIGINAL  "\033[0m"
#define DEBUG_LIGHT_RED "\033[31;1m"
#define DEBUG_RED       "\033[31;2m"
#define DEBUG_GREEN     "\033[32;1m"
#define DEBUG_YELLOW    "\033[33;1m"
#define DEBUG_BLUE      "\033[34;1m"
#define DEBUG_PURPLE    "\033[35;1m"
#define DEBUG_AZURE     "\033[36;1m"
#define DEBUG_WHITE     "\033[37;1m"
#define DEBUG_GRAY      "\033[38;1m"
#define DEBUG_URED      "\033[31;1;4m"
#define DEBUG_UGREEN    "\033[32;1;4m"
#define DEBUG_UYELLOW   "\033[33;1;4m"
#define DEBUG_UBLUE     "\033[34;1;4m"
#define DEBUG_UPURPLE   "\033[35;1;4m"
#define DEBUG_UCYAN     "\033[36;1;4m"
#define DEBUG_UWHITE    "\033[37;1;4m"

#ifndef DYNAMIC_DEBUG
//select modules you want to log  
//#define DEBUG_MOD_ESSENTIAL                    // 0x1
//#define DEBUG_MOD_SAX_EVENTS                   // 0x2
//#define DEBUG_MOD_XMLPARSER                    // 0x4
//#define DEBUG_MOD_WSSPARSER                    // 0x8
//#define DEBUG_MOD_XPATHPARSER                  // 0x10
//#define DEBUG_MOD_FETCHPARSER                  // 0x20
//#define DEBUG_MOD_CANONICALIZER                // 0x40
//#define DEBUG_MOD_TOKENIZER                    // 0x80
//#define DEBUG_MOD_CONFIG                       // 0x100
//#define DEBUG_MOD_CALLBACK        	         // 0x200	
//#define DEBUG_MOD_KEYSTORE                     // 0x400 
//#define DEBUG_MOD_SCHEMA_LOAD                  // 0x800
//#define DEBUG_MOD_JSONPARSER                   // 0x1000
//#define DEBUG_MOD_SCHEMA_VALIDATOR             // 0x2000
//#define DEBUG_MOD_SCHEMA_EVENTS                // 0x8000
//#define DEBUG_MOD_XMLPARSER_TRIE_DUMP           // 0x10000
//#define DEBUG_MOD_TRIE                          // 0x20000
//#define DEBUG_MOD_HASH                          // 0x40000
//#define DEBUG_MOD_MEMORY_FOOTPRINT              // 0x80000
//#define DEBUG_MOD_DYNAMIC_MEMORY                // 0x100000
//#define DEBUG_MOD_MEMORY_UTIL                   // 0x200000
//#define DEBUG_MOD_CACHING                       // 0x400000
//#define DEBUG_MOD_XPATH_COMPILE                 // 0x800000
//#define DEBUG_MOD_XPATH_ALGORITHM               // 0x1000000
//#define DEBUG_MOD_WSSPARSER_ESSENTIAL           // 0x2000000
//#define DEBUG_MOD_BUFFERS_MANAGER               // 0x4000000


#else
#define DEBUG_MOD_ESSENTIAL                     0x1
#define DEBUG_MOD_SAX_EVENTS                    0x2
#define DEBUG_MOD_XMLPARSER                     0x4
#define DEBUG_MOD_WSSPARSER                     0x8
#define DEBUG_MOD_XPATHPARSER                   0x10
#define DEBUG_MOD_FETCHPARSER                   0x20
#define DEBUG_MOD_CANONICALIZER                 0x40
#define DEBUG_MOD_TOKENIZER                     0x80
#define DEBUG_MOD_CONFIG                        0x100
#define DEBUG_MOD_CALLBACK                      0x200
#define DEBUG_MOD_KEYSTORE                      0x400
#define DEBUG_MOD_SCHEMA_LOAD                   0x800
#define DEBUG_MOD_JSONPARSER                    0x1000
#define DEBUG_MOD_SCHEMA_VALIDATOR              0x2000
#define DEBUG_MOD_SCHEMA_EVENTS                 0x8000
#define DEBUG_MOD_XMLPARSER_TRIE_DUMP           0x10000
#define DEBUG_MOD_TRIE                          0x20000
#define DEBUG_MOD_HASH                          0x40000
#define DEBUG_MOD_MEMORY_FOOTPRINT              0x80000
#define DEBUG_MOD_DYNAMIC_MEMORY                0x100000
#define DEBUG_MOD_MEMORY_UTIL                   0x200000
#define DEBUG_MOD_CACHING                       0x400000
#define DEBUG_MOD_XPATH_COMPILE                 0x800000
#define DEBUG_MOD_XPATH_ALGORITHM               0x1000000
#define DEBUG_MOD_WSSPARSER_ESSENTIAL           0x2000000
#define DEBUG_MOD_BUFFERS_MANAGER               0x4000000
// these bitmasks are left over from xml_parser - in order to avoid
// confusion, let's reuse these same numbers, and continue from after
// 0x100000
#endif

extern unsigned int debug_flags;

// 0x1 //////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_ESSENTIAL
	#define MOD_ESSENTIAL
	#define Debug_MOD_ESSENTIAL(...) printf("\n"); printf (__VA_ARGS__); fflush(stdout);
	#define Debug2_MOD_ESSENTIAL(...) printf (__VA_ARGS__); fflush(stdout);
#else
	#define MOD_ESSENTIAL
	#define Debug_MOD_ESSENTIAL(...)
	#define Debug2_MOD_ESSENTIAL(...)
#endif //DEBUG_MOD_ESSENTIAL

// 0x2 /////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_SAX_EVENTS
    #define MOD_SAX_EVENTS
    #define Debug_MOD_SAX_EVENTS(...) printf("\n[SAX EVENT] --- "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_SAX_EVENTS(...) printf (__VA_ARGS__); fflush(stdout); 
#else
    #define MOD_SAX_EVENTS
    #define Debug_MOD_SAX_EVENTS(...)
    #define Debug2_MOD_SAX_EVENTS(...)
#endif //DEBUG_MOD_SAX_EVENTS

// 0x4 //////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_XMLPARSER
    #define MOD_XMLPARSER
    #define Debug_MOD_XMLPARSER(...) printf("\n[XML] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_XMLPARSER(...) printf (__VA_ARGS__); fflush(stdout); 
#else
    #define MOD_XMLPARSER
    #define Debug_MOD_XMLPARSER(...)
    #define Debug2_MOD_XMLPARSER(...)
#endif //DEBUG_MOD_XMLPARSER

// 0x8 //////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_WSSPARSER
    #define MOD_WSSPARSER
    #define Debug_MOD_WSSPARSER(...) printf("\n[WSSPARSER] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_WSSPARSER(...) printf (__VA_ARGS__); fflush(stdout); 
#else
    #define MOD_WSSPARSER
    #define Debug_MOD_WSSPARSER(...)
    #define Debug2_MOD_WSSPARSER(...)
#endif //DEBUG_MOD_WSSPARSER

// 0x10 /////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_XPATHPARSER
    #define MOD_XPATHPARSER
    #define Debug_MOD_XPATHPARSER(...) printf("\n[XPATH] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_XPATHPARSER(...) printf (__VA_ARGS__); fflush(stdout); 
#else
    #define MOD_XPATHPARSER
    #define Debug_MOD_XPATHPARSER(...)
    #define Debug2_MOD_XPATHPARSER(...)
#endif //DEBUG_MOD_XPATHPARSER

// 0x20 //////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_FETCHPARSER
    #define MOD_FETCHPARSER
    #define Debug_MOD_FETCHPARSER(...) printf("\n[FETCHPARSER] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_FETCHPARSER(...) printf (__VA_ARGS__); fflush(stdout); 
#else
    #define MOD_FETCHPARSER
    #define Debug_MOD_FETCHPARSER(...)
    #define Debug2_MOD_FETCHPARSER(...)
#endif //DEBUG_MOD_FETCHPARSER

// 0x40 ///////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_CANONICALIZER
    #define MOD_CANONICALIZER
    #define Debug_MOD_CANONICALIZER(...) printf("\n[CANONICALIZER] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_CANONICALIZER(...) printf (__VA_ARGS__); fflush(stdout); 
#else
    #define MOD_CANONICALIZER
    #define Debug_MOD_CANONICALIZER(...)
    #define Debug2_MOD_CANONICALIZER(...)
#endif //DEBUG_MOD_CANONICALIZER  

// 0x80 ///////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_TOKENIZER
    #define MOD_TOKENIZER
    #define Debug_MOD_TOKENIZER(...) printf("\n[TOKENIZER] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_TOKENIZER(...) printf (__VA_ARGS__); fflush(stdout); 
#else
    #define MOD_TOKENIZER
    #define Debug_MOD_TOKENIZER(...)
    #define Debug2_MOD_TOKENIZER(...)
#endif //DEBUG_MOD_TOKENIZER

// 0x100 ////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_CONFIG
    #define MOD_CONFIG
    #define Debug_MOD_CONFIG(...) printf("\n[CONFIG] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_CONFIG(...) printf (__VA_ARGS__); fflush(stdout); 
#else
    #define MOD_CONFIG
    #define Debug_MOD_CONFIG(...)
    #define Debug2_MOD_CONFIG(...)
#endif //DEBUG_MOD_CONFIG

// 0x200 ////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_CALLBACK
    #define MOD_CALLBACK
    #define Debug_MOD_CALLBACK(...) printf("\n[CALLBACK] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_CALLBACK(...) printf (__VA_ARGS__); fflush(stdout); 
#else
    #define MOD_CALLBACK
    #define Debug_MOD_CALLBACK(...)
    #define Debug2_MOD_CALLBACK(...)
#endif //DEBUG_MOD_CALLBACK

// 0x400 ///////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_KEYSTORE
    #define MOD_KEYSTORE
    #define Debug_MOD_KEYSTORE(...) printf("\n[KEYSTORE] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_KEYSTORE(...) printf (__VA_ARGS__); fflush(stdout); 
#else
    #define MOD_KEYSTORE
    #define Debug_MOD_KEYSTORE(...)
    #define Debug2_MOD_KEYSTORE(...)
#endif //DEBUG_MOD_KEYSTORE

// 0x800 ////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_SCHEMA_LOAD
    #define MOD_SCHEMA_LOAD
    #define Debug_MOD_SCHEMA_LOAD(...) printf("\n[SCHEMA LOAD] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_SCHEMA_LOAD(...) printf (__VA_ARGS__); fflush(stdout); 
#else
    #define MOD_SCHEMA_LOAD
    #define Debug_MOD_SCHEMA_LOAD(...)
    #define Debug2_MOD_SCHEMA_LOAD(...)
#endif // MOD_SCHEMA_LOAD

// 0x1000 ///////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_JSONPARSER
    #define MOD_JSONPARSER
    #define Debug_MOD_JSONPARSER(...) printf("\n[JSON] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_JSONPARSER(...) printf (__VA_ARGS__); fflush(stdout); 
#else
    #define MOD_JSONPARSER
    #define Debug_MOD_JSONPARSER(...)
    #define Debug2_MOD_JSONPARSER(...)
#endif //DEBUG_MOD_JSONPARSER

// 0x2000 ///////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_SCHEMA_VALIDATOR
    #define MOD_SCHEMA_VALIDATOR
    #define Debug_MOD_SCHEMA_VALIDATOR(...) printf("\n[SCHEMA] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_SCHEMA_VALIDATOR(...) printf (__VA_ARGS__); fflush(stdout); 
#else
    #define MOD_SCHEMA_VALIDATOR
    #define Debug_MOD_SCHEMA_VALIDATOR(...)
    #define Debug2_MOD_SCHEMA_VALIDATOR(...)
#endif //DEBUG_MOD_SCHEMA_VALIDATOR







// 0x100 ////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_SCHEMA_EVENTS
    #define MOD_SCHEMA_EVENTS
    #define Debug_MOD_SCHEMA_EVENTS(...) printf("\n[SCHEMA_EVENT] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_SCHEMA_EVENTS(...) printf (__VA_ARGS__); fflush(stdout); 
#else
    #define MOD_SCHEMA_EVENTS
    #define Debug_MOD_SCHEMA_EVENTS(...)
    #define Debug2_MOD_SCHEMA_EVENTS(...)
#endif //DEBUG_MOD_SCHEMA_EVENTS

// 0x2000 ///////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_XMLPARSER_TRIE_DUMP
    #define MOD_XMLPARSER_TRIE_DUMP
    #define Debug_MOD_XMLPARSER_TRIE_DUMP(...) printf("\n[TRIE_DUMP] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_XMLPARSER_TRIE_DUMP(...) printf (__VA_ARGS__); fflush(stdout); 
#else
    #define MOD_XMLPARSER_TRIE_DUMP
    #define Debug_MOD_XMLPARSER_TRIE_DUMP(...)
    #define Debug2_MOD_XMLPARSER_TRIE_DUMP(...)
#endif //DEBUG_MOD_XMLPARSER_TRIE_DUMP

// 0x4000 ///////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_TRIE
    #define MOD_TRIE
    #define Debug_MOD_TRIE(...) printf("\n[TRIE] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_TRIE(...) printf (__VA_ARGS__); fflush(stdout); 
#else
    #define MOD_TRIE
    #define Debug_MOD_TRIE(...)
    #define Debug2_MOD_TRIE(...)
#endif //DEBUG_MOD_TRIE

// 0x8000 ///////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_HASH
    #define MOD_HASH
    #define Debug_MOD_HASH(...) printf("\n[HASH] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_HASH(...) printf (__VA_ARGS__); fflush(stdout); 
#else
    #define MOD_HASH
    #define Debug_MOD_HASH(...)
    #define Debug2_MOD_HASH(...) 
#endif //DEBUG_MOD_HASH

// 0x10000 //////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_MEMORY_FOOTPRINT
    #define CALCULATE_DYNAMIC_MEMORY 1 // used in memory_utils.c to calculate the total dynamic-memory
    #define MOD_MEMORY_FOOTPRINT
    #define Debug_MOD_MEMORY_FOOTPRINT(...) printf("\n[MEMORY_FOOTPRINT] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_MEMORY_FOOTPRINT(...) printf (__VA_ARGS__); fflush(stdout);
#else
    #define CALCULATE_DYNAMIC_MEMORY 0
    #define MOD_MEMORY_FOOTPRINT
    #define Debug_MOD_MEMORY_FOOTPRINT(...)
    #define Debug2_MOD_MEMORY_FOOTPRINT(...)
#endif //DEBUG_MOD_MEMORY_FOOTPRINT

// 0x20000 //////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_DYNAMIC_MEMORY
    #define MOD_DYNAMIC_MEMORY
    #define Debug_MOD_DYNAMIC_MEMORY(...) printf("\n[DYNAMIC_MEMORY] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_DYNAMIC_MEMORY(...) printf (__VA_ARGS__); fflush(stdout); 
#else
    #define MOD_DYNAMIC_MEMORY
    #define Debug_MOD_DYNAMIC_MEMORY(...)
    #define Debug2_MOD_DYNAMIC_MEMORY(...)
#endif //DEBUG_MOD_DYNAMIC_MEMORY

// 0x40000 //////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_MEMORY_UTIL
    #define MOD_MEMORY_UTIL
    #define Debug_MOD_MEMORY_UTIL(...) printf("\n[MEMORY_UTIL] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_MEMORY_UTIL(...) printf (__VA_ARGS__); fflush(stdout); 
#else
    #define MOD_MEMORY_UTIL
    #define Debug_MOD_MEMORY_UTIL(...)
    #define Debug2_MOD_MEMORY_UTIL(...)
#endif //DEBUG_MOD_MEMORY_UTIL

// 0x80000 //////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_CACHING
    #define MOD_CACHING
    #define Debug_MOD_CACHING(...) printf("\n[CACHING] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_CACHING(...) printf (__VA_ARGS__); fflush(stdout); 
#else
    #define MOD_CACHING
    #define Debug_MOD_CACHING(...)
    #define Debug2_MOD_CACHING(...)
#endif //DEBUG_MOD_CACHING

// 0x200000 /////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_XPATH_COMPILE
    #define MOD_XPATH_COMPILE
    #define Debug_MOD_XPATH_COMPILE(...) printf("\n[XPATH_COMPILE] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_XPATH_COMPILE(...) printf (__VA_ARGS__); fflush(stdout); 
#else
    #define MOD_XPATH_COMPILE
    #define Debug_MOD_XPATH_COMPILE(...)
    #define Debug2_MOD_XPATH_COMPILE(...)
#endif //DEBUG_MOD_XPATH_COMPILE

// 0x400000 /////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_XPATH_ALGORITHM
    #define MOD_XPATH_ALGORITHM
    #define Debug_MOD_XPATH_ALGORITHM(...) printf("\n[XPATH_ALGORITHM] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_XPATH_ALGORITHM(...) printf (__VA_ARGS__); fflush(stdout); 
#else
    #define MOD_XPATH_ALGORITHM
    #define Debug_MOD_XPATH_ALGORITHM(...)
    #define Debug2_MOD_XPATH_ALGORITHM(...)
#endif //DEBUG_MOD_XPATH_ALGORITHM

// 0x4000000 /////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_WSSPARSER_ESSENTIAL
    #define MOD_WSSPARSER_ESSENTIAL
    #define Debug_MOD_WSSPARSER_ESSENTIAL(...) printf("\n[WSSPARSER_ESSENTIAL] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_WSSPARSER_ESSENTIAL(...) printf (__VA_ARGS__); fflush(stdout);
#else
    #define MOD_WSSPARSER_ESSENTIAL
    #define Debug_MOD_WSSPARSER_ESSENTIAL(...)
    #define Debug2_MOD_WSSPARSER_ESSENTIAL(...)
#endif //DEBUG_MOD_WSSPARSER_ESSENTIAL

// 0x8000000 /////////////////////////////////////////////////////////////////////////////////////////////
#ifdef DEBUG_MOD_BUFFERS_MANAGER
    #define MOD_BUFFERS_MANAGER
    #define Debug_MOD_BUFFERS_MANAGER(...) printf("\n[BUFFERS_MANAGER] "); SHOW_FILE_AND_LINE printf (__VA_ARGS__); fflush(stdout); 
    #define Debug2_MOD_BUFFERS_MANAGER(...) printf (__VA_ARGS__); fflush(stdout);
#else
    #define MOD_BUFFERS_MANAGER
    #define Debug_MOD_BUFFERS_MANAGER(...)
    #define Debug2_MOD_BUFFERS_MANAGER(...)
#endif //DEBUG_MOD_BUFFERS_MANAGER

#ifdef DYNAMIC_DEBUG
#define DebugClause(_mod) debug_flags & DEBUG_##_mod
#define DebugPrint(_mod, args...) if (debug_flags & DEBUG_##_mod) { Debug_##_mod(args); }
#define DebugPrint2(_mod, args...) if (debug_flags & DEBUG_##_mod) { Debug2_##_mod(args); }
#else
#define DebugClause(_mod) 1
#define DebugPrint(_mod, args...) Debug_##_mod(args)
#define DebugPrint2(_mod, args...) Debug2_##_mod(args)
#endif

#ifndef PRECOMPILED_DEBUG_ONLY
#undef  DebugClause
#undef  DebugPrint
#undef  DebugPrint2
#define DebugClause(_mod) 1
#define DebugPrint(_mod, args...) DebugPrintBDInternalBuffer(XML_PARSER, TS_DEBUG, args)
#define DebugPrint2(_mod, args...) DebugPrint2BDInternalBuffer(XML_PARSER, TS_DEBUG, args)
#endif

#define DebugClauseBD(_mod) debug_flags & DEBUG_##_mod
#define DebugPrintBD(_buf, _mod, _level, args...) \
        if (g_f5debugCallback && _buf) { \
            int _buf_len = snprintf (_buf, MAX_DEBUG_BUF_SIZE - 4, args); \
            if (_buf_len > 0){\
               _buf[MIN(_buf_len, MAX_DEBUG_BUF_SIZE - 5)] = '\0'; \
               g_f5debugCallback (_mod, _level, _buf); \
        }}

#define DebugPrint2BD(_buf, _mod, _level, args...) \
        if (g_f5debugCallback && _buf) { \
            int _buf_len = snprintf (_buf, MAX_DEBUG_BUF_SIZE - 4, args); \
            if (_buf_len > 0){\
               _buf[MIN(_buf_len, MAX_DEBUG_BUF_SIZE - 5)] = '\0'; \
               g_f5debugCallback (_mod, _level, _buf); \
        }}

#define DebugPrintBDString(_buf, _mod, _level, args...) \
        if (g_f5debugCallback && _buf) { \
            int _buf_len = sprintf (_buf, "%.*s",MAX_DEBUG_BUF_SIZE - 4,args); \
            if (_buf_len > 0){\
               _buf[MIN(_buf_len, MAX_DEBUG_BUF_SIZE - 5)] = '\0';\
               g_f5debugCallback (_mod, _level, _buf); \
        }}

#define DebugPrintBDInternalBuffer(_mod, _level, args...) \
        if (g_f5debugCallback) { \
            char _buf[MAX_DEBUG_BUF_SIZE]; \
            int _buf_len = snprintf (_buf, MAX_DEBUG_BUF_SIZE - 4, args); \
            if (_buf_len > 0){ \
               _buf[MIN(_buf_len, MAX_DEBUG_BUF_SIZE - 5)] = '\0'; \
               g_f5debugCallback (_mod, _level, _buf); \
        }}

#define DebugPrint2BDInternalBuffer(_mod, _level, args...) \
        if (g_f5debugCallback) { \
            char _buf[MAX_DEBUG_BUF_SIZE]; \
            int _buf_len = snprintf (_buf, MAX_DEBUG_BUF_SIZE - 4, args); \
            if (_buf_len > 0){ \
               _buf[MIN(_buf_len, MAX_DEBUG_BUF_SIZE - 5)] = '\0'; \
               g_f5debugCallback (_mod, _level, _buf); \
        }}

#define DebugPrintBDStringInternalBuffer(_mod, _level, args...) \
        if (g_f5debugCallback) { \
            char _buf[MAX_DEBUG_BUF_SIZE]; \
            int _buf_len = sprintf (_buf, "%.*s",MAX_DEBUG_BUF_SIZE - 4,args); \
            if (_buf_len > 0){ \
               _buf[MIN(_buf_len, MAX_DEBUG_BUF_SIZE - 5)] = '\0'; \
               g_f5debugCallback (_mod, _level, _buf); \
        }}

#endif //_DEBUG_H_
