#ifndef _F5_XML_H_
#define _F5_XML_H_

#include <stdio.h>
#include "xmlerrors_desc.h"
#include "encoding_types.h"
#include "logger_consts.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef void * F5xmlParser;
typedef void * F5xmlConfig;
typedef void * F5wssKeyStore;
typedef void * F5wssXPathData;
typedef void * F5wssParser;
typedef void * F5jsonParser;
typedef void * xmlParserJob;
typedef void * jsonParserJob;
typedef void * xmlXpathResult;
typedef void * F5xpathData;
typedef void * wssParserJob;
typedef void * wssContextHandle;


typedef int (*F5_pfn_invalidDocumentReportCallback) (void *, xmlParserJob, xml_parser_error_info, xml_error_location, xml_parser_error_info_ex, void *);
typedef int (*F5_pfn_internalErrorReportCallback) (void *, xmlParserJob, xml_parser_error_info, void *);
typedef int (*F5_pfn_alertReportCallback) (void *, xmlParserJob, xml_parser_error_info, xml_error_conf_property, void *);
typedef int (*F5_pfn_checkNegsigCallback) (void *, xmlParserJob, char * data, unsigned int data_len, char * raw_data, unsigned int raw_data_len, unsigned int hash_value, int location, void * userptr, unsigned int is_sensitive, unsigned int depth);
typedef int (*F5_pfn_soapMethodCallback) (void *, xmlParserJob, char * namespace_name, int namespace_len, char * data, int data_len, void * userptr, encoding_type_t);
typedef int (*F5_pfn_sensitiveDataCallback) (void *, xmlParserJob, unsigned int start_data_index, unsigned int data_length, void * userptr);
typedef int (*F5_pfn_wssErrorCallback) (void *, xmlParserJob, xml_parser_error_info, xml_error_location, void *userptr);
typedef int (*F5_pfn_debugCallback) (MODULE_NAME module_id, LOG_LEVEL log_level, const char * data);
typedef void * (*F5_pfn_malloc) (size_t);
typedef void (*F5_pfn_free) (void *);
typedef void * (*F5_pfn_realloc) (void*, size_t);
typedef int (*F5_pfn_usernameTokenCallback) (void *,xmlParserJob,  char * ,int , char * , int, void*);
                    


typedef struct F5xmlParserCallbacks {
	F5_pfn_invalidDocumentReportCallback   	invalidDocumentReportCallback;
	F5_pfn_internalErrorReportCallback		internalErrorReportCallback;
	F5_pfn_alertReportCallback		        alertReportCallback;
	F5_pfn_checkNegsigCallback		        checkNegsigCallback;
	F5_pfn_soapMethodCallback		        soapMethodCallback;
	F5_pfn_sensitiveDataCallback	        sensitiveDataCallback;
	F5xmlParser                             parser;
} F5xmlParserCallbacks;

typedef struct F5WssParserCallbacks {
    F5_pfn_wssErrorCallback   	      wssErrorCallback;
	F5_pfn_alertReportCallback        alertReportCallback;
    F5_pfn_usernameTokenCallback  usernameTokenCallback;
} F5WssParserCallbacks;

typedef struct F5jsonParserCallbacks {
	F5_pfn_invalidDocumentReportCallback	malformedDocument;
	F5_pfn_alertReportCallback		        defenseViolation;
	F5_pfn_checkNegsigCallback		        negativeSignatures;
	F5_pfn_sensitiveDataCallback	        sensitiveDataCallback;
} F5jsonParserCallbacks;

typedef struct F5JsonConfigOverrides{
    int isPolicyInsensitive;
    unsigned int maxElementsLimit;
    unsigned int graphql_max_val_len;
    unsigned int graphql_max_depth;
    unsigned int graphql_flg_tolerate;
    unsigned int is_graphql_override;
} F5JsonConfigOverrides;

// XML config create/delete 
F5xmlConfig  F5xmlConfigCreate (char * config_file);
void         F5xmlConfigDelete (F5xmlConfig config);
int          F5xmlSetInternal(F5xmlConfig xml_config, const char * int_name, int int_value);
int          F5xmlParserSetInternal(F5xmlParser xml_parser, const char * int_name, int int_value);

/* XML Lite API */
F5xmlParser  F5xmlLiteParserCreate (F5xmlParserCallbacks callbacks); 
xmlParserJob F5xmlLitePrepareJob   (F5xmlParser parser, F5xmlConfig xml_config, void * userptr);
int          F5xmlLiteParseChunk   (F5xmlParser parser, xmlParserJob job, unsigned char * buf, int len, int last_chunk);
void         F5xmlLiteParserDelete (F5xmlParser parser);

/* XML Firewall API */
F5xmlParser  F5xmlParserCreate (char * schema_filename, char * config_file, F5xmlParserCallbacks callbacks);
xmlParserJob F5xmlPrepareJob   (F5xmlParser parser, void * userptr);
int          F5xmlParseChunk   (F5xmlParser parser, xmlParserJob job, unsigned char * buf, int len, int last_chunk);
void         F5xmlResetJob     (F5xmlParser parser, xmlParserJob job);
void         F5xmlParserDelete (F5xmlParser parser);

/* XML XPath API */
F5xpathData    F5xmlXpathParserDataCreate (char * xquery_file, char * xquery_buffer, int xquery_buffer_len);
void           F5xmlXpathParserDataDestroy (F5xpathData * xpath_data);
F5xmlParser    F5xmlXpathParserCreate (char * config_file, F5xmlParserCallbacks callbacks);
xmlParserJob   F5xmlXpathPrepareJob (F5xmlParser parser, F5xpathData xpath_data, void * userptr);
int	           F5xmlXpathEvaluate (F5xmlParser parser, xmlParserJob job, unsigned char * buf, int len, xmlXpathResult pResult);
void	       F5xmlXpathResetJob (F5xmlParser parser, xmlParserJob job);
int	           F5xmlXpathFetchValue (xmlXpathResult pResult, unsigned char ** pValue, unsigned int * pValueLen, unsigned char * buf, int len);
void           F5xmlXpathParserDelete (F5xmlParser parser);
xmlXpathResult F5xmlXpathAllocateResult (int max_query_size);
void           F5xmlXpathFreeResult (xmlXpathResult pResult);
int 	       F5xmlXpathResultGetError (xmlXpathResult pResult);
const char *   F5xmlXpathResultGetErrorDesc (xmlXpathResult pResult);
int            F5xmlXpathResultGetNumQueries (xmlXpathResult pResult);
int            F5xmlXpathResultGetQueryId (xmlXpathResult pResult, int index);

/* JSON API */
F5jsonParser F5jsonParserCreate (F5jsonParserCallbacks callbacks); 
xmlParserJob F5jsonPrepareJob   (F5jsonParser parser, F5xmlConfig xml_config, void * userptr, F5JsonConfigOverrides* config_override);
int          F5jsonParseChunk   (F5jsonParser parser, xmlParserJob job, unsigned char * buf, int len, int last_chunk);
void         F5jsonParserDelete (F5jsonParser parser);
int          F5jsonIsSensitive  (jsonParserJob job, char *name);

/* service API*/
unsigned long F5xmlLiteParserGetLastPosition(xmlParserJob job);
unsigned long F5xmlParserGetLastPosition(xmlParserJob job);
unsigned int  F5xmlParserGetHasSensitive(xmlParserJob job);
unsigned int  F5xmlParserGetIsICAPData(xmlParserJob job);
unsigned int  F5xmlLiteParserGetIsICAPData(xmlParserJob job);

void         F5xmlSetMemoryCallbacks   (F5_pfn_malloc pMalloc, F5_pfn_free pFree, F5_pfn_realloc pRealloc);
void         F5xmlSetDebugCallback     (F5_pfn_debugCallback pDebug);


// WSS API
// WS security key store create/delete 
F5wssKeyStore  F5wssKeystoreCreate(F5xmlConfig xml_config);
void           F5wssKeystoreDelete(F5wssKeyStore wss_keystore);
// WS security releted Xpath structures create/delete 
F5wssXPathData F5wssXPathDataCreate(F5xmlConfig xml_config);
void           F5wssXPathDataDelete(F5wssXPathData wss_xpathdata);
// WS session-context (saved between request and response).
void *         F5wssInitContext(void);
void           F5wssFreeContext(void *context_handle);
void           F5wssSetContext(wssParserJob job, void *context_handle);
// WS parser API
F5wssParser    F5WssParserCreate (F5WssParserCallbacks wss_callbacks);
wssParserJob   F5WssPrepareJob (F5wssParser parser, F5xmlConfig config, F5wssKeyStore wss_keystore, F5wssXPathData wss_xpathdata, void * userptr);
int            F5WssEvaluate (F5wssParser parser, wssParserJob job, unsigned char * in_buf, int len, unsigned char ** out_buf, int * out_buf_len, int is_request);
void           F5WssParserDelete (F5wssParser parser); 


#ifdef __cplusplus
}
#endif


#endif //_F5_XML_H_



