#ifndef _XMLERRORS_DESC_H_
#define _XMLERRORS_DESC_H_

#define MAX_VALUE_TEXT_LEN                  256

//xpath errors
#define XPATH_SUCCESS                       0
#define XPATH_CHUNK_END                     1

#define XPATH_DOCUMENT_ERROR               ((int)-1)
#define XPATH_INTERNAL_ERROR               ((int)-2)

// wss
#define WSS_INTERNAL_ERROR                 ((int)-3)
#define WSS_SUCCESS                         0
#define WSS_FAILURE                        ((int)-1)

#define JSON_PARSER_SUCCESS                 0
#define JSON_PARSER_ABORTED                ((int)-1)
#define JSON_PARSER_ERROR                  ((int)-2)

#define JSON_PARSE_CONTINUE                 0
#define JSON_PARSE_ABORT                    1
#define JSON_PARSE_NEGSIG_ABORT             2
#define JSON_PARSE_MASK_GRAPHQL             3

#define JSON_LOCATION_VALUE                 0
#define JSON_LOCATION_NAME                  1
#define JSON_LOCATION_NON_STRING            2

//return values of callback function
#define XML_PARSE_CONTINUE                  0 
#define XML_PARSE_ABORT                     1

#define XML_PARSE_RESET_TAG                 2       //tell parser to start looking for a new tag start (can used to parse html docs were tags not always close)
#define XML_PARSER_TREAT_AS_LEGAL_CHAR      3       //treat as legal char (allow an forbidden xml metachar e.g &#20z;)

//error level
#define XMLPARSER_ERROR_LEVEL_FATAL         1
#define XMLPARSER_ERROR_LEVEL_NONFATAL      2

//error level
#define JSONPARSER_ERROR_LEVEL_FATAL         1
#define JSONPARSER_ERROR_LEVEL_NONFATAL      2

//
#define XMLPARSER_ERROR_TYPE_DOCUMENT_MALFORMED                 1
#define XMLPARSER_ERROR_TYPE_DOCUMENT_SCHEMA_VALIDATION         2
#define XMLPARSER_ERROR_TYPE_ILLEGAL_ENCODING                   3
#define XMLPARSER_ERROR_TYPE_DEFENCE_ALERT                      4
#define XMLPARSER_ERROR_TYPE_INTERNAL                           5

#define JSONPARSER_ERROR_TYPE_DOCUMENT_MALFORMED                1
#define JSONPARSER_ERROR_TYPE_ILLEGAL_REQUEST                   2

typedef enum
{
    XMLPARSER_LOCATION_NONE = 0,
    XMLPARSER_LOCATION_ELEMENT_VALUE,
    XMLPARSER_LOCATION_ATTRIBUTE_VALUE,
    XMLPARSER_LOCATION_COMMENT_VALUE,
    XMLPARSER_LOCATION_PI_VALUE
}xml_location_type_t;

typedef struct xml_parser_error_info {
    int code;
    unsigned int level; //FATAL, NON-FATAL
    unsigned int type;  //document malformed, schema, encoding, defence, internal
    const char * error_desc;
}xml_parser_error_info;

typedef struct xml_error_location{
    unsigned char * buf;    // pointer to buffer where error occured
    int             index;  // offset from start of the chunk 
    int             len;    // length in bytes
}xml_error_location;

typedef struct xml_parser_error_info_ex {
    const char * extended_error_info;
}xml_parser_error_info_ex;

typedef struct xml_error_conf_property {
    const char * property_name;
    const char * property_xpointer;
    char actual_val[MAX_VALUE_TEXT_LEN];
    char expected_val[MAX_VALUE_TEXT_LEN];
    unsigned long json_payload_offset;
}xml_error_conf_property;

//parser statuses  - what parser_returns
enum {
    XML_PARSER_STATUS_SUCCESS = 0,                  // 0
    XML_PARSER_STATUS_CALLBACK_ABORT,               // 1
    XML_PARSER_STATUS_SYNTAX_ERROR,                 // 2
    XML_PARSER_STATUS_PARSER_INTERNAL_ERROR         // 3
};

//parser errors - what callback receives
enum {
    XML_PARSER_ERROR_SUCCESS = XML_PARSER_STATUS_SUCCESS,      // 0
    XML_PARSER_ERROR_TAG_NOT_CLOSED,                           // 1
    XML_PARSER_ERROR_NO_ROOT_ELEMENT,                          // 2
    XML_PARSER_ERROR_INPUT_STREAM_CORRUPT,                     // 3
    XML_PARSER_ERROR_SYNTAX_ERROR,                             // 4
    XML_PARSER_ERROR_ILLEGAL_XML_ENCODING,                     // 5
    XML_PARSER_ERROR_ILLEGAL_BOM_PLACE,                        // 6
    XML_PARSER_ERROR_ENCODING_OVERRIDES_BOM,                   // 7
    XML_PARSER_ERROR_ILLEGAL_DATA_BETWEEN_TAGS,                // 8
    XML_PARSER_ERROR_WHITESPACE_IN_TAG,                        // 9
    XML_PARSER_ERROR_NEW_LINE_IN_TAG,                          //10
    XML_PARSER_ERROR_TAG_OUTSIDE_ROOT_ELEMENT,                 //11
    XML_PARSER_ERROR_ILLEGAL_EMPTY_TAG,                        //12
    XML_PARSER_ERROR_ILLEGAL_PI_NAME,                          //13
    XML_PARSER_ERROR_ILLEGAL_CDATA_CONTENT,                    //14
    XML_PARSER_ERROR_ILLEGAL_CDATA_CLOSURE,                    //15
    XML_PARSER_ERROR_UNDEFINED_PREFIX,                         //16
    XML_PARSER_ERROR_DUPLICATE_ATTRIBUTE,                      //17
    XML_PARSER_ERROR_NAME_NOT_MATCH,                           //18
    XML_PARSER_ERROR_ILLEGAL_XML_VERSION,                      //19
    XML_PARSER_ERROR_ILLEGAL_XML_PROLOG_STANDALONE,            //20
    XML_PARSER_ERROR_ILLEGAL_PROLOG_QUOTATION_OR_APOSTROPHE,   //21
    XML_PARSER_ERROR_ILLEGAL_ESCAPE_CHARACTER,                 //22
    XML_PARSER_ERROR_ATTRIBUTE_WITHOUT_VALUE,                  //23
    XML_PARSER_ERROR_NAMESPACE_URI_NOT_ALLOWED,                //24
    XML_PARSER_ERROR_ILLEGAL_XSI_PREFIX,                       //25
    XML_PARSER_ERROR_NON_XSI_PREFIX,                           //26
    XML_PARSER_ERROR_ILLEGAL_XML_PREFIX,                       //27
    XML_PARSER_ERROR_NON_XML_PREFIX,                           //28
    XML_PARSER_ERROR_ILLEGAL_XLINK_PREFIX,                     //29
    XML_PARSER_ERROR_NON_XLINK_PREFIX,                         //30

    XML_PARSER_ERROR_INTERNAL,                                 //31
    XML_PARSER_ERROR_DEFENCE_PROFILE,                          //32

    // SCHEMA ERRORS (must appear as last id) 
    XML_PARSER_SCHEMA_ERRORS                                   //33
};

//defense parser status/error - what defense parser returns
enum {
    XML_DEFENSE_PARSER_STATUS_SUCCESS = 0,                // 0

    // internal errors
    XML_DEFENSE_PARSER_ERROR_INTERNAL,                    // 1
    XML_DEFENSE_PARSER_ERROR_DEFENCE_EXIT,                // 2

    // malformed document errors
    XML_DEFENSE_PARSER_ERROR_ENCODING_SEQUENCE,           // 3
    XML_DEFENSE_PARSER_ERROR_TAG_NOT_CLOSED,              // 4
    XML_DEFENSE_PARSER_ERROR_NO_ROOT_ELEMENT,             // 5
    XML_DEFENSE_PARSER_ERROR_SYNTAX_ERROR,                // 6
    XML_DEFENSE_PARSER_ERROR_ILLEGAL_XML_ENCODING,        // 7
    XML_DEFENSE_PARSER_ERROR_ILLEGAL_XML_VERSION,         // 8
    XML_DEFENSE_PARSER_ERROR_TAG_OUTSIDE_ROOT_ELEMENT,    // 9
    XML_DEFENSE_PARSER_ERROR_ILLEGAL_EMPTY_TAG,           // 10
    XML_DEFENSE_PARSER_ERROR_ILLEGAL_PI_NAME,             // 11
    XML_DEFENSE_PARSER_ERROR_ILLEGAL_PROPERTY_NAME,       // 12 
    XML_DEFENSE_PARSER_ERROR_DUPLICATE_ATTRIBUTE,         // 13
    XML_DEFENSE_PARSER_ERROR_CLOSE_TAG_NOT_MATCH_OPEN_TAG,// 14
    XML_DEFENSE_PARSER_ERROR_ILLEGAL_CDATA_CLOSURE,       // 15
    XML_DEFENSE_PARSER_ERROR_ENCODING_OVERRIDES_BOM,      // 16
    XML_DEFENSE_PARSER_ERROR_ILLEGAL_ESCAPE_CHARACTER,    // 17
    XML_DEFENSE_PARSER_ERROR_ILLEGAL_NUMERIC_NAME,        // 18
    XML_DEFENSE_PARSER_ERROR_ILLEGAL_LEADING_WHITESPACES, // 19
    XML_DEFENSE_PARSER_ERROR_ILLEGAL_CLOSE_SHORTHAND,     // 20

    // defense
    XML_DEFENSE_PARSER_ERROR_DEFENCE_PROFILE,             // 21

    NUM_XML_DEFENSE_STATUS
};

enum {
    XML_CANONICALIZATION_SUCCESS = 0,                  // = 0
    XML_CANONICALIZATION_ERROR,                        // = 1
    XML_CANONICALIZATION_INTERNAL_ERROR,               // = 2

    XML_CANONICALIZATION_RET_CODES_NUM
};

//wss parser parser status/error - what wss parser returns
enum {                                                  //subviolation code
    XML_WSS_INTERNAL_ERROR = 0,                         
    XML_WSS_MALFORMED_ERROR,                            
    XML_WSS_CERTIFICATE_EXPIRED,                        
    XML_WSS_CERTIFICATE_ERROR,                          
    XML_WSS_DECRYPTION_ERROR,                           
    XML_WSS_ENCRYPTION_ERROR,
    XML_WSS_SIGNING_ERROR,                              //64 
    XML_WSS_VERFICATION_ERROR,                          //128
    XML_WSS_MISSING_TIMESTAMP,                          //256
    XML_WSS_INVALID_TIMESTAMP,                          //512
    XML_WSS_EXPIRED_TIMESTAMP,                          //1024
    XML_WSS_TIMESTAMP_EXPIRATION_TOO_FAR_IN_THE_FUTURE, //2048
    XML_WSS_UNSIGNED_TIMESTAMP,                         //4096   

    XML_WSS_ERRORS_NUM
};

// JSON error codes
enum {
    // non-fatal
    JSONPARSER_ERROR_TAG_NOT_CLOSED = 0,         // = 0
    JSONPARSER_ERROR_NO_ROOT_ELEMENT,            // = 1 
    JSONPARSER_ERROR_WRONG_ARRAY_END,            // = 2
    JSONPARSER_ERROR_WRONG_OBJECT_END,           // = 3
    JSONPARSER_ERROR_ILLEGAL_STRING_CHARACTER,   // = 4
    JSONPARSER_ERROR_ILLEGAL_ESCAPING_SEQUENCE,  // = 5

    // fatal
    JSONPARSER_ERROR_SHORT_DOCUMENT,             // = 6
    JSONPARSER_ERROR_WRONG_ENCODING,             // = 7
    JSONPARSER_ERROR_ENCODING_SEQUENCE,          // = 8
    JSONPARSER_ERROR_MALFORMED_NUMBER,           // = 9 
    JSONPARSER_ERROR_OBJECT_END_IN_ARRAY_CONTEXT,// = 10
    JSONPARSER_ERROR_ARRAY_END_IN_OBJECT_CONTEXT,// = 11
    JSONPARSER_ERROR_SYNTAX_ERROR,               // = 12

    NUM_JSON_PARSER_ERRORS
};

// Error codes and types descriptions
// error types
#define PARSER_ERROR_MALFORMED_DOC                 "Malformed document"
#define PARSER_ERROR_MALFORMED_ENCODING            "Malformed document - illegal encoding"
#define PARSER_ERROR_SCHEMA_VALIDATION_FAILED      "Malformed document - schema validation failed"

// no context error messages
#define XMLERR_EMPTY_CONTEXT                       "Calling XML-parser with empty context!"
#define XMLERR_EMPTY_RESULT_CONTEXT                "Calling XML-parser with empty result context!"

// internal (callback) errors
#define XMLERR_NOMEM_FOR_ELEMENTS_OR_ATTRIBUTES    "Not enough memory for element(s) or attribute(s) value"
#define XMLERR_NOMEM_FOR_COMMENT                   "Not enough memory for comment(s) value"
#define XMLERR_NOMEM_FOR_PI                        "Not enough memory for PI(s) value"
#define XMLERR_NOMEM_FOR_URI                       "Not enough memory for namespace URI"
#define XMLERR_NOMEM_FOR_XPATH_STACK               "Not enough memory for XML stack growth"
#define XMLERR_FAILED_TO_DUP_CONTAINERS            "Failed to duplicate containers"
#define XMLERR_NOMEM                               "Not enough memory!"

// invalid encoding errors
#define XMLERR_UNSUPPORTED_ENCODING                "Unsupported encoding detected"
#define XMLERR_ENCODING_ILLEGAL_BOM                "Illegal BOM place"
#define XMLERR_ENCODING_OVERRIDES_BOM              "Encoding in prolig overrides BOM"

// schema errors desc
#define XMLERR_INVALID_ATTRIBUTE_VALUE             "Invalid attribute value"
#define XMLERR_ELEMENT_NOT_DEFINED_IN_SCHEMA	   "Element is not defined in schema"
#define XMLERR_ATTRIBUTE_NOT_DEFINED_IN_SCHEMA     "Attribute is not defined in schema"
#define XMLERR_NIL_VALUE_INVALID                   "xsi:nil value invalid"
#define XMLERR_NIL_TRUE_BUT_NOT_EMPTY              "xsi:nil='true' but value not empty"
#define XMLERR_NIL_NOT_ALLOWED_IN_ELEMENT          "xsi:nil not allowed in element"
#define XMLERR_INVALID_ELEMENT_VALUE               "Invalid element value"

// xpath errors
#define XMLERR_TAG_CONTAINS_NO_DATA                "Tag contains no data inside"
#define XMLERR_TAG_OUTSIDE_ROOT_ELEMENT            "Tag appears outside of root element"
#define XMLERR_INPUT_STREAM_CORRUPT                "Input stream corrupt"
#define XMLERR_MALFORMED_TAG_CLOSE                 "Close tag not match open tag"
#define XMLERR_ILLEGAL_NS_DECL                     "Illegal namespace declaration"
#define XMLERR_MALFORMED_ENCODING                  "Malformed encoding"
#define XMLERR_OUT_OF_MEMORY                       "Failed to allocate memory"
#define XMLERR_DEFENSE_ERROR                       "Defense value reached maximum"


#endif //_XMLERRORS_DESC_H_
