#ifndef _XML_PARSER_STATES_H_
#define _XML_PARSER_STATES_H_

typedef struct {
  int cdata_state;
  int element_value_state;
} cks_table_t;

#define SIZE_OF_DOCTYPE_SEQUENCE 7

static const short doctype_sequence[SIZE_OF_DOCTYPE_SEQUENCE] = { 
	XML_CHAR_CAPITAL_LETTER_D,
	XML_CHAR_CAPITAL_LETTER_O,
	XML_CHAR_CAPITAL_LETTER_C, 
	XML_CHAR_CAPITAL_LETTER_T, 
	XML_CHAR_CAPITAL_LETTER_Y,
	XML_CHAR_CAPITAL_LETTER_P,
	XML_CHAR_CAPITAL_LETTER_E
};

//#define syntax_parser_state_t unsigned int
typedef unsigned int syntax_parser_state_t;

#define XML_TOKENIZER_STATE_BASE 	0

// syntax parser state id
enum {
	XML_TOKENIZER_STATE_INITIAL = XML_TOKENIZER_STATE_BASE,
	XML_TOKENIZER_STATE_TOKEN_RESOLVE,

	XML_TOKENIZER_STATE_RESOLVE_ETD_OR_COMMENT_OR_DTD,
	XML_TOKENIZER_STATE_SCAN_TAG_OR_PROLOG_START,
	XML_TOKENIZER_STATE_SCAN_TAG_OR_PROLOG,
    XML_TOKENIZER_STATE_RESOLVE_PROLOG_X,
    XML_TOKENIZER_STATE_RESOLVE_PROLOG_M,
    XML_TOKENIZER_STATE_RESOLVE_PROLOG_L,
	XML_TOKENIZER_STATE_RESOLVE_PROLOG_S,

	XML_TOKENIZER_STATE_SCAN_TAG_START,
	XML_TOKENIZER_STATE_RESOLVE_TAG_TYPE, 

    XML_TOKENIZER_STATE_SCAN_COMMENT_BEGIN,
	XML_TOKENIZER_STATE_SCAN_COMMENT_VALUE_START,
	XML_TOKENIZER_STATE_SCAN_COMMENT_VALUE_END,
	XML_TOKENIZER_STATE_RESOLVE_COMMENT_END,
	XML_TOKENIZER_STATE_RESOLVE_COMMENT_TAG_CLOSE,

    XML_TOKENIZER_STATE_RESOLVE_CDATA_LEFT_SQUARE_BRACKET_1,
    XML_TOKENIZER_STATE_RESOLVE_CDATA_LETTER_C,
    XML_TOKENIZER_STATE_RESOLVE_CDATA_LETTER_D,
    XML_TOKENIZER_STATE_RESOLVE_CDATA_LETTER_A,
    XML_TOKENIZER_STATE_RESOLVE_CDATA_LETTER_T,
    XML_TOKENIZER_STATE_RESOLVE_CDATA_LETTER_A_2,

    XML_TOKENIZER_STATE_SCAN_CDATA_VALUE,
    XML_TOKENIZER_STATE_SCAN_CDATA_ONLY_WHITESPACE,

    XML_TOKENIZER_STATE_SCAN_CDATA_VALUE_START_WITH_COPY,
    XML_TOKENIZER_STATE_SCAN_CDATA_VALUE_END_WITH_COPY,
    XML_TOKENIZER_STATE_SKIP_WHITESPACE_IN_CDATA_VALUE_WITH_COPY,
    XML_TOKENIZER_STATE_SCAN_CDATA_ONLY_WHITESPACE_END,

    XML_TOKENIZER_STATE_RESOLVE_END_CDATA_1,
    XML_TOKENIZER_STATE_RESOLVE_END_CDATA_2,

    XML_TOKENIZER_STATE_SCAN_CDATA_VALUE_START,
    XML_TOKENIZER_STATE_SCAN_CDATA_VALUE_NUMBER,
    XML_TOKENIZER_STATE_SCAN_CDATA_VALUE_TOKEN,
    XML_TOKENIZER_STATE_SCAN_CDATA_VALUE_END,
    XML_TOKENIZER_STATE_RESOLVE_END_CDATA_1_WITH_HASH,
    XML_TOKENIZER_STATE_RESOLVE_END_CDATA_2_WITH_HASH,
	
	XML_TOKENIZER_STATE_RESOLVE_EXCLAMATION,
	XML_TOKENIZER_STATE_SCAN_DTD_END,
	XML_TOKENIZER_STATE_SCAN_DTD_INTERNAL_ELT,

    XML_TOKENIZER_STATE_ELEMENT,
	XML_TOKENIZER_STATE_START_TAG_ELEMENT_PREFIX_SCAN_END,
	XML_TOKENIZER_STATE_START_TAG_ELEMENT_NAME_SCAN_START,
	XML_TOKENIZER_STATE_START_TAG_ELEMENT_NAME_SCAN_END,

	//attribute related states
	XML_TOKENIZER_STATE_START_TAG_ATTRIBUTE_PREFIX_START_SCAN,
	XML_TOKENIZER_STATE_START_TAG_ATTRIBUTE_PREFIX_END_SCAN,
	XML_TOKENIZER_STATE_START_TAG_ATTRIBUTE_NAME_START_SCAN,
	XML_TOKENIZER_STATE_START_TAG_ATTRIBUTE_NAME_END_SCAN,
	XML_TOKENIZER_STATE_START_TAG_ATTRIBUTE_SCAN_EQUAL_SIGN,
	XML_TOKENIZER_STATE_START_TAG_ATTRIBUTE_SCAN_VALUE_START,
	
	//delimiters between attribute values and the attribute name that
	//follows or end of open tag
	XML_TOKENIZER_STATE_ATTRIBUTE_DELIM,

    // xsi type value scan
    XML_TOKENIZER_STATE_SCAN_XSITYPE_VALUE_START,
    XML_TOKENIZER_STATE_SCAN_XSITYPE_VALUE_PREFIX,
    XML_TOKENIZER_STATE_SCAN_XSITYPE_VALUE_NAME,
    XML_TOKENIZER_STATE_SCAN_XSITYPE_VALUE_END,

	//namespace definition attribute value
	XML_TOKENIZER_STATE_SCAN_NAMESPACE_VALUE,
	XML_TOKENIZER_STATE_SCAN_NAMESPACE_VALUE_START,
	XML_TOKENIZER_STATE_SCAN_NAMESPACE_VALUE_END,
	XML_TOKENIZER_STATE_SCAN_NAMESPACE_VALUE_SKIP_WS,

	//attribute value
	XML_TOKENIZER_STATE_SCAN_ATTRIBUTE_VALUE,
	XML_TOKENIZER_STATE_SCAN_ATTRIBUTE_VALUE_SKIP_WS,

	//attribute value with calculatin of hash value
	XML_TOKENIZER_STATE_SCAN_ATTRIBUTE_VALUE_START,
	XML_TOKENIZER_STATE_SCAN_ATTRIBUTE_VALUE_NUMBER,
	XML_TOKENIZER_STATE_SCAN_ATTRIBUTE_VALUE_TOKEN,
	XML_TOKENIZER_STATE_SCAN_ATTRIBUTE_VALUE_END,

	//element value
	XML_TOKENIZER_STATE_SCAN_ELEMENT_VALUE,
	XML_TOKENIZER_STATE_SKIP_WHITESPACE_IN_ELEMENT_VALUE,
	XML_TOKENIZER_STATE_SCAN_ELEMENT_VALUE_RIGHT_SQUARE_BRACKET_1,
	XML_TOKENIZER_STATE_SCAN_ELEMENT_VALUE_RIGHT_SQUARE_BRACKET_2,

	//element value with copy to buffer
	XML_TOKENIZER_STATE_SCAN_ELEMENT_VALUE_START_WITH_COPY,
	XML_TOKENIZER_STATE_SCAN_ELEMENT_VALUE_END_WITH_COPY,
	XML_TOKENIZER_STATE_SKIP_WHITESPACE_IN_ELEMENT_VALUE_WITH_COPY,

	//element value with calculatin of hash value
	XML_TOKENIZER_STATE_SCAN_ELEMENT_VALUE_START,
	XML_TOKENIZER_STATE_SCAN_ELEMENT_VALUE_NUMBER,
	XML_TOKENIZER_STATE_SCAN_ELEMENT_VALUE_TOKEN,
	XML_TOKENIZER_STATE_SCAN_ELEMENT_VALUE_END,

	// Post-validation of attributes.
	XML_TOKENIZER_STATE_POST_VALIDATION_START_VALUE,
	XML_TOKENIZER_STATE_POST_VALIDATION_START,
	XML_TOKENIZER_STATE_POST_VALIDATION_END,
	XML_TOKENIZER_STATE_POST_VALIDATION_SKIP_WS,

	//ignores validation
	XML_TOKENIZER_STATE_SCAN_ELEMENT_VALUE_START_AS_MIXED_CONTENT,
	XML_TOKENIZER_STATE_SKIP_WHITESPACE_IN_UNRESTRICTED_VALUE,
	XML_TOKENIZER_STATE_SCAN_ELEMENT_VALUE_END_AS_MIXED_CONTENT,

    //element end tag
	XML_TOKENIZER_STATE_END_TAG_ELEMENT_NAME_PREFIX_SCAN_START,
	XML_TOKENIZER_STATE_END_TAG_ELEMENT_NAME_PREFIX_SCAN_END,
	XML_TOKENIZER_STATE_END_TAG_ELEMENT_NAME_SCAN_START,
	XML_TOKENIZER_STATE_END_TAG_ELEMENT_NAME_SCAN_END,
	XML_TOKENIZER_STATE_END_TAG_ELEMENT_NAME_SKIP_WHITESPACE,
	XML_TOKENIZER_STATE_EMPTY_TAG_VERIFY_TAG_END,
	
	//prolog
	XML_TOKENIZER_STATE_PROLOG_SCAN_PROPERTY_NAME_START,
	XML_TOKENIZER_STATE_PROLOG_SCAN_PROPERTY_NAME_END,
	XML_TOKENIZER_STATE_PROLOG_SCAN_EQUAL_SIGN,
	XML_TOKENIZER_STATE_PROLOG_SCAN_PROPERTY_VALUE_START,
	XML_TOKENIZER_STATE_PROLOG_SCAN_PROPERTY_VALUE_END,
	XML_TOKENIZER_STATE_PROLOG_DECLARATION_VALIDATE_END,
	
	XML_TOKENIZER_STATE_START_TAG_NAMESPACE_NAME_START_SCAN,

	// "<?"  start PI (processing instructions)
	XML_TOKENIZER_STATE_RESOLVE_PI_OR_PROLOG,
	XML_TOKENIZER_STATE_PI_OR_PROLOG_SCAN_NAME_END,

	XML_TOKENIZER_STATE_RESOLVE_PI,
	XML_TOKENIZER_STATE_PI_SCAN_NAME_END,
	XML_TOKENIZER_STATE_PI_SCAN_END,
	XML_TOKENIZER_STATE_PI_SCAN_END2,
	XML_TOKENIZER_STATE_PI_SCAN_GT,
	XML_TOKENIZER_STATE_PI_SCAN_GT2,
	XML_TOKENIZER_STATE_PI_SCAN_GT3,

	// Normalization group
	XML_TOKENIZER_STATE_NORM_START,
	XML_TOKENIZER_STATE_NORM_LOOK_FOR_T,
	XML_TOKENIZER_STATE_NORM_LOOK_FOR_M_P,
	XML_TOKENIZER_STATE_NORM_LOOK_FOR_P,
	XML_TOKENIZER_STATE_NORM_LOOK_FOR_O,
	XML_TOKENIZER_STATE_NORM_LOOK_FOR_S,
	XML_TOKENIZER_STATE_NORM_LOOK_FOR_U,
	XML_TOKENIZER_STATE_NORM_FINISH,
	XML_TOKENIZER_STATE_NORM_CHAR_REF_START,
	XML_TOKENIZER_STATE_NORM_HEX_CHAR_REF,
	XML_TOKENIZER_STATE_NORM_DEC_CHAR_REF,

	// Functions
	XML_TOKENIZER_STATE_FUNCTION,

    // decimal automat
    XML_DECIMAL_STATE__AUTOMAT_NOT_OK,
    XML_DECIMAL_STATE__SCAN_SIGN_OR_DOT_OR_DIGIT,
    XML_DECIMAL_STATE__SKIP_LEADING_ZERO_SEQUENCE,
    XML_DECIMAL_STATE__SCAN_DIGIT_OR_DOT,
    XML_DECIMAL_STATE__SCAN_FIRST_FRACTION_DIGIT,
    XML_DECIMAL_STATE__SCAN_ZERO_SEQUENCE_END,
    XML_DECIMAL_STATE__SCAN_INT_END,
    XML_DECIMAL_STATE__SCAN_FRACTION_END,

    // fast skip states
    XML_FAST_SKIP_STATE_TAG_TYPE,
    XML_FAST_SKIP_STATE_DTD_COMMENT,
    XML_FAST_SKIP_STATE_SCAN_COMMENT_BEGIN,
	XML_FAST_SKIP_STATE_SCAN_COMMENT_VALUE_START,
	XML_FAST_SKIP_STATE_SCAN_COMMENT_VALUE_END,
	XML_FAST_SKIP_STATE_RESOLVE_COMMENT_END,
	XML_FAST_SKIP_STATE_RESOLVE_COMMENT_TAG_CLOSE,
    XML_FAST_SKIP_STATE_SCAN_ELEMENT_VALUE,
	XML_FAST_SKIP_STATE_PI_SCAN,
	XML_FAST_SKIP_STATE_PI_SCAN_END,
	XML_FAST_SKIP_STATE_RESOLVE_CDATA,
	XML_FAST_SKIP_STATE_SCAN_CDATA_VALUE,
	XML_FAST_SKIP_STATE_SCAN_CDATA_CLOSE_1,
	XML_FAST_SKIP_STATE_SCAN_CDATA_CLOSE_2,
	XML_FAST_SKIP_STATE_SCAN_DTD_END,      
	XML_FAST_SKIP_STATE_SCAN_DTD_INTERNAL_ELT,
	XML_FAST_SKIP_STATE_SCAN_START_TAG,
    XML_FAST_SKIP_STATE_SCAN_START_TAG_ATTR_1,
    XML_FAST_SKIP_STATE_SCAN_START_TAG_ATTR_2,
    XML_FAST_SKIP_STATE_FETCH_ATTR_1,
    XML_FAST_SKIP_STATE_FETCH_ATTR_2,
	XML_FAST_SKIP_STATE_SCAN_END_TAG,
	XML_FAST_SKIP_STATE_EMPTY_TAG_VERIFY_TAG_END,

    XML_TOKENIZER_STATE_SCAN_DTD_NAME,

    // states for fetching hash-value.
    XML_FAST_SKIP_STATE_SCAN_CALC_HASH_VALUE,
    XML_FAST_SKIP_STATE_CALC_HASH_FOR_ATTR_1,
    XML_FAST_SKIP_STATE_CALC_HASH_FOR_ATTR_2,
    XML_FAST_SKIP_STATE_IGNORE_NUMBER_SIGN, // when reading '#'
    
    MAX_TOKENIZER_STATES
};


#define XMLTOKENIZER_JT_DESC()   static const char tokenizer_jt_desc[MAX_TOKENIZER_STATES][64] = {\
	"XML_TOKENIZER_STATE_INITIAL",\
	"XML_TOKENIZER_STATE_TOKEN_RESOLVE",\
	"XML_TOKENIZER_STATE_RESOLVE_ETD_OR_COMMENT_OR_DTD",\
	"XML_TOKENIZER_STATE_SCAN_TAG_OR_PROLOG_START",\
	"XML_TOKENIZER_STATE_SCAN_TAG_OR_PROLOG",\
    "XML_TOKENIZER_STATE_RESOLVE_PROLOG_X",\
    "XML_TOKENIZER_STATE_RESOLVE_PROLOG_M",\
    "XML_TOKENIZER_STATE_RESOLVE_PROLOG_L",\
	"XML_TOKENIZER_STATE_RESOLVE_PROLOG_S",\
	"XML_TOKENIZER_STATE_SCAN_TAG_START",\
	"XML_TOKENIZER_STATE_RESOLVE_TAG_TYPE",\
    "XML_TOKENIZER_STATE_SCAN_COMMENT_BEGIN",\
	"XML_TOKENIZER_STATE_SCAN_COMMENT_VALUE_START",\
	"XML_TOKENIZER_STATE_SCAN_COMMENT_VALUE_END",\
	"XML_TOKENIZER_STATE_RESOLVE_COMMENT_END",\
	"XML_TOKENIZER_STATE_RESOLVE_COMMENT_TAG_CLOSE",\
    "XML_TOKENIZER_STATE_RESOLVE_CDATA_LEFT_SQUARE_BRACKET_1",\
    "XML_TOKENIZER_STATE_RESOLVE_CDATA_LETTER_C",\
    "XML_TOKENIZER_STATE_RESOLVE_CDATA_LETTER_D",\
    "XML_TOKENIZER_STATE_RESOLVE_CDATA_LETTER_A",\
    "XML_TOKENIZER_STATE_RESOLVE_CDATA_LETTER_T",\
    "XML_TOKENIZER_STATE_RESOLVE_CDATA_LETTER_A_2",\
    "XML_TOKENIZER_STATE_SCAN_CDATA_VALUE",\
    "XML_TOKENIZER_STATE_SCAN_CDATA_ONLY_WHITESPACE",\
    "XML_TOKENIZER_STATE_SCAN_CDATA_VALUE_START_WITH_COPY",\
    "XML_TOKENIZER_STATE_SCAN_CDATA_VALUE_END_WITH_COPY",\
    "XML_TOKENIZER_STATE_SKIP_WHITESPACE_IN_CDATA_VALUE_WITH_COPY",\
    "XML_TOKENIZER_STATE_SCAN_CDATA_ONLY_WHITESPACE_END",\
    "XML_TOKENIZER_STATE_RESOLVE_END_CDATA_1",\
    "XML_TOKENIZER_STATE_RESOLVE_END_CDATA_2",\
    "XML_TOKENIZER_STATE_SCAN_CDATA_VALUE_START",\
    "XML_TOKENIZER_STATE_SCAN_CDATA_VALUE_NUMBER",\
    "XML_TOKENIZER_STATE_SCAN_CDATA_VALUE_TOKEN",\
    "XML_TOKENIZER_STATE_SCAN_CDATA_VALUE_END",\
    "XML_TOKENIZER_STATE_RESOLVE_END_CDATA_1_WITH_HASH",\
    "XML_TOKENIZER_STATE_RESOLVE_END_CDATA_2_WITH_HASH",\
	"XML_TOKENIZER_STATE_RESOLVE_EXCLAMATION",\
	"XML_TOKENIZER_STATE_SCAN_DTD_END",\
	"XML_TOKENIZER_STATE_SCAN_DTD_INTERNAL_ELT",\
    "XML_TOKENIZER_STATE_ELEMENT",\
	"XML_TOKENIZER_STATE_START_TAG_ELEMENT_PREFIX_SCAN_END",\
	"XML_TOKENIZER_STATE_START_TAG_ELEMENT_NAME_SCAN_START",\
	"XML_TOKENIZER_STATE_START_TAG_ELEMENT_NAME_SCAN_END",\
	"XML_TOKENIZER_STATE_START_TAG_ATTRIBUTE_PREFIX_START_SCAN",\
	"XML_TOKENIZER_STATE_START_TAG_ATTRIBUTE_PREFIX_END_SCAN",\
	"XML_TOKENIZER_STATE_START_TAG_ATTRIBUTE_NAME_START_SCAN",\
	"XML_TOKENIZER_STATE_START_TAG_ATTRIBUTE_NAME_END_SCAN",\
	"XML_TOKENIZER_STATE_START_TAG_ATTRIBUTE_SCAN_EQUAL_SIGN",\
	"XML_TOKENIZER_STATE_START_TAG_ATTRIBUTE_SCAN_VALUE_START",\
	"XML_TOKENIZER_STATE_ATTRIBUTE_DELIM",\
    "XML_TOKENIZER_STATE_SCAN_XSITYPE_VALUE_START",\
    "XML_TOKENIZER_STATE_SCAN_XSITYPE_VALUE_PREFIX",\
    "XML_TOKENIZER_STATE_SCAN_XSITYPE_VALUE_NAME",\
    "XML_TOKENIZER_STATE_SCAN_XSITYPE_VALUE_END",\
	"XML_TOKENIZER_STATE_SCAN_NAMESPACE_VALUE",\
	"XML_TOKENIZER_STATE_SCAN_NAMESPACE_VALUE_START",\
	"XML_TOKENIZER_STATE_SCAN_NAMESPACE_VALUE_END",\
	"XML_TOKENIZER_STATE_SCAN_NAMESPACE_VALUE_SKIP_WS",\
	"XML_TOKENIZER_STATE_SCAN_ATTRIBUTE_VALUE",\
	"XML_TOKENIZER_STATE_SCAN_ATTRIBUTE_VALUE_SKIP_WS",\
	"XML_TOKENIZER_STATE_SCAN_ATTRIBUTE_VALUE_START",\
	"XML_TOKENIZER_STATE_SCAN_ATTRIBUTE_VALUE_NUMBER",\
	"XML_TOKENIZER_STATE_SCAN_ATTRIBUTE_VALUE_TOKEN",\
	"XML_TOKENIZER_STATE_SCAN_ATTRIBUTE_VALUE_END",\
	"XML_TOKENIZER_STATE_SCAN_ELEMENT_VALUE",\
	"XML_TOKENIZER_STATE_SKIP_WHITESPACE_IN_ELEMENT_VALUE",\
	"XML_TOKENIZER_STATE_SCAN_ELEMENT_VALUE_RIGHT_SQUARE_BRACKET_1",\
	"XML_TOKENIZER_STATE_SCAN_ELEMENT_VALUE_RIGHT_SQUARE_BRACKET_2",\
	"XML_TOKENIZER_STATE_SCAN_ELEMENT_VALUE_START_WITH_COPY",\
	"XML_TOKENIZER_STATE_SCAN_ELEMENT_VALUE_END_WITH_COPY",\
	"XML_TOKENIZER_STATE_SKIP_WHITESPACE_IN_ELEMENT_VALUE_WITH_COPY",\
	"XML_TOKENIZER_STATE_SCAN_ELEMENT_VALUE_START",\
	"XML_TOKENIZER_STATE_SCAN_ELEMENT_VALUE_NUMBER",\
	"XML_TOKENIZER_STATE_SCAN_ELEMENT_VALUE_TOKEN",\
	"XML_TOKENIZER_STATE_SCAN_ELEMENT_VALUE_END",\
	"XML_TOKENIZER_STATE_POST_VALIDATION_START_VALUE",\
	"XML_TOKENIZER_STATE_POST_VALIDATION_START",\
	"XML_TOKENIZER_STATE_POST_VALIDATION_END",\
	"XML_TOKENIZER_STATE_POST_VALIDATION_SKIP_WS",\
	"XML_TOKENIZER_STATE_SCAN_ELEMENT_VALUE_START_AS_MIXED_CONTENT",\
	"XML_TOKENIZER_STATE_SKIP_WHITESPACE_IN_UNRESTRICTED_VALUE",\
	"XML_TOKENIZER_STATE_SCAN_ELEMENT_VALUE_END_AS_MIXED_CONTENT",\
	"XML_TOKENIZER_STATE_END_TAG_ELEMENT_NAME_PREFIX_SCAN_START",\
	"XML_TOKENIZER_STATE_END_TAG_ELEMENT_NAME_PREFIX_SCAN_END",\
	"XML_TOKENIZER_STATE_END_TAG_ELEMENT_NAME_SCAN_START",\
	"XML_TOKENIZER_STATE_END_TAG_ELEMENT_NAME_SCAN_END",\
	"XML_TOKENIZER_STATE_END_TAG_ELEMENT_NAME_SKIP_WHITESPACE",\
	"XML_TOKENIZER_STATE_EMPTY_TAG_VERIFY_TAG_END",\
	"XML_TOKENIZER_STATE_PROLOG_SCAN_PROPERTY_NAME_START",\
	"XML_TOKENIZER_STATE_PROLOG_SCAN_PROPERTY_NAME_END",\
	"XML_TOKENIZER_STATE_PROLOG_SCAN_EQUAL_SIGN",\
	"XML_TOKENIZER_STATE_PROLOG_SCAN_PROPERTY_VALUE_START",\
	"XML_TOKENIZER_STATE_PROLOG_SCAN_PROPERTY_VALUE_END",\
	"XML_TOKENIZER_STATE_PROLOG_DECLARATION_VALIDATE_END",\
	"XML_TOKENIZER_STATE_START_TAG_NAMESPACE_NAME_START_SCAN",\
	"XML_TOKENIZER_STATE_RESOLVE_PI_OR_PROLOG",\
	"XML_TOKENIZER_STATE_PI_OR_PROLOG_SCAN_NAME_END",\
    "XML_TOKENIZER_STATE_RESOLVE_PI",\
	"XML_TOKENIZER_STATE_PI_SCAN_NAME_END",\
	"XML_TOKENIZER_STATE_PI_SCAN_END",\
	"XML_TOKENIZER_STATE_PI_SCAN_END2",\
	"XML_TOKENIZER_STATE_PI_SCAN_GT",\
	"XML_TOKENIZER_STATE_PI_SCAN_GT2",\
	"XML_TOKENIZER_STATE_PI_SCAN_GT3",\
	"XML_TOKENIZER_STATE_NORM_START",\
	"XML_TOKENIZER_STATE_NORM_LOOK_FOR_T",\
	"XML_TOKENIZER_STATE_NORM_LOOK_FOR_M_P",\
	"XML_TOKENIZER_STATE_NORM_LOOK_FOR_P",\
	"XML_TOKENIZER_STATE_NORM_LOOK_FOR_O",\
	"XML_TOKENIZER_STATE_NORM_LOOK_FOR_S",\
	"XML_TOKENIZER_STATE_NORM_LOOK_FOR_U",\
	"XML_TOKENIZER_STATE_NORM_FINISH",\
	"XML_TOKENIZER_STATE_NORM_CHAR_REF_START",\
	"XML_TOKENIZER_STATE_NORM_HEX_CHAR_REF",\
	"XML_TOKENIZER_STATE_NORM_DEC_CHAR_REF",\
	"XML_TOKENIZER_STATE_FUNCTION",\
    "XML_DECIMAL_STATE__AUTOMAT_NOT_OK",\
    "XML_DECIMAL_STATE__SCAN_SIGN_OR_DOT_OR_DIGIT",\
    "XML_DECIMAL_STATE__SKIP_LEADING_ZERO_SEQUENCE",\
    "XML_DECIMAL_STATE__SCAN_DIGIT_OR_DOT",\
    "XML_DECIMAL_STATE__SCAN_FIRST_FRACTION_DIGIT",\
    "XML_DECIMAL_STATE__SCAN_ZERO_SEQUENCE_END",\
    "XML_DECIMAL_STATE__SCAN_INT_END",\
    "XML_DECIMAL_STATE__SCAN_FRACTION_END",\
    "XML_FAST_SKIP_STATE_TAG_TYPE",\
    "XML_FAST_SKIP_STATE_DTD_COMMENT",\
    "XML_FAST_SKIP_STATE_SCAN_COMMENT_BEGIN",\
	"XML_FAST_SKIP_STATE_SCAN_COMMENT_VALUE_START",\
	"XML_FAST_SKIP_STATE_SCAN_COMMENT_VALUE_END",\
	"XML_FAST_SKIP_STATE_RESOLVE_COMMENT_END",\
	"XML_FAST_SKIP_STATE_RESOLVE_COMMENT_TAG_CLOSE",\
    "XML_FAST_SKIP_STATE_SCAN_ELEMENT_VALUE",\
	"XML_FAST_SKIP_STATE_PI_SCAN",\
	"XML_FAST_SKIP_STATE_PI_SCAN_END",\
	"XML_FAST_SKIP_STATE_RESOLVE_CDATA",\
	"XML_FAST_SKIP_STATE_SCAN_CDATA_VALUE",\
	"XML_FAST_SKIP_STATE_SCAN_CDATA_CLOSE_1",\
	"XML_FAST_SKIP_STATE_SCAN_CDATA_CLOSE_2",\
	"XML_FAST_SKIP_STATE_SCAN_DTD_END", \
	"XML_FAST_SKIP_STATE_SCAN_DTD_INTERNAL_ELT",\
	"XML_FAST_SKIP_STATE_SCAN_START_TAG",\
    "XML_FAST_SKIP_STATE_SCAN_START_TAG_ATTR_1",\
    "XML_FAST_SKIP_STATE_SCAN_START_TAG_ATTR_2",\
    "XML_FAST_SKIP_STATE_FETCH_ATTR_1",\
    "XML_FAST_SKIP_STATE_FETCH_ATTR_2",\
	"XML_FAST_SKIP_STATE_SCAN_END_TAG",\
	"XML_FAST_SKIP_STATE_EMPTY_TAG_VERIFY_TAG_END",\
    "XML_TOKENIZER_STATE_SCAN_DTD_NAME",\
}; 


#endif //_XML_PARSER_STATES_H_

