package F5::ASM::COAPI::File;
use strict;
use warnings;

use F5::Deploy;

my $login_name = 'root';

sub set_login_name {
    my($class, $new_login_name) = @_;
    $login_name = $new_login_name;
}

sub Login {
    my($new_login_name) = @_;

    $login_name = $new_login_name;
    return $login_name;
}

sub bypass_validation {
    if (!$login_name || substr($login_name,0,1) ne '%') {
        Login('%asmconfig'); #Special name that will still trigger CMI sync events (tm_daemon/mcp/framework/request_group.cpp)
    }
}


sub TransactionalSave {
    my ($transaction) = @_;

    my %seen_tables = ();
    for my $object (@$transaction) {
        F5::ASM::COAPI::File::Storage::store_object(object => $object);
        $seen_tables{$object->{__db_table}} = 1;
    }
    for my $table (keys %seen_tables) {
        F5::ASM::COAPI::File::Storage::save_table($table);
    }
    return scalar @$transaction;
}

sub is_fflag_enabled {
    my ($flag, %params) = @_;
    return 1;
}

#----------------------------------------------------------------------


sub SaveAll {
    my(%params) = @_;
    my $logger = $params{logger};

    return;
}

#======================================================================

package F5::ASM::COAPI::File::Storage;

use YAML::XS;
use File::Basename;

my $DB_DIR = F5::Deploy::map_path('/ts/var/coapi/');
my $table_cache = {};
our $FOLDERIZE = 1;

sub get_table {
    my ($table) = @_;

    if (!$table_cache->{$table}) {
        $table_cache->{$table} = {};
        my $db_file = "$DB_DIR/$table.yaml";
        if (-f $db_file) {
            my $data = YAML::XS::LoadFile($db_file);
            %{$table_cache->{$table}} = map { (($_->{name} || $_->{category}) => $_) } @$data;
        }
    }
    my @data = values %{$table_cache->{$table}};
    return \@data;
}

sub store_object {
    my (%params) = @_;
    my $object = $params{object};
    my $save_table = $params{save_table};

    my $table = $object->{__db_table};
    if (!exists ($table_cache->{$table})) {
        get_table($table);
    }
    my $key = $object->{name} || $object->{category};
    if ($FOLDERIZE) {
        my @parts = split('/', $key);
        $object->{partition_id} ||= $parts[1] || "";
        $object->{leaf_name} ||= $parts[-1];
        $object->{folder_name} ||= $parts[1] ? dirname($key) : "";
    }

    require F5::ASMConfig::Entity::Base;
    my $entity = F5::ASMConfig::Entity::Base->get_cached_entity_by_mcp_type($table);
    if (defined $entity) {
        my $mcp_index_field_name = $entity->mcp_index_field_name;
        if ($mcp_index_field_name && ! defined $object->{$mcp_index_field_name}) {
            my $max_value = 0;
            for my $rec (values (%{$table_cache->{$table}})) {
                if ($rec->{$mcp_index_field_name} > $max_value) {
                    $max_value = $rec->{$mcp_index_field_name};
                }
            }
            $object->{$mcp_index_field_name} = $max_value+1;
        }
    }
    
    if ($object->{__delete}) {
        delete $table_cache->{$table}->{$key};  
    }
    else {
        $table_cache->{$table}->{$key} = { %{$table_cache->{$table}->{$key} || {}}, %$object };
    }

    if ($save_table) {
        save_table($table);
    }
}

sub save_table {
    my ($table) = @_;
    if (exists ($table_cache->{$table})) {
        my $db_file = "$DB_DIR/$table.yaml";
        my @data = values %{$table_cache->{$table}};
        YAML::XS::DumpFile($db_file, \@data);
    }
}

sub clear_cache {
    my ($self, %params) = @_;
    if ($params{table}) {
        delete $table_cache->{$params{table}};
    }
    else {
        $table_cache = {};
    }
}


#======================================================================

package F5::ASM::COAPI::File::Query;

sub new {
    my ($self, $table, $filter, %params) = @_;

    my $query = F5::ASM::COAPI::File::Storage::get_table($table);

    if ($filter && ref($filter)) {
        $query = [ grep { filter_match(filter => $filter, rec => $_) } @$query ];
    }

    $query = [ map { $_ = bless {%$_, __db_table => $table}, 'F5::ASM::COAPI::File::Object' } @$query ];
    return $query;
}

sub clear_cache {
    my ($self, %params) = @_;
    F5::ASM::COAPI::File::Storage::clear_cache(%params);   
}

sub filter_match {
    my(%params) = @_;
    my $filter = $params{filter};
    my $rec = $params{rec};

    for my $k (keys %$filter) {
        if ($rec->{$k} ne $filter->{$k}) {
            return;
        }
    }
    return 1;
}

#======================================================================

package F5::ASM::COAPI::File::Object;

sub new {
    my($class, $table) = @_;

    return bless({__db_table => $table}, $class);
}

sub save {
    my($self, @params) = @_;
    F5::ASM::COAPI::File::Storage::store_object(object => $self,
                                                save_table => 1);
}

sub delete {
    my($self, @params) = @_;
    $self->{__delete} = 1;
    $self->save(@params);
}

sub delete_on_save {
    my($self, @params) = @_;
    $self->{__delete} = 1;
}

# this is a setter that will set only defined values
# setting undefined values into a coapi object throws an error
sub set_value {
    my ($self, $field, $value) = @_;

    $self->{$field} = $value if defined $value;
}

#======================================================================

1;
