package F5::ASMConfig::ContentProfileBase;

use strict;
use warnings;

use base qw/F5::ASMConfig::Entity::Base/;
use F5::ASMConfig::Entity::ContentProfile;

our $DEFAULT_PROFILE_NAME = 'Default';

#----------------------------------------------------------------------

#Always returns the subclassed Content Profile, even if called from F5::ASMConfig::Entity::ContentProfile
sub get_default_content_profile {
    my ($class, %params) = @_;

    my $dbh = $params{dbh};
    my $policy = $params{policy};
    my $created_by = $params{created_by} || 'GUI';
    my $content_profile_type = $params{content_profile_type} || $class->fields->{content_profile_type}->{const_value};

    if (! $content_profile_type) {
        my $content_profile_error_message = "Uknown content profile type for $class.";
        die { error_message => $content_profile_error_message,
              display_message => $content_profile_error_message,
              error_level     => 'info',
              error_code      => F5::ASMConfig::ErrorCode::UNKNOWN(),
            };
    }

    my $class_name = F5::ASMConfig::Entity::ContentProfile->fields->{content_profile_type}->{recast_map}->{$content_profile_type};

    my $default_profile = $class_name->new(
                                            dbh => $dbh,
                                            policy => $policy,
                                            get_criteria => {
                                                profile_name => $DEFAULT_PROFILE_NAME,
                                                content_profile_type => $content_profile_type,
                                            });
    if (! $default_profile) {
        $default_profile = $class_name->new(dbh => $dbh,
                                            policy => $policy,
                                            preload_default => 1,                                                    
                                            object => {
                                                        profile_name => $DEFAULT_PROFILE_NAME,
                                                        created_by => $created_by,
                                                        content_profile_type => $content_profile_type,
                                            });
        $default_profile->{__new_entity} = 1;
    }

    return $default_profile;
}

#----------------------------------------------------------------------

1;
