package F5::ASMConfig::EasyClient;

use strict;
use warnings;

use POSIX;
use IO::Socket::INET6;
use IO::Select;
use Thrift::BufferedTransport;
use Thrift::FramedTransport;
use Thrift::Socket;
use Thrift::BinaryProtocol;

my $DEFAULT_RPC_SERVER_PORT = 9781;
my $DEFAULT_EVENT_QUEUE_PORT = 9783;

use F5::ASMConfig::ASMConfig;
use base qw/F5::ASMConfig::ASMConfigClient/;

#----------------------------------------------------------------------

sub new {
  my($class, %params) = @_;
  my $host = $params{host} || '127.0.0.1';
  my $rpc_server_port = $params{rpc_server_port} || $DEFAULT_RPC_SERVER_PORT;
  my $transport = Thrift::BufferedTransport->new( Thrift::Socket->new({
                                                                       host => $host,
                                                                       port => $rpc_server_port,
                                                                       recvTimeout => 3600000,
                                                                       sendTimeout => 3600000,
                                                                      }),
                                                  1024, 1024);
  my $self = $class->SUPER::new( Thrift::BinaryProtocol->new($transport) );

  $self->{rpc_server_port} = $rpc_server_port;
  $self->{event_queue_port} = $params{event_queue_port} || $DEFAULT_EVENT_QUEUE_PORT;
  $transport->open();
  $self->{session} = $self->login($params{username});

  $self->{session}->{gui_user_name} = $params{gui_user_name} || '';
  $self->{session}->{partition} = $params{partition} || 'Common';
  $self->{host} = $host;
  return $self;
}

#----------------------------------------------------------------------

sub init_event_queue {
  my($self) = @_;
  $self->{event_socket} = IO::Socket::INET6->new(PeerAddr => $self->{host},
                                                 PeerPort => $self->{event_queue_port});
  #  nonblock($self->{event_socket});
#  $self->{event_socket} = Thrift::Socket->new('127.0.0.1', $ASM_CONFIG_EVENT_QUEUE_PORT);
#  $self->{event_socket}->open;
  $self->{event_socket}->send($self->{session}->{session_id} . "\n");

  $self->{event_select} = IO::Select->new($self->{event_socket});
  my $transport = Thrift::FramedTransport->new(Thrift::Socket->new({socket => $self->{event_socket}}));
  $self->{event_protocol} = Thrift::BinaryProtocol->new($transport);
}

#----------------------------------------------------------------------

sub has_waiting_events {
  my($self) = @_;
  if ($self->{event_select}->can_read(0)) {
    return 1;
  }
  return;
}

#----------------------------------------------------------------------

sub get_next_event {
  my($self) = @_;

#  my $buf = '';
#  my $rv = $self->{event_socket}->recv($buf,POSIX::BUFSIZ , 0);
#  print "Buffer = $buf\n";
#  return $buf;
  my $event = F5::ASMConfig::Event->new;
  $event->read($self->{event_protocol});
  return $event;
}

#----------------------------------------------------------------------
# nonblock($socket) puts socket into nonblocking mode
sub nonblock {
  my $socket = shift;
  my $flags;

  $flags = fcntl($socket, F_GETFL, 0)
    or die "Can't get flags for socket: $!\n";
  fcntl($socket, F_SETFL, $flags | O_NONBLOCK)
    or die "Can't make socket nonblocking: $!\n";
}

#----------------------------------------------------------------------

1;
