package F5::ASMConfig::Entity::ActionItem;

use strict;
use warnings;

use F5::ASMConfig::ASMConfig;
use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::ActionItem/;
use F5::ASMConfig::Entity::Policy;

#----------------------------------------------------------------------

sub retrieve_all {
    my($class, %params) = @_;

    my $dbh = $params{dbh};
    my $logger = $params{logger};
    my $get_criteria = $params{get_criteria} || {};
    my $fields = $class->fields;


    my @objs = $class->SUPER::retrieve_all(%params);
    my @extra_objs;

    my $get_policy_summary_items = !$params{for_populate_uuids};

    if ($get_criteria->{action_item_type} && $get_criteria->{action_item_type} != F5::ASMConfig::ActionItemType::POLICY_ACTION_ITEM_SUMMARY()) {
        $get_policy_summary_items = 0;
    }
    if ($params{search_filters}) {        
        for my $filter (grep {$_->{field_name} eq "action_item_type"} @{$params{search_filters}}) {
            if (!$class->is_filter_match(search_filter => $filter, field_value => F5::ASMConfig::ActionItemType::POLICY_ACTION_ITEM_SUMMARY())) {
                $get_policy_summary_items = 0;
            }
        }
    }
    if ( $get_policy_summary_items ) {

        my @policies = F5::ASMConfig::Entity::Policy->retrieve_all(dbh => $params{dbh},
                                                                   logger => $params{logger},
                                                                   get_criteria => {is_active_policy => 1},
                                                                   search_filters => $params{search_filters},
                                                                   page_size => 0,
                                                                  );
        for my $policy (@policies) {
            my @policy_objs = F5::ASMConfig::Entity::PolicyActionItem->retrieve_all(dbh => $params{dbh},
                                                                                    logger => $params{logger},
                                                                                    page_size => 0,
                                                                                    policy => $policy);
            if (@policy_objs) {
                 my $count = scalar @policy_objs;
                 my $obj = $class->new(dbh => $params{dbh},
                                       logger => $params{logger},
                                       object => {created_time => 0,
                                                  last_updated => 0,
                                                  severity => 0,
                                                  next_display_time => 0,
                                                  policy => $policy,
                                                  action_item_type => F5::ASMConfig::ActionItemType::POLICY_ACTION_ITEM_SUMMARY,
                                                  message => "Policy $policy->{policy_name} has $count outstanding configuration action item" . ($count > 1 ? "s" : ""),
                                                 });

                 for my $policy_obj (@policy_objs) {
                     if ($policy_obj->{severity} >= $obj->{severity}) {
                         $obj->{severity} = $policy_obj->{severity};
                          if ($policy_obj->{created_time} > $obj->{created_time}) {
                             $obj->{created_time} = $policy_obj->{created_time};
                             $obj->{next_display_time} = $policy_obj->{next_display_time};
                         }
                     }
                     if ($policy_obj->{last_updated} > $obj->{last_updated}) {
                         $obj->{last_updated} = $policy_obj->{last_updated};
                     }
                 }
                 $obj->set_rest_uuid();
                 push @extra_objs, $obj;
            }
            $class->apply_search_filters(result_set => \@extra_objs,
                                         ignore_nonexistent_fields => 1,
                                         %params);
        }

        if (@extra_objs) {
            $class->apply_sort_order(result_set => \@extra_objs,
                                     %params);
            push @objs, @extra_objs;
        }

    }
    my $total_record_count = scalar @objs;
    $class->apply_paging(result_set => \@objs,
                         %params);
    if (wantarray) {
        return @objs;
    }
    else {
        my $iterator = F5::ASMConfig::Entity::Iterator->new(
                                                            get_next_function => sub {return shift(@objs);},
                                                            get_all_function => sub {return @objs;},
                                                            total_record_count => $total_record_count,
                                                           );
        return $iterator;
    }
}

#----------------------------------------------------------------------

sub rest_search_filters {
    my ($class) = @_;
    return [ F5::ASMConfig::SearchFilter->new(field_name => 'next_display_time',
                                              field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_LE(),
                                              field_value => time()),
           ];
}

#----------------------------------------------------------------------

1;

