package F5::ASMConfig::Entity::AsmHttpMethod;

use strict;
use warnings;

use base qw/F5::ASMConfig::McpBase F5::ASMConfig::AsmHttpMethod/;


#Cover all bases for how this object is created/retrieved
#----------------------------------------------------------------------

sub load_mcp_fields {
    my($self, %params) = @_;
    $self->SUPER::load_mcp_fields(%params);
    $self->{flg_is_user_defined} = !$self->{config_source};
}

#----------------------------------------------------------------------

sub map_fields_to_entity {
    my($class_or_entity, %params) = @_;
    my $rec = $params{rec} || $class_or_entity;
    $rec->{flg_is_user_defined} = !$rec->{config_source};
}

#----------------------------------------------------------------------

sub delete {
    my($self, %params) = @_;

    my $rv = $self->SUPER::delete(%params);

    my $dbh = $params{dbh} || $self->{dbh};
    my $exists_allowed_method_codes = F5::DbUtils::exists_in_table(dbh => $dbh,
                                                                   field_name => 'method_code',
                                                                   field_value => $self->{method_code},
                                                                   table_name => 'PLC.PL_ALLOWED_METHODS'); 

    my $exists_allowed_method_names = F5::DbUtils::exists_in_table(dbh => $dbh,
                                                                   field_name => 'method_name',
                                                                   field_value => $self->{method_name},
                                                                   table_name => 'PLC.PL_OBJECT_METHOD_OVERRIDES');

    if ($exists_allowed_method_codes || $exists_allowed_method_names) {
        my $error_message =  "HTTP Method \"$self->{method_name}\" cannot be deleted while still referenced by other entities";
        die { error_message => $error_message,
              display_message => $error_message,
              error_level     => 'info',
              error_code      => F5::ASMConfig::ErrorCode::FOREIGN_KEY_RESTRICTION(),
       };
    }

    return $rv;
}


#----------------------------------------------------------------------

sub get_rest_uuid {
    my ($self, %params) = @_;
    my $object = $params{object} || $self;
    my $rest_uuid = $object->{rest_uuid} || $object->{method_name} || $self->{rest_uuid} || $self->{method_name};
    return $rest_uuid;
}

#----------------------------------------------------------------------

sub delete_many_as_iterative {
    return 1;
}

#----------------------------------------------------------------------

sub add_post_commit_hook {
    my ($class, %params) = @_;
    my $writes_to_mcp = $params{writes_to_mcp};
    my $handler = $params{handler};
    if ($writes_to_mcp) {
        $handler->add_post_commit_hook(name => 'update_method_pool',
                                       func => sub {
                                           my $prepare_base = F5::PrepareConf::Base->new(dbh => $handler->{dbh},
                                                         logger => $handler->{logger},
                                                         debug => 1, # to no override log_level
                                                        );
                                           $prepare_base->prepare_method_pool();
                                       });
    }
}

#----------------------------------------------------------------------

# objects, which are not fully in the mcp_object model, override the mcp_folder function
sub mcp_folder {
    return '/Common';
}

#----------------------------------------------------------------------

1;

