package F5::ASMConfig::Entity::AssociateGraphqlIdlFilesTask;

use base qw/F5::ASMConfig::TaskBase F5::ASMConfig::AssociateGraphqlIdlFilesTask/;
use F5::ASMConfig::Entity::IdlFile;
use F5::ASMConfig::Entity::GraphqlProfile;

#----------------------------------------------------------------------

sub before_store {
    my($self, %params) = @_;
    my $dbh = $self->{dbh};

    my $policy =  $self->{policy} || $params{policy};

    my $policy_id = $policy->{policy_id};

    if (!$policy_id || $policy_id == -1) {
        die "Valid policy required.\n";
    }

    my $graphql_profile = $self->get_new_value(%params, field_name => 'graphql_profile');
    my $primary_idl_file = $self->get_new_value(%params, field_name => 'primary_idl_file');
    my $graphql_idl_files = $self->get_new_value(%params, field_name => 'graphql_profile_idl_files') || [];

    if (!$graphql_profile) {
        die "Graphql Profile is required\n";
    }

    my %seen_policy_ids = ($policy_id => 1);
    if (!$graphql_profile->{policy}) {
        $graphql_profile = F5::ASMConfig::Entity::GraphqlProfile->new(dbh => $self->{dbh}, get_criteria => $graphql_profile, policy => $policy);
    }
    $seen_policy_ids{$graphql_profile->{policy}->{policy_id}}++;

    my @other_idl_files = map { $_->{idl_file} } @{$graphql_idl_files};

    if (@other_idl_files && !$primary_idl_file) {
        die "Primary Graphql Schema is required\n";
    }

    for my $idl_file ($primary_idl_file, @other_idl_files) {
        if (!$idl_file->{policy}) {
            $idl_file = F5::ASMConfig::Entity::IdlFile->new(dbh => $self->{dbh}, get_criteria => $idl_file, policy => $policy);
        }
        $seen_policy_ids{$idl_file->{policy}->{policy_id}}++;
    }

    if (keys %seen_policy_ids > 1) {
        die "Graphql Profile and Graphql Schema Files must be from the same Security Policy ($policy->{policy_name})\n";
    }

    return $self->SUPER::before_store(%params);
}

#----------------------------------------------------------------------

sub self_link { #Ensure policy is not used as part of the self link path
    my ($self, %params) = @_;
    my $link = $params{base_link} || '';
    $link .= '/' . $self->element_rest_path_name() . "/" . $self->{rest_uuid};
    $link = $self->add_ver_to_self_link(%params,
                                        'link' => $link);
    return $link;
}

#----------------------------------------------------------------------


1;
