package F5::ASMConfig::Entity::AssociateJsonValidationFilesTask;

use base qw/F5::ASMConfig::TaskBase F5::ASMConfig::AssociateJsonValidationFilesTask/;

#----------------------------------------------------------------------

sub before_store {
    my($self, %params) = @_;
    my $dbh = $self->{dbh};

    my $policy =  $self->{policy} || $params{policy};

    my $policy_id = $policy->{policy_id};

    if (!$policy_id || $policy_id == -1) {
        die "Valid policy required.\n";
    }

    my $json_profile = $self->get_new_value(%params, field_name => 'json_profile');
    my $primary_json_validation_file = $self->get_new_value(%params, field_name => 'primary_json_validation_file');
    my $json_profile_validation_files = $self->get_new_value(%params, field_name => 'json_profile_validation_files') || [];

    if (!$json_profile) {
        die "JSON Profile is required\n";
    }

    my %seen_policy_ids = ($policy_id => 1);
    if (!$json_profile->{policy}) {
        $json_profile = F5::ASMConfig::Entity::JsonProfile->new(dbh => $self->{dbh}, get_criteria => $json_profile, policy => $policy);
    }
    $seen_policy_ids{$json_profile->{policy}->{policy_id}}++;

    my @other_validation_files = map { $_->{json_validation_file} } @{$json_profile_validation_files};

    if (@other_validation_files && !$primary_json_validation_file) {
        die "Primary JSON Schema is required\n";
    }

    for my $validation_file ($primary_json_validation_file, @other_validation_files) {
        if (!$validation_file->{policy}) {
            $validation_file = F5::ASMConfig::Entity::JsonValidationFile->new(dbh => $self->{dbh}, get_criteria => $schema, policy => $policy);
        }
        $seen_policy_ids{$validation_file->{policy}->{policy_id}}++;
    }

    if (keys %seen_policy_ids > 1) {
        die "JSON Profile and JSON Schema Files must be from the same Security Policy ($policy->{policy_name})\n";
    }

    return $self->SUPER::before_store(%params);
}

#----------------------------------------------------------------------

sub self_link { #Ensure policy is not used as part of the self link path
    my ($self, %params) = @_;
    my $link = $params{base_link} || '';
    $link .= '/' . $self->element_rest_path_name() . "/" . $self->{rest_uuid};
    $link = $self->add_ver_to_self_link(%params,
                                        'link' => $link);
    return $link;
}

#----------------------------------------------------------------------


1;
