package F5::ASMConfig::Entity::AssociateXmlValidationFilesTask;

use base qw/F5::ASMConfig::TaskBase F5::ASMConfig::AssociateXmlValidationFilesTask/;

#----------------------------------------------------------------------

sub before_store {
    my($self, %params) = @_;
    my $dbh = $self->{dbh};

    my $policy =  $self->{policy} || $params{policy};
        
    my $policy_id = $policy->{policy_id};

    if (!$policy_id || $policy_id == -1) {
        die "Valid policy required.\n";
    }

    my $xml_profile = $self->get_new_value(%params, field_name => 'xml_profile');
    my $primary_xml_validation_file = $self->get_new_value(%params, field_name => 'primary_xml_validation_file');
    my $xml_profile_validation_files = $self->get_new_value(%params, field_name => 'xml_profile_validation_files') || [];
    
    if (!$xml_profile) {
        die "XML Profile is required\n";
    }

    my %policy_ids = ($policy_id => 1);
    if (!$xml_profile->{policy}) {
        $xml_profile = F5::ASMConfig::Entity::XmlProfile->new(dbh => $self->{dbh}, get_criteria => $xml_profile, policy => $policy);
    }
    $policy_ids{$xml_profile->{policy}->{policy_id}}++;
        
    my @other_validation_files = map { $_->{xml_validation_file} } @{$xml_profile_validation_files};

    if (@other_validation_files && !$primary_xml_validation_file) {
        die "Primary XML Schema is required\n";
    }

    for my $validation_file ($primary_xml_validation_file, @other_validation_files) {
        if (!$validation_file->{policy}) {
            $validation_file = F5::ASMConfig::Entity::XmlValidationFile->new(dbh => $self->{dbh}, get_criteria => $schema, policy => $policy);
        }
        $policy_ids{$validation_file->{policy}->{policy_id}}++;
    }
  
    if (keys %policy_ids > 1) {
        die "XML Profile and XML Schema Files must be from the same Security Policy ($policy->{policy_name})\n";
    }
    
    return $self->SUPER::before_store(%params);
}

#----------------------------------------------------------------------

sub self_link { #Ensure policy is not used as part of the self link path
    my ($self, %params) = @_;
    my $link = $params{base_link} || '';
    $link .= '/' . $self->element_rest_path_name() . "/" . $self->{rest_uuid};
    $link = $self->add_ver_to_self_link(%params,
                                        'link' => $link);
    return $link;
}

#----------------------------------------------------------------------


1;
