package F5::ASMConfig::Entity::BotDefenseEventLog;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::ASMConfig;
use F5::DbUtils;

use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::BotDefenseEventLog/;

#----------------------------------------------------------------------

sub additional_where_clauses {
    my ($class, %params) = @_;
    my @additional_where_clauses;
    if ($params{hide_deleted} || ($params{call_flags} && $params{call_flags}->{hide_deleted})) {
        my $dbh = $params{dbh};
        my ($deleted_id) = $dbh->selectrow_array(<<EndQuery);
SELECT bot_defense_event_log_id FROM PRX.BOT_DEFENSE_EVENT_LOG_CLEARED
EndQuery

        if ($deleted_id) {
            push(@additional_where_clauses, "bot_defense_event_log_internal_id > $deleted_id");
        }

        if (F5::DbUtils::exists_in_table(dbh => $dbh,
                                         table_name => 'PRX.BOT_DEFENSE_EVENT_LOG_PROPERTIES',
                                         where_fields => { flg_is_deleted => 1 },)) {

            push (@additional_where_clauses,
                "(bot_defense_event_log_internal_id NOT IN (SELECT deleted_ids.bot_defense_event_log_id FROM PRX.BOT_DEFENSE_EVENT_LOG_PROPERTIES deleted_ids WHERE deleted_ids.flg_is_deleted = 1))");
        }
    }
    return @additional_where_clauses;
}

#----------------------------------------------------------------------

sub before_store {
    my($self, %params) = @_;
    my $call_flags = $params{call_flags};

    my $partition = $self->get_new_value(field_name => 'partition_id', %params);

    if (! $partition && $call_flags->{partition_was_defined}) {
        $partition = $params{session}->{partition};
    }

    if (! $partition) {
        $partition = "Common";

        my $virtual_server_name = $self->get_new_value(field_name => 'virtual_server_name', %params);
        if ($virtual_server_name && $virtual_server_name =~ m|/(\w+)/|) {
            $partition = $1;
        }
    }

    $params{store_obj}->{partition_id} = $partition;
}

#----------------------------------------------------------------------

sub get_rest_uuid {
    my ($self, %params) = @_;
    my $object = $params{object} || $self;
    my $rest_uuid = $object->{rest_uuid} || $object->{support_id} || $self->{rest_uuid} || $self->{support_id};
    return $rest_uuid;
}

#----------------------------------------------------------------------

sub self_link {
    my ($self, %params) = @_;
    my $link = $params{base_link} || '';
    $link .= '/' . $self->element_rest_path_name() . "/" . $self->get_rest_uuid;
    $link = $self->add_ver_to_self_link(%params,
                                        'link' => $link);
    return $link;
}

#----------------------------------------------------------------------

1;

