package F5::ASMConfig::Entity::BruteForce;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::ASMConfig;
use F5::ASMConfig::Entity::LoginObject;
use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::BruteForce/;

#----------------------------------------------------------------------

sub get_external_entities {
    my ($self, %params) = @_;
    my @external_entities = @{$self->SUPER::get_external_entities(%params)};
    if ($self->{url} && 
        (my $login_object = F5::ASMConfig::Entity::LoginObject->new(dbh => $self->{dbh},
                                                                   policy => $self->{policy},
                                                                   logger => $self->{logger},
                                                                   get_criteria => { url => $self->{url} }))) {
        push (@external_entities, $login_object);
    }
    return \@external_entities;
}

#----------------------------------------------------------------------

sub unique_record_parts {
    my ($class, %params) = @_;

    my $get_criteria = $params{get_criteria};
    my @unique_record_parts;

    if ($get_criteria && $get_criteria->{flg_is_default}) {
        push (@unique_record_parts, 'flg_is_default');
    }
    else {
        @unique_record_parts = grep { $class->fields->{$_}->{unique_record_part} } keys %{$class->fields};
    }
    return \@unique_record_parts;
}

#----------------------------------------------------------------------

sub before_store {
    my($self, %params) = @_;

    my $dbh = $self->{dbh};
    my $db_rec = $params{db_rec};
    my $store_mode = $params{store_mode};
    my $store_obj = $params{store_obj};
    my $existing_obj = $params{existing_obj};

    my $login_object = F5::ASMConfig::Entity::LoginObject->new(%params,
                                                               dbh => $self->{dbh},
                                                               logger => $self->{logger},
                                                               get_criteria => { url => $self->get_new_value(%params, field_name => 'url') });

    my $brute_force_display_name = $self->get_element_type_display_name;
    my $login_object_display_name = F5::ASMConfig::Entity::LoginObject->get_element_type_display_name;

    if ($params{validate_value} && ! $self->{flg_is_default}) {
        if ($login_object) {
            my $auth_type_display_name = $login_object->get_field_display_name_string(field_name => 'auth_type');
            if ($login_object->{auth_type} == F5::ASMConfig::LoginAuthType::LOGIN_AUTH_TYPE_NONE()) {
                die "$brute_force_display_name can only be defined for a $login_object_display_name with $auth_type_display_name set\n"
            }
         }
         else {
             die "$brute_force_display_name can only be defined for an existing $login_object_display_name\n";
         }
    }

    if ($self->{flg_is_default}) {
        $store_obj->{flg_is_default} = 1;
    }
}

#----------------------------------------------------------------------

sub get_display_name {
    my ($self, %params) = @_;
    if ($self->{flg_is_default}) {
        return 'Default';
    }
    return $self->SUPER::get_display_name(%params);
}
#----------------------------------------------------------------------

sub from_declarative {
    my ($self, %params) = @_;

    my $object = $params{object};
    my $field_name = $params{field_name};

    my $rv;
    if ($field_name eq 'url' && $object->{url}) {
        $rv = { policy_reference => 'login-pages' };
    }

    return $rv;
}

#----------------------------------------------------------------------

sub declarative_required_fields {
    my ($class) = @_;

    return [];
}

#----------------------------------------------------------------------

1;

