package F5::ASMConfig::Entity::DataProtection;

use strict;
use warnings;

use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::DataProtection/;

#----------------------------------------------------------------------

sub before_store {
    my($self, %params) = @_;

    my $store_obj = $params{store_obj};
    my $existing_obj = $params{existing_obj};

    my @fields = qw(grace_signing_end_time
                    grace_accepting_end_time);

   my %field_values;
   my %field_display_name;

   for my $field (@fields) {
      $field_values{$field} = $self->get_new_value( store_obj => $store_obj,
                                                    existing_obj => $existing_obj,
                                                    field_name => $field, );

      $field_display_name{$field} = $self->get_field_display_name_string(field_name => $field);
   }

   if ( $field_values{grace_signing_end_time} > $field_values{grace_accepting_end_time} ){
          die "$field_display_name{grace_signing_end_time} should be less than or equal to $field_display_name{grace_accepting_end_time}\n";
   }

   # Retain any existing data not explicitly set in the store object.
   if ($store_obj->{data_protection_cookie_configurations}) {
       my $nested_def = $self->fields->{data_protection_cookie_configurations};
       my $existing_nested_structs = $self->hashify_nested_struct_list(nested_def => $nested_def,
                                                                       nested_recs => $existing_obj->{data_protection_cookie_configurations});
       my $store_nested_structs = $self->hashify_nested_struct_list(nested_def => $nested_def,
                                                                    nested_recs => $store_obj->{data_protection_cookie_configurations});
       my @data_protection_cookie_configurations;
       for my $key (keys %$existing_nested_structs) {
           my $config =  {%{$existing_nested_structs->{$key}}, %{$store_nested_structs->{$key} || {}}};
           push (@data_protection_cookie_configurations, $config);
       }
       $store_obj->{data_protection_cookie_configurations} = \@data_protection_cookie_configurations;
   }

}

#----------------------------------------------------------------------

1;
