package F5::ASMConfig::Entity::DeceptionResponsePage;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::ASMConfig;

use HTTP::Response;

use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::DeceptionResponsePage/;

#----------------------------------------------------------------------

my $HEADER_NAME_MAX_LENGTH = 256;
my $HEADER_VALUE_MAX_LENGTH = 1024;

sub before_store {
    my ($self, %params) = @_;

    my $call_flags = $params{call_flags};
    my $store_obj = $params{store_obj};
    my $existing_obj = $params{existing_obj};

    if($params{validate_value}) {
        my $headers = $self->get_new_value( store_obj => $store_obj,
                                            existing_obj => $existing_obj,
                                            call_flags => $call_flags,
                                            field_name => 'response_headers', );

        local $HTTP::Headers::TRANSLATE_UNDERSCORE;

        $headers =~ s/(?:\r\n)*$//g;
        $headers.= "\r\n\r\n";

        my $error_message;
        my $parsed_response;
        eval {$parsed_response = HTTP::Response->parse($headers)};
        if (($@) ||
            (! $parsed_response)||
            (! defined $parsed_response->code) ) {
            $error_message = "Failed to parse response headers";
            die { error_message => $error_message,
                  display_message => $error_message,
                  error_level     => 'info',
                  error_code      => F5::ASMConfig::ErrorCode::INVALID_FIELD_FORMAT(),
                };
        }

        for my $header_name ($parsed_response->header_field_names) {
            if (length($header_name) >= $HEADER_NAME_MAX_LENGTH) {
                $error_message = "Header name is too long (limit = $HEADER_NAME_MAX_LENGTH bytes)";
            }
            elsif (length($parsed_response->header($header_name)) >= $HEADER_VALUE_MAX_LENGTH) {
                $error_message = "Header value is too long (limit = $HEADER_VALUE_MAX_LENGTH bytes)";
            }
        }


        if($error_message) {
           die { error_message => $error_message,
                 display_message => $error_message,
                 error_level     => 'info',
                 error_code      => F5::ASMConfig::ErrorCode::INVALID_FIELD_FORMAT(),
               };
        }
    }
}

#----------------------------------------------------------------------

1;
