package F5::ASMConfig::Entity::DeceptionSettings;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::ASMConfig;

use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::DeceptionSettings/;

#----------------------------------------------------------------------

sub before_store {
    my($self, %params) = @_;
    my $store_obj = $params{store_obj};

    my $flg_user_defined_responses = $self->get_new_value(%params, field_name => 'flg_user_defined_responses');
    my $server_technology = $self->get_new_value(%params, field_name => 'server_technology');

    my $error_message;
    if (!$flg_user_defined_responses) {
        my $field_name = $self->get_field_display_name_string(%params, field_name => 'server_technology');
        if (!$server_technology) {
            $error_message = "$field_name must be specified";
        }
        elsif (!$server_technology->{flg_has_deception_responses}) {
            $error_message = "No deception response pages are available for the selected $field_name";
        }
    }

    if ($error_message) {
        die { error_message => $error_message,
              display_message => $error_message,
              error_level     => 'info',
              error_code      => F5::ASMConfig::ErrorCode::MISSING_FIELD(),
            };
    }
}

#----------------------------------------------------------------------

1;
