package F5::ASMConfig::Entity::Device;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::ASMConfig;
use F5::BigipVersionUtils;
use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::Device/;

my $EM_IMPLEMENTED_FROM_VERSION = '0011.0003.0000';

#----------------------------------------------------------------------

sub get_retrieve_join_tables {
    my($class, %params) = @_;

    my $retval = <<EndQuery;
LEFT JOIN f5em.device_module as dev_mod
  ON (main_table.uid = dev_mod.device_uid)
LEFT JOIN f5em.tmos_module as tm_mod
  ON (dev_mod.module_id = tm_mod.id)
LEFT JOIN f5em.device_trust as dev_trust
  ON (main_table.uid = dev_trust.device_uid)
EndQuery

    return $retval;
}

#----------------------------------------------------------------------

sub additional_where_clauses {
    return <<EndQuery;
dev_mod.provisioned = '1'
AND
tm_mod.description = 'Application Security (ASM)'
EndQuery
}

#----------------------------------------------------------------------

sub retrieve_all {
    my ($class, %params) = @_;

    my @devices = grep { F5::BigipVersionUtils::zero_pad_segments($_->{version}) ge $EM_IMPLEMENTED_FROM_VERSION }
                  $class->SUPER::retrieve_all(%params, page_size => 0);

    my $total_record_count = scalar @devices;

    $class->apply_paging(result_set => \@devices,
                         %params);

    if (wantarray) {
        return @devices;
    }
    else {
        my $iterator = F5::ASMConfig::Entity::Iterator->new(
                                                            get_next_function => sub {return shift(@devices);},
                                                            get_all_function => sub {return @devices;},
                                                            total_record_count => $total_record_count,
                                                           );
        return $iterator;
    }
}

#----------------------------------------------------------------------

1;

