package F5::ASMConfig::Entity::Diff;

use strict;
use warnings;

use F5::ASMConfig::ASMConfig;
use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::Diff/;
use F5::ASMConfig::Entity::Policy;

#----------------------------------------------------------------------

sub additional_field_mapping {
    my ($class, %params) = @_;
    my %additional_field_mapping;
    if ($params{extra_fields} && grep {$_ eq "diff_summary"} (@{$params{extra_fields}})) {
        $additional_field_mapping{diff_summary} = {};
    }
    return \%additional_field_mapping;
}

#----------------------------------------------------------------------

sub retrieve_nested_structures {
    my ($self, %params) = @_;
    if ($params{fields}->{diff_summary}) {
        my $plc_dbname = 'PLC';

        my $summary_query =  <<EndQuery;
SELECT element_type, SUM(IF(diff_type = @{[F5::ASMConfig::DiffType::DIFF_TYPE_ONLY_FIRST]}, 1, 0)) AS only_first_count, 
                     SUM(IF(diff_type = @{[F5::ASMConfig::DiffType::DIFF_TYPE_ONLY_SECOND]}, 1, 0)) AS only_second_count, 
                     SUM(IF(diff_type = @{[F5::ASMConfig::DiffType::DIFF_TYPE_CONFLICT]}, 1, 0)) AS conflict_count
 FROM $plc_dbname.PL_DIFF_ITEMS 
   WHERE diff_id = ? 
 GROUP BY element_type
EndQuery

        my @diff_summary;
        my $summary_sth = $self->{dbh}->prepare($summary_query);
        $summary_sth->execute($self->{diff_id});        
        while (my $summary_row = $summary_sth->fetchrow_hashref) {
            push (@diff_summary, F5::ASMConfig::DiffSummary->new($summary_row));
        }  
        $self->{diff_summary} = \@diff_summary;
    }
}

#----------------------------------------------------------------------
sub get_policies {
    my ($self) = @_;
    my @policies;
    if (defined($self->{first_policy}) && defined($self->{second_policy})) {
        push (@policies, F5::ASMConfig::Cache::get_cached_policy(dbh => $self->{dbh}, policy => $self->{first_policy}));
        push (@policies, F5::ASMConfig::Cache::get_cached_policy(dbh => $self->{dbh}, policy => $self->{second_policy}));
    }
    elsif (my $diff = F5::ASMConfig::Entity::Diff->new(dbh => $self->{dbh}, get_criteria => $self)) {
        push (@policies, F5::ASMConfig::Cache::get_cached_policy(dbh => $self->{dbh}, policy => $diff->{first_policy}));
        push (@policies, F5::ASMConfig::Cache::get_cached_policy(dbh => $self->{dbh}, policy => $diff->{second_policy}));
    }
    return @policies;
}

#----------------------------------------------------------------------
#

1;

