package F5::ASMConfig::Entity::DiffItem;

use strict;
use warnings;

use F5::ASMConfig::ASMConfig;
use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::DiffItem/;
use F5::ASMConfig::Entity::Policy;

#----------------------------------------------------------------------

sub get_policies {
    my ($self) = @_;
    my $diff_id = $self->{diff_id};
    if (!$diff_id && $self->{diff_item_id}) {
        if (my $diff_item = F5::ASMConfig::Entity::DiffItem->new(dbh => $self->{dbh}, get_criteria => $self)) {
            $diff_id = $diff_item->{diff_id};
        }        
    }

    if ($diff_id) {
       return F5::ASMConfig::Entity::Diff->new(dbh => $self->{dbh}, object => {diff_id => $diff_id})->get_policies();
    }
    return;
}

#----------------------------------------------------------------------

sub map_fields_to_entity {
    my($class_or_entity, %params) = @_;
    my $dbh = $params{dbh};
    my $rec = $params{rec} || $class_or_entity;

    $class_or_entity->SUPER::map_fields_to_entity(%params);

    if ( $rec && $rec->{diff_item_id} ) {
        my $diff_item = $class_or_entity->new(dbh => $dbh,
                                              object => $rec);
        my $element_type = $class_or_entity->get_cached_element_type(element_type => $diff_item->{element_type});
        if ( $element_type && $element_type->{class} ) {
            my $element_class = $element_type->{class};
            if ( $diff_item->{first_element_id} ) {
                $rec->{first_element} = $element_class->new(dbh => $dbh,
                                                            policy=> $diff_item->{diff}->{first_policy},
                                                            get_criteria => { $element_class->unique_id_field => $diff_item->{first_element_id} });
            }
            if ( $diff_item->{second_element_id} ) {
                $rec->{second_element} = $element_class->new(dbh => $dbh,
                                                             policy=> $diff_item->{diff}->{second_policy},
                                                             get_criteria => { $element_class->unique_id_field => $diff_item->{second_element_id} });
            }
        }
        $rec->{merge_status} = F5::ASMConfig::DiffDirection::DIFF_DIRECTION_NONE();
    }
}

#----------------------------------------------------------------------

sub after_nested_struct_load {
    my ($self, %params) = @_;
    if ($self->{diff_item_details}) {
        $_->{merge_status} = F5::ASMConfig::DiffDirection::DIFF_DIRECTION_NONE() foreach @{$self->{diff_item_details}};
    }
    return;
}

#----------------------------------------------------------------------

1;

