package F5::ASMConfig::Entity::EmVirtualServer;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::ASMConfig;
use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::EmVirtualServer/;

#----------------------------------------------------------------------

sub return_class {
    my ($class) = @_;
    return "F5::ASMConfig::Entity::VirtualServer";
}

#----------------------------------------------------------------------

sub get_retrieve_join_tables {
    my($class, %params) = @_;

    my $retval = <<EndQuery;
LEFT JOIN f5em.device_config_class as dev_conf_class
  ON (main_table.device_config_class_uid = dev_conf_class.uid)
LEFT JOIN f5em.partition as partition
  ON (dev_conf_class.partition_uid = partition.partition_uid)
LEFT JOIN f5em.device as device
  ON (partition.device_uid = device.uid)
EndQuery

    return $retval;
}

#--------------------------------------------------------------------

sub additional_where_clauses {
    return <<EndQuery;
 dev_conf_class.name = 'Local Traffic / Virtual Server'
EndQuery
}

#----------------------------------------------------------------------

sub map_fields_to_entity {
    my($class, %params) = @_;

    my $dbh = $params{dbh};
    my $rec = $params{rec};

    if ($rec->{destination} =~ /destination ([^:]*):/) {
         $rec->{destination} = $1;
    }

    my $instance_data = $rec->{flg_is_asm_enabled};
    $rec->{flg_is_asm_enabled} = 0;
    $instance_data =~ s/\n//g;

    if ($instance_data =~ /http-class \{ *([^}]*) *\}/) {
        my @http_classes = split (/ +/,$1);
        # This is really ugly, take a deep breath. Here it comes:
        my $instance_names = $dbh->selectcol_arrayref(<<EndQuery, undef, $rec->{device__device},);
SELECT main_table.instance_name
  FROM f5em.device_config_instance as main_table
       LEFT JOIN f5em.device_config_class as dev_conf_class
              ON (main_table.device_config_class_uid = dev_conf_class.uid)
       LEFT JOIN f5em.partition as partition
              ON (dev_conf_class.partition_uid = partition.partition_uid)
       LEFT JOIN f5em.device as device
              ON (partition.device_uid = device.uid)
 WHERE device.uid = ?
   AND dev_conf_class.name = 'Local Traffic / Profiles / HTTP Class'
   AND main_table.instance_data like '%asm enabled%'
EndQuery
        for my $http_class (@http_classes) {
            if (grep {basename($_) eq $http_class} @{$instance_names}) {
                $rec->{flg_is_asm_enabled} = 1;
                last;
            }
        }
    }

    $class->SUPER::map_fields_to_entity(%params);
}

#----------------------------------------------------------------------

1;

