package F5::ASMConfig::Entity::Filetype;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::ASMConfig;
use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::Filetype/;

#----------------------------------------------------------------------

sub update_many_as_iterative {
    return 1;
}

#----------------------------------------------------------------------

sub before_store {
  my($self, %params) = @_;
  my $store_obj = $params{store_obj};
  my $existing_obj = $params{existing_obj};

  $self->adjust_wildcards(%params);
  $self->sync_from_pb_config(%params, pb_config_field => "flg_filetypes", table_name => F5::ASMConfig::Entity::PBConfigFiletype->db_table());
  
  my @fields = qw(
                     is_wildcard
                     filetype_name
                   );

  my %field_values;
  my %field_display_name;

  for my $field (@fields) {
    $field_values{$field} = $self->get_new_value( store_obj => $store_obj,
                                                existing_obj => $existing_obj,
                                                field_name => $field, );

    $field_display_name{$field} = $self->get_field_display_name_string(%params, field_name => $field);
  }

  if ($params{validate_value} && $field_values{is_wildcard} && $field_values{filetype_name} eq 'no_ext') {
      my $error_message = "Cannot use reserved name 'no_ext' as the name of a wilcard file type";
      die { error_message => $error_message,
            display_message => $error_message,
            error_level     => 'info',
            error_code      => F5::ASMConfig::ErrorCode::INVALID_FIELD_FORMAT(),
          };
  }

  if ( !$field_values{is_wildcard} && exists $store_obj->{has_tightening} ) {
      delete $store_obj->{has_tightening};
  }
  
}

#----------------------------------------------------------------------

sub is_pure_wildcard {
    my ($self, %params) = @_;
    return ($self->get_new_value(%params, field_name => 'filetype_name') &&
            $self->get_new_value(%params, field_name => 'filetype_name') eq "*" &&            
            $self->get_new_value(%params, field_name => 'is_wildcard'));
}

#----------------------------------------------------------------------

1;
