package F5::ASMConfig::Entity::GrpcProfile;

use strict;
use warnings;

use Memoize;

use F5::ASMConfig::ASMConfig;
use F5::DbUtils;
use F5::ASMConfig::ContentProfileBase;
use F5::ASMConfig::Entity::GrpcDefenseAttribute;
use base qw/F5::ASMConfig::ContentProfileBase F5::ASMConfig::GrpcProfile/;


#----------------------------------------------------------------------

sub before_store {
    my($self, %params) = @_;
    my $store_obj = $params{store_obj};

    if ( defined($store_obj->{idl_files}) ) {
        my $nested_def = $self->fields->{idl_files};
        my $store_idl_files = $self->hashify_nested_struct_list(nested_def => $nested_def,
                                                                       nested_recs => $store_obj->{idl_files},
                                                                      );

        for my $key (keys %$store_idl_files) {
            if ( ! $store_idl_files->{$key}->{idl_file} ) {
                delete $store_idl_files->{$key}
            } elsif (defined($store_idl_files->{$key}->{import_url}) &&
                    ! length($store_idl_files->{$key}->{import_url})
                    ) {
                delete $store_idl_files->{$key}->{import_url};
            }
        }

        $store_obj->{idl_files} = [ values %$store_idl_files ];
    }

    $self->populate_defense_attributes(store_obj => $store_obj);
    if ($params{validate_value}) {
        $self->validate_defense_attributes(defense_settings => $store_obj->{grpc_defense});
    }

    for my $grpc_defense (@{$store_obj->{grpc_defense}}) {
        if (defined $grpc_defense->{def_attrib_value} &&
            $grpc_defense->{value_type} eq F5::ASMConfig::DefenseAttributeValueType::DEFENSE_ATTRIBUTE_VALUE_TYPE_INTEGER) {

            $grpc_defense->{def_attrib_value} += 0;
        }
    }

}

#----------------------------------------------------------------------

sub after_nested_struct_store {
  my ($self, %params) = @_;
  my $dbh = $self->{dbh};
  my $store_obj = $params{store_obj};
  if ($params{store_mode} eq 'update') {
      $self->populate_defense_attributes(store_obj => $store_obj);
  }
}

#----------------------------------------------------------------------

sub after_nested_struct_load {
    my ($self, %params) = @_;
    $self->populate_defense_attributes();
    return;
}

#----------------------------------------------------------------------

sub populate_defense_attributes {
    my ($self, %params) = @_;
    my $store_obj = $params{store_obj} || $self;

    if (exists($store_obj->{grpc_defense})) {
        my $defense_attributes = $self->get_defense_attributes();
        for my $grpc_defense (@{$store_obj->{grpc_defense}}) {
            %$grpc_defense = ( %$grpc_defense, %{$defense_attributes->{$grpc_defense->{def_attrib_id}}} );
        }
    }
}


#----------------------------------------------------------------------

sub get_defense_attributes {
    my ($self, %params) = @_;
    my $dbh = $params{dbh} || $self->{dbh};

    my %retval = map {$_->{def_attrib_id} => $_} F5::ASMConfig::Entity::GrpcDefenseAttribute->retrieve_all( dbh => $dbh, );

    return $self->install_accessor('get_defense_attributes', \%retval);
}

#----------------------------------------------------------------------

sub update_parent_staging {
    my ($self, %params) = @_;
    return $self->update_content_profile_staging(%params);
}

#----------------------------------------------------------------------

1;
