package F5::ASMConfig::Entity::InternalConfigItem;

use strict;
use warnings;

use base qw(F5::ASMConfig::Entity::Base F5::ASMConfig::InternalConfigItem);

#----------------------------------------------------------------------

sub before_store {
    my($self, %params) = @_;

    return unless $params{validate_value};

    my $store_obj = $params{store_obj};
    my $existing_obj = $params{existing_obj};

    if ($params{call_flags}->{rest_call} && $existing_obj->{flg_is_factory_setting}) {
        my $rest_fields = $self->rest_fields;
        for my $rest_field (keys %$rest_fields) {
            delete $store_obj->{$rest_field} if $rest_field ne 'current_value';
        }
    }

    my $value_type = $self->get_new_value( store_obj => $store_obj,
                                           existing_obj => $existing_obj,
                                           field_name => 'config_item_value_type',);

    if ( $value_type == F5::ASMConfig::ConfigItemValueType::CONFIG_ITEM_VALUE_TYPE_NUMBER() ) {
        my $min_value = $self->get_new_value( store_obj => $store_obj,
                                              existing_obj => $existing_obj,
                                              field_name => 'value_min',);

        my $max_value = $self->get_new_value( store_obj => $store_obj,
                                              existing_obj => $existing_obj,
                                              field_name => 'value_max',);

        my $err_message;
        my @numeric_fields = qw(current_value factory_value);
        for my $field (@numeric_fields) {
            if (defined $store_obj->{$field}) {
                my $value = $store_obj->{$field};
                my $field_display_name = $self->get_field_display_name_string(%params, field_name => $field);
                my $err_prefix = "Failed validating value '$value' for field '$field_display_name': ";
                if ($value !~ /^-?\d+$/) {
                    $err_message = $err_prefix . "Value must be an integer.";
                    last;
                }

                if ( $value < $min_value ) {
                    $err_message = $err_prefix . "The value is out of range (minimum value = $min_value)";
                    last;
                }

                if ( $value > $max_value ) {
                    $err_message = $err_prefix . "The value is out of range (maximum value = $max_value)";
                    last;
                }
            }
        }

        if ($err_message) {
            die  { error_message => $err_message,
                   display_message => $err_message,
                   error_level     => 'info',
                   error_code      => F5::ASMConfig::ErrorCode::INVALID_FIELD_FORMAT(),
                 };
        }
    }
}

#----------------------------------------------------------------------

sub rest_search_filters {
    my ($class) = @_;
    return [ F5::ASMConfig::SearchFilter->new(field_name => 'section_id',
                                              field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_EQ(),
                                              field_value => 1),
             F5::ASMConfig::SearchFilter->new(field_name => 'flg_is_used_in_asm_full',
                                              field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_EQ(),
                                              field_value => 1),
           ];
}

#----------------------------------------------------------------------

sub map_rest_internal_config_item {
    my ($self, %params) = @_;
    my $field_value = $params{field_value};
    my $direction = $params{direction};

    return if !defined($field_value);

    if ($direction eq 'to_rest') {
        if (defined($field_value)) {
           if ($self->{config_item_value_type} && $self->{config_item_value_type} eq F5::ASMConfig::ConfigItemValueType::CONFIG_ITEM_VALUE_TYPE_NUMBER()) {
                $field_value = ($field_value || 0) + 0;
           }
        }
    }
    return $field_value;
}

#----------------------------------------------------------------------

1;
