package F5::ASMConfig::Entity::Iterator;

use strict;
use warnings;

#----------------------------------------------------------------------

sub new {
  my($class, %params) = @_;
  my $self = {};
  bless $self, $class;
  $self->init(%params);

  return $self;
}

#----------------------------------------------------------------------

sub init {
  my($self, %params) = @_;

  $self->{sth} = $params{sth};
  $self->{object_constructor} = $params{object_constructor};
  $self->{total_record_count} = $params{total_record_count};
  $self->{get_next_function} = $params{get_next_function};
  $self->{get_all_function} = $params{get_all_function};
}

#----------------------------------------------------------------------

sub get_next {
  my($self) = @_;
  if (defined $self->{get_next_function}) {
    return $self->{get_next_function}->();
  }
  return unless $self->{sth}->{Active};
  my $rec = $self->{sth}->fetchrow_hashref;
  unless ($rec) {
    $self->{sth}->finish;
    return;
  }

  return $self->{object_constructor}->(rec => $rec);
}

#----------------------------------------------------------------------

sub get_all {
  my($self) = @_;
  if (defined $self->{get_all_function}) {
    return $self->{get_all_function}->();
  }
  my @records;
  while (my $next_record = $self->get_next) {
    push(@records, $next_record);
  }
  return @records;
}

#----------------------------------------------------------------------

sub total_record_count {
  my($self) = @_;
  return $self->{total_record_count};
}

#----------------------------------------------------------------------

1;
