package F5::ASMConfig::Entity::JsonProfile;

use strict;
use warnings;

use Memoize;

use F5::ASMConfig::ASMConfig;
use F5::DbUtils;
use F5::ASMConfig::ContentProfileBase;
use F5::ASMConfig::Entity::JsonDefenseAttribute;
use base qw/F5::ASMConfig::ContentProfileBase F5::ASMConfig::JsonProfile/;


#----------------------------------------------------------------------

sub before_store {
    my($self, %params) = @_;
    my $store_obj = $params{store_obj};

    if ( defined($store_obj->{validation_files}) ) {
        my $nested_def = $self->fields->{validation_files};
        my $store_validation_files = $self->hashify_nested_struct_list(nested_def => $nested_def,
                                                                       nested_recs => $store_obj->{validation_files},
                                                                      );

        for my $key (keys %$store_validation_files) {
            if ( ! $store_validation_files->{$key}->{json_validation_file} ) {
                delete $store_validation_files->{$key}
            } elsif (defined($store_validation_files->{$key}->{import_url}) &&
                    ! length($store_validation_files->{$key}->{import_url})
                    ) {
                delete $store_validation_files->{$key}->{import_url};
            }
        }

        $store_obj->{validation_files} = [ values %$store_validation_files ];
    }

    $self->populate_defense_attributes(store_obj => $store_obj);
    if ($params{validate_value}) {
        $self->validate_defense_attributes(defense_settings => $store_obj->{json_defense});
    }

    for my $json_defense (@{$store_obj->{json_defense}}) {
        if (defined $json_defense->{def_attrib_value} &&
            $json_defense->{value_type} eq F5::ASMConfig::DefenseAttributeValueType::DEFENSE_ATTRIBUTE_VALUE_TYPE_INTEGER) {

            $json_defense->{def_attrib_value} += 0;
        }
    }

}

#----------------------------------------------------------------------

sub after_nested_struct_store {
  my ($self, %params) = @_;
  my $dbh = $self->{dbh};
  my $store_obj = $params{store_obj};
  if ($params{store_mode} eq 'update') {
      $self->populate_defense_attributes(store_obj => $store_obj);
  }
}

#----------------------------------------------------------------------

sub after_nested_struct_load {
    my ($self, %params) = @_;
    $self->populate_defense_attributes();
    return;
}

#----------------------------------------------------------------------

sub populate_defense_attributes {
    my ($self, %params) = @_;
    my $store_obj = $params{store_obj} || $self;

    if (exists($store_obj->{json_defense})) {
        my $defense_attributes = $self->get_defense_attributes();
        for my $json_defense (@{$store_obj->{json_defense}}) {
            %$json_defense = ( %$json_defense, %{$defense_attributes->{$json_defense->{def_attrib_id}}} );
        }
    }
}


#----------------------------------------------------------------------

sub get_defense_attributes {
    my ($self, %params) = @_;
    my $dbh = $params{dbh} || $self->{dbh};

    my %retval = map {$_->{def_attrib_id} => $_} F5::ASMConfig::Entity::JsonDefenseAttribute->retrieve_all( dbh => $dbh, );

    return $self->install_accessor('get_defense_attributes', \%retval);
}

#----------------------------------------------------------------------

sub update_parent_staging {
    my ($self, %params) = @_;
    return $self->update_content_profile_staging(%params);
}

#----------------------------------------------------------------------

sub to_declarative {
    my ($self, %params) = @_;

    my $entity = $self->SUPER::to_declarative(%params);

    for my $validation_file (@{$entity->{validationFiles}}) {
        delete $validation_file->{jsonValidationFile}->{contents};
    }

    return $entity;
}

#----------------------------------------------------------------------

1;
