package F5::ASMConfig::Entity::JsonValidationFile;
use base qw(F5::ASMConfig::Entity::Base F5::ASMConfig::JsonValidationFile);

use F5::Deploy;
use F5::JSON::Schema::Validate;
my $TEMP_SCHEMA_BASE_DIR = F5::Deploy::map_path("/ts/var/tmp");

#----------------------------------------------------------------------

sub before_store {
    my ($self, %params) = @_;

    my $logger = $params{logger} || $self->{logger};

    my $file_contents = $self->get_new_value( store_obj => $params{store_obj},
                                              existing_obj => $params{existing_obj},
                                              field_name => 'file_contents', );

    my $profile_validation_file;
    $profile_validation_file->{local_filename} = "$TEMP_SCHEMA_BASE_DIR/json_schema_file_to_validate_$$";

    open(my $outf, '>', $profile_validation_file->{local_filename});
    print $outf $file_contents;
    close $outf;

    my $rv = F5::JSON::Schema::Validate::is_json_schema_valid(profile_validation_files => [ $profile_validation_file ],
                                                              tolerate_reference_errors => 1,
                                                              logger => $logger,
                                                             );

    unlink $profile_validation_file->{local_filename};

    if (! $rv->{completed_successfully}) {
      my $error_message = "Malformed JSON";
      $logger->error( join('; ', ($error_message, @{$rv->{messages}}))) if $logger;
      die { error_message => $error_message,
            display_message => $error_message,
            error_level     => 'info',
            error_code      => F5::ASMConfig::ErrorCode::INVALID_FIELD_FORMAT(),
          };
    }
}

#----------------------------------------------------------------------


1;
