package F5::ASMConfig::Entity::L7Policy;

use strict;
use warnings;

use F5::Mcp::ProfileType;
use F5::Mcp::PolicyAction;
use F5::Mcp::PolicyTarget;
use base qw/F5::ASMConfig::McpBase F5::ASMConfig::L7Policy/;

#----------------------------------------------------------------------

sub map_fields_to_entity {
    my ($class_or_self, %params) = @_;
    my $obj = $params{rec} || $class_or_self;

    # complex L7 policy association is:
    #    - association of L7plc  that has conditions defined
    #    - association of L7plc, that has an          assigned and enabled ASMplc, to more than 1 VS
    #    - association of L7plc, that has more than 1 assigned and enabled ASMplc, to a VS
    # anything else is a simple association.

    $obj->{type} = F5::ASMConfig::VirtualServerAssocType::VIRTUAL_SERVER_ASSOC_TYPE_L7_COMPLEX();

    if ((! $obj->{conditions} || !@{$obj->{conditions}}) &&
        $obj->{actions} && (scalar @{$obj->{actions}} == 1) &&
        $obj->{actions}->[0]->{target} == F5::Mcp::PolicyTarget::L7_POLICY_TARGET_ASM &&
        $obj->{actions}->[0]->{action} == F5::Mcp::PolicyAction::L7_POLICY_ACTION_ENABLE &&
        (! $obj->{virtual_servers} || (scalar @{$obj->{virtual_servers}} < 2)) ) {

        $obj->{type} = F5::ASMConfig::VirtualServerAssocType::VIRTUAL_SERVER_ASSOC_TYPE_L7_SIMPLE();
    }
    if ($obj->{virtual_servers}) {
         my @virtuals;
         for my $virtual_server_policy (@{$obj->{virtual_servers}}) {
             my $full_name = $virtual_server_policy->{vs_name};
             my @path_parts = split("/", $full_name);
             my $partition = $path_parts[1];
             my $leaf_name = pop @path_parts;
             my $folder = join ("/", @path_parts);
             push (@virtuals, F5::ASMConfig::VirtualServer->new(virtual_server_name => $leaf_name,
                                                                folder => $folder,
                                                                full_name => $full_name,
                                                                partition_id => $partition,
                                                                asm_enabled_l7_policy_name => $obj->{l7_policy_name},));

         }
         $obj->{virtual_servers} = \@virtuals;
    }
}

#----------------------------------------------------------------------

1;

