package F5::ASMConfig::Entity::LogoutObject;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::ASMConfig;
use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::LogoutObject/;

#----------------------------------------------------------------------
sub before_store {
    my($self, %params) = @_;
    my $dbh = $self->{dbh};
    my $store_obj = $params{store_obj};
    my $existing_obj = $params{existing_obj};

    return unless $params{validate_value};

    my $url = $self->get_new_value(%params, field_name => 'url');

    if (! $url || ! ref($url) || ! $url->{url_id} ||  $url->{is_wildcard}) {
        my $error_message = 'A valid explicit URL must be specified';
        die { error_message => $error_message,
              display_message => $error_message,
              error_level     => 'info',
              error_code      => F5::ASMConfig::ErrorCode::INVALID_FIELD_FORMAT(),
            };
    };

    if(! $url->{flg_is_allowed}){
        my $error_message = 'The specified URL [' . $url->{url_name} . ']  is Disallowed.';
        die { error_message => $error_message,
              display_message => $error_message,
              error_level     => 'info',
              error_code      => F5::ASMConfig::ErrorCode::INVALID_FIELD_FORMAT(),
            };
    }

   my @fields = qw(
                   request_contains
                   request_omits
                   );

   my %field_values;
   my %field_display_name;

   for my $field (@fields) {
      $field_values{$field} = $self->get_new_value( store_obj => $store_obj,
                                                    existing_obj => $existing_obj,
                                                    field_name => $field, );

      $field_display_name{$field} = $self->get_field_display_name_string(%params, field_name => $field);
   }

   if ( defined $field_values{request_contains} && defined $field_values{request_omits} &&
        length($field_values{request_contains}) && length($field_values{request_omits}) &&
        $field_values{request_contains} eq $field_values{request_omits} ){
          my $error_message =  "$field_display_name{request_contains} and $field_display_name{request_omits} must differ";
          die { error_message => $error_message,
                display_message => $error_message,
                error_level     => 'info',
                error_code      => F5::ASMConfig::ErrorCode::INVALID_FIELD_FORMAT(),
              };
   }
}
#----------------------------------------------------------------------
sub update_many_as_iterative {
    return 1;
}
#----------------------------------------------------------------------
1;
