package F5::ASMConfig::Entity::Microservice;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::ASMConfig;
use F5::License;

use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::Microservice/;

our %SUB_VIOLATION_CODES = (  F5::Violation::VIOL_EVASION() => 1,
                              F5::Violation::VIOL_HTTP_PROTOCOL() => 1,
                         );

#----------------------------------------------------------------------
sub before_store {
  my($self, %params) = @_;
  my $store_obj = $params{store_obj};
  my $existing_obj = $params{existing_obj};
  my $store_mode = $params{store_mode};

  if ($self->get_new_value(%params, field_name=> 'microservice_host_name') =~ /^\*+$/ &&
      $self->get_new_value(%params, field_name=> 'microservice_uri') =~  m|^/?\*+$| &&
      $self->get_new_value(%params, field_name=> 'is_wildcard_host') &&
      $self->get_new_value(%params, field_name=> 'is_wildcard_uri')) {


      my $error_message =  "Microservice defined by wildcards alone is not allowed";
      die { error_message => $error_message,
              display_message => $error_message,
              error_level     => 'info',
              error_code      => F5::ASMConfig::ErrorCode::ACTION_NOT_ALLOWED(),
          };
  }

  if ( $store_mode eq 'add' ) {
      my $microservice_uri_display_name = $self->get_field_display_name_string(%params, field_name => 'microservice_uri');
      if ( !$store_obj->{microservice_uri} ){
           my $error_message = "$microservice_uri_display_name must be defined";
           die { error_message => $error_message,
                 display_message => $error_message,
                 error_level     => 'info',
                 error_code      => F5::ASMConfig::ErrorCode::MISSING_FIELD(),
           };
      }

      if ($store_obj->{microservice_uri} !~ m|^/| ) {
           my $error_message =  "$microservice_uri_display_name must start with '/'";
           die { error_message => $error_message,
              display_message => $error_message,
              error_level     => 'info',
              error_code      => F5::ASMConfig::ErrorCode::INVALID_FIELD_FORMAT(),
          };

      }
  }

  $store_obj->{is_wildcard} = 1;
  $self->adjust_wildcards(%params, name_field => 'microservice_host_name', add_default_wildcard_as_last => 0);

  my $blocking_settings_overrides = $self->get_new_value(%params, field_name => 'blocking_settings_overrides') || [];

  for my $blocking_settings_override ( @$blocking_settings_overrides ) {
     my $viol_index = $blocking_settings_override->{violation}->{viol_index};

     if (!$SUB_VIOLATION_CODES{$viol_index} ) {
          my $error_message =  "Blocking settings overrides are not allowed for $blocking_settings_override->{violation}->{icontrol_name}";
          die { error_message => $error_message,
                display_message => $error_message,
                error_level     => 'info',
                error_code      => F5::ASMConfig::ErrorCode::ACTION_NOT_ALLOWED(),
              };
     }

  }
}

#----------------------------------------------------------------------
sub update_many_as_iterative {
    return 1;
}
#----------------------------------------------------------------------
sub dependent_rest_field_value {
    my ($self, %params) = @_;
    my $object = $params{object};
    my $field_def = $params{field_def};

    my $code = $object->{$field_def->{dependent_rest_field}};
    my $dependent_value = $field_def->{dependent_rest_field_value};

    return ($code && ( $code != $dependent_value) ) ? $code : 0;
}
#----------------------------------------------------------------------

1;
