package F5::ASMConfig::Entity::PBConfigParameter;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::ASMConfig;

use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::PBConfigParameter/;

#----------------------------------------------------------------------
sub before_store {
    my($self, %params) = @_;

    my $store_obj = $params{store_obj};
    my $existing_obj = $params{existing_obj};

    my @fields = qw(
                     flg_parameters
                     level_parameters
                     flg_parameters_integer_value
                     flg_dynamic_parameters_hidden
                     flg_dynamic_parameters_statistics_forms
                     flg_dynamic_parameters_statistics_links
                   );

   my %field_values;
   my %field_display_name;

   for my $field (@fields) {
      $field_values{$field} = $self->get_new_value( store_obj => $store_obj,
                                                    existing_obj => $existing_obj,
                                                    field_name => $field, );

      $field_display_name{$field} = $self->get_field_display_name_string(field_name => $field);
   }


   if ( $self->has_value_changed(%params, check_fields => [ qw(flg_parameters level_parameters) ])
          && $field_values{flg_parameters} == F5::ASMConfig::TighteningMode::TIGHTENING_MODE_COMPACT
              && $field_values{level_parameters} == F5::ASMConfig::ParamLevel::PARAM_LEVEL_URL ){
       die "$field_display_name{flg_parameters} may not be set to 'compact' when $field_display_name{level_parameters} is set to 'url'.\n";
   }

   if ( $self->has_value_changed(%params, check_fields => [ qw(flg_parameters_integer_value
                                                               flg_dynamic_parameters_hidden
                                                               flg_dynamic_parameters_statistics_forms
                                                               flg_dynamic_parameters_statistics_links ) ])
          && $field_values{flg_parameters_integer_value} && ( $field_values{flg_dynamic_parameters_statistics_forms}
              || $field_values{flg_dynamic_parameters_statistics_links} || $field_values{flg_dynamic_parameters_hidden} )){
       die "Enabling both $field_display_name{flg_parameters_integer_value} and any field from Learn Dynamic Parameters section is not allowed.\n";
   }
}

#----------------------------------------------------------------------
1;

