package F5::ASMConfig::Entity::ParameterContentProfile;
use strict;
use warnings;

use base qw(F5::ASMConfig::Entity::Base F5::ASMConfig::ParameterContentProfile);
use F5::ASMConfig::Entity::Parameter;

#----------------------------------------------------------------------

sub before_store {
    my ($self, %params) = @_;
    my $store_obj = $params{store_obj};
    my $policy = $params{policy} || $self->{policy};

    my $rv;
    my @modified_entities;
    if (!$self->get_new_value(%params, field_name => 'content_profile')) {
        my $parameter = $self->get_new_value(%params, field_name => 'parameter');
        if (!$parameter->{param_value_type}) {
            $parameter = F5::ASMConfig::Entity::Parameter->new(dbh => $self->{dbh},
                                                               policy => $policy,
                                                               get_criteria => $parameter);
        }
        if( $parameter->{param_value_type} == F5::ASMConfig::ParamValueType::PARAM_VALUE_XML_VALUE() || 
            $parameter->{param_value_type} == F5::ASMConfig::ParamValueType::PARAM_VALUE_JSON_VALUE() ) {
            my $profile_type = ( $parameter->{param_value_type} == F5::ASMConfig::ParamValueType::PARAM_VALUE_XML_VALUE() ? F5::ASMConfig::ContentProfileType::CONTENT_PROFILE_TYPE_XML() 
                                                                                                                          : F5::ASMConfig::ContentProfileType::CONTENT_PROFILE_TYPE_JSON() );
            my $default_content_profile = F5::ASMConfig::Entity::ContentProfile->get_default_content_profile(
                                                                           dbh => $self->{dbh},
                                                                           policy => $policy,
                                                                           content_profile_type => $profile_type,
                                                                          );

            if ($default_content_profile->{__new_entity}) {
                $default_content_profile->store_add(validate_value => $params{validate_value});
                push(@modified_entities, {event_type => F5::ASMConfig::EventType::ADD, entity => $default_content_profile, existing_entity => $default_content_profile});
            }

            $store_obj->{content_profile} = $default_content_profile->uncast();
           }
    }
    $rv->{modified_entities} = \@modified_entities;
    return $rv;
}

#----------------------------------------------------------------------

sub update_parent_staging {
    my ($self, %params) = @_;
    ## Update parent Parameter
    if (ref $self) { #Nothing should be using update_many_parameter_content_profiles, but just in case.
        my $parameter = $self->get_new_value(%params, field_name => 'parameter');
        $parameter = F5::ASMConfig::Entity::Parameter->new(dbh => $self->{dbh},
                                               policy => $self->{policy},
                                               object => $parameter,); #In case it is a Thrift object and not an Entity
        return $parameter->store_update(store_obj => {last_updated => time()},
                                        validate_value => 0,);
    }
    return;
}

#----------------------------------------------------------------------

1;
