package F5::ASMConfig::Entity::PolicyAttributes;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::ASMConfig;
use F5::ASMConfig::Entity::AllowedResponseCode;
use F5::ASMConfig::Entity::CustomXffHeader;
use F5::ASMConfig::Entity::PolicyNegsigAttributes;
use F5::ASMConfig::Entity::TrustXff;
use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::PolicyAttributes/;

#----------------------------------------------------------------------

sub before_store {
    my($self, %params) = @_;

    my $store_obj = $params{store_obj};
    my $existing_obj = $params{existing_obj};

    if ($params{validate_value}) {

        my @fields = qw(
                    flg_use_dynamic_session_in_url
                    dynamic_session_pattern
                    custom_xff_headers
                   );



        my %field_values;
        my %field_display_name;

        for my $field (@fields) {
            $field_values{$field} = $self->get_new_value(store_obj => $store_obj,
                                                         existing_obj => $existing_obj,
                                                         field_name => $field, );

            $field_display_name{$field} = $self->get_field_display_name_string(%params,field_name => $field);
        }

        if (ref($field_values{custom_xff_headers}) eq "ARRAY" && scalar(@{$field_values{custom_xff_headers}}) > 5) {
            my $error_message = "Number of $field_display_name{custom_xff_headers} is limited to 5.";
            die { error_message => $error_message,
                  display_message => $error_message,
                  error_level     => 'info',
                  error_code      => F5::ASMConfig::ErrorCode::INVALID_FIELD_FORMAT(),
                };
        }

        if ( $field_values{flg_use_dynamic_session_in_url} && ! $field_values{dynamic_session_pattern} ){
            die "$field_display_name{dynamic_session_pattern} can't be empty when $field_display_name{flg_use_dynamic_session_in_url} is set\n";
        }

    }

}
#----------------------------------------------------------------------

1;

