package F5::ASMConfig::Entity::PolicyBrowser;

use strict;
use warnings;

use base qw(F5::ASMConfig::Entity::Base F5::ASMConfig::PolicyBrowser);

#----------------------------------------------------------------------

sub before_store {
    my($self, %params) = @_;
    my $store_mode = $params{store_mode};
    my $store_obj = $params{store_obj};
    my $existing_obj = $params{existing_obj};

    my @fields = qw(
                     browser_definition
                     min_version
                     max_version
                   );

    my %field_values;
    for my $field (@fields) {
       $field_values{$field} = $self->get_new_value( store_obj => $store_obj,
                                                     existing_obj => $existing_obj,
                                                     field_name => $field, );
    }

    if ( !$field_values{browser_definition}->{flg_is_user_defined} ) {
        if ( defined $field_values{min_version} && defined $field_values{max_version} && ($field_values{min_version} > $field_values{max_version}) ) {
            my $error_message = 'Min Version cannot be greater than Max Version'; 
            die { error_message => $error_message,
                  display_message => $error_message,
                  error_level     => 'info',
                  error_code      => F5::ASMConfig::ErrorCode::ACTION_NOT_ALLOWED(),
                };
        }
    }
    else {
        undef $store_obj->{min_version}; 
        undef $store_obj->{max_version};
    }
}

#----------------------------------------------------------------------

1;
