package F5::ASMConfig::Entity::PolicyBrowserDefinition;

use strict;
use warnings;

use F5::Browsers;

use Digest::MD5 qw(md5_hex);

use base qw(F5::ASMConfig::Entity::Base F5::ASMConfig::PolicyBrowserDefinition);

#----------------------------------------------------------------------

sub before_store {
    my($self, %params) = @_;
    my $store_mode = $params{store_mode};
    my $store_obj = $params{store_obj};
    my $existing_obj = $params{existing_obj};

    my @fields = qw(
                     browser_name
                     match_string
                     match_regex
                   );

    my %field_values;
    for my $field (@fields) {
       $field_values{$field} = $self->get_new_value( store_obj => $store_obj,
                                                     existing_obj => $existing_obj,
                                                     field_name => $field, );
    }

    if (defined $field_values{browser_name}) {
        my ($flg_is_factory_browser) = $self->{dbh}->selectrow_array(<<EndQuery, undef, $field_values{browser_name});
SELECT COUNT(*)
  FROM PLC.FACTORY_BROWSERS
 WHERE browser_name = ?
EndQuery

        if ($flg_is_factory_browser) {
            if ( defined $field_values{match_string} || defined $field_values{match_regex} ) {
                my $error_message = 'User-defined browser $field_name{browser_name} conflicts with an existing factory defined browser';
                die { error_message => $error_message,
                      display_message => $error_message,
                      error_level     => 'info',
                      error_code      => F5::ASMConfig::ErrorCode::ACTION_NOT_ALLOWED(),
                    };
            }
        }
        else {
            if ( !( (defined $field_values{match_string}) ^ (defined $field_values{match_regex}) ) ) {
                my $error_message = 'User Defined Browsers must have exactly one of Match Regex or Match String';
                die { error_message => $error_message,
                      display_message => $error_message,
                      error_level     => 'info',
                      error_code      => F5::ASMConfig::ErrorCode::ACTION_NOT_ALLOWED(),
                    };
            }
            if ($self->has_value_changed(%params, check_fields => [ qw(match_string match_regex) ])) {
                $store_obj->{rule} = F5::Browsers::get_browser_rule(browser => \%field_values);
                my $md5_calc = Digest::MD5->new;
                $md5_calc->add($store_obj->{rule});
                $md5_calc->add($field_values{browser_name});
                $store_obj->{browser_md5} = $md5_calc->hexdigest;
            }
        }
    }
}

#----------------------------------------------------------------------

1;
