package F5::ASMConfig::Entity::PolicyHistory;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::ASMConfig;
use F5::PolicyHistory;
use F5::Cfg;

use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::PolicyHistory/;

use constant POLICY_HISTORY_CFG_FILE  =>  '/etc/ts/tools/policy_history.cfg';

my $MAX_SIZE_OF_SAVED_VERSIONS = F5::Cfg::cfg_get_config_item(POLICY_HISTORY_CFG_FILE, 'CleanPolicyHistory', 'maxSizeOfSavedVersions');
my $MIN_RETAINED_FILES_IN_DIR =  F5::Cfg::cfg_get_config_item(POLICY_HISTORY_CFG_FILE, 'CleanPolicyHistory', 'minRetainedFilesInDir') || 2;


#---------------------------------------------------
sub after_store {
    my ($self) = @_;
    return $self->cleanup;
}
#---------------------------------------------------
sub cleanup {
    my ($self) = @_;
    my @versions_to_delete = ref($self)->retrieve_all(dbh => $self->{dbh},
                                                      policy => $self->{policy},
                                                      sort_orders => [ F5::ASMConfig::SortOrder->new({field_name => "revision",
                                                                                                      sort_direction => F5::ASMConfig::SortDirection::SORT_DIRECTION_DESC(),})
                                                                     ],
                                                      page_size => F5::PolicyHistory::MAX_NUMBER_OF_SAVED_VERSIONS,
                                                      start_page => 2);

    for my $version_to_delete (@versions_to_delete) {
        my $filename_to_unlink =  F5::PolicyHistory->construct_filename(revision => $version_to_delete->{revision},
                                                                        policy_id => $version_to_delete->{policy_id},);
        unlink($filename_to_unlink);
        $version_to_delete->delete(suppress_events => 1);
    }

    if ( defined $MAX_SIZE_OF_SAVED_VERSIONS ) {
        F5::Utils::File::cleanup_files(dir => F5::PolicyHistory::DEFAULT_BASE_DIRECTORY,
                                       max_size => $MAX_SIZE_OF_SAVED_VERSIONS,
                                       min_retained_files_in_dir => $MIN_RETAINED_FILES_IN_DIR);
    }

    return;
}

#---------------------------------------------------

sub map_fields_to_entity {
    my($class, %params) = @_;
    my $dbh = $params{dbh};
    my $rec = $params{rec};

    $rec->{is_available_to_be_restored} = (-f F5::PolicyHistory->construct_filename(revision => $rec->{revision},
                                                                                    policy_id => $rec->{policy_id},) ) ? 1 : 0;

    return $class->SUPER::map_fields_to_entity(%params);
}

#---------------------------------------------------
1;
