package F5::ASMConfig::Entity::PolicyTemplate;

use strict;
use warnings;

use F5::Deploy;
use F5::ASMConfig::ASMConfig;
use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::PolicyTemplate/;

#----------------------------------------------------------------------

sub rest_search_filters {
    my ($class) = @_;

    my @search_filters = ( F5::ASMConfig::SearchFilter->new(field_name => 'flg_is_visible',
                                                            field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_EQ(),
                                                            field_value => 1),
                         );

    # When security.tap.allow db var is disabled we hide behavioral waf template
    my $tap_var = F5::ASM::COAPI::Query->new("db_variable", { name => "cloud.licensed.bewaf"})->[0];

    if ($tap_var && defined $tap_var->{value}) {
        my $value = $tap_var->{value} ne '' ? $tap_var->{value} : $tap_var->{default};
        if ($value eq 'disable') {
            push (@search_filters, F5::ASMConfig::SearchFilter->new(field_name => 'icontrol_name',
                                                                    field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_NE(),
                                                                    field_value => 'POLICY_TEMPLATE_BEHAVIORAL_WAF') );
        }
    }

    return \@search_filters;
}

#----------------------------------------------------------------------

# objects, which are not fully in the mcp_object model, override the mcp_folder function
sub mcp_folder {
    return '/Common';
}

#----------------------------------------------------------------------

sub map_fields_to_entity {
    my($class, %params) = @_;

    $class->SUPER::map_fields_to_entity(%params);
    my $rec = $params{rec};
    if ($rec->{template_file}) {
        $rec->{template_file} = F5::Deploy::map_path($rec->{template_file});
    }
}

#----------------------------------------------------------------------

1;

