package F5::ASMConfig::Entity::PolicyThreatCampaign;

use strict;
use warnings;

use F5::ASMConfig::ASMConfig;
use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::PolicyThreatCampaign/;

#----------------------------------------------------------------------

sub get_staging_period_start_time {
    my($class, %params) = @_;
    my $policy = $params{policy};

    my $staging_period = $params{dbh}->selectrow_array(<<EndQuery, undef, $params{policy}->{policy_id});
SELECT threat_campaign_quarantined_period
  FROM PLC.PL_POLICY_THREAT_CAMPAIGN_ATTRIBUTES
  WHERE policy_id = ?
EndQuery

    my $now = time();
    my $staging_start_time = $now - ($staging_period * 60*60*24);

    return $staging_start_time;
}

#----------------------------------------------------------------------

sub rest_search_filters {
    return [
         F5::ASMConfig::SearchFilter->new(field_name => 'threat_campaign_status',
                                          field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_EQ(),
                                          field_value => F5::ASMConfig::ThreatCampaignStatus::THREAT_CAMPAIGN_STATUS_ACTIVE()),
           ];
}

#----------------------------------------------------------------------

sub declarative_search_filters {
    my ($class, %params) = @_;
    my $dbh = $params{dbh};
    my $policy = $params{policy};
    my $search_filters = $class->SUPER::declarative_search_filters(%params);

    if (!$policy) {
        return $search_filters;
    }

    my $threat_campaigns_atributes = F5::ASMConfig::Entity::PolicyThreatCampaignAttributes->new(%params,
                                                                                                get_criteria => { policy_id => $policy->{id} });
    my $db_table = $class->db_table;
    my $where_clause = 'flg_enabled = 0';
    if ($threat_campaigns_atributes->{flg_quarantine_threat_campaigns}) {
        $where_clause = $where_clause . ' OR flg_is_staging = 0';
    }
    my $threat_campaigns = $dbh->selectall_hashref(<<EndQuery, 'threat_campaign_id', undef, $policy->{id});
SELECT threat_campaign_id
  FROM $db_table
 WHERE policy_id = ? AND ($where_clause)
EndQuery

    my @ids = keys %$threat_campaigns;
    push (@$search_filters, F5::ASMConfig::SearchFilter->new(field_name => 'threat_campaign__threat_campaign_id',
                                                             field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_IN(),
                                                             field_value_list => \@ids)
         );

    return $search_filters;
}

#----------------------------------------------------------------------

1;


