package F5::ASMConfig::Entity::RedirectionDomain;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::ASMConfig;
use F5::ASMConfig::Entity::RedirectionProtection;
use F5::ASMConfig::Entity::PBConfigRedirectionProtection;
use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::RedirectionDomain/;

#----------------------------------------------------------------------
sub normalize_redirection_domain {
    my ($self, %params) = @_;
    my $domain_name = $params{field_value};

    # Only explicit strings and pure wildcard are supported
    if ($domain_name eq '*') {
        return {
                completed_successfully => 1,
                normalized_value => $domain_name,
               }
    }

    return $self->normalize_host(%params);
}

#----------------------------------------------------------------------
sub before_store {
   my($self, %params) = @_;
   my $store_obj = $params{store_obj};
   my $existing_obj = $params{existing_obj};

   if ( $self->get_new_value(field_name => 'domain_name', %params) eq '*' ){
       $store_obj->{is_wildcard} = 1;
       $store_obj->{include_subdomains} = 0;
   }else{
       $store_obj->{is_wildcard} = 0;
   }

   $self->adjust_wildcards(%params);

   if ($self->is_pure_wildcard(%params)){
       $self->sync_from_pb_config(%params, 
                                  pb_config_field => "tighten_redirection_domains",
                                  table_name => F5::ASMConfig::Entity::PBConfigRedirectionProtection->db_table());
   }

}

#----------------------------------------------------------------------

sub is_pure_wildcard {
    my ($self, %params) = @_;
    return ($self->get_new_value(%params, field_name => 'domain_name') &&
            $self->get_new_value(%params, field_name => 'domain_name') eq "*");
}

#----------------------------------------------------------------------

sub get_tightening_subquery {
    my($class, %params) = @_;
    my $policy = $params{policy};
    my $want_suggestions = $params{has_suggestions} ? 1 : 0;

    my $plc_dbname = 'PLC';
    my $query = <<EndQuery;
LEFT JOIN (
       SELECT DISTINCT suggestion_id, 1 as matched
       FROM  PLC.PL_SUGGESTIONS
       WHERE policy_id = ? AND suggestion_status = ? AND element_type = ?
    ) tightening_suggestions
    ON (main_table.flg_is_pattern = 1)
EndQuery
    return { query => $query,
             bind_values => [ $policy->{policy_id}, F5::ASMConfig::SuggestionStatus::SUGGESTION_STATUS_PENDING(), $class->element_type],
             db_field => "CASE WHEN tightening_suggestions.matched THEN $want_suggestions ELSE NOT $want_suggestions END"};

}

#----------------------------------------------------------------------

#
1;
