package F5::ASMConfig::Entity::RequestLogger;

use strict;
use warnings;

use JSON;

use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::RequestLogger/;

my %ALLOWED_FILTER_VALUES = ('request_status' => { map { ($_ => 1) } qw(illegal
                                                                        alerted
                                                                        blocked
                                                                      ) },
                            );

#----------------------------------------------------------------------

sub before_store {
    my ($self, %params) = @_;

    my $dbh = $self->{dbh};

    my $store_obj = $params{store_obj};
    my $existing_obj = $params{existing_obj};
    my $call_flags = $params{call_flags};
    my $policy = $params{policy} || $self->{policy};

    my $rv;
    if ($params{validate_value}) {
        my $filter = $self->get_new_value( store_obj => $store_obj,
                                           existing_obj => $existing_obj,
                                           field_name => 'filter' );

        if ($filter && @$filter) {
            for my $single_filter (@$filter) {
                my $error_message;
                if ($single_filter->{field_name} && ! $ALLOWED_FILTER_VALUES{$single_filter->{field_name}}) {
                    $error_message = "Filter field name is invalid ('$single_filter->{field_name}')";
                }
                else {
                    if (! $single_filter->{field_values} || ! @{$single_filter->{field_values}}) {
                        $error_message = "Filter field values cannot be empty";
                    }
                    else {
                        for my $field_value (@{$single_filter->{field_values}}) {
                            if (! $ALLOWED_FILTER_VALUES{$single_filter->{field_name}}->{$field_value}) {
                                $error_message = "Filter field value is invalid ('$field_value')";
                                last;
                            }
                        }
                    }
                }

                if ($error_message) {
                    die { error_message => $error_message,
                            display_message => $error_message,
                            error_level     => 'info',
                            error_code      => F5::ASMConfig::ErrorCode::INVALID_FIELD_FORMAT(),
                        };
                }
            }
        }
    }
}

#----------------------------------------------------------------------

1;
