package F5::ASMConfig::Entity::ResponseScrubbing;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::ASMConfig;
use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::ResponseScrubbing/;

my $MAX_PATTERN_LENGTH = 1000;
#----------------------------------------------------------------------
sub before_store {
    my ($self, %params) = @_;
    my $store_obj = $params{store_obj};
    my $existing_obj = $params{existing_obj};
    my $call_flags = $params{call_flags};

    return unless $params{validate_value};

    my @fields = qw(
                    check_file_content
                    response_scrubbing_formats
                    flg_check_custom_patterns
                    patterns
                    flg_check_exception_patterns
                    exceptions
                    flg_is_enabled
                    flg_check_for_credit_card
                    flg_check_for_ssn
                   );

   my %field_values;
   my %field_display_name;
   my $error_message;

   for my $field (@fields) {
      $field_values{$field} = $self->get_new_value( store_obj => $store_obj,
                                                    existing_obj => $existing_obj,
                                                    call_flags => $call_flags,
                                                    field_name => $field, );

      $field_display_name{$field} = $self->get_field_display_name_string(field_name => $field,%params);
   }

   if ( $field_values{flg_check_custom_patterns} &&  (@{$field_values{patterns}} == 0) ){
       if ($call_flags->{cascade_disable_flags}) {
           $store_obj->{flg_check_custom_patterns} = $field_values{flg_check_custom_patterns} = 0;
       }
       else {
           $error_message =  "$field_display_name{patterns} list must not be empty when $field_display_name{flg_check_custom_patterns} is set";
       }
   }

   if ( $field_values{flg_check_exception_patterns} &&  (@{$field_values{exceptions}} == 0) ){
       if ($call_flags->{cascade_disable_flags}) {
           $store_obj->{flg_check_exception_patterns} = $field_values{flg_check_exception_patterns} = 0;
       }
       else {
          $error_message =  "$field_display_name{exceptions} list must not be empty when $field_display_name{flg_check_exception_patterns} is set";
       }
   }

   if ( $field_values{check_file_content} &&  (@{$field_values{response_scrubbing_formats}} == 0) ){
       if ($call_flags->{cascade_disable_flags}) {
           $store_obj->{check_file_content} = $field_values{check_file_content} = 0;
       }
       else {
           $error_message = "$field_display_name{response_scrubbing_formats} list must not be empty when $field_display_name{check_file_content} is set";
       }
   }

   if ( $field_values{flg_is_enabled} && ! ( $field_values{check_file_content} ||
                                             $field_values{flg_check_custom_patterns} ||
                                             $field_values{flg_check_for_credit_card} || $field_values{flg_check_for_ssn} )){
       if ($call_flags->{cascade_disable_flags}) {
           $store_obj->{flg_is_enabled} = $field_values{flg_is_enabled} = 0;
       }
       else {
           $error_message =  "At least one of the Data Guard features must be enabled ($field_display_name{check_file_content}, $field_display_name{flg_check_custom_patterns}, $field_display_name{flg_check_for_credit_card}, $field_display_name{flg_check_for_ssn})";
       }
   }

   if ($error_message){
          die { error_message => $error_message,
                display_message => $error_message,
                error_level     => 'info',
                error_code      => F5::ASMConfig::ErrorCode::INVALID_FIELD_FORMAT(),
              };
   }

}

#----------------------------------------------------------------------

sub after_store {
    my($self, %params) = @_;

    my $dbh = $self->{dbh};
    my $store_obj = $params{store_obj};
    my $existing_obj = $params{existing_obj};

    return unless $params{validate_value};

    if ($store_obj->{patterns}) {
        my $total_patterns_len =  $dbh->selectrow_array(<<EndQuery, undef, $self->{policy_id});
SELECT SUM(LENGTH(pattern)) 
  FROM $self->{plc_dbname}.PL_RESPONSE_SCRUBBING_PATTERNS
 WHERE policy_id = ?
EndQuery
    
        if ( $total_patterns_len > $MAX_PATTERN_LENGTH ){
            die "Maximum Total Custom Patterns length is $MAX_PATTERN_LENGTH\n";
        }
    }

    if ($store_obj->{exceptions}) {
         my $total_patterns_len =  $dbh->selectrow_array(<<EndQuery, undef, $self->{policy_id});
SELECT SUM(LENGTH(pattern))
  FROM $self->{plc_dbname}.PL_RESPONSE_SCRUBBING_EXCEPTIONS
 WHERE policy_id = ?
EndQuery

        if ( $total_patterns_len > $MAX_PATTERN_LENGTH ){
             die "Maximum Total Exception Patterns length is $MAX_PATTERN_LENGTH\n";
        }
    }
}

#----------------------------------------------------------------------

sub clean_from_parent {
    my ($class, %params) = @_;
    my $policy = $params{policy};
    my $dbh    = $params{dbh};
    my @search_filters;
    if (my $response_scrubbing = $class->new(dbh => $dbh, policy => $policy, get_criteria => {})) {
        $response_scrubbing->store_update(store_obj => { target_patterns => [] },
                                          call_flags => {suppress_events => 1, cascade_disable_flags => 1},
                                         )
    }
}

#----------------------------------------------------------------------

1;
