package F5::ASMConfig::Entity::ScannerConfig;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::Entity::Policy;
use F5::ASMConfig::Entity::ScannerVuln;
use F5::ASMConfig::ASMConfig;
use F5::ASMConfig::Entity::GlobalScannerConfig;
use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::ScannerConfig/;

#----------------------------------------------------------------------

sub get_retrieve_join_tables {
    my($class, %params) = @_;

    my $retval = <<EndQuery;
LEFT JOIN PLC.GLOBAL_SCANNER_CONFIG global
  ON (main_table.scanner_type = global.scanner_type)
EndQuery

    return $retval;
}

#----------------------------------------------------------------------------------

sub additional_field_mapping {
    my($class, %params) = @_;

    my %additional_field_mapping = (
                                    "api_key" => { full_db_field_name => 'IFNULL(global.api_key, main_table.api_key)' },
                                    "scanner_site_name" => { full_db_field_name => 'CASE WHEN (global.scanner_site_name IS NULL) THEN main_table.scanner_site_name WHEN (global.scanner_site_name = \'\') THEN main_table.scanner_site_name ELSE global.scanner_site_name END' },
                                    "flg_is_setting_global" => { full_db_field_name => 'CASE WHEN (global.scanner_site_name IS NULL) THEN 0 WHEN global.scanner_site_name = \'\' THEN 0 ELSE 1 END' },
                                   );

    return \%additional_field_mapping;
}

#----------------------------------------------------------------------------------

sub before_store {
    my($self, %params) = @_;

    my $store_obj = $params{store_obj};
    my $existing_obj = $params{existing_obj};

    my $policy = $self->{policy};
    my $dbh = $self->{dbh};

    my $old_scanner_type = $existing_obj->{scanner_type};
    my $new_scanner_type = $store_obj->{scanner_type};

    if ( $self->has_value_changed(%params, check_fields => [ qw(scanner_type) ]) ) {
        my $vuln_count = F5::ASMConfig::Entity::ScannerVuln->retrieve_all(dbh => $dbh,
                                                                          policy => $policy,
                                                                         )->{total_record_count};

        if ($vuln_count) {
            my $error_message = "Cannot change scanner type from '$old_scanner_type' to '$new_scanner_type' once vulnerabilities were imported";
            die { error_message => $error_message,
                  display_message => $error_message,
                  error_level     => 'info',
                  error_code      => F5::ASMConfig::ErrorCode::ACTION_NOT_ALLOWED(),
                };
        }
    }
}
1;
