package F5::ASMConfig::Entity::ScannerVuln;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::Entity::Policy;
use F5::ASMConfig::ASMConfig;
use F5::Logger;
use F5::NegativeSignatures::Utils;
use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::ScannerVuln/;

#----------------------------------------------------------------------

sub map_fields_to_entity {
    my($class, %params) = @_;
    my $dbh = $params{dbh};
    my $rec = $params{rec};

    $rec->{attack_type_name} = F5::NegativeSignatures::Utils->get_attack_type_name( attack_type_id =>  $rec->{attack_type_id}, dbh => $dbh);

    return $class->SUPER::map_fields_to_entity(%params);
}

#----------------------------------------------------------------------

sub retrieve_all {
    my ($class, %params) = @_;

    if ( ! grep {$_->{field_name} eq 'attack_type_name'} @{$params{sort_orders}}) {
        return $class->SUPER::retrieve_all(%params);
    }

    my @vulns = $class->SUPER::retrieve_all(%params, page_size => 0);
    
    $class->apply_sort_order(result_set => \@vulns,
                             %params);

    my $total_record_count = scalar @vulns;

    $class->apply_paging(result_set => \@vulns,
                         %params);

    if (wantarray) {
        return @vulns;
    }
    else {
        my $iterator = F5::ASMConfig::Entity::Iterator->new(
                                                            get_next_function => sub {return shift(@vulns);},
                                                            get_all_function => sub {return @vulns;},
                                                            total_record_count => $total_record_count,
                                                           );
        return $iterator;
    }
}

1;
