package F5::ASMConfig::Entity::SignatureSet;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::ASMConfig;
use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::SignatureSet/;

my  %system_name_cache;
my  %system_id_cache;

#----------------------------------------------------------------------

sub update_many_as_iterative {
    return 1;
}

#----------------------------------------------------------------------

sub delete_many_as_iterative {
    return 1;
}

#----------------------------------------------------------------------

sub rest_search_filters {
    my ($class) = @_;
    return [ F5::ASMConfig::SearchFilter->new(field_name => 'flg_is_hidden',
                                              field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_NE(),
                                              field_value => 1),
           ];
}

#----------------------------------------------------------------------

sub linked_entity_filters {
    my ($class_or_self, %params) = @_;
    my $entities = $params{entities} || [$class_or_self];
    my @entity_filters = ({
                           filter => F5::ASMConfig::SearchFilter->new({
                                                          field_name => 'set_id',
                                                          field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_IN(),
                                                          field_value_list => [ map { $_->get_id() } @$entities ],}),
                           entity_types => ['F5::ASMConfig::Entity::PolicySignatureSet'],
                          },);
    return @entity_filters;
}

#----------------------------------------------------------------------
sub reload_cache {
  my($class, %params) = @_;

  my $dbh;
  if (ref($class)) {
    $dbh = $class->{dbh};
  }
  else {
    $dbh = $params{dbh};
  }
  %system_name_cache = ();
  %system_id_cache = ();

  my $plc_dbname = 'PLC';
  my $sth = $dbh->prepare(<<EndQuery);
SELECT system_display_name,system_id
  FROM $plc_dbname.NEGSIG_SYSTEMS
EndQuery
  $sth->execute;

  while (my $row = $sth->fetchrow_hashref) {
    $system_name_cache{$row->{system_display_name}} = $row->{system_id};
    $system_id_cache{$row->{system_id}} = $row->{system_display_name};
  }
}
#-----------------------------------------------------------------------
sub get_system_name {
  my($class, %params) = @_;

  my $system_id = $params{system_id};
  unless ($system_id_cache{$system_id}) {
    $class->reload_cache(%params);
  }

  return $system_id_cache{$system_id};
}
#-----------------------------------------------------------------------
sub get_system_id {
  my($class, %params) = @_;

  my $system_name = $params{system_name};
  unless ($system_name_cache{$system_name}) {
    $class->reload_cache(%params);
  }

  return $system_name_cache{$system_name};
}
#------------------------------------------------------------------------
sub retrieve_nested_structures {
  my($self,%params) = @_;

  $self->SUPER::retrieve_nested_structures(%params);

  if ($self->{systems}) {
   for my $set_system (@{$self->{systems}}){
     $set_system->{system_name} = $self->get_system_name( system_id => $set_system->{system_id});
   }
  }
 
}
#------------------------------------------------------------------------

1;

