package F5::ASMConfig::Entity::Suggestion;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::ASMConfig;
use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::Suggestion/;

#----------------------------------------------------------------------

sub before_store {
  my($self, %params) = @_;
  my $store_obj = $params{store_obj};
  my $existing_obj = $params{existing_obj};
  my $call_flags = $params{call_flags} || {};

  $self->SUPER::before_store(%params);

  my $element = $self->get_new_value(%params, field_name => 'element', );
  my $policy = $params{policy} || $self->{policy};
  my $dbh = $params{dbh} || $self->{dbh};

  if ($store_obj->{element}) {
      my $element_type_info = $self->get_cached_element_type(element_type => $element->{element_type});
      my $element_class = $element_type_info->{class};
      my $element_type_name = $element_type_info->{element_type_name}; 
      my $entity = $element_class->new(%params, dbh => $dbh, policy => $policy, object => $element->{$element_type_name}, instantiate_entities => 1);
      my $original_entity = $entity;

      ($store_obj->{sig_id}) = $entity->get_sig_ids(%params);
      ($store_obj->{metachar}) = $entity->get_metachars(%params);
      $store_obj->{section_type} = $entity->get_section_type();

      my $parent_diff_struct = $entity->parent_diff_struct(%params);
      if ($parent_diff_struct) {
          $entity = $entity->get_diff_parent_entity(%params, parent_diff_struct => $parent_diff_struct, remove_non_unique_record_parts => 1);
          if ($store_obj->{suggestion_action}) {
              my $subfield = $parent_diff_struct->{subfield};
              if (ref $entity->{$subfield} eq 'ARRAY') {  #Nested diff structs should appear as append/remove actions
                  if ($store_obj->{suggestion_action} == F5::ASMConfig::SuggestionAction::SUGGESTION_ACTION_DELETE() || 
                      $store_obj->{suggestion_action} == F5::ASMConfig::SuggestionAction::SUGGESTION_ACTION_UPDATE_REMOVE()) {
                      $store_obj->{suggestion_action} = F5::ASMConfig::SuggestionAction::SUGGESTION_ACTION_UPDATE_REMOVE();
                  }
                  else {
                      $store_obj->{suggestion_action} = F5::ASMConfig::SuggestionAction::SUGGESTION_ACTION_UPDATE_APPEND();
                  }
              }
          }
      }

      if ($parent_diff_struct && $parent_diff_struct->{entity_name_from_child}) {
          $store_obj->{element_name} =  $original_entity->get_rest_display_name(%params);
      }
      else {
          $store_obj->{element_name} = $entity->get_rest_display_name(%params);
      }
      $store_obj->{element_type} = $element->{element_type};
      $store_obj->{element_rest_uuid} = $entity->get_rest_uuid(%params);
      $store_obj->{element_kind} = $entity->get_kind(%params);

      my %call_flag_overrides = %{$params{call_flags}};
      $call_flag_overrides{append_nested_struct_on_update} = ($store_obj->{suggestion_action} eq F5::ASMConfig::SuggestionAction::SUGGESTION_ACTION_UPDATE_APPEND());
      $call_flag_overrides{remove_nested_struct_on_update} = ($store_obj->{suggestion_action} eq F5::ASMConfig::SuggestionAction::SUGGESTION_ACTION_UPDATE_REMOVE());

      if (!$store_obj->{description}) {
          my $description_entity = ($parent_diff_struct && $parent_diff_struct->{display_values_from_child}) ? $original_entity
                                                                                                         : $entity;
          if ($store_obj->{suggestion_action} != F5::ASMConfig::SuggestionAction::SUGGESTION_ACTION_DELETE()) {
              $description_entity->instantiate_nested_struct_entities(store_obj => $description_entity,
                                                                      policy => $policy);
              if (defined($description_entity->{flg_is_allowed}) && !$description_entity->{flg_is_allowed}) {
                  #Add disallowed entity
                  my $entity_name = $description_entity->get_element_type_display_name();
                  $store_obj->{description} = "Add this $entity_name to the disallowed $entity_name list.";
                  if (!$store_obj->{support_id_list} || !@{$store_obj->{support_id_list}}) {
                      $store_obj->{description} .= " No valid traffic requested it."
                  }
              }
              else {
                  $store_obj->{description} = $description_entity->get_entity_changes_string(%params, 
                                                                                             call_flags => \%call_flag_overrides,
                                                                                             future_tense => 1, 
                                                                                             for_suggestion => 1,
                                                                                             get_rest_display_name => 1,
                                                                                            );
              }
          }

          if (!$store_obj->{description} || $self->get_new_value(%params, field_name => 'matched_wildcard_element')) {
              my $element_type = $description_entity->element_type();
              my $action = ($store_obj->{suggestion_action} == F5::ASMConfig::SuggestionAction::SUGGESTION_ACTION_DELETE() ) ? 'Delete'
                       : $store_obj->{description} ? 'Add/Update'
                       : 'Add';
              my $entity_name = $description_entity->get_element_type_display_name();
              if ( $element_type == F5::ASMConfig::ElementType::FILETYPE || $element_type == F5::ASMConfig::ElementType::_URL ||
                      $element_type == F5::ASMConfig::ElementType::WEBSOCKET_URL ){
                  if ( $element_type == F5::ASMConfig::ElementType::_URL ) {
                      my $app_id = $store_obj->{element}->{$element_type_name}->{app_id};
                      $entity_name = $app_id ? "HTTPS $entity_name" : "HTTP $entity_name";
                  }
                  elsif ( $element_type == F5::ASMConfig::ElementType::WEBSOCKET_URL ) {
                      my $protocol = $store_obj->{element}->{$element_type_name}->{protocol};
                      $entity_name = $protocol ? "WSS $entity_name" : "WS $entity_name";
                  }
                  if ( $action eq 'Delete' ) {
                      my $thrift_obj = $store_obj->{element}->{$element_type_name};
                      my $element_class = $self->get_cached_element_type(element_type => $element_type)->{class};
                      my $entity = $element_class->new(dbh => $self->{dbh},
                                                       selected_fields => ['flg_is_allowed'],
                                                       get_criteria => $thrift_obj);
                      if ( $entity && $entity->{flg_is_allowed} == 0 ) {
                          $entity_name = "Disallowed $entity_name";
                      }
                  }
              }
              if ( $action ne 'Delete' && $store_obj->{element}->{$element_type_name}->{is_wildcard} ) {
                  $entity_name = "Wildcard $entity_name";
              }

              $store_obj->{description} = $store_obj->{description} ? "$action $entity_name. $store_obj->{description}"
                                                                    : "$action $entity_name";
          }
      }
  }
  if ($params{store_mode} eq 'add') {
      if (my $matched_wildcard_element = $self->get_new_value(%params, field_name => 'matched_wildcard_element', )) {
          my $element_type_info = $self->get_cached_element_type(element_type => $matched_wildcard_element->{element_type});
          my $element_class = $element_type_info->{class};
          my $element_type_name = $element_type_info->{element_type_name};

          my $wildcard_entity = $element_class->new(%params, dbh => $dbh, policy => $policy, 
                                                    object => $matched_wildcard_element->{$element_type_name}, instantiate_entities => 1);
          my $original_wildcard_entity = $wildcard_entity;

          my $parent_diff_struct = $wildcard_entity->parent_diff_struct(%params);
          if ($parent_diff_struct) {
              $wildcard_entity = $wildcard_entity->get_diff_parent_entity(%params, parent_diff_struct => $parent_diff_struct, remove_non_unique_record_parts => 1);
          }
          
          $store_obj->{matched_wildcard_element_rest_uuid} = $wildcard_entity->get_rest_uuid(%params);
          if ($parent_diff_struct && $parent_diff_struct->{entity_name_from_child}) {
              $store_obj->{matched_wildcard_element_name} =  $original_wildcard_entity->get_rest_display_name(%params);
          }
          else {
              $store_obj->{matched_wildcard_element_name} = $wildcard_entity->get_rest_display_name(%params);
          }
      }
      
      $store_obj->{last_occurrence_time} ||= time();
      $store_obj->{first_occurrence_time} ||= $store_obj->{last_occurrence_time};

      if( (not defined $store_obj->{suggestion_reason}) 
          || ($store_obj->{suggestion_reason} != F5::ASMConfig::SuggestionReason::SUGGESTION_REASON_POLICY_TIGHTENING())) {
          $store_obj->{suggestion_reason} = $store_obj->{violation} ? F5::ASMConfig::SuggestionReason::SUGGESTION_REASON_VIOLATION_MITIGATION()
                                                                     : F5::ASMConfig::SuggestionReason::SUGGESTION_REASON_POLICY_REFINING();
      }

      if ( $store_obj->{element_type} == F5::ASMConfig::ElementType::HTTP_CHECK || $store_obj->{element_type} == F5::ASMConfig::ElementType::NORMALIZATION ||
           ( $store_obj->{element_type} == F5::ASMConfig::ElementType::BLOCKING &&
                 ( $store_obj->{violation} == F5::Violation::VIOL_HTTP_PROTOCOL() || $store_obj->{violation} == F5::Violation::VIOL_EVASION() )  )
         ) {
          $store_obj->{is_acceptable_on_microservice} =  F5::License::is_waf_licensed();
      }

      $store_obj->{suggesting_child_policy_count} = 0;
  }

  elsif ($params{session} && $params{session}->{user_name} && $params{session}->{user_name} eq $F5::ASMConfig::Entity::Base::POLICY_BUILDER_USER_NAME) {
      $store_obj->{last_occurrence_time} ||= time();
  }
  
  if(!$store_obj->{occurrence_count} || $store_obj->{occurrence_count} <= 1) {
      $store_obj->{first_occurrence_time} ||= $store_obj->{last_occurrence_time};
  }

  if ( !$store_obj->{average_violation_rating} && $store_obj->{violation_rating_counts} && @{$store_obj->{violation_rating_counts}}) {
      my $occurrences;
      my $total_rating;
      for my $violating_rating_count (@{$store_obj->{violation_rating_counts}}) {
          $occurrences += $violating_rating_count->{occurrence_count};
          $total_rating += ($violating_rating_count->{occurrence_count} * $violating_rating_count->{violation_rating});
      }
      if ($occurrences) {
          $store_obj->{average_violation_rating} = $total_rating / $occurrences;
      }
  }

  if ($store_obj->{average_violation_rating}) {
      $store_obj->{average_violation_rating} = sprintf "%.1f", $store_obj->{average_violation_rating};
  }

  if ($store_obj->{suggesting_child_policy_ids}) {
      $store_obj->{suggesting_child_policy_count} = scalar @{$self->get_new_value(%params, field_name => 'suggesting_child_policy_ids')};
  }

  if ($params{store_mode} eq 'update') {
    if ($existing_obj->{suggestion_status} == F5::ASMConfig::SuggestionStatus::SUGGESTION_STATUS_IGNORED &&
        $store_obj->{suggestion_status} && $store_obj->{suggestion_status} == F5::ASMConfig::SuggestionStatus::SUGGESTION_STATUS_PENDING
       ) {

        my $error_message = "It is forbidden to change the status of suggestions removed from Learning from 'Ignored' to 'Pending'";
        die { error_message => $error_message,
              display_message => $error_message,
              error_level     => 'info',
              error_code      => F5::ASMConfig::ErrorCode::ACTION_NOT_ALLOWED(),
            };
    }

    if ( $self->has_value_changed(%params, check_fields => 'suggestion_status') &&
         $store_obj->{suggestion_status} == F5::ASMConfig::SuggestionStatus::SUGGESTION_STATUS_ACCEPT_GLOBALLY() && 
         !$existing_obj->{sig_id} && !defined($existing_obj->{metachar}) ) {

        die { error_message => 'Suggestion can not be Accepted Globally',
              error_level     => 'info',
              error_code      => F5::ASMConfig::ErrorCode::ACTION_NOT_ALLOWED(),
            };
    }

    if ( $self->is_suggestion_status_accept(object => $existing_obj) &&
        !$self->is_suggestion_status_accept(object => $store_obj)
        ) {

        $store_obj->{suggestion_status} = $existing_obj->{suggestion_status};
    }

    if ( $existing_obj->{has_parent} &&
         !$call_flags->{inherited_call} &&
         $self->has_value_changed(%params, check_fields => 'suggestion_status') &&
         ($store_obj->{suggestion_status} == F5::ASMConfig::SuggestionStatus::SUGGESTION_STATUS_ACCEPT_GLOBALLY() ||
           (($store_obj->{suggestion_status} == F5::ASMConfig::SuggestionStatus::SUGGESTION_STATUS_ACCEPT() ||
             $store_obj->{suggestion_status} == F5::ASMConfig::SuggestionStatus::SUGGESTION_STATUS_ACCEPT_AND_STAGE()) &&
            ($existing_obj->{inheritance_acceptability} == F5::ASMConfig::InheritanceAcceptability::INHERITANCE_ACCEPTABILITY_LOCKED()))
         )
       ) {
        my $throw_exception = 1;
        my $section_type = $existing_obj->{section_type};
        if ($store_obj->{suggestion_status} == F5::ASMConfig::SuggestionStatus::SUGGESTION_STATUS_ACCEPT_GLOBALLY()) {
            if ($existing_obj->{sig_id}) {
                $section_type = F5::ASMConfig::SectionType::SECTION_TYPE_ATTACK_SIGNATURES();
                my $policy_sig = F5::ASMConfig::Entity::PolicySignature->new(dbh => $dbh, policy => $policy, get_criteria => { sig_id => $existing_obj->{sig_id} });
                $throw_exception &&= $policy_sig->{is_inherited};
            }
            else {
                my $section = F5::ASMConfig::Entity::Section->new(dbh => $dbh, policy => $policy, get_criteria => {section_type => $section_type});
                $throw_exception &&= $section->is_inherited();
            }
        }

        if ($throw_exception) {
            my $section_display_name = F5::ASMConfig::Entity::Section->get_field_display_string(field_name => 'section_type', field_value => $section_type);
            my $error_message = "Suggestion cannot be accepted due to Inherited '$section_display_name'";
            die { error_message => $error_message,
                  display_message => $error_message,
                  error_level     => 'info',
                  error_code      => F5::ASMConfig::ErrorCode::MODIFIED_INHERITED_VALUE(),
                };
        }
    }
  }

}

#----------------------------------------------------------------------

sub map_fields_to_entity {
    my($class_or_entity, %params) = @_;
    my $dbh = $params{dbh};
    my $rec = $params{rec} || $class_or_entity;
    my $policy = $params{policy};

    $class_or_entity->SUPER::map_fields_to_entity(%params);

    if ($rec->{element}) {
        my $element_type_info = $class_or_entity->get_cached_element_type(element_type => $rec->{element}->{element_type});

        my $element_class = $element_type_info->{class};
        my $element_type_name = $element_type_info->{element_type_name};

        if ($element_class) {
            my $entity = $element_class->new(dbh => $dbh, 
                                             policy => $policy, 
                                             instantiate_entities => 1,
                                             object => $rec->{element}->{$element_type_name})->get_diff_parent_entity(%params,
                                                                                                                  remove_non_unique_record_parts => 1);
            $element_class = ref $entity; #In case the rest entity is different (ie. diff struct)

            my @unique_record_parts = @{$entity->unique_record_parts(%params,
                                                                 get_criteria => $entity)};
            my %unique_fields = map { ($_ => 1) } @unique_record_parts;

            $rec->{entity} = $element_class->new(dbh => $dbh, policy => $policy, object => {});
            $rec->{entity_changes} = $element_class->new(dbh => $dbh, policy => $policy, object => {});

            for my $field (keys %$entity) {
                if ($unique_fields{$field}) {
                    $rec->{entity}->{$field} = $entity->{$field};
                }
                else {
                    $rec->{entity_changes}->{$field} = $entity->{$field};
                }
            }   
            $rec->{entity}->{rest_uuid} = $rec->{element_rest_uuid};
            if ($policy) {
                $rec->{entity}->{policy} = $policy;
            }
        }
    }

    if ($rec->{matched_wildcard_element}) {
        my $element_type_info = $class_or_entity->get_cached_element_type(element_type => $rec->{matched_wildcard_element}->{element_type});

        my $element_class = $element_type_info->{class};
        my $element_type_name = $element_type_info->{element_type_name};

        if ($element_class) {
            $rec->{matched_wildcard_entity} = $element_class->new(dbh => $dbh,
                                                                  policy => $policy,
                                                                  instantiate_entities => 1,
                                                                  object => $rec->{matched_wildcard_element}->{$element_type_name})->get_diff_parent_entity(%params,
                                                                                                                                                            remove_non_unique_record_parts => 1);
        }
    }
}

#----------------------------------------------------------------------

my %ACCEPT_STATUSES = ( F5::ASMConfig::SuggestionStatus::SUGGESTION_STATUS_ACCEPT() => 1,
                        F5::ASMConfig::SuggestionStatus::SUGGESTION_STATUS_ACCEPT_AND_STAGE() => 1,
                        F5::ASMConfig::SuggestionStatus::SUGGESTION_STATUS_ACCEPT_ON_MICROSERVICES() => 1,
                        F5::ASMConfig::SuggestionStatus::SUGGESTION_STATUS_ACCEPT_GLOBALLY() => 1,);

sub is_suggestion_status_accept {
    my ($class_or_self, %params) = @_;
    my $object = $params{object} || $class_or_self;
    my $suggestion_status = $params{suggestion_status} || $object->{suggestion_status};
    return $suggestion_status && $ACCEPT_STATUSES{$suggestion_status};
}

#----------------------------------------------------------------------

sub suppress_events {
    my ($self, %params) = @_;
    if (($params{store_mode} && $params{store_mode} eq 'add') ||
        ($self->{suggestion_status} && $self->{suggestion_status} != F5::ASMConfig::SuggestionStatus::SUGGESTION_STATUS_PENDING()) ||
        ($params{session} && $params{session}->{user_name} && $params{session}->{user_name} ne $F5::ASMConfig::Entity::Base::POLICY_BUILDER_USER_NAME)) {
        return 0;
    }
    return 1;
}

#----------------------------------------------------------------------

sub skip_inheritance {
    my ($self, %params) = @_;
    my $event = $params{event};
    if (($event->{event_type} == F5::ASMConfig::EventType::DELETE()) ||
        ($event->{update_thrift_obj} && 
         $self->is_suggestion_status_accept(object => $event->{update_thrift_obj}))
       ) {
        return 0;
    }
    return 1;
}

#----------------------------------------------------------------------

sub relayable {
    my ($class_or_self, %params) = @_;
    if (ref $class_or_self) {
        return ($class_or_self->{suggestion_status} == F5::ASMConfig::SuggestionStatus::SUGGESTION_STATUS_IGNORED()) ? F5::ASMConfig::RelayabilityType::RELAYABILITY_TYPE_IS_RELAYABLE()
                                                                                                                     : F5::ASMConfig::RelayabilityType::RELAYABILITY_TYPE_IS_NOT_RELAYABLE();
    }
    else {
        return $class_or_self->SUPER::relayable(%params);
    }
}

#----------------------------------------------------------------------

sub export_declarative {
    my ($class, %params) = @_;
    my $dbh = $params{dbh};
    my $policy = $params{policy};

    my @select_fields = ( qw/entity entity_changes element_kind suggestion_action description/ );
    return $class->SUPER::export_declarative(%params,
                                             selected_fields =>  \@select_fields);
}

#----------------------------------------------------------------------

sub to_declarative {
    my ($self, %params) = @_;

    my $entity = $self->to_rest();
    my $declarative_entity = { action => $entity->{action},
                               description => $entity->{description} };

    my $entity_type = F5::YAML::Schema::Utils::get_entity_type(entity_kind => $entity->{entityKind});
    $declarative_entity->{entityType} = $entity_type->element_rest_name;

    if (defined $entity->{entityChanges}) {
        my $entity_changes = $entity->{entityChanges};
        delete $entity_changes->{lastUpdateMicros};

        if (keys %{$entity_changes}) {
            $declarative_entity->{entityChanges} = $entity_changes;
        }
        F5::YAML::Schema::Utils::item_references_fixup(item => $declarative_entity->{entityChanges},
                                                       entity_type => $entity_type,
                                                       direction => 'to_declarative',
                                                      );
    }

    F5::YAML::Schema::Utils::item_references_fixup(item => $entity->{entityReference},
                                                   entity_type => $entity_type,
                                                   direction => 'to_declarative',
                                                  );
    $declarative_entity->{entity} = $entity->{entityReference};

    return $declarative_entity;
}

#----------------------------------------------------------------------

1;
