package F5::ASMConfig::Entity::ValidHostName;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::ASMConfig;
use F5::DbUtils;

use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::ValidHostName/;
#----------------------------------------------------------------------

sub before_store {
    my($self, %params) = @_;
    my $store_mode = $params{store_mode};
    my $store_obj = $params{store_obj};
    my $existing_obj = $params{existing_obj};
    my $policy = $params{policy} || $self->{policy};
    my $host_name = $self->get_new_value(%params, field_name => 'host_name');
    my $flg_include_subdomains = $self->get_new_value(%params, field_name => 'flg_include_subdomains');

    my $rv;
    my @modified_entities;
    my @pattern_values = (0);

    if ( ( $store_mode eq 'add' && $store_obj->{transparency_enforcement_mode} &&
           $store_obj->{transparency_enforcement_mode}  ==  F5::ASMConfig::TransparencyEnforcementMode::TRANSPARENCY_ENFORCEMENT_TRANSPARENT() ) ||
           $self->has_value_changed(%params, check_fields => [ qw(transparency_enforcement_mode) ]) ){

        my @pattern_values = (0);

        if ( $flg_include_subdomains ){
            push (@pattern_values, 1);
        }

        for my $flg_is_pattern_host ( @pattern_values ) {
            my $current_host_name = $flg_is_pattern_host ? "*.$host_name" : $host_name;
            $current_host_name = lc($current_host_name);

            my $microservice = F5::ASMConfig::Entity::Microservice->new(dbh => $self->{dbh},
                                                                        policy => $policy,
                                                                        get_criteria => { microservice_host_name => $current_host_name, microservice_uri => '/*' },
                                                                        );
            if (!$microservice){
                my $microservice_store_obj =  F5::ASMConfig::Entity::Microservice->new( dbh => $self->{dbh},
                                                                                        policy => $policy,
                                                                                        object => {
                                                                                             microservice_host_name        => $current_host_name,
                                                                                             microservice_uri              => '/*',
                                                                                             flg_is_pattern_uri            => 1,
                                                                                             flg_is_pattern                => 1,
                                                                                             pattern_order                 => 1,
                                                                                             flg_is_pattern_host           => $flg_is_pattern_host,
                                                                                             transparency_enforcement_mode => $store_obj->{transparency_enforcement_mode},
                                                                                             description                   => "Auto-generated during $store_mode of hostname.",
                                                                                               }
                                                                                      );

                $rv = $microservice_store_obj->store_add( store_obj => $microservice_store_obj );

                if( $rv ) {
                   push(@modified_entities, {event_type => F5::ASMConfig::EventType::ADD, entity => $microservice_store_obj, existing_entity => $microservice_store_obj});
                }
            }
            else {
                my $microservice_store_obj =  F5::ASMConfig::Entity::Microservice->new( dbh => $self->{dbh},
                                                                                        policy => $policy,
                                                                                        object => {
                                                                                                 transparency_enforcement_mode => $store_obj->{transparency_enforcement_mode},
                                                                                               }
                                                                                      );

                $rv = $microservice->store_update(store_obj => $microservice_store_obj );

                if( $rv && $rv->{db_change_made} ) {
                    push(@modified_entities, {event_type => F5::ASMConfig::EventType::UPDATE, entity => $microservice_store_obj, existing_entity => $microservice});
                }
            }
        }
    }

   $rv->{modified_entities} = \@modified_entities;
   return $rv;
}

#----------------------------------------------------------------------

sub to_rest {
    my ($self, %params) = @_;

    my $object = $params{object} || $self;
    my $TRANSPARENCY_ENFORCEMENT_HOST_VERSION = '14.1.0';
    my $ver = $params{ver} || $params{call_flags}->{rest_version};
    my $policy = $params{policy} || $self->{policy};

    #if rest_version < 14.1.0
    if ( $ver && F5::BigipVersionUtils::compare_versions(version1 => $TRANSPARENCY_ENFORCEMENT_HOST_VERSION,
                                                         version2 => $ver) > 0) {
        my ($microservice_enforcement_mode) = $self->{dbh}->selectrow_array(<<EndQuery, undef, $policy->{policy_id}, lc($object->{host_name}), '/*');
SELECT transparency_enforcement_mode
  FROM PLC.PL_MICROSERVICES
 WHERE policy_id = ?
   AND microservice_host_name = ?
   AND microservice_uri = ?
EndQuery

       $object->{transparency_enforcement_mode} = $microservice_enforcement_mode ? $microservice_enforcement_mode :
                                                                               F5::ASMConfig::TransparencyEnforcementMode::TRANSPARENCY_ENFORCEMENT_POLICY_DEFAULT;
    }

    return $self->SUPER::to_rest(%params);;
}

#----------------------------------------------------------------------


1;
