package F5::ASMConfig::Entity::VirtualServer;

use strict;
use warnings;

use F5::Mcp::ProfileType;
use F5::Mcp::PolicyAction;
use F5::Mcp::PolicyTarget;
use base qw/F5::ASMConfig::McpBase F5::ASMConfig::VirtualServer/;

#----------------------------------------------------------------------

sub map_fields_to_entity {
    my ($class_or_self, %params) = @_;
    my $obj = $params{rec} || $class_or_self;
    $obj->{flg_is_asm_enabled} = 0;
    $obj->{flg_is_psm_enabled} = 0;
    $obj->{has_ssl_profile} = 0;
    $obj->{has_http_profile} = 0;
    $obj->{http_classes} ||= [];

    my %security_profiles = (F5::Mcp::ProfileType::PROFILE_FTP()      => 'profile_ftp',
                             F5::Mcp::ProfileType::PROFILE_INSTANCE() => 'profile_smtp');

    for my $vs_profile (@{F5::ASM::COAPI::Query->new('virtual_server_profile', {vs_name => $obj->{full_name} })}) {
        my $profile_class = $security_profiles{$vs_profile->{profile_type}};
        if ($vs_profile->{profile_type} == F5::Mcp::ProfileType::PROFILE_WEBSECURITY) {
            for my $vs_policy (@{F5::ASM::COAPI::Query->new('virtual_server_policy', { vs_name => $obj->{full_name} })}) {                        
                if (my ($policy_action) = @{F5::ASM::COAPI::Query->new("policy_action", {target => F5::Mcp::PolicyTarget::L7_POLICY_TARGET_ASM,
                                                                                         action => F5::Mcp::PolicyAction::L7_POLICY_ACTION_ENABLE,
                                                                                         policy_name => $vs_policy->{policy_name}},
                                                                                         do_cache => 1)}) {
                    $obj->{flg_is_asm_enabled} = 1;
                    #For backwards compatibility with EM
                    push (@{$obj->{http_classes}}, F5::ASMConfig::HttpClass->new(http_class_name => $policy_action->{policy},
                                                                                 l7_policy_name => $vs_policy->{policy_name}));
                    $obj->{asm_enabled_l7_policy_name} = $vs_policy->{policy_name};
                }
            }                    
        }
        elsif ($vs_profile->{profile_type} == F5::Mcp::ProfileType::PROFILE_HTTPSECURITY ||
                ($vs_profile->{profile_type} == F5::Mcp::ProfileType::PROFILE_FTP &&
                scalar @{F5::ASM::COAPI::Query->new($profile_class, {name => $vs_profile->{profile_name},
                                                                     security_enabled => 1})}) ||
               ($vs_profile->{profile_type} == F5::Mcp::ProfileType::PROFILE_INSTANCE &&
                scalar @{F5::ASM::COAPI::Query->new('ltcfg_instance_field', {class_name => $profile_class,
                                                                             field_name => 'security_enabled',
                                                                             value => 'true',
                                                                             instance_name => $vs_profile->{profile_name}})})) {
            $obj->{flg_is_psm_enabled} = 1;
        }
        elsif ($vs_profile->{profile_type} == F5::Mcp::ProfileType::PROFILE_SERVERSSL ||
               $vs_profile->{profile_type} == F5::Mcp::ProfileType::PROFILE_CLIENTSSL) {
            $obj->{has_ssl_profile} = 1;
        }

        if ($vs_profile->{profile_type} == F5::Mcp::ProfileType::PROFILE_HTTP) {
            $obj->{has_http_profile} = 1;
        }
    }     
}

#----------------------------------------------------------------------

1;

