package F5::ASMConfig::Entity::XmlWssConfiguration;

use strict;
use warnings;

use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::ASMConfig;
use base qw/F5::ASMConfig::Entity::Base F5::ASMConfig::XmlWssConfiguration/;

#----------------------------------------------------------------------
sub before_store {
    my($self, %params) = @_;

    my $db_rec = $params{db_rec}; 
    my $store_obj = $params{store_obj};
    my $store_mode = $params{store_mode};

    if ($store_mode eq 'add'){
        $db_rec->{profile_id} = $store_obj->{xml_profile}->{profile_id};
    }

    if ($store_obj->{client_certificates}) {
        for my $client_certificate (@{$store_obj->{client_certificates}}) {
            if ( !$client_certificate->{cert_type} || !$client_certificate->{key_name} ) {
                $client_certificate = F5::ASMConfig::Entity::Certificate->new(dbh => $self->{dbh},
                                                                              logger => $self->{logger},
                                                                              policy => $self->{policy},
                                                                              get_criteria => $client_certificate, );
            }
            if ( $client_certificate->{cert_type} == F5::ASMConfig::CertificateType::CERTIFICATE_TYPE_SERVER ) {
                my $error_message = "$client_certificate->{key_name} is a Server Certificate.";
                die { error_message => $error_message,
                      display_message => $error_message,
                      error_level     => 'info',
                      error_code      => F5::ASMConfig::ErrorCode::INVALID_FIELD_FORMAT(),
                    };
            }
        }
    }

    if ($store_obj->{server_certificate}) {
        for my $server_certificate (@{$store_obj->{server_certificate}}) {
            if ( !$server_certificate->{cert_type} || !$server_certificate->{key_name} ) {
                $server_certificate = F5::ASMConfig::Entity::Certificate->new(dbh => $self->{dbh},
                                                                              logger => $self->{logger},
                                                                              policy => $self->{policy},
                                                                              get_criteria => $server_certificate, );
            }
            if ( $server_certificate->{cert_type} == F5::ASMConfig::CertificateType::CERTIFICATE_TYPE_CLIENT ) {
                my $error_message = "$server_certificate->{key_name} is a Client Certificate.";
                die { error_message => $error_message,
                      display_message => $error_message,
                      error_level     => 'info',
                      error_code      => F5::ASMConfig::ErrorCode::INVALID_FIELD_FORMAT(),
                    };
            }
        }
    }
}

#----------------------------------------------------------------------


1;

