package F5::ASMConfig::Handler::AllowedMethod;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::ASMConfig;
use F5::ASMConfig::Entity::AllowedMethod;

our $entity_class = 'F5::ASMConfig::Entity::AllowedMethod';

my $default_table = 'PLC.PL_ALLOWED_METHODS_DEFAULTS';
our @handled_functions = (
                            {
                             function_name => 'delete_allowed_method',
                             relayable => 1,
                            },
                            {
                             function_name => 'reset_defaults_allowed_methods',
                             relayable => 1,
                            },
                            {
                             function_name => 'get_default_allowed_method',
                            },
                            {
                             function_name => 'delete_many_allowed_methods',
                             relayable => 1,
                            },
                            {
                             function_name => 'update_many_allowed_methods',
                             relayable => 1,
                             criteria_index => 3,
                             store_index => 4,
                            },
                         );

#----------------------------------------------------------------------

sub delete_allowed_method{
    my($self,$session, $call_flags,$policy, $allowed_method) = @_;

    my $rv;
    my $saved_error;
    eval {
        $rv = $self->base_delete( entity_class => $entity_class,
                                  rpc_params => [ $session, $call_flags, $policy, $allowed_method]);
    };
    if ($@) {
        $saved_error = $@;
    }

    if (! $call_flags->{skip_mcp}) {
        F5::ASMConfig::Entity::AllowedMethod->cleanup_user_defined_methods(dbh => $self->{dbh});
    }

    if ($saved_error) {
        die $saved_error;
    }

    return $rv;
}
#----------------------------------------------------------------------

sub  reset_defaults_allowed_methods{
  my($self,$session, $call_flags,$policy) = @_;
  my $dbh = $self->{dbh};

  return $self->base_reset_default_rows( entity_class => $entity_class,
                                         default_table => $default_table,
                                         rpc_params => [ $session, $call_flags, $policy]);
}
#----------------------------------------------------------------------
sub get_default_allowed_method{
  my($self,$session, $call_flags) = @_;
  my $dbh = $self->{dbh};


  my $res = $self->base_get_default_rows( entity_class => $entity_class,
                                          default_table => $default_table,
                                         );

   return {return_value => $res};
}
#-----------------------------------------------------------------------

sub delete_many_allowed_methods {
    my($self, $session, $call_flags, $policy, $allowed_method_criteria) = @_;

    my $local_call_flags = F5::ASMConfig::CallFlags->new($call_flags);

    if (!defined $local_call_flags->{search_filters}){
       $local_call_flags->{search_filters} = [];
    }

    push(@{$local_call_flags->{search_filters}},
                 { field_name => 'method_name',
                   field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_NOT_IN(),
                   field_value_list => [qw(GET POST)] });

    my $rv = $self->base_delete_many(entity_class => $entity_class,
                                     rpc_params => [ $session, $local_call_flags, $policy, $allowed_method_criteria ],);

    return $rv;
}

#-----------------------------------------------------------------------

sub update_many_allowed_methods {
    my($self, $session, $call_flags, $policy, $allowed_method_criteria, $store_allowed_method) = @_;

    my $local_call_flags = F5::ASMConfig::CallFlags->new($call_flags);

    if (!$local_call_flags->{inherited_call}) { #Allow disassociate call to set is_inherited to false on _all_ headers
        if (!defined $local_call_flags->{search_filters}){
           $local_call_flags->{search_filters} = [];
        }

        push(@{$local_call_flags->{search_filters}},
                     { field_name => 'method_name',
                       field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_NOT_IN(),
                       field_value_list => [qw(GET POST HEAD)] });
    }

    my $rv = $self->base_update_many(entity_class => $entity_class,
                                     rpc_params => [ $session, $local_call_flags,$policy, $allowed_method_criteria, $store_allowed_method ],);

    return $rv;
}

#-----------------------------------------------------------------------
1;

