package F5::ASMConfig::Handler::ApplyPolicyTask;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::TaskBase/;
use F5::ASMConfig::Entity::ApplyPolicyTask;
use F5::ASM::COAPI;

our @handled_functions = (
                            {
                             function_name => 'add_apply_policy_task',
                             criteria_index => 3,
                             store_index    => 4,
                            },
                            {
                              function_name => 'update_apply_policy_task',
                              criteria_index => 3,
                              store_index => 4
                            },
                            { 
                             function_name => 'execute_apply_policy_task',
                             exclusive_lock_file => $F5::DbUtils::LOCK_FILE,
                            },
                            {
                             function_name => 'delete_apply_policy_task',
                            },
                            {
                             function_name => 'delete_many_apply_policy_tasks',
                            },
                         );

our $entity_class = 'F5::ASMConfig::Entity::ApplyPolicyTask';

#----------------------------------------------------------------------

sub add_apply_policy_task {
  my($self, @rpc_params) = @_;
  return F5::ASMConfig::Handler::TaskBase::add_task(self => $self,
                                                    rpc_params => \@rpc_params,
                                                    entity_class => $entity_class,
                                                    task_type => 'apply_policy',);
}

#----------------------------------------------------------------------

sub update_apply_policy_task {
    my ($self, @rpc_params) = @_;
    return F5::ASMConfig::Handler::TaskBase::update_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                         task_type => 'apply_policy',);
}

#----------------------------------------------------------------------

sub apply_policy {
    my (%params) = @_;
    my $self       = $params{self};
    my $session    = $params{session};
    my $call_flags = $params{call_flags};
    my $task       = $params{task};

    my $policy = $task->{target_policy};
    if (!$policy) {
        return $self->handle_error(action => 'execute',
                                   error_thrown => 'Policy not found',
                                   error_code => F5::ASMConfig::ErrorCode::MISSING_POLICY(),
                                   call_flags => $call_flags,
                                   entity_class => $entity_class,
                                   thrift_obj => $task,
                                  );
    }

    my $set_active_flags = F5::ASMConfig::SetActiveFlags->new($call_flags->{set_active_flags});
    if (!$policy->{is_active_policy}) {
        $set_active_flags->{activate_policy} = 1;
        $call_flags->{writes_to_mcp} = 1;
    }

    $self->begin_work($self->{session});
    $self->set_active($session, $call_flags, $policy, $set_active_flags);
    $self->commit($self->{session});

    $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_COMPLETED();
}


#----------------------------------------------------------------------
sub execute_apply_policy_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::execute_task(self => $self,
                                                          rpc_params => \@rpc_params,
                                                          entity_class => $entity_class,
                                                          task_type => 'apply_policy',
                                                          execute_func => sub { apply_policy(@_); },
                                                          );
}

#----------------------------------------------------------------------

sub delete_apply_policy_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                        );
}

#----------------------------------------------------------------------

sub delete_many_apply_policy_tasks {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_many_tasks(self => $self,
                                                               rpc_params => \@rpc_params,
                                                               entity_class => $entity_class,
                                                              );
}

#----------------------------------------------------------------------

1;
