package F5::ASMConfig::Handler::ApplyServerTechnologiesTask;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::TaskBase/;
use F5::ASMConfig::Entity::ApplyServerTechnologiesTask;

our @handled_functions = (
                            {
                             function_name => 'add_apply_server_technologies_task',
                             criteria_index => 3,
                             store_index    => 4,
                            },
                            {
                              function_name => 'update_apply_server_technologies_task',
                              criteria_index => 3,
                              store_index => 4
                            },
                            {
                             function_name => 'execute_apply_server_technologies_task',
                             exclusive_lock_file => $F5::DbUtils::LOCK_FILE,
                            },
                            {
                             function_name => 'delete_apply_server_technologies_task',
                            },
                            {
                             function_name => 'delete_many_apply_server_technologies_tasks',
                            },
                         );

our $entity_class = 'F5::ASMConfig::Entity::ApplyServerTechnologiesTask';

#----------------------------------------------------------------------
sub add_apply_server_technologies_task {
  my($self, @rpc_params) = @_;
  return F5::ASMConfig::Handler::TaskBase::add_task(self => $self,
                                                    rpc_params => \@rpc_params,
                                                    entity_class => $entity_class,
                                                    task_type => 'apply_server_technologies',);
}

#----------------------------------------------------------------------
sub update_apply_server_technologies_task {
    my ($self, @rpc_params) = @_;
    return F5::ASMConfig::Handler::TaskBase::update_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                         task_type => 'apply_server_technologies',);
}

#----------------------------------------------------------------------
sub apply_server_technologies {
    my (%params) = @_;
    my $self       = $params{self};
    my $dbh        = $self->{dbh};
    my $session    = $params{session};
    my $call_flags = $params{call_flags};
    my $task       = $params{task};
    my $policy     = $task->{target_policy};
    my $server_technologies = $task->{server_technologies} || [] ;
    my $get_messages_only = $task->{get_messages_only};
    my $include_implied_server_technologies = $task->{include_implied_server_technologies};
    my $rv;
    my @messages;

    $call_flags->{apply_side_effects} = 1;
    $call_flags->{include_implied_server_technologies} = defined $include_implied_server_technologies ? $include_implied_server_technologies : 1;

    push @{$call_flags->{ignored_errors}}, F5::ASMConfig::IgnoredError->new({error_code => F5::ASMConfig::ErrorCode::DUPLICATE_RECORD()});
    $self->begin_work($session);

    eval {
        for my $server_technology ( @{$server_technologies} ) {
            my $policy_server_technology = F5::ASMConfig::PolicyServerTechnology->new(server_technology => $server_technology);
            if ( $get_messages_only ) {
                 $rv = $self->get_messages_only_for_add_policy_server_technology($session, $call_flags, $policy, $policy_server_technology);
                 push (@messages, @$rv);
            }
            else {
                 $rv = $self->add_policy_server_technology($session, $call_flags, $policy, $policy_server_technology);
            }
        }

        $self->commit($session);
     };

     my @unique_messages = do { my %seen; grep { !$seen{$_}++ }  @messages };

     if ($@) {
         $task->{message}  = $@->{error_message};
         $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_FAILURE();
     }
     else {
         $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_COMPLETED();
         $task->{message} = join("\n", @unique_messages);
     }
}

#----------------------------------------------------------------------
sub execute_apply_server_technologies_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::execute_task(self => $self,
                                                          rpc_params => \@rpc_params,
                                                          entity_class => $entity_class,
                                                          task_type => 'apply_server_technologies',
                                                          execute_func => sub { apply_server_technologies(@_); },
                                                          );
}

#----------------------------------------------------------------------

sub delete_apply_server_technologies_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                        );
}

#----------------------------------------------------------------------

sub delete_many_apply_server_technologies_tasks {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_many_tasks(self => $self,
                                                               rpc_params => \@rpc_params,
                                                               entity_class => $entity_class,
                                                              );
}

#----------------------------------------------------------------------

1;
