package F5::ASMConfig::Handler::AsmConfigSettings;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::ASMConfig;
use F5::ASMConfig::Entity::AsmConfigSettings;
use F5::Cluster::Sync;

our @handled_functions = (
                            {
                             function_name => 'reset_defaults_asm_config_settings',
                             relayable => 1,
                            },
                            {
                             function_name => 'get_sync_status',
                            },
                            {
                             function_name => 'get_sync_status_file',
                            },
                            {
                             function_name => 'apply_sync',
                            },
                         );

our $entity_class = 'F5::ASMConfig::Entity::AsmConfigSettings';

#----------------------------------------------------------------------

sub reset_defaults_asm_config_settings {
    my ($self, $session, $call_flags) = @_;

    my $dbh = $self->{dbh};

    my $default_asm_config_settings = F5::ASMConfig::Entity::AsmConfigSettings->new(dbh => $self->{dbh}, preload_default => 1);

    return $self->base_update(rpc_params => [$session, $call_flags, undef, $default_asm_config_settings],
                              entity_class => $entity_class,
                             );
}

#----------------------------------------------------------------------

sub get_sync_status{
    my ($self, $session, $call_flags) = @_;
    my $dbh = $self->{dbh};

    my $cluster_sync = F5::Cluster::Sync->new(dbh => F5::DbUtils::get_dbh(),
                                              logger => $self->{logger});

    my @retval = ();

    my $sync_status = $cluster_sync->get_sync_status;

    if(ref $sync_status eq 'ARRAY'){
       for my $status (@{$sync_status}) {
          push @retval, F5::ASMConfig::SyncStatusRes->new($status);
       }
    }

   return { return_value => \@retval };
}

#----------------------------------------------------------------------

sub get_sync_status_file {
    my ($self, $session, $call_flags) = @_;

    my $cluster_sync = F5::Cluster::Sync->new(dbh => $self->{dbh},
                                              logger => $self->{logger});

    $cluster_sync->get_sync_status_file;

    return { return_value => 1 };
}

#----------------------------------------------------------------------

sub apply_sync {
    my ($self, $session, $call_flags, $placeholder, $apply_sync_flags) = @_;

    my $cluster_sync = F5::Cluster::Sync->new(dbh => $self->{dbh},
                                              %$apply_sync_flags);


    #Use internal transaction
    $self->{dbh}->commit;
    my $rv;
    eval {
        $rv = $cluster_sync->apply_sync(asm_config_handler => $self,
                                        %$apply_sync_flags,
                                       );
    };
    if ($self->{dbh}->{AutoCommit}) {
        $self->{dbh}->begin_work;
    }
    if (! $rv) {
        $rv = {};
    }

    return { return_value => F5::ASMConfig::ApplySyncRes->new(%$rv) };
}


#----------------------------------------------------------------------

1;
