package F5::ASMConfig::Handler::AsmHttpMethod;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use File::Basename;
use F5::ASM::COAPI;
use F5::BigipUtils;

our @handled_functions = (
                           {
                             function_name => "add_asm_http_method",
                             write_access => 1,
                             relayable => 1,
                             writes_to_mcp => 1,
                             store_index   => 3,
                             allow_action_on_all_partitions => 1,
                           },
                         );

our $entity_class = 'F5::ASMConfig::Entity::AsmHttpMethod';

#----------------------------------------------------------------------
sub add_asm_http_method {
    my ($self, $session, $call_flags, $place_holder, $asm_http_method) = @_;

    if ($call_flags->{skip_mcp}) {
        return {return_value => $asm_http_method};
    }

    my $method_name = $asm_http_method->{method_name};

    if ( !$method_name){
        return $self->handle_error(action => 'add',
                                   display_message => "Method name is a required field",
                                   entity_class => $entity_class);
    }

    if ( $method_name =~ /([^-a-zA-Z0-9._%])/) {
        return $self->handle_error(action => 'add',
                                   display_message => "The value contains an illegal character '$1'",
                                   entity_class => $entity_class);
    }
    
    my $found_methods = F5::ASM::COAPI::Query->new('asm_http_method', {  leaf_name => $method_name });
    my $found_method = $found_methods->[0] if ($found_methods && @$found_methods);

    if ($found_method) {
        return $self->handle_error(action => 'add',
                                   display_message => "Method '$method_name' already exists in the pool.\n",
                                   entity_class => $entity_class,
                                  );
    }
    
    $asm_http_method->{name} = "/Common/$method_name";
    return $self->base_add(rpc_params => [$session, $call_flags, $place_holder, $asm_http_method],
                           entity_class => $entity_class);
 
}
#----------------------------------------------------------------------
1;
