package F5::ASMConfig::Handler::AssociateGraphqlIdlFilesTask;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::TaskBase/;
use F5::ASMConfig::Entity::AssociateGraphqlIdlFilesTask;
use F5::ASMConfig::Entity::GraphqlProfile;

our @handled_functions = (
                            {
                             function_name => 'add_associate_graphql_idl_files_task',
                             criteria_index => 3,
                             store_index    => 4,
                             write_access   => 1,
                            },
                            {
                              function_name => 'update_associate_graphql_idl_files_task',
                              criteria_index => 3,
                              store_index => 4
                            },
                            {
                             function_name => 'execute_associate_graphql_idl_files_task',
                            },
                            {
                             function_name => 'delete_associate_graphql_idl_files_task',
                            },
                            {
                             function_name => 'delete_many_associate_graphql_idl_files_tasks',
                            },
                         );

our $entity_class = 'F5::ASMConfig::Entity::AssociateGraphqlIdlFilesTask';

#----------------------------------------------------------------------

sub add_associate_graphql_idl_files_task {
  my($self, @rpc_params) = @_;
  return F5::ASMConfig::Handler::TaskBase::add_task(self => $self,
                                                    rpc_params => \@rpc_params,
                                                    entity_class => $entity_class,
                                                    task_type => 'associate_graphql_idl_files',);
}

#----------------------------------------------------------------------

sub update_associate_graphql_idl_files_task {
    my ($self, @rpc_params) = @_;
    return F5::ASMConfig::Handler::TaskBase::update_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                         task_type => 'associate_graphql_idl_files',);
}

#----------------------------------------------------------------------

sub associate_graphql_idl_files {
    my (%params) = @_;
    my $self       = $params{self};
    my $session    = $params{session};
    my $call_flags = $params{call_flags};
    my $task       = $params{task};

    my $graphql_profile = $task->{graphql_profile};
    my $primary_idl_file = $task->{primary_idl_file};

    my $policy = $params{policy};

    my $rv;
    eval {
        my $store_graphql_profile = F5::ASMConfig::Entity::GraphqlProfile->new(dbh => $self->{dbh},
                                                                       object => {  idl_files => [ F5::ASMConfig::GraphqlProfileIdlFile->new({ flg_is_primary_enforced => 1,
                                                                                                                                               idl_file => $primary_idl_file})]});
        $self->begin_work($self->{session});
        $rv = $self->update_graphql_profile($session, $call_flags, $policy, $graphql_profile, $store_graphql_profile);
        $self->commit($self->{session});
    };

    if ($@) {
        $task->{message} = $@->{error_message};
        if ($@->{additional_error_messages}) {
            $task->{missing_files} = $@->{additional_error_messages};
        }
        $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_FAILURE();
    }
    else {
        $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_COMPLETED();
    }

}


#----------------------------------------------------------------------
sub execute_associate_graphql_idl_files_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::execute_task(self => $self,
                                                          rpc_params => \@rpc_params,
                                                          entity_class => $entity_class,
                                                          task_type => 'associate_graphql_idl_files',
                                                          execute_func => sub { associate_graphql_idl_files(@_); },
                                                          );
}

#----------------------------------------------------------------------

sub delete_associate_graphql_idl_files_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                        );
}

#----------------------------------------------------------------------

sub delete_many_associate_graphql_idl_files_tasks {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_many_tasks(self => $self,
                                                               rpc_params => \@rpc_params,
                                                               entity_class => $entity_class,
                                                              );
}

#----------------------------------------------------------------------

1;
