package F5::ASMConfig::Handler::AssociateJsonValidationFilesTask;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::TaskBase/;
use F5::ASMConfig::Entity::AssociateJsonValidationFilesTask;

our @handled_functions = (
                            {
                             function_name => 'add_associate_json_validation_files_task',
                             criteria_index => 3,
                             store_index    => 4,
                             write_access   => 1,
                            },
                            {
                              function_name => 'update_associate_json_validation_files_task',
                              criteria_index => 3,
                              store_index => 4
                            },
                            {
                             function_name => 'execute_associate_json_validation_files_task',
                            },
                            {
                             function_name => 'delete_associate_json_validation_files_task',
                            },
                            {
                             function_name => 'delete_many_associate_json_validation_files_tasks',
                            },
                         );

our $entity_class = 'F5::ASMConfig::Entity::AssociateJsonValidationFilesTask';

#----------------------------------------------------------------------

sub add_associate_json_validation_files_task {
  my($self, @rpc_params) = @_;
  return F5::ASMConfig::Handler::TaskBase::add_task(self => $self,
                                                    rpc_params => \@rpc_params,
                                                    entity_class => $entity_class,
                                                    task_type => 'associate_json_validation_files',);
}

#----------------------------------------------------------------------

sub update_associate_json_validation_files_task {
    my ($self, @rpc_params) = @_;
    return F5::ASMConfig::Handler::TaskBase::update_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                         task_type => 'associate_json_validation_files',);
}

#----------------------------------------------------------------------

sub associate_json_validation_files {
    my (%params) = @_;
    my $self       = $params{self};
    my $session    = $params{session};
    my $call_flags = $params{call_flags};
    my $task       = $params{task};

    my $json_profile = $task->{json_profile};
    my $primary_validation_file = $task->{primary_json_validation_file};
    my $json_profile_validation_files =  $task->{json_profile_validation_files} || [];
    my $policy = $params{policy};

    my $rv;
    eval {
        my $store_json_profile = F5::ASMConfig::Entity::JsonProfile->new(dbh => $self->{dbh},
                                                                       object => { flg_follow_schema_links => $task->{flg_follow_schema_links},
                                                                                   validation_files => [ F5::ASMConfig::JsonProfileValidationFile->new({ flg_is_primary_enforced => 1,
                                                                                                                                                        json_validation_file => $primary_validation_file})]});
        for my $json_profile_validation_file (@$json_profile_validation_files) {
            push (@{$store_json_profile->{validation_files}}, $json_profile_validation_file);
        }

        $self->begin_work($self->{session});
        $rv = $self->update_json_profile($session, $call_flags, $policy, $json_profile, $store_json_profile);
        $self->commit($self->{session});
    };

    if ($@) {
        $task->{message} = $@->{error_message};
        if ($@->{additional_error_messages}) {
            $task->{missing_files} = $@->{additional_error_messages};
        }
        $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_FAILURE();
    }
    else {
        $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_COMPLETED();
    }

}


#----------------------------------------------------------------------
sub execute_associate_json_validation_files_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::execute_task(self => $self,
                                                          rpc_params => \@rpc_params,
                                                          entity_class => $entity_class,
                                                          task_type => 'associate_json_validation_files',
                                                          execute_func => sub { associate_json_validation_files(@_); },
                                                          );
}

#----------------------------------------------------------------------

sub delete_associate_json_validation_files_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                        );
}

#----------------------------------------------------------------------

sub delete_many_associate_json_validation_files_tasks {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_many_tasks(self => $self,
                                                               rpc_params => \@rpc_params,
                                                               entity_class => $entity_class,
                                                              );
}

#----------------------------------------------------------------------

1;
