package F5::ASMConfig::Handler::BehavioralWafFile;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::Deploy;
use F5::DbUtils;
use F5::Logger;
use F5::ASMConfig::Entity::ServerTechnology;
use F5::SystemUtils;
use F5::LockFactory;
use F5::PrepareConf::Policy;
use F5::SetActive;
use File::Find;
use File::Path;
use F5::BehavioralWafFile;

my $TEMP_BASE_DIR = F5::Deploy::map_path("/ts/var/tmp");

our @handled_functions = (
                           {
                             function_name => 'update_behavioral_waf_file',
                             filename_arguments => ['3.filename'],
                             relayable => 1,
                             write_access => 1,
                           },
                           {
                             function_name => 'set_behavioral_waf_active',
                           },
                         );

#----------------------------------------------------------------------

sub update_behavioral_waf_file {
    my ($self, $session, $call_flags, $place_holder, $update_file) = @_;

    my $rv = F5::ASMConfig::BehavioralWafUpdateRes->new();
    eval {
        my $behavioral_waf_file = F5::BehavioralWafFile->new(dbh => $self->{dbh},
                                                             logger => $self->{logger},
                                                             filename => $update_file->{filename});

        $behavioral_waf_file->update_behavioral_waf_file();
        $self->set_behavioral_waf_active($session, $call_flags);
        $rv->{completed_successfully} = 1;
    };
    if (my $err = $@) {
        $rv->{completed_successfully} = 0;
        $rv->{error_msg} = $err->{error_message};
    }

    return { return_value => $rv };
}

#----------------------------------------------------------------------

sub set_behavioral_waf_active {
    my($self, $session, $call_flags) = @_;

    my $logger = $self->{logger};
    my $dbh = $self->{dbh};
    my $lock_file =  $F5::DbUtils::LOCK_FILE;
    my $locker = F5::LockFactory::get_locker(lock_filename => $lock_file,
                                             logger => $logger);

    $locker->lock();

    my $active_policies = $dbh->selectall_hashref('SELECT account_id, id as active_policy_id FROM PLC.PL_POLICIES WHERE account_id > 0', 'account_id');

    my $prepareconf_policy = F5::PrepareConf::Policy->new(dbh => $dbh,
                                                          logger => $logger, );

    $prepareconf_policy->prepare_deception_response_pages();
    $prepareconf_policy->prepare_deception_traps();
    $prepareconf_policy->prepare_negsig_enforcer_names();

    for my $active_policy (values %$active_policies) {
        $prepareconf_policy->prepare(account_id => $active_policy->{account_id},
                                     policy_id => $active_policy->{active_policy_id},
                                     policy_parts_to_prepare => [ 'blocking',
                                                                  'deception_traps',
                                                                  'server_technology_responses',
                                                                  'signature_responses' ], );
    }

    $dbh->commit unless $dbh->{AutoCommit};

    F5::SetActive::set_active(dbh => $dbh,
                              logger => $logger,
                              do_update_core => 1,
                              send_sync => 1,
                              asm_config_handler => $self, );

    $locker->unlock();

    return { return_value => 1 };
}

#----------------------------------------------------------------------

1;
