package F5::ASMConfig::Handler::BruteForce;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::ASMConfig;
use F5::ASMConfig::Entity::BruteForce;

our @handled_functions = (
                           {
                             function_name => 'delete_many_brute_forces',
                             relayable => 1,
                           },
                           {
                             function_name => 'delete_brute_force',
                             relayable => 1,
                           },
                         );

our $entity_class = 'F5::ASMConfig::Entity::BruteForce';
#----------------------------------------------------------------------
sub delete_many_brute_forces {
    my($self, $session, $call_flags, $policy, $brute_force_criteria) = @_;

    if (!defined $call_flags){
        $call_flags = F5::ASMConfig::CallFlags->new();
    }

    if (!defined $call_flags->{search_filters}){
       $call_flags->{search_filters} = [];
    }

    push(@{$call_flags->{search_filters}},
                 { field_name => 'flg_is_default',
                   field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_EQ(),
                   field_value => 0 });

    my $rv = $self->base_delete_many(entity_class => $entity_class,
                                     rpc_params => [ $session, $call_flags,$policy, $brute_force_criteria ],);

    return $rv;
}

#----------------------------------------------------------------------
sub delete_brute_force {
  my($self, $session, $call_flags, $policy,$brute_force) = @_;

  my $deleted_brute_force = F5::ASMConfig::Entity::BruteForce->new(  dbh => $self->{dbh},
                                                                     policy => $policy,
                                                                     get_criteria => $brute_force );


  if ( $deleted_brute_force->{flg_is_default} ){
        return $self->handle_error(action => 'delete',
                                   error_thrown => 'Default brute_force cannot be deleted',
                                   entity_class => $entity_class,
                                   thrift_obj  => $brute_force,
                                   error_code   => F5::ASMConfig::ErrorCode::ACTION_NOT_ALLOWED(),
                                   call_flags => $call_flags,
                                  );
  }

  return $self->base_delete(rpc_params => [$session, $call_flags, $policy,$brute_force],
                            entity_class => $entity_class,
                           );
}
#----------------------------------------------------------------------
1;

