package F5::ASMConfig::Handler::CentralConfigurationAddress;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::ASMConfig;
use File::Basename;


our @handled_functions = (
                           {
                             function_name => 'add_central_configuration_address',
                             store_index   => 3,
                           },

                           {
                             function_name => 'update_central_configuration_address',
                             criteria_index => 3,
                             store_index => 4,
                           },
                           {
                             function_name => 'delete_central_configuration_address',
                           },
                         );

our $entity_class = 'F5::ASMConfig::Entity::CentralConfigurationAddress';

#----------------------------------------------------------------------
sub add_central_configuration_address {
    my($self, $session, $call_flags, $policy, $central_configuration_address) = @_;

    my $rv = $self->base_add(entity_class => $entity_class,
                              rpc_params => [ $session, $call_flags,$policy,$central_configuration_address ],
                            );

    if( $central_configuration_address && $central_configuration_address->{is_connected} ){
         $self->write_to_syslog(msg =>"Adding Central Configuration Address $central_configuration_address->{ip_address} port $central_configuration_address->{port} as connected");
    }

    return $rv;
}

#----------------------------------------------------------------------
sub delete_central_configuration_address {
    my($self, $session, $call_flags, $policy, $central_configuration_address) = @_;

    my $deleted_central_configuration_address = F5::ASMConfig::Entity::CentralConfigurationAddress->new(get_criteria => $central_configuration_address, dbh => $self->{dbh});

    if( $deleted_central_configuration_address && $deleted_central_configuration_address->{is_connected} ){
         $self->write_to_syslog(msg =>"Deleting Central Configuration Address $deleted_central_configuration_address->{ip_address} port $deleted_central_configuration_address->{port} which was connected");
    }

    my $rv = $self->base_delete(entity_class => $entity_class,
                                rpc_params => [ $session, $call_flags,$policy,$central_configuration_address ],
                              );

    return $rv;
}

#----------------------------------------------------------------------

sub update_central_configuration_address {
    my($self, $session, $call_flags, $policy, $central_configuration_address_criteria, $store_central_configuration_address) = @_;

    my $updated_central_configuration_address = F5::ASMConfig::Entity::CentralConfigurationAddress->new(get_criteria => $central_configuration_address_criteria, dbh => $self->{dbh});

    if( $updated_central_configuration_address && $store_central_configuration_address ){
        if ( $updated_central_configuration_address->{is_connected} && $store_central_configuration_address->{is_connected} == 0 ) {
            $self->write_to_syslog(msg =>"Disconnecting Central Configuration Address $updated_central_configuration_address->{ip_address} port $updated_central_configuration_address->{port}");
        }
        elsif ( !$updated_central_configuration_address->{is_connected} && $store_central_configuration_address->{is_connected} ) {
            $self->write_to_syslog(msg =>"Connecting Central Configuration Address $updated_central_configuration_address->{ip_address} port $updated_central_configuration_address->{port}");
        }
    }

    my $rv = $self->base_update(rpc_params => [ $session, $call_flags, $policy, $central_configuration_address_criteria, $store_central_configuration_address ],
                                entity_class => $entity_class,
                               );


    return $rv;
}



#----------------------------------------------------------------------

1;
