package F5::ASMConfig::Handler::Certificate;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::ASMConfig;
use F5::Certificates;

our @handled_functions =  (
                            {
                             function_name => 'verify_certificate',
                             filename_arguments => [3],
                            },
                          );
#---------------------------------------------------------
sub verify_certificate{
  my($self, $session, $call_flags,$place_holder, $filename) = @_;

  my $dbh = $self->{dbh};
  my $logger = $self->{logger};
  my $pem_file = $filename;

  if(not defined $pem_file){
     return $self->handle_error(action => "verify_certificate",
                                error_code => F5::ASMConfig::ErrorCode::FILE_NOT_FOUND(),
                                error_thrown => "Verification Error: Certificate file is missing",
                                error_level  => 'error',
                        );
  }

   my $fh;
   if(!  open($fh, "<", $pem_file) ) {
       return $self->handle_error(action => "verify_certificate",
                                  error_code => F5::ASMConfig::ErrorCode::FILE_NOT_FOUND(),
                                  error_thrown => "Could not open '$pem_file' : $!",
                                  error_level  => 'error',
                                 );
    }

    my $pem_file_content = join('', <$fh>);
    close $fh;

   my $res = F5::Certificates::verify_certificate(pem_file_content => $pem_file_content, logger => $logger);
  

   if(!$res->{completed_successfully}){
      my $error_thrown = "Failed to verify the certificate";
      if($res->{error_message} ne "") {
         $error_thrown .= ", $res->{error_message}";	
      }
      return $self->handle_error(action => "verify_certificate",
                                 error_code => F5::ASMConfig::ErrorCode::INVALID_CERTIFICATE(),
                                 error_thrown => $error_thrown,
                                 error_level  => 'info',
                                );
   }

   my $verification_result = F5::ASMConfig::CertificateVerificationRes->new();
   $verification_result->{is_expired} = $res->{is_expired};
   $verification_result->{is_untrusted} = $res->{is_untrusted};
   $verification_result->{error_msg} = $res->{error_message};
   $verification_result->{completed_successfully} = $res->{completed_successfully};
   $verification_result->{expiration_date_before_convert} = $res->{expiration_date_before_convert};
   $verification_result->{validation_report} = $res->{validation_report};
   $verification_result->{local_expiration_date} = $res->{local_expiration_date};


   return { return_value => $verification_result};
}
#-------------------------------------------------------------------------------------
1;
