package F5::ASMConfig::Handler::Charset;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;

our @handled_functions =  (
                            {
                             function_name => 'get_default_charset_for_encoding',
                            },
                            {
                             function_name => 'update_charset',
                             relayable => 1,
                             criteria_index => 3,
                             store_index => 4,
                            },
                            {
                             function_name => 'update_many_charsets',
                             relayable => 1,
                             criteria_index => 3,
                             store_index => 4,
                            },
                          );

our $entity_class = 'F5::ASMConfig::Entity::Charset';
#----------------------------------------------------------------------

sub get_default_charset_for_encoding {
  my($self, $session, $call_flags, undef, $charset, $encoding) = @_;

  my $dbh = $self->{dbh};
  
  my $rec = { %$charset };
  my $ary_ref = $dbh->selectcol_arrayref(<<EndQuery, { Columns=>[1, 2] }, $encoding, $charset->{charset_type});
SELECT A.charset, B.encoding_type
  FROM $self->{dcc_dbname}.DEFAULT_CHARSETS A 
       JOIN $self->{dcc_dbname}.ENCODINGS B
    ON A.encoding_id = B.encoding_id
 WHERE B.encoding = ?
   AND A.charset_type = ?
EndQuery

  $rec->{charset} = $ary_ref->[0];
  $rec->{encoding_type} = $ary_ref->[1];

  F5::ASMConfig::Entity::Charset->map_fields_to_entity(rec => $rec, dbh => $dbh); 
  my $return_charset = F5::ASMConfig::Entity::Charset->new(dbh => $dbh,
                                                           object => $rec);

  my $retval = { return_value => $return_charset,
               };

  return $retval;
}

#----------------------------------------------------------------------

sub update_charset {
    my($self, $session, $call_flags, $policy, $charset_criteria, $store_charset) = @_;

    my $local_call_flags = F5::ASMConfig::CallFlags->new(%$call_flags, append_nested_struct_on_update => 1);
    return $self->base_update(entity_class => $entity_class,
                              rpc_params => [ $session, $local_call_flags, $policy, $charset_criteria, $store_charset ],);
}

#----------------------------------------------------------------------

sub update_many_charsets {
    my($self, $session, $call_flags, $policy, $charset_criteria, $store_charset) = @_;

    my $local_call_flags = F5::ASMConfig::CallFlags->new(%$call_flags, append_nested_struct_on_update => 1);
    return $self->base_update_many(entity_class => $entity_class,
                                   rpc_params => [ $session, $local_call_flags, $policy, $charset_criteria, $store_charset ],);
}

#----------------------------------------------------------------------

1;
