package F5::ASMConfig::Handler::CheckSignaturesTask;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::TaskBase/;
use F5::ASMConfig::Entity::CheckSignaturesTask;
use F5::Utils::Rest;

our @handled_functions = (
                            {
                             function_name => 'add_check_signatures_task',
                             criteria_index => 3,
                             store_index    => 4,
                            },
                            {
                              function_name => 'update_check_signatures_task',
                              criteria_index => 3,
                              store_index => 4
                            },
                            { 
                             function_name => 'execute_check_signatures_task',
                            },
                            {
                             function_name => 'delete_check_signatures_task',
                            },
                            {
                             function_name => 'delete_check_signatures_tasks',
                            },
                         );

our $entity_class = 'F5::ASMConfig::Entity::CheckSignaturesTask';

#----------------------------------------------------------------------

sub add_check_signatures_task {
  my($self, @rpc_params) = @_;
  return F5::ASMConfig::Handler::TaskBase::add_task(self => $self,
                                                    rpc_params => \@rpc_params,
                                                    entity_class => $entity_class,
                                                    task_type => 'check_signatures',);
}

#----------------------------------------------------------------------

sub update_check_signatures_task {
    my ($self, @rpc_params) = @_;
    return F5::ASMConfig::Handler::TaskBase::update_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                         task_type => 'check_signatures',);
}

#----------------------------------------------------------------------

my $LIVE_UPDATE_URL = "/mgmt/tm/live-update";

sub check_signatures {
    my (%params) = @_;
    my $self       = $params{self};
    my $session    = $params{session};
    my $call_flags = $params{call_flags};
    my $task       = $params{task};

    my $current = F5::Utils::Rest::local_request(uri => $LIVE_UPDATE_URL, logger => $self->{logger} );
    my ($asm_signature_status) = grep { $_->{name} eq 'ASM Attack Signatures' } (@{$current->{items}});
    $task->{update_available} = $asm_signature_status->{isUpdateAvailable} ? 1 : 0;
    $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_COMPLETED();
}


#----------------------------------------------------------------------

sub execute_check_signatures_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::execute_task(self => $self,
                                                          rpc_params => \@rpc_params,
                                                          entity_class => $entity_class,
                                                          task_type => 'check_signatures',
                                                          execute_func => sub { check_signatures(@_); },
                                                          );
}

#----------------------------------------------------------------------

sub delete_check_signatures_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                        );
}

#----------------------------------------------------------------------

sub delete_many_check_signatures_tasks {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_many_tasks(self => $self,
                                                               rpc_params => \@rpc_params,
                                                               entity_class => $entity_class,
                                                              );
}

#----------------------------------------------------------------------

1;
