package F5::ASMConfig::Handler::CheckThreatCampaignFileTask;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::TaskBase/;
use F5::ASMConfig::Entity::CheckThreatCampaignFileTask;
use F5::Utils::Rest;

our @handled_functions = (
                            {
                             function_name => 'add_check_threat_campaign_file_task',
                             criteria_index => 3,
                             store_index    => 4,
                            },
                            {
                              function_name => 'update_check_threat_campaign_file_task',
                              criteria_index => 3,
                              store_index => 4
                            },
                            { 
                             function_name => 'execute_check_threat_campaign_file_task',
                            },
                            {
                             function_name => 'delete_check_threat_campaign_file_task',
                            },
                            {
                             function_name => 'delete_check_threat_campaign_file_tasks',
                            },
                         );

our $entity_class = 'F5::ASMConfig::Entity::CheckThreatCampaignFileTask';

#----------------------------------------------------------------------

sub add_check_threat_campaign_file_task {
  my($self, @rpc_params) = @_;
  return F5::ASMConfig::Handler::TaskBase::add_task(self => $self,
                                                    rpc_params => \@rpc_params,
                                                    entity_class => $entity_class,
                                                    task_type => 'check_threat_campaign_file',);
}

#----------------------------------------------------------------------

sub update_check_threat_campaign_file_task {
    my ($self, @rpc_params) = @_;
    return F5::ASMConfig::Handler::TaskBase::update_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                         task_type => 'check_threat_campaign_file',);
}

#----------------------------------------------------------------------

my $LIVE_UPDATE_URL = "/mgmt/tm/live-update";

sub check_threat_campaign_file {
    my (%params) = @_;
    my $self       = $params{self};
    my $session    = $params{session};
    my $call_flags = $params{call_flags};
    my $task       = $params{task};


    my $current = F5::Utils::Rest::local_request(uri => $LIVE_UPDATE_URL, logger => $self->{logger});
    my ($threat_campaign_status) = grep { $_->{name} eq 'Threat Campaigns' } (@{$current->{items}});
    $task->{update_available} = $threat_campaign_status->{isUpdateAvailable} ? 1 : 0;
    $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_COMPLETED();
}


#----------------------------------------------------------------------

sub execute_check_threat_campaign_file_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::execute_task(self => $self,
                                                          rpc_params => \@rpc_params,
                                                          entity_class => $entity_class,
                                                          task_type => 'check_threat_campaign_file',
                                                          execute_func => sub { check_threat_campaign_file(@_); },
                                                          );
}

#----------------------------------------------------------------------

sub delete_check_threat_campaign_file_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                        );
}

#----------------------------------------------------------------------

sub delete_many_check_threat_campaign_file_tasks {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_many_tasks(self => $self,
                                                               rpc_params => \@rpc_params,
                                                               entity_class => $entity_class,
                                                              );
}

#----------------------------------------------------------------------

1;
