package F5::ASMConfig::Handler::ClusterState;

use strict;
use warnings;

use F5::Cluster::Utils;
use F5::Cluster::Sync;
use base qw/F5::ASMConfig::Handler::Base/;

our @handled_functions =  (
                            {
                             function_name => 'update_cluster_state',
                            },
                            {
                             function_name => 'send_cluster_sync',
                             relayable => 1,
                             update_mcp_timestamp => 0,
                            },
                          );

#----------------------------------------------------------------------

sub update_cluster_state {
  my($self, $session, $call_flags, undef, $cluster_state) = @_;

  my $dbh = $self->{dbh};

  my $retval = { return_value => $cluster_state,
                 events => [ $self->create_event(event_type => F5::ASMConfig::EventType::UPDATE,
                                                 session => $session,
                                                 element => F5::ASMConfig::Element->new({
                                                                                         element_type => F5::ASMConfig::ElementType::CLUSTER_STATE,
                                                                                         cluster_state => $cluster_state,
                                                                                        }),
                                                 call_flags => $call_flags,
                                                 user_name => $session->{user_name},
                                                 suppress_save_to_log => 1,
                                                )
                           ],
               };
  return $retval;
}

#----------------------------------------------------------------------

sub send_cluster_sync {
    my ($self, $session, $call_flags) = @_;

    if (F5::Cluster::Utils::is_clustering_enabled() &&
        F5::Cluster::Utils::is_master_blade()) {

        my $cluster_sync = F5::Cluster::Sync->new(dbh => $self->{dbh},
                                                  logger => F5::Logger->get_logger(logger_name => 'clustering'));

        if ( !$cluster_sync) {
            die "Can't sync - ASM is not ready";
        }

        $cluster_sync->send_sync();
    }
    return { return_value => 1 };
}

#----------------------------------------------------------------------

1;
