package F5::ASMConfig::Handler::Cookie;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::ASMConfig;

our @handled_functions = (
                           {
                             function_name => 'delete_many_cookies',
                             relayable => 1,
                           },
                           {
                             function_name => 'delete_cookie',
                             relayable => 1,
                           },
                           {
                             function_name => 'update_cookie',
                             relayable => 1,
                             criteria_index => 3, 
                             store_index => 4
                           },
                         );

our $entity_class = 'F5::ASMConfig::Entity::Cookie';

#----------------------------------------------------------------------

sub delete_many_cookies {
    my($self, $session, $call_flags, $policy, $cookie_criteria) = @_;

    if (!defined $call_flags){
        $call_flags = F5::ASMConfig::CallFlags->new();
    }

    if (!defined $call_flags->{search_filters}){
       $call_flags->{search_filters} = [];
    }

    push(@{$call_flags->{search_filters}},
                 { field_name => 'cookie_name',
                   field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_NE(),
                   field_value => '*' });

    my $cookie_iter = $entity_class->retrieve_all(dbh => $self->{dbh},
                                                  call_flags => $call_flags,
                                                  policy => $policy,
                                                  search_filters => $call_flags->{search_filters},
                                                  get_criteria => $cookie_criteria);
 
    while (my $cookie = $cookie_iter->get_next()) {
        $self->update_many_scanner_vulns($session,
                                         F5::ASMConfig::CallFlags->new(set_fields_to_null => [qw/asm_cookie/]),
                                         $policy,
                                         F5::ASMConfig::ScannerVuln->new({asm_cookie => $cookie}),
                                         F5::ASMConfig::ScannerVuln->new({asm_vuln_status => F5::ASMConfig::ScannerAsmVulnStatus::SCANNER_ASM_VULN_STATUS_PENDING()}));
    }

    

    return $self->base_delete_many(entity_class => $entity_class,
                                   rpc_params => [ $session, $call_flags,$policy, $cookie_criteria],);

}

#----------------------------------------------------------------------

sub delete_cookie {
    my($self, $session, $call_flags, $policy,$cookie) = @_;

    my $dbh = $self->{dbh};
    my $logger = $self->{logger};
  
  
   $self->update_many_scanner_vulns($session,
                                    F5::ASMConfig::CallFlags->new(set_fields_to_null => [qw/asm_cookie/]),
                                    $policy,
                                    F5::ASMConfig::ScannerVuln->new({asm_cookie => $cookie}),
                                    F5::ASMConfig::ScannerVuln->new({asm_vuln_status => F5::ASMConfig::ScannerAsmVulnStatus::SCANNER_ASM_VULN_STATUS_PENDING()}));

    my $rv = $self->base_delete(entity_class => $entity_class,
                                rpc_params => [ $session, $call_flags,$policy,$cookie],
                              );

    return $rv;
}

#----------------------------------------------------------------------

sub update_cookie {
    my($self, $session, $call_flags, $policy, $cookie_criteria, $store_cookie) = @_;

    my $rv = $self->base_update(rpc_params => [ $session, $call_flags, $policy, $cookie_criteria, $store_cookie ],
                                entity_class => $entity_class,
                               );
    my $cookie = $rv->{return_value};

    my $status;
    if ($cookie->{in_staging}) {
        $status = F5::ASMConfig::ScannerAsmVulnStatus::SCANNER_ASM_VULN_STATUS_HANDLED_STAGING();
    }
    else {
        $status = F5::ASMConfig::ScannerAsmVulnStatus::SCANNER_ASM_VULN_STATUS_HANDLED();
    }

    $self->update_many_scanner_vulns($session,
                                     undef,
                                     $policy,
                                     F5::ASMConfig::ScannerVuln->new({asm_cookie => $cookie}),
                                     F5::ASMConfig::ScannerVuln->new({asm_cookie => undef, asm_vuln_status => $status}));

    return $rv;
}

1;
