package F5::ASMConfig::Handler::DeviceGroupMember;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use File::Basename;
use F5::DeviceGroup;

our @handled_functions = (
                          {
                              function_name => "update_device_group_member",
                              relayable => 0,
                              write_access => 1,
                          },
                         );

our $entity_class = 'F5::ASMConfig::Entity::DeviceGroupMember';

#----------------------------------------------------------------------
sub update_device_group_member {
    my ($self, $session, $call_flags, $place_holder, $device_group_member) = @_;
    if ($self->{is_relay_handler_process}) {
        #Persistent store from relay_listener
        my $rv =  $self->base_update(entity_class => $entity_class,
                                     rpc_params => [ $session, $call_flags,$place_holder, $device_group_member ],);
        return $rv;
    }

    my $folder_name = F5::DeviceGroup::get_asm_sync_folder_name($self->{device_group});
    if (!$self->{device_group} ||
        ($device_group_member->{host} !~ /^$folder_name\//)) {
        $self->{logger}->debug("MCP update received for '$device_group_member->{host}:$device_group_member->{known_time}', but it is not in our device group");
        return { return_value => $device_group_member };
    }
    
    $device_group_member->{host} = "/Common/". basename($device_group_member->{host});  #remove asm_sync directory path to get device name
   
    if ( !$device_group_member->{known_time} ||
        $device_group_member->{host} eq $self->{local_device_name} ||
        (defined($self->{known_times}->{$device_group_member->{host}}) && $self->{known_times}->{$device_group_member->{host}} ge $device_group_member->{known_time})) {
        $self->{logger}->debug("MCP update received for '$device_group_member->{host}:$device_group_member->{known_time}', already seen locally");
        return { return_value => $device_group_member };
    }

    if ($self->{shared_state}{sync_init} && $self->{shared_state}{sync_init} < $device_group_member->{known_time}) {
        $self->{logger}->info("MCP update received for '$device_group_member->{host}:$device_group_member->{known_time}' when in sync init state.");
        $self->{shared_state}{sync_init} = 0;
    }

    $call_flags->{originator} = $device_group_member->{host};
    $call_flags->{originator_time} = $device_group_member->{known_time};
    $self->send_to_relay(function_name       => "mcp_asm_update",
                         device_group_member => $device_group_member,
                         rpc_params          => [ $session, $call_flags ],
                         group_id            => $F5::Cgc::Channel::DeviceGroup::ASM_CGC_LOCAL_GROUP,
                         service             => $F5::Cgc::Channel::DeviceGroup::ASM_CGC_MCP_EVENTS,
                         mark_time           => 0);

    return { return_value => $device_group_member };
}
#----------------------------------------------------------------------


1;
