package F5::ASMConfig::Handler::DisabledActionItem;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::Entity::DisabledActionItem;

our @handled_functions = (
                            {
                             function_name => 'add_disabled_action_item',
                             relayable => 1,
                            },
                         );

our $entity_class = 'F5::ASMConfig::Entity::DisabledActionItem';

#----------------------------------------------------------------------

sub add_disabled_action_item {
    my ($self, $session, $call_flags, $policy, $disabled_action_item) = @_;
    my $rv;
    my @policies;
    if (!$policy) {
       @policies = F5::ASMConfig::Entity::Policy->retrieve_all(dbh => $self->{dbh},
                                                               get_criteria => { is_active_policy => 1},
                                                              ); 
       push @{$call_flags->{ignored_errors}}, F5::ASMConfig::IgnoredError->new({error_code => F5::ASMConfig::ErrorCode::DUPLICATE_RECORD()});
    }
    else {
        push @policies, $policy;
    }
    for my $p (@policies) {
        my $p_rv = $self->base_add(rpc_params => [$session, $call_flags, $p, $disabled_action_item],
                          entity_class => $entity_class,
                          );
        $rv->{return_value} = $p_rv->{return_value};
        if ($p_rv->{events}) {
            push(@{$rv->{events}}, @{$p_rv->{events}} );
        }

        my $policy_action_item = F5::ASMConfig::Entity::PolicyActionItem->new(dbh => $self->{dbh}, 
                                                                              policy => $p,
                                                                              get_criteria => {action_item_type => $disabled_action_item->{action_item_type}},
                                                                              );
        if ($policy_action_item) {
            # This delete actually only dismisses the item, but it creates an event, which we want.
            # We execute the actual delete afterwards
            $self->delete_policy_action_item($session, $call_flags, $p, 
                                             F5::ASMConfig::PolicyActionItem->new(action_item_type => $policy_action_item->{action_item_type}));
            $policy_action_item->delete();
        }
    }
    return $rv;
}
  
#----------------------------------------------------------------------

1;
