package F5::ASMConfig::Handler::EmAsmIpAddressList;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use XML::Twig;
use File::Basename;

use F5::ASMConfig::Entity::EmAsmIpAddressList;
use F5::ASMConfig::Client;
use F5::DeviceGroup;

our @handled_functions = (
                           {
                             function_name => 'fetch_em_asm_ip_address_list',
                           },
                           {
                             function_name => 'deploy_em_asm_ip_address_list',
                           },
                         );

our $entity_class = 'F5::ASMConfig::Entity::EmAsmIpAddressList';

#----------------------------------------------------------------------

sub fetch_em_asm_ip_address_list {
    my($self, $session, $call_flags, $place_holder, $device, $policy) = @_;


    my $client_to_remote = F5::ASMConfig::Client->new( username => 'my_test', host => $device->{mng_ip_address}, is_ssl => 1);
    eval {
        $policy = $client_to_remote->search_policy($session, $call_flags,$policy)->{element_list}->[0]->{policy};
    };
    if ($@) {
        return $self->handle_error(action => 'search',
                                   error_thrown => $F5::ASMConfig::Handler::NONEXISTENT_RECORD_ERROR,
                                   call_flags => $call_flags,
                                   entity_class => 'F5::ASMConfig::Entity::Policy',
                                   thrift_obj => $policy,
                                  );
    }



    my $whitelist_ips = $client_to_remote->search_whitelist_ip(
                                                               $session, 
                                                               $call_flags, 
                                                               $policy, 
                                                               F5::ASMConfig::WhitelistIp->new());

    
    $call_flags->{add_on_updating_nonexistent_record} = 1;
    my @ip_addresses;

    for my $element (@{$whitelist_ips->{element_list}}){
        my $actual_ip_address = $element->{whitelist_ip};
        my $em_asm_ip_address = F5::ASMConfig::EmAsmIpAddress->new();
        $em_asm_ip_address->{ip_address} = $actual_ip_address->{ip_address};
        $em_asm_ip_address->{mask} = $actual_ip_address->{mask};
        $em_asm_ip_address->{flg_pb_trusted} = $actual_ip_address->{flg_pb_trusted};
        $em_asm_ip_address->{flg_ignore_anomaly} = $actual_ip_address->{flg_ignore_anomaly};
        $em_asm_ip_address->{flg_never_log} = $actual_ip_address->{flg_never_log};
        $em_asm_ip_address->{flg_never_block} = $actual_ip_address->{flg_never_block};
        $em_asm_ip_address->{flg_never_learn} = $actual_ip_address->{flg_never_learn};
        $em_asm_ip_address->{description} = $actual_ip_address->{description};
        $em_asm_ip_address->{flg_ignore_ip_reputation} = $actual_ip_address->{flg_ignore_ip_reputation};

        push (@ip_addresses, $em_asm_ip_address);
    }

    my $ip_address_list = F5::ASMConfig::EmAsmIpAddressList->new({
                                                                  ip_address_list_name => $device->{mng_ip_address} . "-" . $policy->{policy_name},
                                                                  ip_addresses => \@ip_addresses,
                                                                });
  

    my $rv = $self->base_update(
                                rpc_params => [$session, $call_flags, undef, $ip_address_list],
                                entity_class => $entity_class
                               )->{return_value};

    my $retval = { return_value => $rv, };
    return $retval;
}

#----------------------------------------------------------------------

sub deploy_em_asm_ip_address_list {
    my($self, $session, $call_flags, $place_holder, $deploy_properties, $ip_address_list) = @_;

    $call_flags->{originator} = basename(F5::DeviceGroup::get_local_device_name());

    my %target_devices;
    for my $deploy_target (@{$deploy_properties->{deploy_targets}}) {
        my $device = $self->get_device($session, $call_flags, $deploy_target->{device});
        if (! exists $target_devices{$device->{mng_ip_address}}) {
            $target_devices{$device->{mng_ip_address}} = $device;
            $device->{http_classes} = {};
        }
        $device = $target_devices{$device->{mng_ip_address}};
        next unless defined $deploy_target->{http_class};

        my $http_class = $deploy_target->{http_class};

        my $account_name = '/' . $http_class->{folder} . '/' . $http_class->{http_class_name};
        $device->{http_classes}->{$account_name} = 1;
    }

    $ip_address_list = F5::ASMConfig::Entity::EmAsmIpAddressList->new(dbh => $self->{dbh},
                                                                      get_criteria => $ip_address_list);

    my $multi_client = F5::ASMConfig::Client->new( 
        devices => [ values %target_devices ],
        is_async => 1,
        is_ssl => 1,
        logger => $self->{logger},
        deploy_request_fields => {
                                  deploy_type => F5::ASMConfig::ElementType::EM_ASM_IP_ADDRESS_LIST(),
                                  em_asm_ip_address_list => $ip_address_list,
                                 },
        deploy_targets => $deploy_properties->{deploy_targets},
        async_calls => [
                        {
                         call_sub => sub {
                                          my($client, $retvals) = @_;

                                          if (%{$client->{device}->{http_classes}}) {
                                              $client->search_account(
                                                  $client->{session},
                                                  $call_flags,
                                                 # F5::ASMConfig::Account->new
                                             );
                                          }
                                      },
                         call_name => 'Account search',
                         return_value => 'target_accounts',
                        },
                        {
                         call_sub => sub {
                                          my($client, $retvals) = @_;
                                          if (!$retvals->{target_accounts}->{total_record_count}) {
                                              die "target account not found";
                                          }

                                          my %accounts;
                                          if ($retvals->{target_accounts}) {
                                              for my $account (map { $_->{account} } @{$retvals->{target_accounts}->{element_list}}) {
                                                  if (exists $client->{device}->{http_classes}->{$account->{account_name}}) {
                                                      $accounts{$account->{account_name}} = $account;
                                                  }
                                              }
                                          }
                                          $retvals->{accounts} = \%accounts;

                                          my $search_call_flags = F5::ASMConfig::CallFlags->new(%$call_flags);
                                          $search_call_flags->{search_filters} = [
                                                                                  F5::ASMConfig::SearchFilter->new(
                                                                                                                   field_name => 'policy_name',
                                                                                                                   field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_IN(),
                                                                                                                   field_value_list => [map { $_->{active_policy_name} } values %accounts ],
                                                                                                                  )
                                                                                 ];

                                          $client->search_policy(
                                                                 $client->{session},
                                                                 $search_call_flags,
                                                                 F5::ASMConfig::Policy->new());
                                      },
                         call_name => 'Policy search',
                         return_value => 'target_policies',
                        },
                        {
                         call_sub => sub {
                                          my($client, $retvals) = @_;
                                          if (!$retvals->{target_policies}->{total_record_count}) {
                                              die "target policy not found";
                                          }

                                          for my $target_policy (map {$_->{policy}} @{$retvals->{target_policies}->{element_list}}) {
                                              $client->delete_many_whitelist_ips($client->{session}, 
                                                                                 $call_flags, 
                                                                                 $target_policy,
                                                                                 F5::ASMConfig::WhitelistIp->new());
                                          }
                                         },
                         call_name => 'Delete Whitelist IP addresses from policy',
                        },
                        {
                         call_sub => sub {
                             my($client, $retvals) = @_;
                             if (!$retvals->{target_policies}->{total_record_count}) {
                                 die "target policy not found";
                             }

                             for my $target_policy (map {$_->{policy}} @{$retvals->{target_policies}->{element_list}}) {
                                 for my $ip_address (@{$ip_address_list->{ip_addresses}}) {
                                     my $whitelist_ip = F5::ASMConfig::WhitelistIp->new();
                                     $whitelist_ip->{ip_address} = $ip_address->{ip_address};
                                     $whitelist_ip->{mask} = $ip_address->{mask};
                                     $whitelist_ip->{flg_pb_trusted} = $ip_address->{flg_pb_trusted};
                                     $whitelist_ip->{flg_ignore_anomaly} = $ip_address->{flg_ignore_anomaly};
                                     $whitelist_ip->{flg_never_log} = $ip_address->{flg_never_log};
                                     $whitelist_ip->{flg_never_block} = $ip_address->{flg_never_block};
                                     $whitelist_ip->{flg_never_learn} = $ip_address->{flg_never_learn};
                                     $whitelist_ip->{description} = $ip_address->{description};
                                     $whitelist_ip->{flg_ignore_ip_reputation} = $ip_address->{flg_ignore_ip_reputation};
                                     $whitelist_ip->{flg_is_em_originated} = 1;
                                     #TODO: handle in F5::Thrift
                                     delete $whitelist_ip->{__short_class};
                                     $client->add_whitelist_ip(
                                                               $client->{session}, 
                                                               $call_flags, 
                                                               $target_policy,
                                                               $whitelist_ip);
                                 }

                                 $self->write_to_syslog(msg => "IP Address Exceptions List for Security Policy \"$target_policy->{policy_name}\" was deployed to device $client->{device}->{device_name}.");

                             }
                         },
                         call_name => 'Adding Whitelist IP addresses to policy',
                        },
                        {
                         call_sub => sub {
                                          my($client, $retvals) = @_;
                                          if (!$retvals->{target_policies}->{total_record_count}) {
                                              die "target policy not found";
                                          }

                                          for my $target_policy (map {$_->{policy}} @{$retvals->{target_policies}->{element_list}}) {
                                              $client->set_active($client->{session}, $call_flags, $target_policy, F5::ASMConfig::SetActiveFlags->new());
                                          }
                                         },
                         call_name => 'Set Active Policy',
                        },

                    ],
    );

    return {return_value => $multi_client->run_async_calls(dbh => $self->{dbh})};
}

1;
