package F5::ASMConfig::Handler::EventSubscription;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;

use F5::ASMConfig::Cache;

our @handled_functions =  ( 
                            {
                             function_name => 'add_event_subscription',
                            },
                            {
                             function_name => 'delete_event_subscription',
                            },
                            {
                             function_name => 'delete_event_subscription_by_policy',
                            },
                            {
                             function_name => 'delete_all_event_subscriptions',
                            },
                          );

#----------------------------------------------------------------------

sub add_event_subscription {
    my($self, $session, $event_subscription) = @_;
    $event_subscription->{event_subscription_id} = ++$self->{session_last_event_subscription_id}->{$session->session_id};

    if ($event_subscription->{policy} && $event_subscription->{policy_name} && ! $event_subscription->{policy}->{policy_id}) {
        my $policy = F5::ASMConfig::Cache::get_cached_policy(dbh => $self->{dbh},
                                                             policy => $event_subscription->{policy});

        $event_subscription->{policy}->{policy_id} = $policy->{policy_id};
    }
    return {
            return_value => $event_subscription,
            events => [
                       $self->create_event(event_type => F5::ASMConfig::EventType::ADD,
                                           session => $session,
                                           element => F5::ASMConfig::Element->new({
                                                                                   element_type => F5::ASMConfig::ElementType::EVENT_SUBSCRIPTION,
                                                                                   event_subscription => $event_subscription,
                                                                                  }),
                                           suppress_save_to_log => 1,
                                          ),
                      ],
           };
}

#----------------------------------------------------------------------

sub delete_event_subscription {
    my($self, $session, $event_subscription) = @_;
    $self->{logger}->debug("Deleting event subscription (event_subscription_id = $event_subscription->{event_subscription_id})") if $self->{logger};

    return {
            return_value => 1,
            events => [
                       $self->create_event(event_type => F5::ASMConfig::EventType::DELETE,
                                           session => $session,
                                           element => F5::ASMConfig::Element->new({
                                                                                   element_type => F5::ASMConfig::ElementType::EVENT_SUBSCRIPTION,
                                                                                   event_subscription => $event_subscription,
                                                                                  }),
                                           suppress_save_to_log => 1,
                                          ),
                      ],
           };
}

#----------------------------------------------------------------------

sub delete_event_subscription_by_policy {
    my($self, $session, $policy) = @_;

    $policy = F5::ASMConfig::Cache::get_cached_policy(dbh => $self->{dbh},
                                                      policy => $policy);

    if (! $policy || ! $policy->{policy_id}) {
        $self->{logger}->debug("Couldn't delete event subscription for unknown policy") if $self->{logger};
        return {
                return_value => 0,
               };
    }

    $self->{logger}->debug("Deleting event subscriptions for policy (policy_id = $policy->{policy_id})") if $self->{logger};

    return {
            return_value => 1,
            events => [
                       $self->create_event(event_type => F5::ASMConfig::EventType::DELETE,
                                           session => $session,
                                           policy => $policy,
                                           element => F5::ASMConfig::Element->new({
                                                                                   element_type => F5::ASMConfig::ElementType::EVENT_SUBSCRIPTION,
                                                                                  }),
                                           suppress_save_to_log => 1,
                                          ),
                      ],
           };
}

#----------------------------------------------------------------------

sub delete_all_event_subscriptions {
    my($self, $session) = @_;
    $self->{logger}->debug("Deleting all event subscriptions for session $session->{session_id}") if $self->{logger};

    return {
            return_value => 1,
            events => [
                       $self->create_event(event_type => F5::ASMConfig::EventType::DELETE,
                                           session => $session,
                                           element => F5::ASMConfig::Element->new({
                                                                                   element_type => F5::ASMConfig::ElementType::EVENT_SUBSCRIPTION,
                                                                                  }),
                                           suppress_save_to_log => 1,
                                          ),
                      ],
           };
}

#----------------------------------------------------------------------

1;
