package F5::ASMConfig::Handler::ExportDataProtectionTask;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::TaskBase/;
use F5::ASMConfig::Entity::ExportDataProtectionTask;
use F5::Utils::Rest;
use File::Copy;
use MIME::Base64;

our @handled_functions = (
                            {
                             function_name => 'add_export_data_protection_task',
                             criteria_index => 3,
                             store_index    => 4,
                            },
                            {
                              function_name => 'update_export_data_protection_task',
                              criteria_index => 3,
                              store_index => 4
                            },
                            {
                             function_name => 'execute_export_data_protection_task',
                            },
                            {
                             function_name => 'delete_export_data_protection_task',
                            },
                            {
                             function_name => 'delete_many_export_data_protection_tasks',
                            },
                         );

our $entity_class = 'F5::ASMConfig::Entity::ExportDataProtectionTask';

#----------------------------------------------------------------------

sub add_export_data_protection_task {
  my($self, @rpc_params) = @_;
  return F5::ASMConfig::Handler::TaskBase::add_task(self => $self,
                                                    rpc_params => \@rpc_params,
                                                    entity_class => $entity_class,
                                                    task_type => 'export_data_protection',);
}

#----------------------------------------------------------------------

sub update_export_data_protection_task {
    my ($self, @rpc_params) = @_;
    return F5::ASMConfig::Handler::TaskBase::update_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                         task_type => 'export_data_protection',);
}

#----------------------------------------------------------------------

sub export_data_protection {
    my (%params) = @_;
    my $self       = $params{self};
    my $session    = $params{session};
    my $call_flags = $params{call_flags};
    my $task       = $params{task};


    my $filename = F5::Utils::Rest::get_full_file_path(username => $task->{username}, filename => ($task->{filename} || $task->{rest_uuid}) );

    my $data_protection_import_export = F5::ASMConfig::DataProtectionImportExport->new(
                                                                                          filename => $filename,
                                                                                      );

    my $rv = $self->export_data_protection($session, $call_flags, $data_protection_import_export);

    if ($task->{result_inline}) {
        open (my $FH, '<', $filename);
        {
            local $/;
            $task->{exported_data_protection} = <$FH>;
        }
        close ($FH);
    }
    $task->{file_size} = $rv->{file_size};
    if (!$task->{filename}) {
        unlink($filename);
    }
    $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_COMPLETED();
}

#----------------------------------------------------------------------

sub execute_export_data_protection_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::execute_task(self => $self,
                                                          rpc_params => \@rpc_params,
                                                          entity_class => $entity_class,
                                                          task_type => 'export_data_protection',
                                                          execute_func => sub { export_data_protection(@_); },
                                                          );
}

#----------------------------------------------------------------------

sub delete_export_data_protection_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                        );
}

#----------------------------------------------------------------------

sub delete_many_export_data_protection_tasks {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_many_tasks(self => $self,
                                                               rpc_params => \@rpc_params,
                                                               entity_class => $entity_class,
                                                              );
}

#----------------------------------------------------------------------

1;
