package F5::ASMConfig::Handler::ExportPolicyTemplateTask;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::TaskBase/;
use F5::ASMConfig::Entity::ExportPolicyTemplateTask;
use F5::Utils::Rest;
use File::Copy;
use MIME::Base64;

our @handled_functions = (
                            {
                             function_name => 'add_export_policy_template_task',
                             criteria_index => 3,
                             store_index    => 4,
                            },
                            {
                              function_name => 'update_export_policy_template_task',
                              criteria_index => 3,
                              store_index => 4
                            },
                            { 
                             function_name => 'execute_export_policy_template_task',
                            },
                            {
                             function_name => 'delete_export_policy_template_task',
                            },
                            {
                             function_name => 'delete_many_export_policy_template_tasks',
                            },
                         );

our $entity_class = 'F5::ASMConfig::Entity::ExportPolicyTemplateTask';

#----------------------------------------------------------------------

sub add_export_policy_template_task {
  my($self, @rpc_params) = @_;
  return F5::ASMConfig::Handler::TaskBase::add_task(self => $self,
                                                    rpc_params => \@rpc_params,
                                                    entity_class => $entity_class,
                                                    task_type => 'export_policy_template',);
}

#----------------------------------------------------------------------

sub update_export_policy_template_task {
    my ($self, @rpc_params) = @_;   
    return F5::ASMConfig::Handler::TaskBase::update_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                         task_type => 'export_policy_template',);
}

#----------------------------------------------------------------------

sub export_policy_template {
    my (%params) = @_;
    my $self       = $params{self};
    my $session    = $params{session};
    my $call_flags = $params{call_flags};
    my $task       = $params{task};

    my $policy_template = $task->{target_policy_template};
    if (!$policy_template) {
        return $self->handle_error(action => 'execute',
                                   error_thrown => 'Policy Template not found',
                                   error_code => F5::ASMConfig::ErrorCode::MISSING_POLICY(),
                                   call_flags => $call_flags,
                                   entity_class => $entity_class,
                                   thrift_obj => $task,
                                  );
    }

    my $filename = F5::Utils::Rest::get_full_file_path(username => $task->{username}, filename => ($task->{filename} || $task->{rest_uuid}) );

    copy($policy_template->{template_file}, $filename);

    $task->{file_size} = -s $filename;
    if ($task->{result_inline}) {
        open (my $FH, '<', $filename);
        { 
            local $/;
            $task->{exported_policy_template} = <$FH>;
        }
        close ($FH);
        if ($task->{is_base64}) {
           $task->{exported_policy_template} = MIME::Base64::encode_base64($task->{exported_policy_template});
           $task->{file_size} = length($task->{exported_policy_template});
        }
    }
    if (!$task->{filename}) {
        unlink($filename);
    }
    $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_COMPLETED();
}


#----------------------------------------------------------------------

sub execute_export_policy_template_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::execute_task(self => $self,
                                                          rpc_params => \@rpc_params,
                                                          entity_class => $entity_class,
                                                          task_type => 'export_policy_template',
                                                          execute_func => sub { export_policy_template(@_); },
                                                          );
}

#----------------------------------------------------------------------

sub delete_export_policy_template_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                        );
}

#----------------------------------------------------------------------

sub delete_many_export_policy_template_tasks {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_many_tasks(self => $self,
                                                               rpc_params => \@rpc_params,
                                                               entity_class => $entity_class,
                                                              );
}

#----------------------------------------------------------------------

1;
