package F5::ASMConfig::Handler::ExportSignaturesTask;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::TaskBase/;
use F5::ASMConfig::Entity::ExportSignaturesTask;
use F5::Utils::Rest;
use MIME::Base64;

our @handled_functions = (
                            {
                             function_name => 'add_export_signatures_task',
                             criteria_index => 3,
                             store_index    => 4,
                            },
                            {
                              function_name => 'update_export_signatures_task',
                              criteria_index => 3,
                              store_index => 4
                            },
                            { 
                             function_name => 'execute_export_signatures_task',
                            },
                            {
                             function_name => 'delete_export_signatures_task',
                            },
                            {
                             function_name => 'delete_many_export_signatures_tasks',
                            },
                         );

our $entity_class = 'F5::ASMConfig::Entity::ExportSignaturesTask';

#----------------------------------------------------------------------

sub add_export_signatures_task {
  my($self, @rpc_params) = @_;
  return F5::ASMConfig::Handler::TaskBase::add_task(self => $self,
                                                    rpc_params => \@rpc_params,
                                                    entity_class => $entity_class,
                                                    task_type => 'export_signatures',);
}

#----------------------------------------------------------------------

sub update_export_signatures_task {
    my ($self, @rpc_params) = @_;   
    return F5::ASMConfig::Handler::TaskBase::update_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                         task_type => 'export_signatures',);
}

#----------------------------------------------------------------------

sub export_signatures {
    my (%params) = @_;
    my $self       = $params{self};
    my $session    = $params{session};
    my $call_flags = $params{call_flags};
    my $task       = $params{task};

    my $filename = F5::Utils::Rest::get_full_file_path(username => $task->{username}, filename => ($task->{filename} || $task->{rest_uuid}) );

    my $local_call_flags = F5::ASMConfig::CallFlags->new();
    if ( $task->{sig_filter} ) {
        my $filters = F5::ASMConfig::Entity::Signature->convert_rest_filter_string(%params,
                                                                                   dbh => $self->{dbh},
                                                                                   filter_string => $task->{sig_filter});
        push (@{$local_call_flags->{search_filters}}, @$filters);
    }

    my $file_format = $task->{file_format} || 'xml';
    my $rv = $self->generate_sigfile($session, $local_call_flags, undef, F5::ASMConfig::Sigfile->new(flg_is_user_defined => 1,
                                                                                                     filename => $filename,
                                                                                                     file_format => $file_format,
                                                                                                     tag => $task->{sig_tag}, ));

    $task->{file_size} = -s $filename;

    if ($task->{result_inline} || $task->{is_base64}) {
        my $xml_signatures;
        open (my $FH, '<', $filename);
        while (my $line = <$FH>) {
            $xml_signatures .= $line;
        }
        close ($FH);
        if ($task->{is_base64}) {
           $xml_signatures = MIME::Base64::encode_base64($xml_signatures);
           $task->{file_size} = length($xml_signatures);
        }
        if ($task->{result_inline}) {
            $task->{exported_signatures} = $xml_signatures;
        }
    }

    if (!$task->{filename}) {
       unlink($filename);
    }
    $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_COMPLETED();
}

#----------------------------------------------------------------------

sub execute_export_signatures_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::execute_task(self => $self,
                                                          rpc_params => \@rpc_params,
                                                          entity_class => $entity_class,
                                                          task_type => 'export_signatures',
                                                          execute_func => sub { export_signatures(@_); },
                                                          );
}

#----------------------------------------------------------------------

sub delete_export_signatures_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                        );
}

#----------------------------------------------------------------------

sub delete_many_export_signatures_tasks {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_many_tasks(self => $self,
                                                               rpc_params => \@rpc_params,
                                                               entity_class => $entity_class,
                                                              );
}

#----------------------------------------------------------------------

1;
