package F5::ASMConfig::Handler::Filetype;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::Entity::Filetype;

our @handled_functions = (
                            {
                             function_name => 'filetype_increase_length',
                             relayable => 1,
                             criteria_index => 3,
                             store_index    => 4,
                            },
                           {
                             function_name => 'delete_filetype',
                             relayable => 1,
                           },
                           {
                             function_name => 'delete_many_filetypes',
                             relayable => 1,
                           },
                         );

our $entity_class = 'F5::ASMConfig::Entity::Filetype';

#----------------------------------------------------------------------

sub filetype_increase_length {
  my($self, @rpc_params) = @_;
  return $self->base_update(rpc_params => \@rpc_params,
                            entity_class => $entity_class,
                            update_func => sub {

                              my(%params) = @_;
                              my $entity = $params{entity};
                              my $store_obj = $params{store_obj};

                              my @length_fields = qw(url_len
                                                     req_len
                                                     qs_len
                                                     pd_len);
                              my $record_modified = 0;

                              for my $length_field (@length_fields) {
                                  if ($store_obj->{$length_field} &&
                                      $entity->{$length_field} >= $store_obj->{$length_field}) {
                                      # Discard unnecessary update
                                      delete $store_obj->{$length_field};
				  }
                              }
                            });
}

#----------------------------------------------------------------------

sub delete_filetype {
    my($self, $session, $call_flags, $policy, $filetype) = @_;

    my $rv = $self->base_delete(entity_class => $entity_class,
                                rpc_params => [ $session, $call_flags,$policy,$filetype ],
                              );

    my $local_call_flags = F5::ASMConfig::CallFlags->new($call_flags);
    $local_call_flags->{search_filters} = F5::ASMConfig::Entity::Extraction->empty_extraction_filters;

    $self->delete_many_extractions( $self->{session},
                                    $local_call_flags,
                                    $policy, );

    return $rv;
}

#----------------------------------------------------------------------

sub delete_many_filetypes {
    my($self, $session, $call_flags, $policy, $filetype_criteria) = @_;

    my $rv = $self->base_delete_many(entity_class => $entity_class,
                                     rpc_params => [ $session, $call_flags,$policy, $filetype_criteria ],);

    my $local_call_flags = F5::ASMConfig::CallFlags->new($call_flags);
    $local_call_flags->{search_filters} = F5::ASMConfig::Entity::Extraction->empty_extraction_filters;

    $self->delete_many_extractions( $self->{session},
                                    $local_call_flags,
                                    $policy, );

    return $rv;
}

#----------------------------------------------------------------------

1;
