package F5::ASMConfig::Handler::GlobalScannerConfig;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::Entity::GlobalScannerConfig;
use F5::ASMConfig::ASMConfig;
use F5::Scanner;

use XML::Twig;
use LWP::UserAgent;

our @handled_functions = (
                           {
                             function_name => 'update_global_scanner_config',
                             criteria_index => 3,
                             store_index    => 4,
                             relayable => 1,
                           },
                           {
                             function_name => 'add_global_scanner_config',
                             relayable => 1,
                             store_index => 3,
                           },
                         );

our $entity_class = 'F5::ASMConfig::Entity::GlobalScannerConfig';
my @site_list;

#----------------------------------------------------------------------

sub update_global_scanner_config {
    my ($self, $session, $call_flags, $place_holder, $global_scanner_config_criteria, $store_global_scanner_config) = @_;

    my @relevant_scanner_config_elements = @{$self->search_scanner_config($session, 
                                                                          $call_flags, 
                                                                          F5::ASMConfig::Policy->new(),
                                                                          F5::ASMConfig::ScannerConfig->new(scanner_type => $global_scanner_config_criteria->{scanner_type})
                                                                         )->{element_list}};

    my @events;
    for my $scanner_config (@relevant_scanner_config_elements) {
        push( @events, $self->create_event(event_type => F5::ASMConfig::EventType::UPDATE,
                                           session    => $session,
                                           element => $scanner_config,
                                           call_flags => $call_flags,
                                           user_name  => $session->{user_name},
                                          ));
    }

    my $rv = $self->base_update(rpc_params => [ $session, $call_flags, $place_holder, $global_scanner_config_criteria, $store_global_scanner_config ],
                                entity_class => $entity_class,);

    push (@{$rv->{events}},@events);
    return $rv;
}


#----------------------------------------------------------------------

sub add_global_scanner_config {
    my ($self, $session, $call_flags, $place_holder, $global_scanner_config) = @_;

    $call_flags->{add_on_updating_nonexistent_record} = 1;
    my $retval = $self->update_global_scanner_config($session, $call_flags, $place_holder, $global_scanner_config);

    return { return_value => $retval };
}

1;
