package F5::ASMConfig::Handler::HslFtpProfile;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::ASMConfig;
use F5::ASMConfig::Entity::HslFtpProfile;

our @handled_functions = (
                           {
                             function_name => 'reset_defaults_hsl_ftp_profile',
                             relayable => 1,
                           },
                           {
                             function_name => 'get_default_hsl_ftp_profile',
                           },
                         );

our $entity_class = 'F5::ASMConfig::Entity::HslFtpProfile';
#------------------------------------------------------------------
sub reset_defaults_hsl_ftp_profile{
  my ($self, $session, $call_flags, $place_holder, $ftp_profile) = @_;
  my $dbh = $self->{dbh};

  my $res = get_default_hsl_ftp_profile($self, $session, $call_flags);

  my $default_ftp_profile = $res->{return_value};
  $default_ftp_profile->{hsl_profile_id} = $ftp_profile->{hsl_profile_id};

  $default_ftp_profile->{flg_default} = undef;
  
  return  $self->base_update(rpc_params => [$session, $call_flags,$place_holder, $default_ftp_profile],
                             entity_class => $entity_class,);

}
#------------------------------------------------------------------
sub get_default_hsl_ftp_profile{
  my ($self, $session, $call_flags) = @_;
  my $dbh = $self->{dbh};

  my $default_ftp_profile = F5::ASMConfig::Entity::HslFtpProfile->new(dbh => $dbh, preload_default => 1);

   my $default_command_config = $dbh->selectall_arrayref(<<EndQuery, { Slice => {} });
SELECT command_code, flg_is_enabled
  FROM $self->{plc_dbname}.HSL_COMMAND_CODES
 WHERE hsl_profile_type_id = 1
EndQuery

  for my $command_config (@{$default_command_config}){
    push @{$default_ftp_profile->{hsl_profile_command_config}}, 
               F5::ASMConfig::HslProfileCommandConfig->new( { command_code => $command_config->{command_code},
                                                              flg_is_enabled => $command_config->{flg_is_enabled},
                                                            }
                                                           );
  }


     my $default_violation_config = $dbh->selectall_arrayref(<<EndQuery, { Slice => {} });
SELECT violation_code, flg_alarm, flg_reject
  FROM $self->{plc_dbname}.HSL_PROFILE_VIOLATION_CONFIG_DEFAULTS
 WHERE hsl_profile_type_id = 1
EndQuery

  for my $violation_config (@{$default_violation_config}){
    push @{$default_ftp_profile->{hsl_profile_violation_config}},
               F5::ASMConfig::HslProfileViolationConfig->new( { violation_code => $violation_config->{violation_code},
                                                                flg_alarm => $violation_config->{flg_alarm},
                                                                flg_reject => $violation_config->{flg_reject},
                                                              }
                                                           );
  }


  push @{$default_ftp_profile->{ftp_profile_properties}}, F5::ASMConfig::HslFtpProfileProperties->new( { flg_enforce_username_retries => 1,
                                                                                                        username_retries_limit => 5,
                                                                                                        flg_enforce_host_ip_retries => 0,
                                                                                                        host_ip_retries_limit => 5,
                                                                                                        reenable_login_after_timeout => 15,
                                                                                                        command_length_restriction => 255,
                                                                                                       }
                                                                                                       );

  $default_ftp_profile->{hsl_profile_name} = undef;

  return { return_value => $default_ftp_profile };

}
1;
