package F5::ASMConfig::Handler::HslSmtpProfile;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::ASMConfig;
use F5::ASMConfig::Entity::HslSmtpProfile;

our @handled_functions = (
                           {
                             function_name => 'reset_defaults_hsl_smtp_profile',
                             relayable => 1,
                           },
                           {
                             function_name => 'get_default_hsl_smtp_profile',
                           },
                         );

our $entity_class = 'F5::ASMConfig::Entity::HslSmtpProfile';
#------------------------------------------------------------------
sub reset_defaults_hsl_smtp_profile{
  my ($self, $session, $call_flags, $place_holder, $smtp_profile) = @_;
  my $dbh = $self->{dbh};

  my $res = get_default_hsl_smtp_profile($self, $session, $call_flags);

  my $default_profile = $res->{return_value};
  $default_profile->{hsl_profile_id} = $smtp_profile->{hsl_profile_id};
 
  $default_profile->{flg_default} = undef;

  return  $self->base_update(rpc_params => [$session, $call_flags,$place_holder, $default_profile],
                             entity_class => $entity_class,);

}
#------------------------------------------------------------------
sub get_default_hsl_smtp_profile{
  my ($self, $session, $call_flags) = @_;
  my $dbh = $self->{dbh};

  my $default_smtp_profile = F5::ASMConfig::Entity::HslSmtpProfile->new(dbh => $dbh, preload_default => 1);

   my $default_command_config = $dbh->selectall_arrayref(<<EndQuery, { Slice => {} });
SELECT command_code, flg_is_enabled
  FROM $self->{plc_dbname}.HSL_COMMAND_CODES
 WHERE hsl_profile_type_id = 2
EndQuery

  for my $command_config (@{$default_command_config}){
    push @{$default_smtp_profile->{hsl_profile_command_config}}, 
               F5::ASMConfig::HslProfileCommandConfig->new( { command_code => $command_config->{command_code},
                                                              flg_is_enabled => $command_config->{flg_is_enabled},
                                                            }
                                                           );
  }


     my $default_violation_config = $dbh->selectall_arrayref(<<EndQuery, { Slice => {} });
SELECT violation_code, flg_alarm, flg_reject
  FROM $self->{plc_dbname}.HSL_PROFILE_VIOLATION_CONFIG_DEFAULTS
 WHERE hsl_profile_type_id = 2
EndQuery

  for my $violation_config (@{$default_violation_config}){
    push @{$default_smtp_profile->{hsl_profile_violation_config}},
               F5::ASMConfig::HslProfileViolationConfig->new( { violation_code => $violation_config->{violation_code},
                                                                flg_alarm => $violation_config->{flg_alarm},
                                                                flg_reject => $violation_config->{flg_reject},
                                                              }
                                                           );
  }


  push @{$default_smtp_profile->{smtp_profile_properties}}, 
               F5::ASMConfig::HslSmtpProfileProperties->new( {
                                                               flg_skip_first_message => 1,
                                                               sender_rate_limit => 100, 
                                                               receiver_rate_limit => 200,
                                                               auto_reject_first_message_timeout => 40,
                                                               directory_attack_timeout => 10,
                                                               directory_attack_count => 20,
                                                               hold_period => 60,
                                                             }
                                                           );

  $default_smtp_profile->{smtp_address_config} = [];

  $default_smtp_profile->{hsl_profile_name} = undef;

  return { return_value => $default_smtp_profile };

}
1;
