package F5::ASMConfig::Handler::HttpClass;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::ASMConfig;

our @handled_functions = (
                          {
                            function_name => 'add_http_class',
                            write_access => 1,
                            relayable    => 1,
                          },
                          {
                            function_name => 'search_http_class',
                            # -- only for EM > 3.1 (may never be called)
                          },
                         );

our $entity_class = 'F5::ASMConfig::Entity::HttpClass';

#----------------------------------------------------------------------

sub search_http_class {
    my($self, $session, $call_flags, $place_holder, $http_class) = @_;
# -- only for EM > 3.1 (may never be called)
    if (!F5::BigipUtils::is_device_em()) {
        die "Not implemented on BigIP\n";
    }
    
    my $search_result = $self->base_search(rpc_params => [$session, $call_flags, undef, $http_class],
                                           entity_class => $entity_class,
                                          );
    my $element_list = $search_result->{return_value}->{element_list};

    my %account_names = map { $_->{http_class}->{folder} . '/' . $_->{http_class}->{http_class_name} => $_->{http_class}->{http_class_id}, }
                        @$element_list;

    my $remote_device = $self->get_device($session, $call_flags, F5::ASMConfig::Device->new($http_class->{device}));

    my $remote_call_flags = F5::ASMConfig::CallFlags->new(remote_device => $remote_device);
    $remote_call_flags->{search_filters} = [ F5::ASMConfig::SearchFilter->new(
                                            field_name => 'account_name',
                                            field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_IN(),
                                            field_value_list => [keys %account_names],)];
    my @accounts = map {$_->{account}}
                   @{$self->search_account(
                         $session,
                         $remote_call_flags,
                         F5::ASMConfig::Account->new())->{element_list}};

    my %accounts;
    for my $account (@accounts){
        $accounts{$account_names{$account->{account_name}}} = $account->{active_policy_name};
    }

    my $local_filter_call_flags = F5::ASMConfig::CallFlags->new;
    $local_filter_call_flags->{search_filters} = [ F5::ASMConfig::SearchFilter->new(
                                                            field_name => 'http_class_id',
                                                            field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_IN(),
                                                            field_value_list => [keys %accounts],)];

    my $final_search_result = $self->base_search(rpc_params => [$session, $local_filter_call_flags, undef, $http_class],
                                                  entity_class => $entity_class,);

    my $http_classes_list = $final_search_result->{return_value}->{element_list};

    for my $actual_http_class (map{$_->{http_class}} @$http_classes_list) {
        $actual_http_class->{active_policy_name} = $accounts{$actual_http_class->{http_class_id}};
    }

    return $final_search_result;
}

#----------------------------------------------------------------------

sub add_http_class {
    my($self, $session, $call_flags, $place_holder, $http_class) = @_;

    #Called on BIGIP only
    $self->from_em_slash_replace(obj => $http_class,
                                 field_name => 'http_class_name');
    my $policy_name = $http_class->{http_class_name};
    if ($policy_name !~ m|^/|) {
        $policy_name = "/Common/$policy_name";
    }
    my $action;
    my $policy = F5::ASMConfig::Entity::Policy->new(dbh => $self->{dbh}, get_criteria => {policy_name => $policy_name});
    if (!$policy) {
        $policy = F5::ASMConfig::Policy->new(policy_name => $policy_name, 
                                             is_active_policy => $http_class->{flg_is_asm_enabled});
        $action = "add_policy";
    }
    elsif ($policy->{is_active_policy} != $http_class->{flg_is_asm_enabled}) {
       $policy->{is_active_policy} = $http_class->{flg_is_asm_enabled};
       $action = "update_policy";
    }
  
    if ($action) {
        $policy = $self->$action($self->{session}, $call_flags, $policy);
    }

    $http_class->{account_id} = $policy->{account_id};
    $http_class->{policy_id} = $policy->{policy_id};
    $http_class->{http_class_name} = $policy->{policy_name};

    return { return_value => $http_class };
}

#----------------------------------------------------------------------

1;
