package F5::ASMConfig::Handler::IdlFile;

use strict;
use warnings;

use F5::ASMConfig::Entity::IdlFile;
use F5::ASMConfig::Handler::TaskBase;
use F5::Utils::Rest;
use MIME::Base64;
use File::Basename;

our @handled_functions = (
                            {
                             function_name => 'add_idl_file',
                             relayable => 1,
                             criteria_index => 3,
                             store_index    => 4,
                            },
                            {
                              function_name => 'update_idl_file',
                              criteria_index => 3,
                              store_index => 4
                            },
                         );

our $entity_class = 'F5::ASMConfig::Entity::IdlFile';

#----------------------------------------------------------------------

sub add_idl_file {
    my ($self, $session, $call_flags, $policy, $idl_file) = @_;

    $idl_file->{file_name} = basename($idl_file->{file_name} || "");

    if (!$idl_file->{file_contents}) {
        my $search_file_name = F5::Utils::Rest::get_full_file_path(username => $session->{gui_user_name}, filename => $idl_file->{file_name} );
        open (my $FH, '<', $search_file_name) || $self->handle_error(action => 'add',
                                                                     entity_class => $entity_class,
                                                                     thrift_obj  => $idl_file,
                                                                     error_code  => F5::ASMConfig::ErrorCode::PERMISSION_DENIED(),
                                                                     msg         => "Could not find $idl_file->{file_name}.",
                                                                     error_level  => "info", );
        local $/;
        $idl_file->{file_contents} = <$FH>;
    }

    if ($idl_file->{flg_is_base64}) {
        $idl_file->{file_contents} = MIME::Base64::decode_base64($idl_file->{file_contents});
    }

    $call_flags->{return_existing_on_duplicate_add} = 1;

    return $self->base_add(rpc_params => [$session, $call_flags, $policy, $idl_file],
                           entity_class => $entity_class,
                           );
}

#----------------------------------------------------------------------

1;
