package F5::ASMConfig::Handler::InternalConfigItem;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::ASMConfig;
use F5::ASMConfig::Entity::InternalConfigItem;

our @handled_functions = (
                           {
                             function_name => 'add_internal_config_item',
                             relayable => 1,
                           },
                           {
                             function_name => 'delete_internal_config_item',
                             relayable => 1,
                           },
                           {
                             function_name => 'delete_many_internal_config_items',
                             relayable => 1,
                           },
                         );

our $entity_class = 'F5::ASMConfig::Entity::InternalConfigItem';
#----------------------------------------------------------------------

sub add_internal_config_item {
    my($self, $session, $call_flags, $place_holder, $store_internal_config_item) = @_;

    $store_internal_config_item->{flg_is_factory_setting} = 0;

    if ( !defined($store_internal_config_item->{factory_value}) ) {
        $store_internal_config_item->{factory_value} = ( $store_internal_config_item->{config_item_value_type} == F5::ASMConfig::ConfigItemValueType::CONFIG_ITEM_VALUE_TYPE_NUMBER() ) ? 0 : '';
    }

    return $self->base_add( rpc_params => [$session, $call_flags, $place_holder, $store_internal_config_item],
                            entity_class => $entity_class,
                          );
}

#----------------------------------------------------------------------

sub delete_internal_config_item {
  my($self, $session, $call_flags, $place_holder, $internal_config_item) = @_;

  my $deleted_internal_config_item = F5::ASMConfig::Entity::InternalConfigItem->new( dbh => $self->{dbh},
                                                                                     get_criteria => $internal_config_item );


  if ( $deleted_internal_config_item->{flg_is_factory_setting} ) {
     return $self->handle_error( error_code =>  F5::ASMConfig::ErrorCode::ACTION_NOT_ALLOWED(),
                                 error_thrown => "Variable '$deleted_internal_config_item->{config_item_name}' is not user defined and cannot be deleted.\n",
                                 error_level  => "info",
                                 thrift_obj => $internal_config_item,
                                 call_flags => $call_flags,
                               );
  }

  return $self->base_delete(rpc_params => [$session, $call_flags, $place_holder, $internal_config_item],
                            entity_class => $entity_class,
                           );
}

#----------------------------------------------------------------------

sub delete_many_internal_config_items {
    my($self, $session, $call_flags, $place_holder, $internal_config_item_criteria) = @_;

    my $local_call_flags = F5::ASMConfig::CallFlags->new($call_flags);

    if (!defined $local_call_flags->{search_filters}){
       $local_call_flags->{search_filters} = [];
    }

    push(@{$local_call_flags->{search_filters}},
                 { field_name => 'flg_is_factory_setting',
                   field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_EQ(),
                   field_value => 0 });

    my $rv = $self->base_delete_many(entity_class => $entity_class,
                                     rpc_params => [ $session, $local_call_flags,$place_holder, $internal_config_item_criteria ],);

    return $rv;
}

#----------------------------------------------------------------------

1;
