package F5::ASMConfig::Handler::JsonValidationFile;

use strict;
use warnings;

use F5::ASMConfig::Entity::JsonValidationFile;
use F5::ASMConfig::Handler::TaskBase;
use F5::Utils::Rest;
use MIME::Base64;
use File::Basename;

our @handled_functions = (
                            {
                             function_name => 'add_json_validation_file',
                             relayable => 1,
                             criteria_index => 3,
                             store_index    => 4,
                            },
                            {
                              function_name => 'update_json_validation_file',
                              criteria_index => 3,
                              store_index => 4
                            },
                         );

our $entity_class = 'F5::ASMConfig::Entity::JsonValidationFile';

#----------------------------------------------------------------------

sub add_json_validation_file {
    my ($self, $session, $call_flags, $policy, $json_validation_file) = @_;

    $json_validation_file->{file_name} = basename($json_validation_file->{file_name} || "");

    if (!$json_validation_file->{file_contents}) {
        my $search_file_name = F5::Utils::Rest::get_full_file_path(username => $session->{gui_user_name}, filename => $json_validation_file->{file_name} );
        open (my $FH, '<', $search_file_name) || $self->handle_error(action => 'add',
                                                                     entity_class => $entity_class,
                                                                     thrift_obj  => $json_validation_file,
                                                                     error_code  => F5::ASMConfig::ErrorCode::PERMISSION_DENIED(),
                                                                     msg         => "Could not find $json_validation_file->{file_name}.",
                                                                     error_level  => "info", );
        local $/;
        $json_validation_file->{file_contents} = <$FH>;
    }

    if ($json_validation_file->{flg_is_base64}) {
        $json_validation_file->{file_contents} = MIME::Base64::decode_base64($json_validation_file->{file_contents});
    }

    $call_flags->{return_existing_on_duplicate_add} = 1;

    return $self->base_add(rpc_params => [$session, $call_flags, $policy, $json_validation_file],
                           entity_class => $entity_class,
                           );
}

#----------------------------------------------------------------------

sub update_json_validation_file {
    my ($self, $session, $call_flags, $policy, $json_validation_file, $store_json_validation_file) = @_;

    if (! $call_flags->{force_update_json_validation_file}) {
        return $self->handle_error(action => 'update',
                                   error_code => F5::ASMConfig::ErrorCode::ACTION_NOT_ALLOWED(),
                                   entity_class => $entity_class,
                                   thrift_obj => $json_validation_file,
                                   error_thrown => "Cannot update uploaded JSON validation files",
                                   error_level => "info",
                                  );
    }

    my $rv = $self->base_update(entity_class => $entity_class,
                                rpc_params => [ $session, $call_flags, $policy, $json_validation_file, $store_json_validation_file ]);
    return $rv;
}

#----------------------------------------------------------------------

1;
