package F5::ASMConfig::Handler::LoadComplete;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::Entity::SessionAwarenessDataPoint;
use F5::DbSync;
use F5::ConfigSync;
use F5::PrepareConf::LoggingProfile;
use Time::HiRes;

our @handled_functions = (
                          {
                              function_name => 'load_complete',
                              relayable => 0,
                          },
                         );

#------------------------------------------------------------------
sub load_complete {
    my ($self, $session, $call_flags, $load_complete) = @_;
    my $dbh = $self->{dbh};
    my $logger = $self->{logger};
    $load_complete ||= {};

    if (! defined $load_complete->{do_reload_session_awareness}) {
        $load_complete->{do_reload_session_awareness} = 1;
    }

    if (F5::ConfigSync::is_ucs_post_install_waiting()) {
        my $configsync = F5::ConfigSync->new(ucs_post_install => 1,
                                             dbh => $dbh,
                                             logger => $logger,
                                             asm_config_handler => $self,
                                            );
        $configsync->load_with_account_data;
    }

    my $dbsync = F5::DbSync->new(
        logger     => $logger,
        dbh          => $dbh,
        asm_config_handler => $self,
    );

    $logger->info("populating data into mcp");
    $dbsync->populate_asm_data_to_mcp();
    $logger->info("populating completed successfully");

    $dbsync->sync_policies;
    $dbsync->sync_ftp_profiles;
    $dbsync->sync_smtp_profiles;
    $dbsync->sync_virtual_servers;

    my $prepare_logprof = F5::PrepareConf::LoggingProfile->new(dbh => $dbh,
                                                               logger => $logger,
                                                              );

    $prepare_logprof->prepare(flg_load_complete => 1);

    # Other tsconfd-initiated changes may have occurred in the interim --
    # thus, always send updated config to BD
    F5::SetActive::set_active(do_update_core => 1, 
                              dbh => $dbh,
                              logger => $logger,
                              asm_config_handler => $self);

    if ($load_complete->{do_reload_session_awareness}) {
        $logger->debug("Reloading Session Awareness Data Points") if $logger;
        eval {
            F5::ASMConfig::Entity::SessionAwarenessDataPoint->reload_session_db(dbh => $dbh,
                                                                                logger => $logger,
                                                                               );
        };
        if ($@) {
            $logger->error("Failed to load Session Awareness Data Points ($@)") if $logger;
        }
    }

    if ($call_flags && $call_flags->{update_mcp_timestamp}) {
        $self->{known_times}{$self->{local_device_name}} = Time::HiRes::time();
        $self->{update_mcp_timestamp} = 1;
    }

    # placed by F5::ASMStart on startup
    F5::ASMReady::remove_asm_load_complete_end_lock_file();

    return {return_value => 1, events => []};
}

#------------------------------------------------------------------

1;
