package F5::ASMConfig::Handler::McpLoggingProfile;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::PrepareConf::Base;
use F5::PrepareConf::LoggingProfile;
use F5::SetActive;

our @handled_functions = (
                           {
                             function_name => "update_mcp_logging_profile",
                             relayable => 0,
                           },
                           {
                             function_name => "update_mcp_http_method",
                             relayable => 0,
                           },
                         );

my %pool_member_cache;

#----------------------------------------------------------------------
sub update_mcp_logging_profile {
    my ($self, $session, $call_flags, $place_holder, $mcp_logging_profile) = @_;

    my $update_needed = 0;
    for my $k (keys %$mcp_logging_profile) {
        if ($k eq 'pool_names') {
            if ($mcp_logging_profile->{pool_names}) {
                for my $pool_name (@{$mcp_logging_profile->{pool_names}}) {
                    my($mcp_pool_member) = @{F5::ASM::COAPI::Query->new('pool_member', {pool_name => $pool_name})};
                    my $cache_pool_member = $pool_member_cache{$pool_name};
                    if ($cache_pool_member && $mcp_pool_member &&
                        $cache_pool_member->{addr} eq $mcp_pool_member->{addr} &&
                        $cache_pool_member->{port} eq $mcp_pool_member->{port}) {

                        $self->{logger}->debug("Skipping unnecessary MCP update to pool member $pool_name") if $self->{logger};
                    }
                    else {
                        $update_needed = 1;
                    }
                    $pool_member_cache{$pool_name} = $mcp_pool_member;
                }
            }
        }
        elsif ($k eq 'flg_load_complete' && $mcp_logging_profile->{flg_load_complete}) {
            $update_needed = 1;
        }
        elsif ($mcp_logging_profile->{$k} && ref($mcp_logging_profile->{$k}) && @{$mcp_logging_profile->{$k}}) {
            $update_needed = 1;
        }
    }

    if ($update_needed) {
        my $lock_file =  $F5::DbUtils::LOCK_FILE;
        my $locker = F5::LockFactory::get_locker(lock_filename => $lock_file,
                                                 logger => $self->{logger});

        # keep the lock until after the set_active is complete
        $locker->lock();

        my $prepare_logprof = F5::PrepareConf::LoggingProfile->new(dbh => $self->{dbh},
                                                                   logger => $self->{logger},
                                                                   debug => 1, # to no override log_level
                                                                  );

        my $change_made = $prepare_logprof->prepare(flg_load_complete => $mcp_logging_profile->{flg_load_complete},
                                                    vs_names => $mcp_logging_profile->{vs_names},
                                                    profile_names => $mcp_logging_profile->{profile_names},
                                                    publisher_names => $mcp_logging_profile->{publisher_names},
                                                    destination_names => $mcp_logging_profile->{destination_names},
                                                    dest_pool_names => $mcp_logging_profile->{dest_pool_names},
                                                    pool_names => $mcp_logging_profile->{pool_names});

        if ($change_made) {
            # send_update_core even if no logging profile in use, since last fw_vs_log_profile could be deleted
            my $set_active_rv = F5::SetActive::set_active(dbh => $self->{dbh},
                                                          logger => $self->{logger},
                                                          asm_config_handler => $self,
                                                          do_update_core => 1,
                                                          expanded_return_value => 1,
                                                          prepare_account_tables => 0,
                                                         );

            $locker->unlock();

            if ($set_active_rv->{update_core_rv} ne '0') {
                return $self->handle_error(action => "update_mcp_logging_profile",
                                           error_code => F5::ASMConfig::ErrorCode::SET_ACTIVE_FAIL(),
                                           error_thrown => "Apply Logging profile operation failed. Error: $set_active_rv->{update_core_rv}",
                                           error_level => "error",
                                          );
            }
        }
        else {
            $locker->unlock();
            $self->{logger}->debug("Update to logging profile does not affect ASM logging") if $self->{logger};
        }
    }

    return { return_value => $mcp_logging_profile };
}

#----------------------------------------------------------------------
sub update_mcp_http_method {
    my ($self, @rpc_params) = @_;

    my $prepare_base = F5::PrepareConf::Base->new(dbh => $self->{dbh},
                                                  logger => $self->{logger},
                                                  debug => 1, # to no override log_level
                                                 );
    $prepare_base->prepare_method_pool();
    # send_update_core will be done together with policy or logging profile
    
    return { return_value => 1 };
}

#------------------------------------------------------------------

1;
