package F5::ASMConfig::Handler::Parameter;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::ASMConfig;
use F5::ASMConfig::Entity::Base;
use F5::ASMConfig::Entity::Parameter;

our @handled_functions = (
                           {
                             function_name => 'update_parameter_add_update_value_metachars',
                             relayable => 1,
                             criteria_index => 3,
                             store_index    => 4,
                           },
                           {
                             function_name => 'update_parameter_add_update_name_metachars',
                             relayable => 1,
                             criteria_index => 3,
                             store_index    => 4,
                           },
                           {
                             function_name => 'update_parameter_add_update_negsigs',
                             relayable => 1,
                             criteria_index => 3,
                             store_index    => 4,
                           },
                           {
                             function_name => 'update_parameter_delete_negsigs',
                             relayable => 1,
                             criteria_index => 3,
                             store_index    => 4,
                           },
                           {
                             function_name => 'parameter_increase_length',
                             relayable => 1,
                             criteria_index => 3,
                             store_index    => 4,
                           },
                           {
                             function_name => 'delete_parameter',
                             relayable => 1,
                           },
                           {
                             function_name => 'delete_many_parameters',
                             relayable => 1,
                           },
                           {
                             function_name => 'update_parameter',
                             relayable => 1,
                             criteria_index => 3, 
                             store_index => 4
                           },
                         );

our $entity_class = 'F5::ASMConfig::Entity::Parameter';

#----------------------------------------------------------------------
sub delete_parameter {
  my($self, $session, $call_flags, $policy, $parameter) = @_;
  my $dbh = $self->{dbh};

  $self->update_many_scanner_vulns($session,
                                   F5::ASMConfig::CallFlags->new(set_fields_to_null => [qw/asm_parameter/]),
                                   $policy,
                                   F5::ASMConfig::ScannerVuln->new({asm_parameter => $parameter}),
                                   F5::ASMConfig::ScannerVuln->new({asm_vuln_status => F5::ASMConfig::ScannerAsmVulnStatus::SCANNER_ASM_VULN_STATUS_PENDING()}));


  my $rv = $self->base_delete(entity_class => $entity_class,
                              rpc_params => [ $session, $call_flags,$policy, $parameter ],
                             );

  return $rv;
}

#----------------------------------------------------------------------

sub delete_many_parameters {
    my($self, $session, $call_flags, $policy, $param_criteria) = @_;

    my $param_iter = $entity_class->retrieve_all(dbh => $self->{dbh},
                                                 call_flags => $call_flags,
                                                 policy => $policy,
                                                 search_filters => $call_flags->{search_filters},
                                                 get_criteria => $param_criteria);

    while (my $param = $param_iter->get_next()) {
        $self->update_many_scanner_vulns($session,
                                         F5::ASMConfig::CallFlags->new(set_fields_to_null => [qw/asm_parameter/]),
                                         $policy,
                                         F5::ASMConfig::ScannerVuln->new({asm_parameter => $param}),
                                         F5::ASMConfig::ScannerVuln->new({asm_vuln_status => F5::ASMConfig::ScannerAsmVulnStatus::SCANNER_ASM_VULN_STATUS_PENDING()}));
    }

    my $rv = $self->base_delete_many(entity_class => $entity_class,
                                     rpc_params => [ $session, $call_flags,$policy, $param_criteria ],);

    return $rv;
}

#----------------------------------------------------------------------

sub update_parameter_add_update_value_metachars {
  my($self, @rpc_params) = @_;

  my $call_flags = $rpc_params[1];
  $call_flags->{append_nested_struct_on_update} = 1;
  $call_flags->{remove_nested_struct_on_update} = 0;
  return $self->base_update(rpc_params => \@rpc_params,
                            entity_class => $entity_class,
                           );
}

#----------------------------------------------------------------------

sub update_parameter_add_update_name_metachars {
  my($self, @rpc_params) = @_;

  my $call_flags = $rpc_params[1];
  $call_flags->{append_nested_struct_on_update} = 1;
  $call_flags->{remove_nested_struct_on_update} = 0;
  return $self->base_update(rpc_params => \@rpc_params,
                            entity_class => $entity_class,
                           );
}

#----------------------------------------------------------------------

sub update_parameter_add_update_negsigs {
  my($self, @rpc_params) = @_;

  my $call_flags = $rpc_params[1];
  $call_flags->{append_nested_struct_on_update} = 1;
  $call_flags->{remove_nested_struct_on_update} = 0;
  return $self->base_update(rpc_params => \@rpc_params,
                            entity_class => $entity_class,
                           );
}

#----------------------------------------------------------------------

sub update_parameter {
    my($self, $session, $call_flags, $policy, $param_criteria, $store_param) = @_;

    my $delete_param_content_profile = 0;
    if (defined($store_param->{param_value_type}) 
        && $store_param->{param_value_type} != F5::ASMConfig::ParamValueType::PARAM_VALUE_XML_VALUE()
        && $store_param->{param_value_type} != F5::ASMConfig::ParamValueType::PARAM_VALUE_JSON_VALUE()) {
        my $param = $entity_class->new(dbh => $self->{dbh},
                                       get_criteria => $param_criteria,
                                       policy => $policy,
                                       call_flags => $call_flags);
        if ($param->{param_value_type} == F5::ASMConfig::ParamValueType::PARAM_VALUE_XML_VALUE()
            || $param->{param_value_type} == F5::ASMConfig::ParamValueType::PARAM_VALUE_JSON_VALUE()) {
            $delete_param_content_profile = 1;
        }
    }

    my $rv = $self->base_update(rpc_params => [ $session, $call_flags, $policy, $param_criteria, $store_param ],
                                entity_class => $entity_class,
                               );
    my $param = $rv->{return_value};

    if ($delete_param_content_profile) {
        $self->delete_parameter_content_profile($session, 
                                                F5::ASMConfig::CallFlags->new(ignore_nonexistent_record_on_delete => 1), 
                                                $policy, 
                                                F5::ASMConfig::ParameterContentProfile->new(parameter => $param));
        #Ensure this event arrives after the update parameter event
        push (@{$rv->{events}}, pop @{$self->{events}});
    }

    my $status;
    if ($param->{in_staging}) {
        $status = F5::ASMConfig::ScannerAsmVulnStatus::SCANNER_ASM_VULN_STATUS_HANDLED_STAGING();
    }
    else {
        $status = F5::ASMConfig::ScannerAsmVulnStatus::SCANNER_ASM_VULN_STATUS_HANDLED();
    }

    $self->update_many_scanner_vulns($session,
                                     undef,
                                     $policy,
                                     F5::ASMConfig::ScannerVuln->new({asm_parameter => $param}),
                                     F5::ASMConfig::ScannerVuln->new({asm_parameter => undef, asm_vuln_status => $status}));

     return $rv;
}

#----------------------------------------------------------------------

sub update_parameter_delete_negsigs {
  my($self, @rpc_params) = @_;

  my $call_flags = $rpc_params[1];
  $call_flags->{remove_nested_struct_on_update} = 1;
  $call_flags->{append_nested_struct_on_update} = 0;
  return $self->base_update(rpc_params => \@rpc_params,
                            entity_class => $entity_class,
                           );
}

#----------------------------------------------------------------------

sub parameter_increase_length {
  my($self, @rpc_params) = @_;
  return $self->base_update(rpc_params => \@rpc_params,
                            entity_class => $entity_class,
                            update_func => sub {
                              my(%params) = @_;
                              my $entity = $params{entity};
                              my $store_obj = $params{store_obj};

                              if ($store_obj->{max_value_len} &&
                                  (defined $entity->{max_value_len} &&
                                   $entity->{max_value_len} >= $store_obj->{max_value_len})) {
                                # Discard unnecessary update
                                delete $store_obj->{max_value_len};
                              }
                            },
                           );
}

#----------------------------------------------------------------------

1;
