package F5::ASMConfig::Handler::PolicyActionItem;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::Entity::PolicyActionItem;

our @handled_functions = (
                            {
                             function_name => 'delete_policy_action_item',
                             relayable => 0,
                             write_access => 1,
                            },
                            {
                             function_name => 'delete_many_policy_action_items',
                             relayable => 0,
                             write_access => 1,
                            },
                         );

our $entity_class = 'F5::ASMConfig::Entity::PolicyActionItem';

#----------------------------------------------------------------------

sub delete_policy_action_item {
  my ($self, $session, $call_flags, $policy, $policy_action_item_criteria) = @_;

  my $policy_action_item = $entity_class->new(dbh => $self->{dbh},
                                              policy => $policy,
                                              get_criteria => $policy_action_item_criteria,);
  return unless $policy_action_item;

  my $dismiss_time = $self->{action_item_handler}->get_dismiss_time(type => $policy_action_item->{action_item_type});
  $policy_action_item->{next_display_time} = time() + $dismiss_time;
  return $self->base_update(rpc_params => [$session, $call_flags, $policy, $policy_action_item_criteria, $policy_action_item],
                            entity_class => $entity_class,
                           );
}

#----------------------------------------------------------------------

sub delete_many_policy_action_items {
    my ($self, $session, $call_flags, $policy, $action_item_criteria) = @_;
    my $action_item_iter = $entity_class->retrieve_all(dbh => $self->{dbh},
                                                       call_flags => $call_flags,
                                                       policy => $policy,
                                                       search_filters => $call_flags->{search_filters},
                                                       get_criteria => $action_item_criteria);

    my $delete_call_flags = F5::ASMConfig::CallFlags->new(%$call_flags, search_filters => []);
    my @entities_changed;
    while (my $action_item = $action_item_iter->get_next()) {
          my $rv = $self->delete_policy_action_item($session,
                                                    $delete_call_flags,
                                                    $policy,
                                                    $action_item);
          if ($rv) {
              push(@entities_changed, $rv);
          }
    }

    my $rv;
    if ($call_flags->{rest_call}) {  #Return a search result if this is a rest call
        $rv = $self->entities_to_search_result(entities => \@entities_changed,
                                               call_flags => $call_flags,
                                               entity_class => $entity_class,);
    }
    else {
        $rv = scalar @entities_changed;
    }

    my $retval = { return_value => $rv, entities_changed => \@entities_changed};        
    return $retval;
}

#----------------------------------------------------------------------

1;
