package F5::ASMConfig::Handler::PolicyBuildingDisallowedFiletype;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;

our @handled_functions = (
                           {
                             function_name => 'delete_policy_building_disallowed_filetype',
                             relayable     => 1,
                           },
                           {
                             function_name => 'add_policy_building_disallowed_filetype',
                             relayable     => 1,
                           },
                           {
                             function_name => 'delete_many_policy_building_disallowed_filetypes',
                             relayable     => 1,
                           },
                           {
                             function_name => 'search_policy_building_disallowed_filetype',
                           },

                         );

our $entity_class = 'F5::ASMConfig::Entity::PolicyBuildingDisallowedFiletype';

#----------------------------------------------------------------------
sub delete_policy_building_disallowed_filetype {
    my($self, $session, $call_flags, $place_holder, $policy_building_disallowed_filetype) = @_;

    my $updated_policy_building_disallowed_filetype = $entity_class->new(dbh => $self->{dbh}, get_criteria => $policy_building_disallowed_filetype);
    if (!$updated_policy_building_disallowed_filetype){
        return $self->handle_error(action => 'delete',
                                   entity_class => $entity_class,
                                   error_code   => F5::ASMConfig::ErrorCode::NONEXISTENT_RECORD(),
                                   thrift_obj   => $policy_building_disallowed_filetype,
                                   error_thrown => $F5::ASMConfig::Handler::NONEXISTENT_RECORD_ERROR,
                                   error_level  => 'info',
                                 );
    }

    $updated_policy_building_disallowed_filetype->{is_deleted} = 1;
    my $rv = $self->base_update(entity_class => $entity_class,
                                rpc_params => [ $session, $call_flags,undef, $policy_building_disallowed_filetype,$updated_policy_building_disallowed_filetype]);
    if ( $rv && $rv->{events} ){
        for my $event ( @{$rv->{events}} ){
            $event->{event_type} = F5::ASMConfig::EventType::DELETE();
        }
    }

    return $rv;
}

#----------------------------------------------------------------------

sub add_policy_building_disallowed_filetype {
    my($self, $session, $call_flags, $place_holder, $policy_building_disallowed_filetype) = @_;

    $call_flags->{update_on_adding_duplicate_record} = 1;
    $policy_building_disallowed_filetype->{is_deleted} = 0;
    delete $call_flags->{search_filters};
    my $rv = $self->base_add(entity_class => $entity_class,
                                rpc_params => [ $session, $call_flags,undef, $policy_building_disallowed_filetype]);

    if ( $rv && $rv->{events} ){
        for my $event ( @{$rv->{events}} ){
            $event->{event_type} = F5::ASMConfig::EventType::ADD();
        }
    }

    return $rv;

}

#----------------------------------------------------------------------

sub delete_many_policy_building_disallowed_filetypes {
    my($self, $session, $call_flags, $place_holder, $policy_building_disallowed_filetype_criteria) = @_;

    my $updated_policy_building_disallowed_filetype = $entity_class->new(dbh => $self->{dbh}, object => { is_deleted => 1});
    my $rv = $self->base_update_many(entity_class => $entity_class,
                                     rpc_params => [ $session, $call_flags,undef, $policy_building_disallowed_filetype_criteria,$updated_policy_building_disallowed_filetype]);

    if ( $rv && $rv->{events} ){
        for my $event ( @{$rv->{events}} ){
            $event->{event_type} = F5::ASMConfig::EventType::DELETE();
        }
    }

    return $rv;
}

#----------------------------------------------------------------------

sub search_policy_building_disallowed_filetype {
    my($self, $session, $call_flags, undef,$policy_building_disallowed_filetype_criteria) = @_;
    $call_flags->{search_filters} ||= [];

    push (@{$call_flags->{search_filters}}, @{$entity_class->catchup_search_filters});

    my $rv = $self->base_search(rpc_params => [$session, $call_flags, undef, $policy_building_disallowed_filetype_criteria],
                                entity_class => $entity_class,
                               );
    return $rv;
}
#----------------------------------------------------------------------

1;

