package F5::ASMConfig::Handler::PolicyDiffTask;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::TaskBase/;
use F5::ASMConfig::Entity::PolicyDiffTask;

our @handled_functions = (
                            {
                             function_name => 'add_policy_diff_task',
                             criteria_index => 3,
                             store_index    => 4,
                            },
                            {
                              function_name => 'update_policy_diff_task',
                              criteria_index => 3,
                              store_index => 4
                            },
                            {
                             function_name => 'execute_policy_diff_task',
                            },
                            {
                             function_name => 'delete_policy_diff_task',
                            },
                            {
                             function_name => 'delete_many_policy_diff_tasks',
                            },
                         );

our $entity_class = 'F5::ASMConfig::Entity::PolicyDiffTask';

#----------------------------------------------------------------------

sub add_policy_diff_task {
  my($self, @rpc_params) = @_;
  return F5::ASMConfig::Handler::TaskBase::add_task(self => $self,
                                                    rpc_params => \@rpc_params,
                                                    entity_class => $entity_class,
                                                    task_type => 'policy_diff',);
}

#----------------------------------------------------------------------

sub update_policy_diff_task {
    my ($self, @rpc_params) = @_;
    return F5::ASMConfig::Handler::TaskBase::update_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                         task_type => 'policy_diff',);
}

#----------------------------------------------------------------------

sub policy_diff {
    my (%params) = @_;
    my $self       = $params{self};
    my $session    = $params{session};
    my $call_flags = $params{call_flags};
    my $task       = $params{task};

    my $first_policy = $task->{first_policy};
    my $second_policy = $task->{second_policy};

    if (!$first_policy || !$second_policy) {
        return $self->handle_error(action => 'execute',
                                   error_thrown => 'Policy not found',
                                   error_code => F5::ASMConfig::ErrorCode::MISSING_POLICY(),
                                   call_flags => $call_flags,
                                   entity_class => $entity_class,
                                   thrift_obj => $task,
                                  );
    }

    my $rv;
    eval {
        $rv = $self->add_diff($session, $call_flags, undef, F5::ASMConfig::Entity::Diff->new( object => {first_policy => $first_policy,
                                                                                                         second_policy => $second_policy} ));
    };

    if ($@) {
        $task->{message} = $@->{error_message};
        $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_FAILURE();
    }
    else {
        $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_COMPLETED();
        $task->{result_diff} = $rv;
    }
}


#----------------------------------------------------------------------

sub execute_policy_diff_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::execute_task(self => $self,
                                                          rpc_params => \@rpc_params,
                                                          entity_class => $entity_class,
                                                          task_type => 'policy_diff',
                                                          execute_func => sub { policy_diff(@_); },
                                                          );
}

#----------------------------------------------------------------------

sub delete_policy_diff_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                        );
}

#----------------------------------------------------------------------

sub delete_many_policy_diff_tasks {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_many_tasks(self => $self,
                                                               rpc_params => \@rpc_params,
                                                               entity_class => $entity_class,
                                                              );
}

#----------------------------------------------------------------------

1;
