package F5::ASMConfig::Handler::PolicyMergeTask;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::TaskBase/;
use F5::ASMConfig::Entity::PolicyMergeTask;

our @handled_functions = (
                            {
                             function_name => 'add_policy_merge_task',
                             criteria_index => 3,
                             store_index    => 4,
                            },
                            {
                              function_name => 'update_policy_merge_task',
                              criteria_index => 3,
                              store_index => 4
                            },
                            {
                             function_name => 'execute_policy_merge_task',
                             exclusive_lock_file => $F5::DbUtils::LOCK_FILE,
                            },
                            {
                             function_name => 'delete_policy_merge_task',
                            },
                            {
                             function_name => 'delete_many_policy_merge_tasks',
                            },
                         );

our $entity_class = 'F5::ASMConfig::Entity::PolicyMergeTask';

#----------------------------------------------------------------------

sub add_policy_merge_task {
  my($self, @rpc_params) = @_;
  return F5::ASMConfig::Handler::TaskBase::add_task(self => $self,
                                                    rpc_params => \@rpc_params,
                                                    entity_class => $entity_class,
                                                    task_type => 'policy_merge',);
}

#----------------------------------------------------------------------

sub update_policy_merge_task {
    my ($self, @rpc_params) = @_;
    return F5::ASMConfig::Handler::TaskBase::update_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                         task_type => 'policy_merge',);
}

#----------------------------------------------------------------------

sub policy_merge {
    my (%params) = @_;
    my $self       = $params{self};
    my $session    = $params{session};
    my $call_flags = $params{call_flags};
    my $task       = $params{task};

    my $diff = $task->{diff};
    if (!$diff) {
        return $self->handle_error(action => 'execute',
                                   error_thrown => 'Diff not found',
                                   error_code => F5::ASMConfig::ErrorCode::MISSING_FIELD(),
                                   call_flags => $call_flags,
                                   entity_class => $entity_class,
                                   thrift_obj => $task,
                                  );
    }

    my $local_call_flags = F5::ASMConfig::CallFlags->new();
    if ( $task->{item_filter} ) {
        my $filters = F5::ASMConfig::Entity::DiffItem->convert_rest_filter_string(%params,
                                                                                  dbh => $self->{dbh},
                                                                                  filter_string => $task->{item_filter});
        push (@{$local_call_flags->{search_filters}}, @$filters);
    }

    my $rv;
    my $auto_merge_flags = F5::ASMConfig::AutoMergeFlags->new( add_all_missing_to_first   => $task->{add_missing_entities_to_first},
                                                               add_all_missing_to_second  => $task->{add_missing_entities_to_second},
                                                               accept_conflicts_direction => $task->{handle_common_entities} );
    eval {
        $rv = $self->auto_merge_diff($session, $local_call_flags, undef, $diff, $auto_merge_flags);
    };

    if ($@) {
        $task->{message} = $@->{error_message};
        $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_FAILURE();
    }
    else {
        $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_COMPLETED();
        $task->{result_diff} = $rv;
    }
}

#----------------------------------------------------------------------

sub execute_policy_merge_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::execute_task(self => $self,
                                                          rpc_params => \@rpc_params,
                                                          entity_class => $entity_class,
                                                          task_type => 'policy_merge',
                                                          execute_func => sub { policy_merge(@_); },
                                                          );
}

#----------------------------------------------------------------------

sub delete_policy_merge_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                        );
}

#----------------------------------------------------------------------

sub delete_many_policy_merge_tasks {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_many_tasks(self => $self,
                                                               rpc_params => \@rpc_params,
                                                               entity_class => $entity_class,
                                                              );
}

#----------------------------------------------------------------------

1;
