package F5::ASMConfig::Handler::PolicySignatureSet;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::ASMConfig;

our @handled_functions = (
                           {
                             function_name => 'add_policy_signature_set',
                             relayable => 1,
                             store_index => 3,
                           },
                           {
                             function_name => 'send_policy_signature_set_event',
                           },
                         );

our $entity_class = 'F5::ASMConfig::Entity::PolicySignatureSet';

#----------------------------------------------------------------------

sub send_policy_signature_set_event {
    my($self, $session, $call_flags, $policy) = @_;
    #This function sends a "fake" policy signature set event to trigger PB refreshing all the policy signatures
    my $policy_signature_set = F5::ASMConfig::Entity::PolicySignatureSet->new(dbh => $self->{dbh},
                                                                       policy => $policy,
                                                                       object => {set_id => 0, flg_learn => 1},
                                                                      );
    return { return_value => undef,
             events => [ $self->create_event(event_type => F5::ASMConfig::EventType::UPDATE,
                                             session => $session,
                                             policy => $policy,
                                             element => $policy_signature_set,
                                             call_flags => $call_flags,
                                             user_name => $session->{user_name},
                                             suppress_save_to_log => 1,
                                            )
                       ],
           };
}

#----------------------------------------------------------------------

sub add_policy_signature_set {
  my($self, $session, $call_flags, $policy, $store_signature_set) = @_;
  my $rv = $self->base_add(rpc_params => [$session, $call_flags, $policy, $store_signature_set],
                           entity_class => $entity_class,
                           );

  if ($policy->{policy_type} == F5::ASMConfig::PolicyType::POLICY_TYPE_PARENT()) {
      my $signature_call_flags = F5::ASMConfig::CallFlags->new(%$call_flags, inherited_call => 1);
      my $child_policy_ids = $policy->child_policy_ids();;
      my $enabled_search_filter = F5::ASMConfig::SearchFilter->new({ field_name => 'flg_disabled',
                                                                     field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_NE(),
                                                                   });
      $signature_call_flags->{search_filters} = [
                                         F5::ASMConfig::SearchFilter->new(
                                                    field_name => 'is_inherited',
                                                    field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_EQ(),
                                                    field_value => 1,),
                                         F5::ASMConfig::SearchFilter->new(
                                                    field_name => 'flg_disabled_differs_from_parent_policy_id',
                                                    field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_EQ(),
                                                    field_value => $policy->{policy_id}),
                                         F5::ASMConfig::SearchFilter->new(
                                                    field_name => 'policy_id',
                                                    field_value_list => $child_policy_ids,
                                                    field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_IN()),
                                         $enabled_search_filter
                                         ];
      for my $flg_disabled (0,1) {
          $enabled_search_filter->{field_value} = $flg_disabled;
          $self->update_many_policy_signatures($self->{session}, $signature_call_flags, undef, F5::ASMConfig::PolicySignature->new(),
                                                                                               F5::ASMConfig::PolicySignature->new(flg_disabled => $flg_disabled));
      }
  }
  return $rv;
}

#----------------------------------------------------------------------

1;
