package F5::ASMConfig::Handler::PolicySummary;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
our @handled_functions = (
                           {
                             function_name => 'get_policy_summary',
                             authz_limit_field => 'policy_id',
                           },
                          );

our $entity_class = 'F5::ASMConfig::Entity::Policy';
#----------------------------------------------------------------------

sub get_policy_summary {
  my($self, $session, $call_flags, $place_holder, $policy_criteria) = @_;

  my $dbh = $self->{dbh};
  my $summary = F5::ASMConfig::PolicySummaryRes->new({  
                                                       configured_policies => 0,
                                                       policies => 0,
                                                       file_types => 0,
                                                       urls => 0,
                                                       parameters => 0,
                                                       cookies => 0,
                                                       redirection_domains => 0,
                                                       is_pb_enabled => undef,
                                                       is_learning_enabled => undef,
                                                      });

  $self->add_partition_search_filter(session    => $session,
                                     call_flags => $call_flags);

  my @policies = F5::ASMConfig::Entity::Policy->retrieve_all(dbh => $dbh,
                                                             call_flags => $call_flags,
                                                             search_filters => $call_flags->{search_filters},
                                                             get_criteria => {is_active_policy => 1,
                                                                              %$policy_criteria});
  my $num_policies = scalar @policies;
  for my $policy (@policies) {
      $summary->{policies}++;
      if ($policy->{flg_encoding_selected}) {
          my %params = (dbh => $dbh, policy => $policy, minimal_load => 1, page_size => 1);
          $summary->{configured_policies}++;
          $summary->{file_types} += F5::ASMConfig::Entity::Filetype->retrieve_all(%params)->{total_record_count};
          $summary->{urls} += F5::ASMConfig::Entity::URL->retrieve_all(%params)->{total_record_count};
          $summary->{parameters} += F5::ASMConfig::Entity::Parameter->retrieve_all(%params)->{total_record_count};
          $summary->{cookies} += F5::ASMConfig::Entity::Cookie->retrieve_all(%params)->{total_record_count};
          $summary->{redirection_domains} += F5::ASMConfig::Entity::RedirectionDomain->retrieve_all(%params)->{total_record_count};
      }
      $summary->{is_pb_enabled} += $policy->{is_pb_enabled};
      $summary->{is_learning_enabled} += $policy->{is_learning_enabled};
  } 
  for my $verify_field (qw(is_pb_enabled is_learning_enabled)) {
      if ($summary->{$verify_field} == 0 || $summary->{$verify_field} == $num_policies) {
          $summary->{$verify_field} = $summary->{$verify_field} ? 1 : 0;
      }
      else {
          delete $summary->{$verify_field};
      }
  }

  return {return_value => $summary};
}

#----------------------------------------------------------------------
1;
