package F5::ASMConfig::Handler::RamCache;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::RamCache;
use F5::SetActive;

our @handled_functions = (
                           {
                             function_name => "invalidate_cache",
                             relayable => 0,
                           },
                           {
                             function_name => 'update_ramcache',
                             relayable => 0,
                           },
                         );

#----------------------------------------------------------------------
sub invalidate_cache {
    my($self, $session, $call_flags, $place_holder, $vs_name) = @_;

    # like in set_active (by policy)
    F5::RamCache::invalidate_cache_by_vs_name(vs_name => $vs_name, logger => $self->{logger});

    return { return_value => 1 };
}

#------------------------------------------------------
sub update_ramcache {
    my($self, $session, $call_flags) = @_;

    my $lock_file =  $F5::DbUtils::LOCK_FILE;
    my $locker = F5::LockFactory::get_locker(lock_filename => $lock_file,
                                             logger => $self->{logger});

    # keep the lock until after the set_active is complete
    $locker->lock();

    F5::RamCache::prepare_conf(dbh => $self->{dbh}, logger => $self->{logger});

    my $set_active_rv = F5::SetActive::set_active(dbh => $self->{dbh}, 
                                                  logger => $self->{logger},
                                                  asm_config_handler => $self,
                                                  do_update_core => 1,
                                                  expanded_return_value => 1,
                                                  prepare_account_tables => 0,
                                                 );
    $locker->unlock();

    if ($set_active_rv->{update_core_rv} ne '0') {
        return $self->handle_error(action => "update_ramcache",
                                   error_code => F5::ASMConfig::ErrorCode::SET_ACTIVE_FAIL(),
                                   error_thrown => "Apply RamCache operation failed. Error: $set_active_rv->{update_core_rv}",
                                   error_level => "error",
                                  );
    }           

    return { return_value => 1 };
}

#------------------------------------------------------------------

1;
