package F5::ASMConfig::Handler::ScannerConfig;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::Entity::ScannerConfig;
use F5::ASMConfig::ASMConfig;
use F5::Scanner;
use HTTP::Request;
use F5::DeviceGroup;
use F5::Cluster::Utils;

use F5::XML::Twig;
use LWP::UserAgent;

our @handled_functions = (
                           {
                             function_name => 'get_scanner_site_names',
                           },
                           {
                             function_name => 'update_scanner_config',
                             criteria_index => 3,
                             store_index    => 4,
                             relayable => 1,
                           },
                           {
                             function_name => 'add_scanner_config',
                             relayable => 1,
                             store_index   => 3,
                           },
                         );

our $entity_class = 'F5::ASMConfig::Entity::ScannerConfig';
my @site_list;

#----------------------------------------------------------------------

sub get_scanner_site_names {
    my ($self, $session, $call_flags, $policy, $scanner_config) = @_;

    my $ua = F5::Scanner::get_ua(timeout => $call_flags->{outbound_call_timeout});

    my $scanner = F5::Scanner->new(dbh => $self->{dbh},
                                   scanner_type => $scanner_config->{scanner_type},
                                   api_key => $scanner_config->{api_key},
                                  );

    my $request = HTTP::Request->new( GET => $scanner->get_scanner_site_names_url()); 
    my $response = $ua->request($request);

    if ($response->is_success) {
        my $twig = F5::XML::Twig->new( twig_handlers => { label => sub{ 
                                                                   my ($twig, $label) = @_; 
                                                                   push @site_list, $label->text;}
                                                    });
        $twig->parse($response->content);
        return {return_value => \@site_list};
    }
    else {
        my @events;
        push(@events, $self->create_event(event_type => F5::ASMConfig::EventType::ERROR));
	my $exception = {
			 error_message => "Failed to get site names\n" . $response->status_line,
			 events => \@events,
			};
	die $exception;
    }
}

#----------------------------------------------------------------------

sub update_scanner_config {
    my ($self, $session, $call_flags, $policy, $scanner_config_criteria, $store_scanner_config) = @_;

    if (!$store_scanner_config->{flg_is_api_key_final} && $store_scanner_config->{api_key}) {
        $store_scanner_config->{scan_status} = F5::ASMConfig::ScanStatus::SCAN_STATUS_REGISTRATION_REQUESTED();
    }
    elsif ($store_scanner_config->{flg_is_api_key_final} &&
           $store_scanner_config->{flg_is_api_key_final} == 1 && $store_scanner_config->{scan_requested_time}) {

        $store_scanner_config->{scan_status} = F5::ASMConfig::ScanStatus::SCAN_STATUS_REGISTRATION_COMPLETED();
    }
    else {
        $store_scanner_config->{scan_status} = F5::ASMConfig::ScanStatus::SCAN_STATUS_SCAN_COMPLETED();
    }

    if ($self->{is_relay_handler_process} && $store_scanner_config && $store_scanner_config->{bidir_enabled}
        && !F5::Cluster::Utils::is_failover_standby() && F5::DeviceGroup::is_device_active($call_flags->{originator})) {
        $store_scanner_config->{bidir_enabled} = 0;
    }


    my $rv = $self->base_update(rpc_params => [ $session, $call_flags, $policy, $scanner_config_criteria, $store_scanner_config ],
                                entity_class => $entity_class,);

    return $rv;
}


#----------------------------------------------------------------------

sub add_scanner_config {
    my ($self, $session, $call_flags, $policy, $scanner_config) = @_;

    $call_flags->{add_on_updating_nonexistent_record} = 1;
    if ($self->{is_relay_handler_process} && $scanner_config->{bidir_enabled}
       && !F5::Cluster::Utils::is_failover_standby() && F5::DeviceGroup::is_device_active($call_flags->{originator})) {
       $scanner_config->{bidir_enabled} = 0;
    }


    return { return_value => $self->update_scanner_config($session, $call_flags, $policy, $scanner_config), };
    
}

1;
