package F5::ASMConfig::Handler::ScannerVuln;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::Entity::ScannerVuln;
use F5::ASMConfig::Entity::ScannerConfig;
use F5::ASMConfig::Handler::ScannerActions;
use F5::Scanner;
use HTTP::Request;

use XML::Twig;
use LWP::UserAgent;

our $DIRECTORY_TRAVERSAL_NORMALIZATION_CODE = 4;

our @handled_functions = (
                           {
                             function_name => 'retest_vuln',
                             relayable => 1,
                           },
                           {
                             function_name => 'resolve_vuln',
                             relayable => 1,
                           },
                           {
                             function_name => 'get_pre_resolve_messages',
                           },
                         );

our $entity_class = 'F5::ASMConfig::Entity::ScannerVuln';
my @site_list;

#----------------------------------------------------------------------

sub retest_vuln {
    my($self, $session, $call_flags, $policy, $vuln) = @_;

    ($vuln) = F5::ASMConfig::Entity::ScannerVuln->retrieve_all(
                  dbh => $self->{dbh},
                  get_criteria => $vuln);

    my $response;
    my $logger = F5::Scanner::get_logger();
    if (!$self->{is_relay_handler_process}) {
        my $config_object = $self->base_get(
                                            rpc_params => [$session, $call_flags, $policy, F5::ASMConfig::ScannerConfig->new],
                                            entity_class => 'F5::ASMConfig::Entity::ScannerConfig',
                                           )->{return_value};


        my $scanner = F5::Scanner->new(dbh => $self->{dbh},
                                       client => $self,
                                       policy => $policy,);

        my $ua = F5::Scanner::get_ua(timeout => $call_flags->{outbound_call_timeout});

        my $request_url = $scanner->get_full_url(
                              vuln_ids => [$vuln->{scanner_vuln_id}],
                              url_type => 'retest_status_url',
                          );

        my $request = HTTP::Request->new( PUT => $request_url);

        $logger->debug("Retest_vuln is sending PUT request to $request_url");

        $response = $ua->request($request);
    }

    if ($self->{is_relay_handler_process} || ($response && $response->is_success)) {
        $logger->debug('Success. Updating status to Retest Requested.');

        my $update_vuln = F5::ASMConfig::ScannerVuln->new({
                              retest_status => F5::ASMConfig::ScannerRetestStatus::SCANNER_RETEST_STATUS_REQUESTED,
                              flg_retest_local => $self->{is_relay_handler_process} ? 0 : 1,});

        my $base_update_rv = $self->base_update_many(rpc_params => [
                                                         $session, 
                                                         $call_flags, 
                                                         $policy, 
                                                         F5::ASMConfig::ScannerVuln->new({scanner_vuln_id => $vuln->{scanner_vuln_id}}), 
                                                         $update_vuln],
                                                     entity_class => $entity_class,
                                                     suppress_events => 1,
                                                    );
        
    }
    else {
        $call_flags->{do_not_relay} = 1;
    }

    return { return_value => 1 };
}

#----------------------------------------------------------------------

sub resolve_vuln {
    my($self, $session, $call_flags, $policy, $vuln, $flg_staging) = @_;

    if (! defined $flg_staging) {
        $flg_staging = 0;
    }

    ($vuln) = F5::ASMConfig::Entity::ScannerVuln->retrieve_all(
                  dbh => $self->{dbh},
                  policy => $policy,
                  get_criteria => $vuln,);


    my $scanner = F5::Scanner->new(
                      dbh => $self->{dbh},
                      client => $self,
                      policy => $policy,);

    my $messages = $scanner->resolve_vuln(vuln => $vuln, flg_staging => $flg_staging);

    my @warnings;
    if ($messages) {
        for my $msg (@$messages) {
            my $message = F5::ASMConfig::Message->new();
            $message->{msg} = $msg;
            $message->{level} = 'warning';
            push @warnings, $message;
        }
    }
 
    my $return_value = F5::ASMConfig::ResolveVulnRes->new;
    $return_value->{msgs} = \@warnings;

    return {return_value => $return_value};
    
}

#----------------------------------------------------------------------

sub get_pre_resolve_messages {
    my($self, $session, $call_flags, $policy, $vuln, $flg_staging) = @_;

    ($vuln) = F5::ASMConfig::Entity::ScannerVuln->retrieve_all(
                  dbh => $self->{dbh},
                  policy => $policy,
                  get_criteria => $vuln,);


    my $scanner = F5::Scanner->new(
                      dbh => $self->{dbh},
                      client => $self,
                      policy => $policy,);

    my $messages = $scanner->resolve_vuln(vuln => $vuln, get_messages_only => 1, flg_staging => $flg_staging);

    my @warnings;
    if ($messages) {
        for my $message (@$messages) {
            push (@warnings, F5::ASMConfig::PreResolveMessage->new($message));
        }
    }

    return {return_value => \@warnings};
}

#----------------------------------------------------------------------

1;
