package F5::ASMConfig::Handler::SessionAwarenessDataPoint;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::ASMConfig;

our @handled_functions = (
                           {
                             function_name => 'update_many_session_awareness_data_points',
                             criteria_index => 3,
                             store_index => 4,
                             write_access => 1,
                             relayable => F5::ASMConfig::RelayabilityType::RELAYABILITY_TYPE_CHECK_RELAYABLE(),
                           },
                         );

our $entity_class = 'F5::ASMConfig::Entity::SessionAwarenessDataPoint';

#----------------------------------------------------------------------

sub update_many_session_awareness_data_points {
    my($self, $session, $call_flags, $policy, $session_awareness_data_point_criteria, $store_session_awareness_data_point) = @_;

    my $retval = $self->base_update_many(rpc_params => [$session, $call_flags, $policy, $session_awareness_data_point_criteria, $store_session_awareness_data_point],
                                         entity_class => $entity_class,);

    my $updated_records = 0;
    if ($retval->{events}) {
        if ($store_session_awareness_data_point->{is_persistent} && $self->{device_group}) {
            #These data points may not have been persistent until now so they may not exist on the remote device yet. 
            #Therefore each updated data point needs to be sent as an indivdual call with all of its fields
            $call_flags->{relayable} = 1;
            $call_flags->{add_on_updating_nonexistent_record} = 1;
            for my $event (@{$retval->{events}}) {
                if ($event->{element} && $event->{element}->{session_awareness_data_point}) {
                    push (@{$self->{rpc_calls_to_relay}->{update_many_session_awareness_data_points}}, {function_name => 'update_session_awareness_data_point',
                                                                                      rpc_params    => [$session, $call_flags, $policy, $event->{element}->{session_awareness_data_point}, 
                                                                                                                                        $event->{element}->{session_awareness_data_point}],
                                                                                     }
                    );
                    $updated_records++;
                }
            }
        }
    }
    if ($updated_records) {
           push(@{$retval->{events}},
           $self->create_event(event_type => F5::ASMConfig::EventType::UPDATE,
                               session => $session,
                               element => F5::ASMConfig::Element->new({
                                                                       element_type => F5::ASMConfig::ElementType::SESSION_AWARENESS_DATA_POINT,
                                                                      }),
                               call_flags => $call_flags,
                               user_name => $session->{user_name},
                               description => "$updated_records Session" . ($updated_records > 1 ? "s were" : " was") . " set to 'Block All'.",
                               policy => $policy,
                               do_policy_version_update => 0,
                              )
          );
    }
    return $retval;
}

#----------------------------------------------------------------------

1;
