package F5::ASMConfig::Handler::UCSConfigLoad;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;

our @handled_functions = (
                           {
                             function_name => 'send_ucs_config_load_event',
                           },
                         );

#----------------------------------------------------------------------

sub send_ucs_config_load_event {
  my($self, $session, $call_flags, $placeholder, $placeholder2, $loaded_policies) = @_;

  my $dbh = $self->{dbh};

  my $description = "Loaded configuration from " . ($self->{is_relay_handler_process} ? "Policy Sharing Synchronization." : "UCS.");
  my %loaded_policies_by_id = map { ($_->{policy_id} => $_) } @$loaded_policies;

  my @events;
  push(@events,
       $self->create_event(event_type => F5::ASMConfig::EventType::UPDATE,
                           session => $session,
                           element => F5::ASMConfig::Element->new({
                                                                   element_type => F5::ASMConfig::ElementType::UCS_CONFIG_LOAD,
                                                                  }),
                           call_flags => $call_flags,
                           user_name => $session->{user_name},
                           description => $description,
                          )
      );

  my $iter = F5::ASMConfig::Entity::Policy->retrieve_all(dbh => $self->{dbh}, logger => $self->{logger},);
  while(my $policy = $iter->get_next) {
      my $loaded_policy = $loaded_policies_by_id{$policy->{policy_id}};

      push(@events,
           $self->create_event(event_type => F5::ASMConfig::EventType::UPDATE,
                               session => $session,
                               element => F5::ASMConfig::Element->new({
                                                                       element_type => F5::ASMConfig::ElementType::POLICY,
                                                                       policy => $policy,
                                                                      }),
                               call_flags => $call_flags,
                               user_name => $session->{user_name},
                               description => $description,
                               suppress_send_to_dispatcher => 1,
                               policy => $policy,
                               policy_version_time => $loaded_policy->{policy_version_time},
                               policy_version_device_name => $loaded_policy->{policy_version_device_name},
                               policy_version_policy_name => $loaded_policy->{policy_version_policy_name},
                               policy_version_last_change => $loaded_policy->{policy_version_last_change},
                              )
          );
  }

  my $retval = { return_value => undef,
                 events => \@events,
               };
  return $retval;
}

#----------------------------------------------------------------------

1;
