package F5::ASMConfig::Handler::URL;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::Entity::URL;
use F5::CRC;

our @handled_functions = (
                           {
                             function_name => 'update_url_add_update_metachars',
                             relayable => 1,
                             criteria_index => 3,
                             store_index    => 4,
                           },
                           {
                             function_name => 'delete_url',
                             relayable => 1,
                           },
                           {
                             function_name => 'delete_many_urls',
                             relayable => 1,
                           },
                         );

our $entity_class = 'F5::ASMConfig::Entity::URL';

#----------------------------------------------------------------------

sub update_url_add_update_metachars {
  my($self, @rpc_params) = @_;

  my $call_flags = $rpc_params[1];
  $call_flags->{append_nested_struct_on_update} = 1;
  return $self->base_update(rpc_params => \@rpc_params,
                            entity_class => $entity_class,
                           );
}

#----------------------------------------------------------------------

sub delete_url{
    my($self, $session, $call_flags, $policy,$url) = @_;

    my $dbh = $self->{dbh};
    my $logger = $self->{logger};
  
  
    $self->update_many_scanner_vulns($session,
                                     F5::ASMConfig::CallFlags->new(set_fields_to_null => [qw/asm_url asm_parameter/]),
                                     $policy,
                                     F5::ASMConfig::ScannerVuln->new({asm_url => $url}),
                                     F5::ASMConfig::ScannerVuln->new({asm_vuln_status => F5::ASMConfig::ScannerAsmVulnStatus::SCANNER_ASM_VULN_STATUS_PENDING()}));

    my $rv = $self->base_delete(entity_class => $entity_class,
                                rpc_params => [ $session, $call_flags,$policy,$url],
                              );

    my $local_call_flags = F5::ASMConfig::CallFlags->new($call_flags);
    $local_call_flags->{search_filters} = F5::ASMConfig::Entity::Extraction->empty_extraction_filters;

    $self->delete_many_extractions( $self->{session},
                                    $local_call_flags,
                                    $policy, );

    return $rv;

}

#----------------------------------------------------------------------

sub delete_many_urls {
    my($self, $session, $call_flags, $policy, $url_criteria) = @_;

    my $url_iter = $entity_class->retrieve_all(dbh => $self->{dbh},
                                               call_flags => $call_flags,
                                               policy => $policy,
                                               search_filters => $call_flags->{search_filters},
                                               get_criteria => $url_criteria);

    while (my $url = $url_iter->get_next()) {
        $self->update_many_scanner_vulns($session,
                                         F5::ASMConfig::CallFlags->new(set_fields_to_null => [qw/asm_url asm_parameter/]),
                                         $policy,
                                         F5::ASMConfig::ScannerVuln->new({asm_url => $url}),
                                         F5::ASMConfig::ScannerVuln->new({asm_vuln_status => F5::ASMConfig::ScannerAsmVulnStatus::SCANNER_ASM_VULN_STATUS_PENDING()}));
    }

    my $rv = $self->base_delete_many(entity_class => $entity_class,
                                     rpc_params => [ $session, $call_flags,$policy, $url_criteria ],);

    my $local_call_flags = F5::ASMConfig::CallFlags->new($call_flags);
    $local_call_flags->{search_filters} = F5::ASMConfig::Entity::Extraction->empty_extraction_filters;

    $self->delete_many_extractions( $self->{session},
                                    $local_call_flags,
                                    $policy, );

    return $rv;
}

#----------------------------------------------------------------------

1;
