package F5::ASMConfig::Handler::UpdateEnforcerTask;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::TaskBase/;
use F5::ASMConfig::Entity::UpdateEnforcerTask;
use F5::ASM::COAPI;

our @handled_functions = (
                            {
                             function_name => 'add_update_enforcer_task',
                             criteria_index => 3,
                             store_index    => 4,
                            },
                            {
                              function_name => 'update_update_enforcer_task',
                              criteria_index => 3,
                              store_index => 4
                            },
                            { 
                             function_name => 'execute_update_enforcer_task',
                             exclusive_lock_file => $F5::DbUtils::LOCK_FILE,
                            },
                            {
                             function_name => 'delete_update_enforcer_task',
                            },
                            {
                             function_name => 'delete_many_update_enforcer_tasks',
                            },
                         );

our $entity_class = 'F5::ASMConfig::Entity::UpdateEnforcerTask';

#----------------------------------------------------------------------

sub add_update_enforcer_task {
  my($self, @rpc_params) = @_;
  return F5::ASMConfig::Handler::TaskBase::add_task(self => $self,
                                                    rpc_params => \@rpc_params,
                                                    entity_class => $entity_class,
                                                    task_type => 'update_enforcer',);
}

#----------------------------------------------------------------------

sub update_update_enforcer_task {
    my ($self, @rpc_params) = @_;
    return F5::ASMConfig::Handler::TaskBase::update_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                         task_type => 'update_enforcer',);
}

#----------------------------------------------------------------------

sub update_enforcer {
    my (%params) = @_;
    my $self       = $params{self};
    my $session    = $params{session};
    my $call_flags = $params{call_flags};
    my $task       = $params{task};
    

    my $set_active_flags = F5::ASMConfig::SetActiveFlags->new($call_flags->{set_active_flags});
    $set_active_flags->{do_update_core} = 1;

    $self->begin_work($self->{session});
    $self->set_active($session, $call_flags, undef, $set_active_flags);
    $self->commit($self->{session});

    $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_COMPLETED();
}


#----------------------------------------------------------------------
sub execute_update_enforcer_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::execute_task(self => $self,
                                                          rpc_params => \@rpc_params,
                                                          entity_class => $entity_class,
                                                          task_type => 'update_enforcer',
                                                          execute_func => sub { update_enforcer(@_); },
                                                          );
}

#----------------------------------------------------------------------

sub delete_update_enforcer_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                        );
}

#----------------------------------------------------------------------

sub delete_many_update_enforcer_tasks {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_many_tasks(self => $self,
                                                               rpc_params => \@rpc_params,
                                                               entity_class => $entity_class,
                                                              );
}

#----------------------------------------------------------------------

1;
