package F5::ASMConfig::Handler::UpdateThreatCampaignFileTask;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::TaskBase/;
use F5::ASMConfig::Entity::UpdateThreatCampaignFileTask;
use File::Copy;
use MIME::Base64;
use F5::Utils::Rest;
use Digest::MD5;

our @handled_functions = (
                            {
                             function_name => 'add_update_threat_campaign_file_task',
                             criteria_index => 3,
                             store_index    => 4,
                             write_access   => 1,
                            },
                            {
                              function_name => 'update_update_threat_campaign_file_task',
                              criteria_index => 3,
                              store_index => 4
                            },
                            { 
                             function_name => 'execute_update_threat_campaign_file_task',
                            },
                            {
                             function_name => 'delete_update_threat_campaign_file_task',
                            },
                            {
                             function_name => 'delete_many_update_threat_campaign_file_tasks',
                            },
                         );

our $entity_class = 'F5::ASMConfig::Entity::UpdateThreatCampaignFileTask';

#----------------------------------------------------------------------

sub add_update_threat_campaign_file_task {
  my($self, @rpc_params) = @_;
  return F5::ASMConfig::Handler::TaskBase::add_task(self => $self,
                                                    rpc_params => \@rpc_params,
                                                    entity_class => $entity_class,
                                                    task_type => 'update_threat_campaign_file',);
}

#----------------------------------------------------------------------

sub update_update_threat_campaign_file_task {
    my ($self, @rpc_params) = @_;
    return F5::ASMConfig::Handler::TaskBase::update_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                         task_type => 'update_threat_campaign_file',);
}

#----------------------------------------------------------------------

my $LIVE_UPDATE_URL = "/mgmt/tm/live-update/threat-campaigns";
my $LIVE_UPDATE_DIR = '/var/lib/hsqldb/live-update/update-files';
my $TIMEOUT = 30*60; #30 Minutes
my $CHECK_INTERVAL = 3;

sub update_threat_campaign_file {
    my (%params) = @_;
    my $self       = $params{self};
    my $session    = $params{session};
    my $call_flags = $params{call_flags};
    my $task       = $params{task};
    my $logger     = $self->{logger};

    my $remove_file = 0;
    my $filename = $task->{filename};
    eval {
        if ($filename) {
            $filename = F5::Utils::Rest::get_full_file_path(username => $task->{username}, filename => $task->{filename} );
        }
        if ($task->{file_contents}) {
            if (!$filename) {
                $task->{filename} = $task->{rest_uuid};
                $filename = F5::Utils::Rest::get_full_file_path(username => $task->{username}, filename => $task->{filename} );
                $remove_file = 1;
            }
            open (my $FH, '>', $filename) || $self->handle_error(action => 'execute',
                                                                 entity_class => $entity_class,
                                                                 thrift_obj  => $task,
                                                                 error_code  => F5::ASMConfig::ErrorCode::PERMISSION_DENIED(),
                                                                 msg         => "Could not open $filename for writing.",
                                                                 error_level  => "info", );
           my $contents = $task->{file_contents};
           if ($task->{is_base64}) {
               $contents = MIME::Base64::decode_base64($contents);
           }
           print $FH $contents;
           close ($FH);
        }

        my $installation;
        if ($filename) {
            my $update_file;
            if ($task->{filename}) {
                #Check for existance
                open (my $fh, '<', $filename) || die "File '$task->{filename}' not found\n";
                my $md5 = uc(Digest::MD5->new->addfile($fh)->hexdigest());
                close ($fh);
                my $existing_file_rv = F5::Utils::Rest::local_request(uri => "$LIVE_UPDATE_URL/update-files",
                                                                      parameters => { '$filter' => "md5 eq '$md5' or filename eq '$task->{filename}'", },
                                                                      logger => $logger,);
                if ($existing_file_rv->{totalItems}) {
                    if (($update_file) = grep { $_->{md5} eq $md5 && $_->{filename} eq $task->{filename} } @{$existing_file_rv->{items}}) {
                        $logger->debug("Found existing file with md5 $md5");
                    }
                    else {
                        die "File '$task->{filename}' already exists with a different MD5\n";
                    }
                }
            }

            if (!$update_file) {
                $logger->debug("Adding $task->{filename} to Live Update");
                #copy filename to remove the username from the file
                copy($filename, "$LIVE_UPDATE_DIR/$task->{filename}");
                $filename = "$LIVE_UPDATE_DIR/$task->{filename}";
                if (my ($login,$pass,$tomcat_uid,$tomcat_gid) = getpwnam("tomcat")) {
                    chown $tomcat_uid, $tomcat_gid, $filename;
                } 

                $update_file = F5::Utils::Rest::local_request(uri => "$LIVE_UPDATE_URL/update-files",
                                                              method => "POST",
                                                              body => { fileLocationReference => { link => $filename } },
                                                              logger => $logger,
                                                             );
            }

            my $rv = F5::Utils::Rest::local_request(uri => "$LIVE_UPDATE_URL/installations",
                                                    parameters => { '$filter' => "updateFile.isFileAvailable eq true and updateFile.id eq '$update_file->{id}'",
                                                                    '$top'   => 1,
                                                                    '$orderby' => 'lastUpdateMicros desc',
                                                                    '$select'  => 'status,id',
                                                                  },
                                                    logger => $logger);
            my $create_file = 1;
            if ($rv->{totalItems}) {
                   my ($item) = @{$rv->{items}};
                   if ($item->{status} eq 'install-complete' || $item->{status} eq 'install-in-progress') {
                       $task->{message} = "The provided file is in state: $item->{status}";
                       $create_file = 0;
                   }
                   elsif ($item->{status} ne 'previously-installed' && $item->{status} ne 'error') {
                       $create_file = 0;
                       $installation = $item;
                   }
            }
            if ($create_file) {
                $installation = F5::Utils::Rest::local_request(uri => "$LIVE_UPDATE_URL/installations",
                                                               method => "POST",
                                                               body => { updateFileReference => { link => $update_file->{selfLink} },
                                                                         status => 'install', },
                                                               logger => $logger,
                                                              );
            }
        }
        else {
            $logger->debug("Checking for update for threat-campaigns");
            my $rv = F5::Utils::Rest::local_request(uri => "$LIVE_UPDATE_URL/installations",
                                                    parameters => { '$filter' => 'updateFile.isFileAvailable eq true and updateFile.isFileManuallyUploaded eq false',
                                                                    '$top'   => 1,
                                                                    '$orderby' => 'updateFile.createDateTime desc, lastUpdateMicros desc',
                                                                    '$select'  => 'status,id',
                                                                  },
                                                    logger => $logger);
            if ($rv->{totalItems}) {
               my ($item) = @{$rv->{items}};
               if ($item->{status} eq 'previously-installed' || $item->{status} eq 'error') {
                   #Create a new one
                   $installation = F5::Utils::Rest::local_request(uri => "$LIVE_UPDATE_URL/installations",
                                                                  method => "POST",
                                                                  body => { updateFileReference => $item->{updateFileReference},
                                                                            status => 'install' },
                                                                  logger => $logger,
                                                                 );
               }
               elsif ($item->{status} !~ /install/) {
                   $installation = $item;
               }
               #Otherwise it is already installed
            }
        }

        my %FINAL_STATUS = ('error' => 1,
                            'install-complete' => 1,
                           );

        if ($installation) {
            $logger->debug("Installing update for threat-campaigns");
            if ($installation->{status} ne 'install') {
                $installation = F5::Utils::Rest::local_request(uri => "$LIVE_UPDATE_URL/installations/$installation->{id}",
                                                               method => "PATCH",
                                                               body => { status => 'install' },
                                                               logger => $logger,
                                                              );
            }
            my $expire_time = time() + $TIMEOUT;  #Give up after 30 minutes
            while (!$FINAL_STATUS{$installation->{status}}) {
                if (time() > $expire_time) {
                    die "Maximum time exceeded\n";
                }
                sleep $CHECK_INTERVAL;
                $installation = F5::Utils::Rest::local_request(uri => "$LIVE_UPDATE_URL/installations/$installation->{id}",
                                                               parameters => { '$select'  => 'status,id,errorMessage', },
                                                               logger => $logger,
                                                              );
            }
            if ($installation->{status} eq 'install-complete') {
                $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_COMPLETED();
                my $result = F5::ASMConfig::Entity::ThreatCampaignFileUpdateRes->new(dbh => $self->{dbh},
                                                                                     get_criteria => {},
                                                                                     selected_fields => ['threat_campaign_file_status_id'],
                                                                                     sort_orders => [ { field_name => 'threat_campaign_file_status_id',
                                                                                                        sort_direction => F5::ASMConfig::SortDirection::SORT_DIRECTION_DESC()}],
                                                                                     );
                $task->{threat_campaign_file_update_result} = $result;
            }
            else {
                $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_FAILURE();
                $task->{message} = $installation->{errorMessage};
            }
        }
        else {
            $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_FAILURE();
            $task->{message} ||= "The most recent Threat Campaign file is already installed.";
        }
    };
    my $err = $@;

    if ($remove_file) {
        unlink ($filename);
    }

    die $err if $err;

}

#----------------------------------------------------------------------

sub execute_update_threat_campaign_file_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::execute_task(self => $self,
                                                          rpc_params => \@rpc_params,
                                                          entity_class => $entity_class,
                                                          task_type => 'update_threat_campaign_file',
                                                          execute_func => sub { update_threat_campaign_file(@_); },
                                                          );
}

#----------------------------------------------------------------------

sub delete_update_threat_campaign_file_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                        );
}

#----------------------------------------------------------------------

sub delete_many_update_threat_campaign_file_tasks {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_many_tasks(self => $self,
                                                               rpc_params => \@rpc_params,
                                                               entity_class => $entity_class,
                                                              );
}

#----------------------------------------------------------------------

1;
