package F5::ASMConfig::Handler::ValidHostName;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::ASMConfig;

our @handled_functions = (
                           {
                             function_name => 'delete_many_valid_host_names',
                             relayable => 1,
                           },
                           {
                             function_name => 'delete_valid_host_name',
                             relayable => 1,
                           },
                         );

our $entity_class = 'F5::ASMConfig::Entity::ValidHostName';

#---------------------------------------------------------------------
sub delete_many_valid_host_names {
    my($self, $session, $call_flags, $policy, $valid_host_name_criteria) = @_;

    my $valid_host_name_iter = $entity_class->retrieve_all(dbh => $self->{dbh},
                                                  call_flags => $call_flags,
                                                  policy => $policy,
                                                  get_criteria => $valid_host_name_criteria);

    while (my $valid_host_name = $valid_host_name_iter->get_next()) {
        $self->update_many_scanner_vulns($session,
                                         F5::ASMConfig::CallFlags->new(set_fields_to_null => [qw/asm_valid_host_name/]),
                                         $policy,
                                         F5::ASMConfig::ScannerVuln->new({asm_valid_host_name => $valid_host_name}),
                                         F5::ASMConfig::ScannerVuln->new({asm_vuln_status => F5::ASMConfig::ScannerAsmVulnStatus::SCANNER_ASM_VULN_STATUS_PENDING()}));
    }



    return $self->base_delete_many(entity_class => $entity_class,
                                   rpc_params => [ $session, $call_flags,$policy, $valid_host_name_criteria],);

}

#----------------------------------------------------------------------
sub delete_valid_host_name {
    my($self, $session, $call_flags, $policy,$valid_host_name) = @_;

    my $dbh = $self->{dbh};
    my $logger = $self->{logger};


   $self->update_many_scanner_vulns($session,
                                    F5::ASMConfig::CallFlags->new(set_fields_to_null => [qw/asm_valid_host_name/]),
                                    $policy,
                                    F5::ASMConfig::ScannerVuln->new({asm_valid_host_name => $valid_host_name}),
                                    F5::ASMConfig::ScannerVuln->new({asm_vuln_status => F5::ASMConfig::ScannerAsmVulnStatus::SCANNER_ASM_VULN_STATUS_PENDING()}));

    my $rv = $self->base_delete(entity_class => $entity_class,
                                rpc_params => [ $session, $call_flags,$policy,$valid_host_name],
                              );

    return $rv;
}

#----------------------------------------------------------------------

1;
