package F5::ASMConfig::Handler::VirtualServer;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use File::Basename;
use F5::ASM::COAPI;
use F5::BigipUtils;

our @handled_functions = (
                           {
                             function_name => "search_virtual_server",
                           },
                           {
                             function_name => "update_virtual_server",
                             write_access => 1,
                             relayable => 1,
                             writes_to_mcp => 1,
                             criteria_index => 3,
                             store_index   => 4,
                           },
                         );

our $entity_class = 'F5::ASMConfig::Entity::VirtualServer';

#----------------------------------------------------------------------

sub search_virtual_server {
    my ($self, $session, $call_flags, $place_holder, $virtual_server_criteria) = @_;

    if (F5::BigipUtils::is_device_em()) {
        return $self->base_search(rpc_params => [$session, $call_flags, $place_holder, $virtual_server_criteria],
                                  entity_class => 'F5::ASMConfig::Entity::EmVirtualServer',
                                 );
    }

    $self->add_partition_search_filter(session    => $session,
                                       call_flags => $call_flags);

    return $self->base_search(rpc_params => [$session, $call_flags, $place_holder, $virtual_server_criteria],
                              entity_class => $entity_class,
                              );
}

#----------------------------------------------------------------------
#For old EM support
sub update_virtual_server {
    my ($self, $session, $call_flags, $place_holder, $virtual_server_criteria, $store_virtual_server) = @_;

    if (F5::BigipUtils::is_device_em()) {
        return { return_value => F5::ASMConfig::SearchResult->new({
                                     total_record_count => 0,
                                     max_page => 1,
                                     result_record_count => 0,
                                     start_page => 1,
                                     element_list => []}) };
    }
    my $virtual_server = F5::ASMConfig::Entity::VirtualServer->new(get_criteria => $virtual_server_criteria);
    if ($store_virtual_server->{http_classes}) {
        if (!defined($virtual_server)) {
            return $self->handle_error(action => "update",
                                   entity_class => $entity_class,
                                   thrift_obj  => $virtual_server_criteria,
                                   error_code   => F5::ASMConfig::ErrorCode::MCP_NONEXISTENT_RECORD(),
                                   error_level  => "info",
                                  );
        }
        #Need to replace this policy into an existing L7 policy on this virtual that controls ASM (if one exists)
        my $l7_policy_name;
        my $assoc_type;
        if (defined($virtual_server->{http_classes}) && @{$virtual_server->{http_classes}}) {
            my $l7_policy = shift @{$virtual_server->{http_classes}};
            $l7_policy_name = $l7_policy->{l7_policy_name};
            $assoc_type = F5::ASMConfig::Entity::L7Policy->new(dbh => $self->{dbh}, get_criteria => { l7_policy_name => $l7_policy_name })->{type};
        }
        for my $policy_name (map { $_->{http_class_name} } @{$store_virtual_server->{http_classes}}) {
            if ($policy_name !~ m|^/|) {
                $policy_name = "/Common/$policy_name";
            }
            if (my $policy = F5::ASMConfig::Entity::Policy->new(dbh => $self->{dbh}, get_criteria => { policy_name => $policy_name })) {
                my $assoc = F5::ASMConfig::VirtualServerAssoc->new(virtual_server => $virtual_server,
                                                                   l7_policy_name => $l7_policy_name,
                                                                   assoc_type     => $assoc_type);

                if ( $assoc_type == F5::ASMConfig::VirtualServerAssocType::VIRTUAL_SERVER_ASSOC_TYPE_L7_SIMPLE ) {
                    $policy->{extra_virtual_server_assocs} = [ $assoc ];
                } else {
                    $policy->{extra_virtual_server_assocs_complex} = [ $assoc ];
                }

                $self->update_policy($session, $call_flags, $policy);
            }
        }
    }
    
    return {return_value => $virtual_server};
}

#----------------------------------------------------------------------

1;
