package F5::ASMConfig::Handler::WebsocketUrl;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::ASMConfig::ASMConfig;

our @handled_functions = (
                           {
                             function_name => 'add_websocket_url',
                             relayable     => 1,
                             store_index   => 3,
                           },
                         );

our $entity_class = 'F5::ASMConfig::Entity::WebsocketUrl';

#----------------------------------------------------------------------

sub add_websocket_url {
    my($self, $session, $call_flags, $policy, $websocket_url) = @_;

    my @plain_text_events;
    if ($session->{user_name} eq $F5::ASMConfig::Entity::Base::POLICY_BUILDER_USER_NAME &&
        $entity_class->is_pure_wildcard(store_obj => $websocket_url) ) {
        $websocket_url->{flg_check_payload} = 1;
        $websocket_url->{flg_websocket_message_format_text} = 1;

        my $default_plain_text_profile = F5::ASMConfig::Entity::WebsocketUrl->get_missing_default_text_profile(
                                                          dbh => $self->{dbh},
                                                          policy => $policy,
                                                          store_obj => $websocket_url);

        if ($default_plain_text_profile && $default_plain_text_profile->{__new_entity}) {
            my $text_profile_rv = $self->base_add( entity_class => 'F5::ASMConfig::Entity::PlainTextProfile',
                                              rpc_params => [ $session, $call_flags, $policy, $default_plain_text_profile ],);
            if ($text_profile_rv && $text_profile_rv->{events}) {
                push (@plain_text_events,@{$text_profile_rv->{events}});
            }
        }
    }

    my $return_value = $self->base_add(rpc_params => [$session, $call_flags, $policy, $websocket_url],
                                       entity_class => $entity_class, );
    push (@{$return_value->{events}},@plain_text_events);

    return $return_value;
}

#----------------------------------------------------------------------


1;
