package F5::ASMConfig::Handler::XmlValidationFile;

use strict;
use warnings;

use F5::ASMConfig::Entity::XmlValidationFile;
use F5::ASMConfig::Handler::TaskBase;
use F5::Utils::Rest;
use MIME::Base64;
use File::Basename;

our @handled_functions = (
                            {
                             function_name => 'add_xml_validation_file',
                             relayable => 1,
                             criteria_index => 3,
                             store_index    => 4,
                            },
                            {
                              function_name => 'update_xml_validation_file',
                              criteria_index => 3,
                              store_index => 4
                            },
                         );

our $entity_class = 'F5::ASMConfig::Entity::XmlValidationFile';

#----------------------------------------------------------------------

sub add_xml_validation_file {
    my ($self, $session, $call_flags, $policy, $xml_validation_file) = @_;

    $xml_validation_file->{file_name} = basename($xml_validation_file->{file_name} || "");

    if (!$xml_validation_file->{file_contents}) {
        my $search_file_name = F5::Utils::Rest::get_full_file_path(username => $session->{gui_user_name}, filename => $xml_validation_file->{file_name} );

        open (my $FH, '<', $search_file_name) || $self->handle_error(action => 'add',
                                                                     entity_class => $entity_class,
                                                                     thrift_obj  => $xml_validation_file,
                                                                     error_code  => F5::ASMConfig::ErrorCode::PERMISSION_DENIED(),
                                                                     msg         => "Could not find $xml_validation_file->{file_name}.",
                                                                     error_level  => "info", );
        local $/;
        $xml_validation_file->{file_contents} = <$FH>;
    }

    if ($xml_validation_file->{flg_is_base64}) {
        $xml_validation_file->{file_contents} = MIME::Base64::decode_base64($xml_validation_file->{file_contents});
    }

    $call_flags->{return_existing_on_duplicate_add} = 1;

    return $self->base_add(rpc_params => [$session, $call_flags, $policy, $xml_validation_file],
                           entity_class => $entity_class,
                           );
}

#----------------------------------------------------------------------

sub update_xml_validation_file {
    my ($self, $session, $call_flags, $policy, $xml_validation_file) = @_;
    return $self->handle_error(action => 'update',
                               error_code => F5::ASMConfig::ErrorCode::ACTION_NOT_ALLOWED(),
                               entity_class => $entity_class,
                               thrift_obj => $xml_validation_file,
                               error_thrown => "Cannot update uploaded xml validation files",
                               error_level => "info",
                              );
}

#----------------------------------------------------------------------

1;
