package F5::ASMConfig::LiveUpdateBase;

use strict;
use warnings;

use F5::Utils::Rest;
use base qw/F5::ASMConfig::Entity::Base/;

my $BASE_REST_URL = '/mgmt/tm/live-update';

#----------------------------------------------------------------------

sub retrieve_all {
    my($class, %params) = @_;

    my $logger = $params{logger};
    my $live_update_type = $class->live_update_type;
    my $live_update_rest_path = $class->live_update_rest_path;
    my $uri = "$BASE_REST_URL/$live_update_type/$live_update_rest_path";
    my $rv = F5::Utils::Rest::local_request(uri => $uri, logger => $logger);
    for my $field (@{$class->live_update_fields}) {
        my $live_update_field_name = $class->fields->{$field}->{live_update_field_name};
        if (defined($rv->{$live_update_field_name})) {
            $rv->{$field} = $rv->{$live_update_field_name};
        }
    }
    my @objs = ($class->new(dbh => $params{dbh}, object => $rv));
    my $total_record_count = scalar @objs;

    if (wantarray) {
        return @objs;
    }
    else {
        my $iterator = F5::ASMConfig::Entity::Iterator->new(
                                                            get_next_function => sub {return shift(@objs);},
                                                            get_all_function => sub {return @objs;},
                                                            total_record_count => $total_record_count,
                                                           );
        return $iterator;
    }
}

#----------------------------------------------------------------------

sub store_update {
    my($self, %params) = @_;

    my $db_change_made;
    my $body;
    my $fields = $self->fields;
    for my $live_update_field (@{$self->live_update_fields(%params)}) {
        if ($self->has_value_changed(%params, existing_obj => $self, check_fields => [$live_update_field])) {
            $body->{$fields->{$live_update_field}->{live_update_field_name}} = $params{store_obj}->{$live_update_field};
        }
    }
    if ($body) {
        my $live_update_rest_path = $self->live_update_rest_path;
        for my $live_update_type (@{$self->live_update_write_types}) {
             my $uri = "$BASE_REST_URL/$live_update_type/$live_update_rest_path";
             F5::Utils::Rest::local_request(uri => $uri, 
                                            method => 'PATCH', 
                                            body   => $body,
                                            logger => $self->{logger},);
        }
        $self->retrieve(get_criteria => $self);
        $db_change_made = 1;
    }

    my $retval = { db_change_made => $db_change_made,
                   modified_entities => [],
                 };

    return $retval;
}

#----------------------------------------------------------------------

1;
