package F5::ASMConfig::McpBase;

use strict;
use warnings;

use F5::ASM::COAPI;
use base qw/F5::ASMConfig::Entity::Base/;

sub retrieve_all {
    my ($class, %params) = @_;
    my $logger = $params{logger};
    my $get_criteria = $params{get_criteria} || {};
    my $extra_fields = $params{extra_fields};
    my $find_single_record = (defined($params{find_single_record}) ? $params{find_single_record} : 0);
    my $minimal_load = $params{minimal_load};

    my $fields = $class->fields;

    my $query;
    if ($class->mcp_id_field && $params{get_criteria}->{$class->mcp_id_field}) {
        $query = F5::ASM::COAPI::Query->new($class->mcp_element_type, { $class->mcp_id_field_name => $get_criteria->{$class->mcp_id_field}}, do_cache => 0 );
    }
    else {
        my @mcp_unique_record_parts = @{$class->mcp_unique_record_parts(get_criteria => $get_criteria)};
        my $criteria;
        if (($find_single_record || scalar @mcp_unique_record_parts) &&
             scalar @mcp_unique_record_parts == scalar grep { defined $get_criteria->{$_} } @mcp_unique_record_parts) {
            $criteria = {map { $fields->{$_}->{mcp_field_name} => $get_criteria->{$_} } @mcp_unique_record_parts };
        }
        else {
            for my $mcp_field (@{$class->mcp_fields}) {
                if (exists $get_criteria->{$mcp_field}) {
                    $criteria->{$fields->{$mcp_field}->{mcp_field_name}} = $get_criteria->{$mcp_field};
                }
            }
        }
        $query = F5::ASM::COAPI::Query->new($class->mcp_element_type, $criteria, do_cache => 0); 
    }  

    my @entities = map { $class->new(mcp_object => $_, logger => $logger, dbh => $params{dbh})} @$query;

    for my $entity (@entities) {
        #Externals
        if (!$minimal_load) {
            for my $nested_field (grep { $fields->{$_}->{mcp_nested_struct} } keys %$fields) {
                #Nested
                my $nested_field_def = $fields->{$nested_field};
                my $parent_field_name = $nested_field_def->{parent_mcp_entity_field_name} || $entity->mcp_id_field;
                my @nested_objs = @{F5::ASM::COAPI::Query->new($nested_field_def->{mcp_join_table},
                                                               { $nested_field_def->{parent_mcp_field_name} => $entity->{$parent_field_name} })};
                if ($nested_field_def->{entity_type} && $nested_field_def->{mcp_external_struct}) {
                    my $foreign_field_name = $nested_field_def->{foreign_mcp_entity_field_name} || $nested_field_def->{entity_type}->mcp_id_field;
                    @nested_objs = map { $nested_field_def->{entity_type}->new(dbh => $params{dbh}, 
                                                                               get_criteria => { $foreign_field_name => $_->{$nested_field_def->{foreign_mcp_field_name}} }) } @nested_objs;
                }
                $entity->{$nested_field} = \@nested_objs;
            }
        }
        $entity->map_fields_to_entity(%params, rec => $entity);
    }
 
    $class->apply_search_filters(%params, result_set => \@entities);

    $class->apply_sort_order(%params, result_set => \@entities);

    my $total_record_count = scalar @entities;
    $class->apply_paging(%params, result_set => \@entities);

    if (wantarray) {
        return @entities;
    }
    else {
        my $iterator = F5::ASMConfig::Entity::Iterator->new(
                                                            get_next_function => sub {return shift(@entities);},
                                                            get_all_function => sub {return @entities;},
                                                            total_record_count => $total_record_count,
                                                           );
        return $iterator;
    }

}

#----------------------------------------------------------------#

sub is_mcp_object {
    return 1;
}
1;
