package F5::ASMConfig::Utils;

use strict;
use warnings;

use F5::Deploy;
use F5::ASMConfig::ASMConfig;
use YAML::XS;
use Memoize;
memoize('get_event_type_strings');
memoize('get_entity_schema');
memoize('entity_by_table_map');

my $LIB_PATH = F5::Deploy::map_path('/usr/share/ts/share/asm_config/yaml');
my $ENUM_VALUES_PATH = "/enums";
my $ELEMENT_VALUES_PATH = "/elements";

#----------------------------------------------------------------------

sub get_event_type_strings {
  my %event_types = (
                     F5::ASMConfig::EventType::ADD() => 'add',
                     F5::ASMConfig::EventType::DELETE() => 'delete',
                     F5::ASMConfig::EventType::UPDATE() => 'update',
                     F5::ASMConfig::EventType::ALL_TYPES() => 'all types',
                     F5::ASMConfig::EventType::CATCHUP_COMPLETE() => 'catchup complete',
                     F5::ASMConfig::EventType::ERROR() => 'error',
                     F5::ASMConfig::EventType::INFO() => 'info',
                     F5::ASMConfig::EventType::CATCHUP_START() => 'catchup start',
                     F5::ASMConfig::EventType::CATCHUP_ADD() => 'catchup add',
                     F5::ASMConfig::EventType::ASYNC_RESULT() => 'async result',
                     F5::ASMConfig::EventType::ASYNC_NO_RESULT() => 'async no result',
                    );
  return %event_types;
}

#----------------------------------------------------------------------

sub get_entity_schema {
    my (%params) = @_;
    my $base_dir = $params{base_dir} || '';
    my $lib_path = $params{lib_path} || $LIB_PATH;
    my $extra_paths = $params{extra_paths};
    my @enum_files = glob "$base_dir$lib_path$ENUM_VALUES_PATH/*";
    my @element_files = glob "$base_dir$lib_path$ELEMENT_VALUES_PATH/*";
    my @extra_files;
    if ($extra_paths) {
        @extra_files = map { glob $_ } @$extra_paths;
    }

    my $yaml;
    {
        local $/;
        for my $file (@enum_files, @element_files, @extra_files) {
            open (my $fh, "<", $file);
            $yaml .= <$fh>;
        }
    }

    my $schema;
    eval {
        $schema = YAML::XS::Load($yaml);
    };
    #For debugging failures
    if (my $err = $@) {
       if ($err =~ /line: ([0-9]+)/) {
           my $line_number = $1;
           my @lines = (split /\n/, $yaml);
           my $l = "$lines[$line_number - 2]\n$lines[$line_number - 1]\n$lines[$line_number]\n";
           #die "$err Line: $lines[$line_number - 1]\n";
           die "$err Line: $l";
       }
       die $err;
    }
    return $schema;
}

#----------------------------------------------------------------------

sub entity_by_table_map {
    my ($class, %params) = @_;
    my $schema = get_entity_schema(%params);
    my %entity_by_table_map;
    for my $entity (keys %$schema) {
        if ( exists $schema->{$entity}->{db_table} ) {
            push(@{$entity_by_table_map{$schema->{$entity}->{db_table}}} , $schema->{$entity}->{entity_type});
        }
    }
    return %entity_by_table_map;
}

#----------------------------------------------------------------------

1;
