package F5::Acc::Entity::Column::bitmask;

use strict;
use warnings;
use Carp;
use F5::Acc::Entity::Util;
use base 'F5::Acc::Entity::Column::int';

sub new {
    my $class = shift;
    local $_;
    my ($entity_class, $name, $descriptor) = @_;
    my $self = $class->SUPER::new(@_);
    $self->{maskmap}     = delete $descriptor->{maskmap};
    $self->{prefix}      = delete $descriptor->{prefix};
    $self->{inv_maskmap} = map { $self->{maskmap} => $_ }
        keys %{ $self->{maskmap} };
    return $self;
}

sub make_entity_column {
    my $self = shift;
    $self->SUPER::make_entity_column;
    my $entity_class = $self->entity_class;
    my %maskmap = %{ $self->maskmap };
    my $prefix = $self->prefix;
    for my $key (keys %maskmap) {
        install_sub(
            $entity_class,
            $self->flag_method_name($key),
            sub { $self->get_set_flag($maskmap{$key}, @_) },
        );
    }
    install_sub($entity_class, insert => sub { $self->insert(@_) });
}

# compute the column value by examining all bitmask related flags
sub insert {
    my ($self, $entity_class, $params) = @_;
    my %flags                 = ();
    my %maskmap               = %{ $self->maskmap };
    my $flags_exist_in_params = 0;
    my $column_value          = 0;

    for my $flag_name (keys %maskmap) {
        my $flag_method_name = $self->flag_method_name($flag_name);
        next unless exists $params->{$flag_method_name};
        $flags_exist_in_params = 1;

        my $flag_value = delete $params->{$flag_method_name};
        next unless $flag_value; # user unset a flag, don't do nothin'

        $flags{$flag_name} = 1;
    }
    $column_value |= $maskmap{$_} for keys %flags;

    $params->{ $self->public_name } = $column_value
        if $flags_exist_in_params;
}

sub get_set_flag {
    my ($self, $maskmap_value, $entity, $flag_value) = @_;
    my $name = $self->public_name;
    my $old_maskmap_value = $entity->$name;

    return $old_maskmap_value & $maskmap_value if @_ == 3;

    my $new_maskmap_value = $flag_value?
        $old_maskmap_value | $maskmap_value:
        $old_maskmap_value & ~$maskmap_value;
    $entity->$name($new_maskmap_value);
}

sub _validate {
    my ($self, $value) = @_;
    $self->SUPER::_validate($value);
}

sub maskmap     { shift->{maskmap} }
sub inv_maskmap { shift->{inv_maskmap} }
sub prefix      { shift->{prefix} }

sub get_col_methods {
    my $self   = shift;
    my $prefix = $self->prefix;
    return (
        $self->SUPER::get_col_methods,
        map { $self->flag_method_name($_) } keys %{ $self->maskmap },
    );
}

sub flag_method_name {
    my ($self, $flag_name) = @_;
    return $self->prefix. "_${flag_name}_flag";
}

1;


