package F5::Acc::Entity::Column::enum;

use strict;
use warnings;
use Carp;
use List::Util qw(first);
use base 'F5::Acc::Entity::Column::str';

sub new {
    my $class = shift;
    my ($entity_class, $name, $descriptor) = @_;
    my $self = $class->SUPER::new(@_);
    $self->{any_of} = delete $descriptor->{any_of};
    croak "can't create enum with no any_of key"     unless $self->any_of;
    croak "can't create enum with non array-ref key" unless ref $self->any_of eq 'ARRAY';
    croak "can't create enum with empty any_of key"  unless @{$self->any_of};
    return $self;
}

sub _get_default { shift->{any_of}->[0] }

sub _validate {
    my ($self, $value) = @_;
    my @any_of = @{ $self->any_of };
    $self->invalid($value, "not in [". join(", ", @any_of). "]")
        unless first { $value eq $_ } @any_of;
}

sub any_of { shift->{any_of} }


1;

