package F5::Acc::Entity::Column::hash;

use strict;
use warnings;
use Carp;
use F5::Acc::Entity::Util;
use base 'F5::Acc::Entity::Column::list';

sub new {
    my $class = shift;
    my ($entity_class, $name, $descriptor) = @_;
    my $self = $class->SUPER::new(@_);
    $self->{key_column_name} = delete $descriptor->{index};
    return $self;
}

sub get_from_entity {
    my ($self, $entity) = @_;
    local $_;
    my $column_name       = $self->public_name;
    my $value_column_name = $self->value_column_name;
    my $key_column_name   = $self->key_column_name;
    my %hash = map { $_->$key_column_name => $_->$value_column_name }
        $entity->$column_name;
    return wantarray? %hash: \%hash;
}

sub set_on_entity {
    my $self              = shift;
    my $entity            = shift;
    my %hash              = ref $_[0]? %{ $_[0] }: @_;
    my $column_name       = $self->public_name;
    my $value_column_name = $self->value_column_name;
    my $key_column_name   = $self->key_column_name;
    my $add_to_sub_name   = "add_to_$column_name";
    $entity->$column_name->delete_all;
    for my $key (keys %hash) {
        $entity->$add_to_sub_name({
            # TODO: should inherit from list, see comment there
            policy_id          => $entity->policy_id,
            $key_column_name   => $key,
            $value_column_name => $hash{$key},
        });
    }
}

sub key_column_name { shift->{key_column_name} }

sub clone_deep {
    my ($self, $entity) = @_;
    croak "Shallow cloning not supported on hash columns"
        if $self->shallow_clone;
    my $col_name = $self->public_name;
    my $access   = "${col_name}_list";
    my $original = $entity->$access;
    my %new_hash = %$original;
    return ("${col_name}_list", {%new_hash});
}

1;

