package F5::Acc::Entity::Column::list;

use strict;
use warnings;
use Carp;
use F5::Acc::Entity::Util;
use base 'F5::Acc::Entity::Column::has_many';

sub new {
    my $class = shift;
    my ($entity_class, $name, $descriptor) = @_;
    # TODO: enforce existing has_many relationship, and existence of value column
    my $self = $class->SUPER::new(@_);
#    $self->{alias} = $self->{name}. "_list";
    $self->{value_column_name} = delete $descriptor->{value};
    return $self;
}

sub make_entity_column {
    my $self = shift;
    $self->SUPER::make_entity_column;
    install_sub(
        $self->entity_class,
        $self->name. '_list',
        sub { $self->get_set_from_entity(@_) },
    );
}

sub is_cdbi { 0 }

sub get_set_from_entity {
    my ($self, $entity, @other) = @_;
    if (@_ == 2)
        { $self->get_from_entity($entity) }
    else
        { $self->set_on_entity($entity, @other) }
}

sub get_from_entity {
    my ($self, $entity) = @_;
    local $_;
    my $column_name       = $self->public_name;
    my $value_column_name = $self->value_column_name;
    my @values = sort map { $_->$value_column_name } $entity->$column_name;
    return wantarray? @values: \@values;

}

sub set_on_entity {
    my $self              = shift;
    my $entity            = shift;
    my @values            = ref $_[0]? @{ $_[0] }: @_;
    my $column_name       = $self->public_name;
    my $value_column_name = $self->value_column_name;
    my $add_to_sub_name   = "add_to_$column_name";
    $entity->$column_name->delete_all;
    for my $value (@values) {
        $entity->$add_to_sub_name({
            # TODO: should use the entities specific add_to
            # which for all policy entities, adds policy id
            # having this here, is simply pure evil
            policy_id          => $entity->policy_id,
            $value_column_name => $value,
        });
    }
}

# no validation for this column type
sub _validate {}

sub value_column_name { shift->{value_column_name} }

sub get_col_methods {
    my $self = shift;
    my $name = $self->name;
    return ($self->SUPER::get_col_methods, "${name}_list");
}

sub can_insert { 0 }

sub clone_deep {
    my ($self, $entity) = @_;
    croak "Shallow cloning not supported on list columns"
        if $self->shallow_clone;
    my $col_name = $self->public_name;
    my $access   = "${col_name}_list";
    my $original = $entity->$access;
    my @new_list = @$original;
    return ("${col_name}_list", [@new_list]);
}

1;

