package F5::Acc::Entity::Column::wrapped;

use strict;
use warnings;
use Carp;
use F5::Acc::Entity::Util;
use base 'F5::Acc::Entity::Column::has_a';

sub make_entity_column {
    my $self = shift;
    local $_;
    $self->SUPER::make_entity_column;
    my $entity_class = $self->entity_class;

    # can't do this setup in new, because infinite loop will ensue:
    # Schema::new is the method that calls the column constructor
    my %wrapper_cols = %{ $self->entity_class->schema->cols_hash };
    my %wrapped_cols = %{ $self->to          ->schema->cols_hash };
    my @wrapped_cols =
        grep { !$wrapper_cols{$_} } # e.g ignore id and other common cols
        keys %wrapped_cols;
    $self->{wrapped_methods} = [
        map { $wrapped_cols{$_}->get_col_methods }
        @wrapped_cols
    ];
    $self->{wrapped_public_col_names} = [
        map { $wrapped_cols{$_}->public_name }
        @wrapped_cols
    ];

    # install triggers
    for my $sub (qw(after_delete before_update insert)) {
        install_sub(
            $entity_class,
            $sub,
            sub { $self->$sub(@_) },
         );
    }

    # install delegate accessors/modifiers
    for my $method (@{ $self->wrapped_methods }) {
        install_sub(
            $entity_class,
            $method,
            sub { $self->delegate_method($method, @_) },
         );
    }
}

sub delegate_method {
    my ($self, $method_name, $entity, @params) = @_;
    my $name = $self->public_name;
    return $entity->$name->$method_name(@params);
}

sub after_delete {
    my ($self, $entity) = @_;
    my $name = $self->name;
    $entity->$name->delete;
}

sub before_update {
    my ($self, $entity) = @_;
    my $name = $self->name;
    $entity->$name->update;
}

sub insert {
    my ($self, $entity_class, $params) = @_;

    # extract the wrapped method values from the wrapper insert params
    my $wrapped_method_values = {};
    for my $method (@{ $self->wrapped_methods }) {
        $wrapped_method_values->{$method} = delete $params->{$method}
            if exists $params->{$method};
    }
    # TODO: HACK HACK HACK
    # all inserts should be done through add_to_X, and add_to_X should
    # add policy_id
    $wrapped_method_values->{policy_id} = $params->{policy_id};

    $params->{ $self->public_name } =
        $self->to->insert($wrapped_method_values);
}

sub wrapped_methods { shift->{wrapped_methods} }
sub wrapped_public_col_names { shift->{wrapped_public_col_names} }

# wrapped objects do not clone, becaue their wrapper object clones them
# on insert().
# instead, we pass to the clone constructor the properties of the original.
# since insert on the wrapper inserts a wrapped, the original wrapped
# gets cloned.
sub clone_deep {
    my ($self, $entity) = @_;
    my %wrapped_cols;
    my $col_name = $self->public_name;
    my $orig_wrapped = $entity->$col_name;
    my @insert_params;

    for my $col_name (@{ $self->wrapped_public_col_names }) {
        my $orig_wrapped_col =
            $orig_wrapped->schema->get_column_by_public_name( $col_name );
        my @col_keys_and_values = $orig_wrapped_col->
            clone_deep( $orig_wrapped );
        push @insert_params, @col_keys_and_values;
    }
    return @insert_params;
}

1;

=head1 TODO

 1) enforce only one wrapped in wrapper

=cut

