package F5::Acc::Entity::Util;

use strict;
use warnings;
use Carp;
use base 'Exporter';

our @EXPORT = qw(
    compute_isa_list merge_hash_values
    install_sub install_after_sub
);

# same as _merge_hash_refs (see below) but hashes are given as values
# inside 2 other hashes, and a key used to extract these values
sub merge_hash_values ($$$) {
    my ($key, $primary, $secondary) = @_;
    my ($value_1, $value_2) = ($primary->{$key}, $secondary->{$key});
    return _merge_hash_refs($value_1, $value_2);
}

# given 2 hash refs (or undef instead of a hash ref) will merge them
# into a single hashref, priority given to 1st hash ref given
sub _merge_hash_refs {
    my ($primary, $secondary) = @_;
    return
        defined $primary?
            defined $secondary? {%$secondary, %$primary}: $primary:
            defined $secondary? $secondary              : {};
}

# return list of all superclasses for a class
# multiple inheritance not supported, so always put the Entity class
# 1st in the ISA array
sub compute_isa_list ($) {
    my $thing = shift;
    my $class = ref($thing) || $thing;
    return ($class, _isa_list_of($class));
}

# TODO: only single inheritance supported, should use Class::ISA
sub _isa_list_of {
    my $start_class = shift;
    no strict;
    my $base = ${"$start_class\::ISA"}[0];
    my $package = __PACKAGE__;
    return () if !$base || $base =~ /$package/;
    return ($base, _isa_list_of($base));
}

# install some code on a sub in a package
sub install_sub {
    my ($package_name, $sub_name, $code) = @_;
    my $full_name = "${package_name}::$sub_name";
    croak "can't install [$full_name], because no code given"
        unless ref $code eq 'CODE';
    my $original = $package_name->can($sub_name);
    no strict 'refs';
    no warnings;
    *{$full_name} = $original?
        sub {
            # make what is possibly a list into an array, so that
            # $code can always change it for the benefit of $original
            my @args = @_;

            $code->(@args);
            $original->(@args);
        }:
        $code;
}

sub install_after_sub {
    my ($package_name, $sub_name, $code) = @_;
    my $full_name = "${package_name}::$sub_name";
    croak "can't install [$full_name], because no code given"
        unless ref $code eq 'CODE';
    my $original = $package_name->can($sub_name);
    no strict 'refs';
    no warnings;
    *{$full_name} = $original?
        sub {
            my $return_value = $original->(@_);
            return $code->(@_, $return_value);
        }:
        $code;
}

