package F5::Acc::iControlDispatch;

use strict;
use warnings;


use F5::Provisioning;
use F5::Acc::Exception;

our ($ErrorCode, $ErrorString);

# retry on certain errors - flag to avoid infinite loop
our $inside_retry = 0;

sub new {
    my $class = shift;
    bless {}, $class;
}

sub dispatch {
    my ($self, $package, $method) = (shift, shift, shift);
    my @args = @_;
    $ErrorCode = 0;
    $ErrorString = '';
    my $return_value;

    eval {
        if ( not F5::Provisioning::is_asm_on() ) {
            die F5::Acc::Exception->throw( 'ASMNotEnabled' );
        }

        if ( not $self->{$package} ) {
            eval "require $package";
            if ( $@ ) {
                die F5::Acc::Exception->throw( 'UnableToLoadPackage', $package );
            }
            no strict 'refs';

	    if ($package eq 'F5::EmServices') {
		$self->{$package} = $package->new(second_try => $inside_retry);
            } else {
		$self->{$package} = $package->new();
            }
        }

	$return_value = $self->{$package}->$method( @args );
    };

    if ( $@ ) {

        my $do_retry = 0;

        if ( ref $@ ) {
            $ErrorCode = $@->{code};
            $ErrorString = $@->{string};
            $do_retry = 1 if $@->{key} eq 'DatabaseError';
        }
        else {
            $ErrorCode = 2;
            $ErrorString = $@;
            $do_retry = 1;
        }

        if ( $do_retry and not $inside_retry ) {
            local $inside_retry = 1;
            # re-instantiate the package's object
            delete $self->{$package};
            return $self->dispatch( $package, $method, @args );
        }

        return;
    }

    return $return_value;
}

1;
