package F5::ActionItem::ScannerVulns;

use strict;
use warnings;
use base qw(F5::ActionItem::Base);
use F5::ASMConfig::ASMConfig;
use F5::ASMConfig::Entity::PolicyActionItem;
use F5::ASMConfig::Entity::ScannerVuln;


our $elements = [ F5::ASMConfig::ElementType::SCANNER_ACTIONS,
                  F5::ASMConfig::ElementType::SCANNER_VULN];

our $attributes = { subscriber => 1,
                    policy_action_item => 1,
                    throttle_events => 1,
                    type => F5::ASMConfig::ActionItemType::SCANNER_VULNS,
                    name => "scanner_vulns",
                    severity => F5::ASMConfig::ActionItemSeverity::MEDIUM,
                  };

#----------------------------------------------------------------------

sub check_condition {
    my ($class, %params) = @_;
    return unless $params{policy}->{flg_encoding_selected};
    my $vulns = F5::ASMConfig::Entity::ScannerVuln->retrieve_all(dbh => $params{dbh},
                                                                 logger => $params{logger},
                                                                 page_size => 1,
                                                                 policy => $params{policy},
                                                                 search_filters => [ F5::ASMConfig::SearchFilter->new(
                                                                                         field_name => 'flg_is_resolvable',
                                                                                         field_value => F5::ASMConfig::ScannerVulnType::SCANNER_VULN_TYPE_NOT_RESOLVABLE(),
                                                                                         field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_NE()),
                                                                                   ],
                                                                 get_criteria => { asm_vuln_status => F5::ASMConfig::ScannerAsmVulnStatus::SCANNER_ASM_VULN_STATUS_PENDING,
                                                                                   scanner_vuln_status => F5::ASMConfig::ScannerVulnStatus::SCANNER_VULN_STATUS_OPEN
                                                                                 },
                                                                );

    return $vulns->total_record_count();
}

#----------------------------------------------------------------------

1;
