package F5::ActionItem::SignatureStaging;

use strict;
use warnings;
use base qw(F5::ActionItem::Base);
use F5::ASMConfig::ASMConfig;
use F5::DateTimeUtils;

our $elements = [ F5::ASMConfig::ElementType::POLICY_SIGNATURE, 
                  F5::ASMConfig::ElementType::POLICY_NEGSIG_ATTRIBUTES,
                  F5::ASMConfig::ElementType::POLICY_ATTRIBUTES,
                  F5::ASMConfig::ElementType::PBCONFIG,
                  F5::ASMConfig::ElementType::SET_ACTIVE,
                ];

our $attributes = { subscriber => 1,
                    pb_dependant => 1,
                    policy_action_item => 1,
                    throttle_events => 1,
                    type => F5::ASMConfig::ActionItemType::SIGNATURE_STAGING_PAST_PERIOD,
                    name => "signature_staging",
                    severity => F5::ASMConfig::ActionItemSeverity::MEDIUM,
                    staging_class => "F5::ActionItem::SignatureStaging",
                  };



#----------------------------------------------------------------------

sub check_condition {
    my ($class, %params) = @_;
    return $class->check_condition_staging(%params);
}

#----------------------------------------------------------------------

sub get_next_staging_or_tightening_past_period_time {
    my ($class, %params) = @_;
    my $client = $params{client} || $params{action_item_handler}->{client};
    my $call_flags = F5::ASMConfig::CallFlags->new(page_size => 1,
                                                   search_filters => [F5::ASMConfig::SearchFilter->new(field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_EQ(),
                                                                                                       field_name => "extra_has_suggestions",
                                                                                                       "field_value" => 0) ],
                                                   sort_orders => [F5::ASMConfig::SortOrder->new(field_name => "quarantined_since",
                                                                                                 sort_direction => F5::ASMConfig::SortDirection::SORT_DIRECTION_ASC())]);

    my $search_result = $client->search_policy_signature($client->{session}, $call_flags, $params{policy}, F5::ASMConfig::PolicySignature->new());
    if ($search_result->{total_record_count}) {
        my $signature = $search_result->{element_list}->[0]->{policy_signature};
        return F5::DateTimeUtils::convert_datetime_to_epoch($signature->{quarantined_since});
    }
    return;
}

#----------------------------------------------------------------------
1;
