package F5::ActionItem::SyncDeviceGroup;

use strict;
use warnings;
use base qw(F5::ActionItem::Base);
use F5::ASMConfig::ASMConfig;


our $elements = [ F5::ASMConfig::ElementType::DEVICE_GROUP,
                ];

our $attributes = { 
                    subscriber => 1,
                    type => F5::ASMConfig::ActionItemType::MISSING_SYNC_DEVICE_GROUP,
                    name => "sync_device_group",
                    severity => F5::ASMConfig::ActionItemSeverity::HIGH,
                  };

#----------------------------------------------------------------------

my $DATASYNC_DG = '/Common/datasync-global-dg';

sub check_condition {
    my ($self, %params) = @_;
    if ($params{event} && $params{event}->{element} &&
        $params{event}->{element}->{element_type} == F5::ASMConfig::ElementType::DEVICE_GROUP &&
        $params{event}->{element}->{device_group}->{name}) {
        #The DeviceGroup event always has the name of the asm device group we are a member of.
        #So if there is a name, then we are a member of a DG, and the condition is false
        return;
    }
    my $condition;
    my $client = $params{client} || $params{action_item_handler}->{client};
    if ($client) {
        my $search_device_groups = $client->search_device_group($client->{session});    
        if ($search_device_groups) {
            my @device_groups = grep { $_->{name} ne $DATASYNC_DG } map { $_->{device_group} } @{$search_device_groups->{element_list}};
            my @asm_device_groups = grep {$_->{asm_sync}} @device_groups;
            $condition = @device_groups && ! @asm_device_groups; #In violation if this device is in a device group, but not in any asm enabled ones.
        }
    }
    return $condition;
}

#----------------------------------------------------------------------
1;
