package F5::AlternateResponsesFixup;

use strict;
use warnings;
my $CURRENT_CR_REP = chr(3);
my $CURRENT_LF_REP = chr(4);

my %alt_types = ( 1 => 'default',
                  2 => 'redirect',
                  3 => 'custom',
);
my @causes = ('default', 'persistent flow');
my @plc_fields = qw( policy_id cause response_type alternate_response_header alternate_response_content redirect_url);

sub set_alternate_response {
    my ($dbh, $logger, $new_dbname, $alternate_response) = @_;
    local $" = ',';
    my $alt_response_row;
    if ( $alternate_response->{alternate_response_file_name} ){
        my ($type, $filename) = split /~/, $alternate_response->{alternate_response_file_name};
        $alt_response_row->{response_type} = $alt_types{$type};
    }

    if ($alt_response_row->{response_type} eq 'default'){
        my $sql = "SELECT alternate_response_header, alternate_response_content FROM
                    $new_dbname.PL_ALTERNATE_RESPONSE_DEFAULTS WHERE 
                    response_type = 'default' AND license_type = 'ASM_ENABLED' AND cause = 'default'";
        ($alt_response_row->{alternate_response_header},
        $alt_response_row->{alternate_response_content} ) = $dbh->selectrow_array($sql);
    }
    else {
        ($alt_response_row->{alternate_response_header},
        $alt_response_row->{alternate_response_content} ) = 
            split /${CURRENT_CR_REP}${CURRENT_LF_REP}${CURRENT_CR_REP}${CURRENT_LF_REP}(?:${CURRENT_CR_REP}${CURRENT_LF_REP})*/, 
            $alternate_response->{alternate_response_file}, 2;
    }

    if ($alt_response_row->{response_type} eq 'redirect') {
        $alt_response_row->{redirect_url} = ( $alt_response_row->{alternate_response_header} =~  m/Location: (\S*)/ ) ? $1 : undef;
        $alt_response_row->{alternate_response_header} = undef;
    }

    $alt_response_row->{policy_id} = $alternate_response->{policy_id};
    my @placeholders = ('?') x scalar @plc_fields;
    for my $cause(@causes){
        $alt_response_row->{cause} = $cause;
        my $sql = "INSERT INTO $new_dbname.PL_ALTERNATE_RESPONSES
                   (@plc_fields) VALUES( @placeholders )";
        if(! $dbh->do($sql, undef, @{$alt_response_row}{@plc_fields})) {
            $logger->error("failed to execute query $sql ".$DBI::errstr) if $logger;
            return;
        }
    }

    return 1;
}

1;
