package F5::BdCfgTables;

use strict;

use F5::DbUtils;

use Memoize;

memoize('get_dcc_table_config');

my $DCC_TABLE_CFG;
my $BD_CFG_BASE_DIR = F5::Deploy::map_path("/usr/share/ts/share") . "/bd-cfg";

#----------------------------------------------------------------------

sub get_dcc_table_config {
    my(%params) = @_;
    my $dbh = $params{dbh};
    unless ($DCC_TABLE_CFG) {
        my $dcc_table_yaml = YAML::XS::LoadFile($BD_CFG_BASE_DIR . "/tables.yaml");
        my $dcc_tables_names = $dcc_table_yaml->{table_order};
        my %dcc_tables_names_hash = map { ($_ => 1) } @$dcc_tables_names;
        opendir(my $bd_cfg_tables_dir, $BD_CFG_BASE_DIR . "/tables");
        my @bd_cfg_tables = readdir($bd_cfg_tables_dir);
        my @dcc_missing_tables;
        for my $bd_cfg_table (@bd_cfg_tables) {
            if ($bd_cfg_table =~ s/\.yaml// && ! exists $dcc_tables_names_hash{$bd_cfg_table}) {
                push(@dcc_missing_tables, YAML::XS::LoadFile(sprintf('%s/tables/%s.yaml', $BD_CFG_BASE_DIR, $bd_cfg_table)));
            }
        }
        @dcc_missing_tables = sort { $a->{dcc_table_id} <=> $b->{dcc_table_id} } @dcc_missing_tables;
        for my $dcc_missing_table (@dcc_missing_tables) {
            push(@$dcc_tables_names, $dcc_missing_table->{name});
        }
        my $table_index = 1;
        for my $dcc_table_name (@$dcc_tables_names) {
            my $dcc_table = YAML::XS::LoadFile($BD_CFG_BASE_DIR . "/tables/" . $dcc_table_name . ".yaml");
            $dcc_table->{table_index} = $table_index;
            $table_index++;

            if (! $dcc_table->{table} && ! $dcc_table->{legacy_query}) {
                $dcc_table->{table} = "DCC." . uc($dcc_table->{name});
            }
            elsif ($dcc_table->{legacy_query}) {
                $dcc_table->{output_file} = $dcc_table->{name};
            }
            unless ($dcc_table->{output_file}) {
                $dcc_table->{output_file} = lc $dcc_table->{table};
                $dcc_table->{output_file} =~ s/^dcc\.//;
            }
            $dcc_table->{char_field_indexes} = [];
            $dcc_table->{binary_field_indexes} = [];
            $dcc_table->{nullable_field_indexes} = [];
            my $dcc_table_db_fields;
            if ($dcc_table->{table} && $dbh) {
                $dcc_table_db_fields = F5::DbUtils::get_db_fields($dbh, $dcc_table->{table}, undef, 1);
            }
            my $field_index = 0;
            my @message_fields;
            for my $field (@{$dcc_table->{message_fields}}) {
                if ((($field->{proto_type} && $field->{proto_type} eq 'skip') ||
                       (exists $field->{read_from_dcc_database} && ! $field->{read_from_dcc_database}))) {
                    next;
                }
                $field->{field_index} = $field_index++;
                $dcc_table->{fields_by_name}->{$field->{name}} = $field;
                push @message_fields, $field;

                if ($field->{data_type}) {
                    if (index($field->{data_type}, 'char') == 0) {
                        push(@{$dcc_table->{char_field_indexes}}, $field->{field_index});
                    }
                    if (index($field->{data_type}, 'binary') == 0) {
                        push(@{$dcc_table->{binary_field_indexes}}, $field->{field_index});
                    }
                }
                elsif ($field->{proto_type}) {
                    if (index($field->{proto_type}, 'string') == 0) {
                        push(@{$dcc_table->{char_field_indexes}}, $field->{field_index});
                    }
                    if (index($field->{proto_type}, 'bytes') == 0) {
                        push(@{$dcc_table->{binary_field_indexes}}, $field->{field_index});
                    }
                }
                if ($dcc_table_db_fields->{$field->{name}} &&
                    $dcc_table_db_fields->{$field->{name}}->{nullable}) {

                    push(@{$dcc_table->{nullable_field_indexes}}, $field->{field_index});
                }
            }
            $dcc_table->{message_fields} = \@message_fields;

            if (@{$dcc_table->{char_field_indexes}} ||
                @{$dcc_table->{binary_field_indexes}} ||
                @{$dcc_table->{nullable_field_indexes}}) {

                $dcc_table->{has_transformable_fields} = 1;
            }

            if (! $dcc_table->{send_all_data_in_update}) {
                if (! defined $dcc_table->{split_config_by_account} && $dcc_table->{fields_by_name}->{policy_name_crc}) {
                    $dcc_table->{split_config_by_account} = 1;
                }
                if (! defined $dcc_table->{split_config_by_account} && $dcc_table->{table} &&
                    $dcc_table_db_fields->{policy_name_crc}) {

                    $dcc_table->{split_config_by_account} = 1;
                }
            }

            if ($dcc_table->{order_by_fields}) {
                $dcc_table->{order_by_fields} = [ split(/,\s*/, $dcc_table->{order_by_fields}) ];
            }
            else {
                $dcc_table->{order_by_fields} = [ map { $_->{name} } grep { $_->{is_primary_key} } @{$dcc_table->{message_fields}} ];
                if (@{$dcc_table->{order_by_fields}}) {
                    my %order_by_fields_hash = map { $_ => 1 } @{$dcc_table->{order_by_fields}};
                    if (exists $order_by_fields_hash{policy_name_crc} && $dcc_table->{order_by_fields}->[0] ne 'policy_name_crc') {
                        unshift(@{$dcc_table->{order_by_fields}}, 'policy_name_crc');
                    }
                }

                if (! $dcc_table->{split_config_by_account} && ! $dcc_table->{singleton_row} && ! @{$dcc_table->{order_by_fields}}) {
                    $dcc_table->{order_by_fields} = [ map { $_->{name} } @{$dcc_table->{message_fields}} ];
                }
            }
            $dcc_table->{order_by_field_indexes} = [ map { $dcc_table->{fields_by_name}->{$_}->{field_index} } @{$dcc_table->{order_by_fields}} ];

            # Add policy_name_crc to order by fields after indexes have been set
            if ($dcc_table->{split_config_by_account} && (! @{$dcc_table->{order_by_fields}} || $dcc_table->{order_by_fields}->[0] ne 'policy_name_crc')) {
                unshift(@{$dcc_table->{order_by_fields}}, 'policy_name_crc');
            }

            push(@$DCC_TABLE_CFG, $dcc_table);
        }
    }
    return $DCC_TABLE_CFG;
}
