package F5::Browsers;

use strict;
use warnings;

use F5::Dos::BotSignatures;

use constant BROWSER_SIG_ID_START => 900000000;

#----------------------------------------------------------------------------------------

sub get_user_defined_browsers {
    my (%params) = @_;

    my $dbh = $params{dbh};

    my $where_clause = "";
    if ($params{policy_id}) {
        $where_clause .= " AND defs.policy_id = $params{policy_id}"
    }

    my $browsers = $dbh->selectall_arrayref(<<EndQuery, { Slice => {} });
SELECT browsers.last_updated as last_updated,
       browsers.action,
       defs.browser_name,
       defs.last_updated as definition_last_updated,
       defs.match_string,
       defs.match_regex,
       defs.rule,
       defs.browser_md5
  FROM PLC.PL_BROWSER_DEFINITIONS defs
LEFT JOIN PLC.PL_BROWSERS browsers
       ON (defs.browser_definition_id = browsers.browser_definition_id)
LEFT JOIN PLC.FACTORY_BROWSERS factory
       ON (defs.browser_name = factory.browser_name)
 WHERE factory.factory_browser_id IS NULL $where_clause
EndQuery

    for my $browser (@$browsers) {
        $browser->{last_updated} = List::Util::max($browser->{last_updated} || 0,
                                                   $browser->{definition_last_updated} || 0 );
        delete $browser->{definition_last_updated};
    }

    return $browsers;
}

#----------------------------------------------------------------------------------------

sub get_browser_rule {
    my(%params) = @_;
    my $browser = $params{browser};

    my $sig = {};
    if (defined $browser->{match_string}) {
        $sig->{user_agent_search_string} = $browser->{match_string};
        $sig->{user_agent_match_type} = F5::Mcp::BotDefenseSignatureMatchType::BOT_DEFENSE_SIGNATURE_MATCH_TYPE_CONTAINS;
    }
    elsif (defined $browser->{match_regex}) {
        $sig->{user_agent_search_string} = $browser->{match_regex};
        $sig->{user_agent_match_type} = F5::Mcp::BotDefenseSignatureMatchType::BOT_DEFENSE_SIGNATURE_MATCH_TYPE_REGEXP;
    }
                
    return F5::Dos::BotSignatures::build_rule(sig => $sig);
}

#----------------------------------------------------------------------------------------

sub get_browser_sig_id {
    my (%params) = @_;
    my $dbh = $params{dbh};
    my $browser = $params{browser};

    my $browser_md5 = $browser->{browser_md5};
    my $sig_id = $dbh->selectrow_array(<<EndQuery, undef, $browser_md5);
SELECT sig_id
  FROM DCC.USER_DEFINED_BROWSER_NAMES
 WHERE browser_md5 = ?
EndQuery

    if(! $sig_id) {
        my $rec = { browser_name => $browser->{browser_name},
                    rule => $browser->{rule},
                    browser_md5 => $browser_md5,
                  };

        my ($count) = $dbh->selectrow_array("SELECT COUNT(*) FROM DCC.USER_DEFINED_BROWSER_NAMES");
        if (!$count) {
            $rec->{sig_id} = BROWSER_SIG_ID_START + 1;
        } 

        F5::DbUtils::insert_row(dbh => $dbh,
                                table_name => "DCC.USER_DEFINED_BROWSER_NAMES",
                                rec => $rec,
                                last_insert_id => \$sig_id,
                               );
    }
    return $sig_id;
}

#----------------------------------------------------------------------------------------

sub cleanup {
    my (%params) = @_;

    my $dbh = $params{dbh};

    $dbh->do(<<EndQuery);
DELETE FROM DCC.USER_DEFINED_BROWSER_NAMES
WHERE browser_md5 NOT IN (
   SELECT DISTINCT browser_md5
   FROM PLC.PL_BROWSER_DEFINITIONS
   WHERE browser_md5 IS NOT NULL
   )
EndQuery
}

#----------------------------------------------------------------------------------------

1;
