#######################################################################
# Copyright 2003 MagniFire WebSystems (Israel) Ltd. All rights reserved
# File: CRC.pm
#
# Implements CRC API from libutils
# 
# $Header$
#
#######################################################################

package F5::CRC;

use 5.006;
use strict;
use warnings;

require Exporter;
require DynaLoader;

my %METHODS = (
    get_crc         => { call_back => \&get_crc,      unesc_flg => 1, unesc_arg => 0, min_args => 1, max_args => 2 },
    get_crc16       => { call_back => \&get_crc16,    unesc_flg => 1, unesc_arg => 0, min_args => 1, max_args => 2 },
    get_crc32       => { call_back => \&get_crc32,    unesc_flg => 1, unesc_arg => 0, min_args => 1, max_args => 2 },
    get_crc_obj     => { call_back => \&get_crc_obj,  unesc_flg => 0, unesc_arg => 2, min_args => 3, max_args => 4 },
    get_crc_str     => { call_back => \&get_crc_str,  unesc_flg => 1, unesc_arg => 1, min_args => 2, max_args => 3 },
    get_crc_char    => { call_back => \&get_crc_char, unesc_flg => 0, unesc_arg => 0, min_args => 2, max_args => 2 },
    get_crc_arch    => { call_back => \&get_crc_arch, unesc_flg => 1, unesc_arg => 3, min_args => 5, max_args => 6 },
    get_crc_flow    => { call_back => \&get_crc_flow, unesc_flg => 1, unesc_arg => 0, min_args => 3, max_args => 4 },
);


our @ISA = qw(Exporter DynaLoader);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use F5::CRC ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);
our $VERSION = '0.02';

bootstrap F5::CRC $VERSION;

# Preloaded methods go here.

sub get_crc_any {
    my ($method, @args) = @_;

    if ( ! exists $METHODS{$method} ) {
        print qq{Un defined method F5::CRC::$method \n};
        return 0;
    }

    my $min_args = $METHODS{$method}->{min_args};
    my $max_args = $METHODS{$method}->{max_args};
    if ( ( @args < $min_args ) || ( @args > $max_args ) ) {
        print qq{F5::CRC::$method - Wrong number of arguments (req: $min_args, $max_args(th) is optional)\n};
        return 0;
    }

    my $unescape = $METHODS{$method}->{unesc_flg};
    if( $min_args != $max_args && @args == $max_args ) {
       $unescape = pop @args;
    }
    my $i = $METHODS{$method}->{unesc_arg};
    $args[$i] = unescape($args[$i]) if $unescape;

    return $METHODS{$method}->{call_back}->(@args);
}

sub unescape
{
    my $str = shift;

    $str =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/ge;
    return $str;
}

1;
__END__
# Below is the pod documentation for the CRC module

=head1 NAME

F5::CRC - Perl extension for accessing the CRC routines

=head1 SYNOPSIS

  use F5::CRC;

  my $crc = F5::CRC::get_crc($str);
  my $crc = F5::CRC::get_crc32($str);
  my $crc = F5::CRC::get_crc16($str);
  my $crc = F5::CRC::get_crc_obj($app_id, $policy_name_crc, $str);
  my $crc = F5::CRC::get_crc_str($crc_input, $str);
  my $crc = F5::CRC::get_crc_char($crc_input, $char);
  my $crc = F5::CRC::get_crc_arch($crc_input, $app_id, $policy_name_crc, $str, $method);
  my $crc = F5::CRC::get_crc_flow($crc_input, $crc_input_2, $method);

=head1 DESCRIPTION

This module implements the functions in the CRC portion of the libutils library.

=head1 EXPORT

None by default.

=head1 AUTHOR

Ephraim Dan, E<lt>edan@magnifire.comE<gt>

=head1 SEE ALSO

L<perl>.

=cut
