package F5::ClientSideJS;

use F5::DateTimeUtils;
use F5::Deploy;
use File::Copy;
use File::Path;
use F5::Utils::File;

our $CONF_OUTPUT_TMP_DIR = F5::Deploy::get_path_by_type('clientside_js') . '/tmp';
our $CONF_OUTPUT_DIR = F5::Deploy::get_path_by_type('clientside_js').'/trigger-queue';
our $OBFUSCATED_INPUT_DIR = F5::Deploy::get_path_by_type('clientside_js').'/generated-queue';
our $CLIENT_SIDE_JS_DEST_DIR = F5::Deploy::get_path_by_type('bd_config_files') . '/client-side-js';

# Temporary setting the master key hardcoded (here and in bd). This will be solved in: https://jira.pdsea.f5net.com/browse/MBIPWAF-183225
our $MASTER_KEY = "62de1023fba31f763943ce1cecf86d64528b96417ca1663e4153ce21b067bfd9dfb52da6aa4d1c00f6caeba833a7ba66cc";


#----------------------------------------------------------------------

sub new {
    my($class, %params) = @_;
    my $self = {};
    bless $self, $class;
    $self->init(%params);
    return $self;
}

#----------------------------------------------------------------------

sub init {
    my($self, %params) = @_;
    $self->{logger} = $params{logger};
}

sub generate_obfuscation_config_file {
    my($self, %params) = @_;

    # First making sure required folders exists
    mkpath($CONF_OUTPUT_TMP_DIR);
    mkpath($CONF_OUTPUT_DIR);

    my $cur_time = time;
    my $time_string = F5::DateTimeUtils::convert_epoch_to_file_timestamp($cur_time);
    my $curr_filename = "$CONF_OUTPUT_TMP_DIR/obfuscator-$time_string.json";

    my $rv = F5::SystemUtils::run_system_tool(command_line_params => 
                [ F5::Deploy::get_path_by_type('executables').'/datacrypt_cmd', '-g', $params{generated_variants_count},
                                                                                '-o'.$curr_filename,
                                                                                '-a', $cur_time,
                                                                                '-m', $MASTER_KEY ]);

    if ($rv && !$rv->{completed_successfully}) {
        $self->{logger}->error("Failed to generate js obfuscation config file") if $self->{logger};    
        #die $rv->{standard_error};
        return;
    }

    # Copy created json file to shared volume
    move($curr_filename, $CONF_OUTPUT_DIR);
    
    $self->{logger}->trace("Successfully generated js obfuscation config file in $curr_filename") if $self->{logger};
}

#----------------------------------------------------------------------

sub check_and_move_obfuscated_files {
    my($self, %params) = @_;
  
    # Get the latest obfuscated folder
    my @folders = sort glob("$OBFUSCATED_INPUT_DIR/*");
    if (!@folders) {
        # Folder is empty, nothing to move
        return false;
    }
    my $obfuscated_folder = "$OBFUSCATED_INPUT_DIR/$folders[-1]";
     
    # Copy all files to shared volume (delete later, in case copy failed..)
    my $dest_dir = "$CLIENT_SIDE_JS_DEST_DIR/$folders[-1]";
    mkpath($dest_dir);
    F5::Utils::File::copy_files_recursively(from_dir => $obfuscated_folder, to_dir => $dest_dir);
    
    $self->{logger}->trace("Copied obfuscated files: $obfuscated_folder -> $dest_dir") if $self->{logger};
    
    # Delete all folders
    F5::Utils::File::delete_files_recursively($OBFUSCATED_INPUT_DIR);
    
    return $dest_dir;
}

1;
