package F5::CommonUpgrade::Base;

use F5::ASMConfig::ASMConfig;

use File::Basename;

use F5::BigipVersionUtils;

#----------------------------------------------------------------------

sub new {
  my($class, %params) = @_;
  my $self = {};
  bless $self, $class;
  $self->init(%params);

  return $self;
}

#----------------------------------------------------------------------

sub install_accessor {
  my($self, $field, $val) = @_;

  no strict 'refs';
  no warnings 'redefine';
  my $class = ref($self) || $self;
  *{$class . '::' . $field} = sub {
    return $val;
  };
  return $val;
}

#----------------------------------------------------------------------

sub max_version {
  my($class) = @_;
  no strict 'refs';

  my $max_version_padded;
  if (${$class . '::max_version'}) {
      $max_version_padded = F5::BigipVersionUtils::zero_pad_segments(${$class . '::max_version'}, 3);
  }
  return $class->install_accessor('max_version', $max_version_padded);
}

#----------------------------------------------------------------------

sub min_version {
  my($class) = @_;
  no strict 'refs';

  my $min_version_padded;
  if (${$class . '::min_version'}) {
      $min_version_padded = F5::BigipVersionUtils::zero_pad_segments(${$class . '::min_version'}, 3);
  }
  return $class->install_accessor('min_version', $min_version_padded);
}

#----------------------------------------------------------------------

sub priority {
  my($class) = @_;
  no strict 'refs';

  my $priority;
  if (${$class . '::priority'}) {
      $priority = ${$class . '::priority'};
  }
  return $class->install_accessor('priority', $priority);

}

#----------------------------------------------------------------------

sub tables_handled {
  my($class) = @_;
  no strict 'refs';

  my @tables_handled;
  if (${$class . '::tables_handled'}) {
      @tables_handled = @{${$class . '::tables_handled'}};
  }
  return $class->install_accessor('tables_handled', \@tables_handled);
}

#----------------------------------------------------------------------

sub tables_renamed {
  my($class) = @_;
  no strict 'refs';

  my %tables_renamed;
  if (${$class . '::tables_renamed'}) {
      %tables_renamed = %{${$class . '::tables_renamed'}};
  }
  return $class->install_accessor('tables_renamed', \%tables_renamed);
}

#----------------------------------------------------------------------

sub is_valid_version {
  my($class, $old_version, $new_version) = @_;
  if ((!defined($class->min_version) || ($old_version && $old_version ge $class->min_version)) &&
      (!defined($class->max_version) || ($old_version && $old_version lt $class->max_version))) {
    return 1;
  }
  else {
    return 0;
  }
}

#----------------------------------------------------------------------

sub add_info {
    my($class, %params) = @_;
    if ($params{common_upgrade} && $params{common_upgrade}->{notification_handler}) {
        $params{common_upgrade}->{notification_handler}->add_info(%params);
    }
}

#----------------------------------------------------------------------

sub add_warning {
    my($class, %params) = @_;
    if ($params{common_upgrade} && $params{common_upgrade}->{notification_handler}) {
        $params{common_upgrade}->{notification_handler}->add_warning(%params);
    }
}

#----------------------------------------------------------------------


1;
