package F5::CommonUpgrade::BeWafFalsePositive;

use base 'F5::CommonUpgrade::Base';
use F5::ASMConfig::ASMConfig;

our $max_version = '15.1.0';
our $tables_handled = [ qw(PLC.PL_POLICY_TEMPLATES_SETTINGS) ];

use constant BEHAVIORAL_WAF => 44;

my @bewaf_policy_ids;

sub row_fixup {
    my($self, %params) = @_;
    my $row = $params{row};

    my $policy_id = defined $params{policy_id} ? 
                            $params{policy_id} :
                            $row->{policy_id};

    if ($row->{policy_template_id} == BEHAVIORAL_WAF) {
        push @bewaf_policy_ids, $policy_id;
    }
}

sub after_table_complete {
    my($self, %params) = @_;
    return unless @bewaf_policy_ids;

    local $" = ',';

    my $dbh    = $params{dbh};
    my $dbname = $params{dbname} || 'PLC';
    my @placeholders = ('?') x scalar @bewaf_policy_ids;

    $dbh->do(<<EndQuery, undef, F5::ASMConfig::AttackSignatureFalsePositiveMode::ATTACK_SIGNATURE_FALSE_POSITIVE_MODE_DETECT_AND_ALLOW(), @bewaf_policy_ids)
UPDATE $dbname.PL_POLICY_NEGSIG_ATTRIBUTES
   SET attack_signature_false_positive_mode = ?
 WHERE policy_id IN (@placeholders)
EndQuery
}

1;
