package F5::CommonUpgrade::BotDefenseProfile;

use base 'F5::CommonUpgrade::Base';

use F5::ASM::COAPI;
use F5::PrepareConf::BotDefense;

our $max_version = '13.1.0';
our $tables_handled = [ qw(PLC.PL_POLICIES) ];

#----------------------------------------------------------------------

sub row_fixup {
    my($self, %params) = @_;

    my $idmap = $params{idmap};
    my $row = $params{row};

    my $policy_name = $row->{name};
    return unless $policy_name;

    push(@{$idmap->{'__private_save'}->{save_policy_names}}, $policy_name);
}

#----------------------------------------------------------------------

sub after_all_tables_complete {
    my($self, %params) = @_;
    my $idmap = $params{idmap};
    my $dbname = $params{dbname} || 'PLC';
    my $dbh = $params{dbh};
    my $common_upgrade = $params{common_upgrade} || {};
    my $logger = $params{logger} || $common_upgrade->{logger};
    my $policy_id = $params{policy_id};

    if ($policy_id && $policy_id ne '*') {
        $logger->debug("Bot defense profile deletion is skipped during policy import") if $logger;
        return;
    }

    if ($idmap && $idmap->{'__private_save'} && $idmap->{'__private_save'}->{save_policy_names}) {
        my %mcp_profiles = map { ( $_->{name} => $_ ) } @{F5::ASM::COAPI::Query->new('profile_bot_defense_asm')};

        for my $policy_name (@{$idmap->{'__private_save'}->{save_policy_names}}) {
            my $bot_defense_profile_name = F5::PrepareConf::BotDefense::get_bot_defense_profile_name(policy_name => $policy_name);
            delete $mcp_profiles{$bot_defense_profile_name};
        }
        delete $mcp_profiles{$F5::PrepareConf::BotDefense::BASE_PROFILE_NAME};
        delete $mcp_profiles{$F5::PrepareConf::BotDefense::AGGREGATED_PROFILE_NAME};

        for my $bot_defense_profile_name (keys %mcp_profiles) {
            my $mcp_profile = $mcp_profiles{$bot_defense_profile_name};
            eval {
                $logger->trace("Deleting unused bot defense profile $bot_defense_profile_name") if $logger;
                $mcp_profile->delete;
            };
            if (my $err = $@) {
                $logger->info("Error encountered while trying to delete a bot defense profile named $bot_defense_profile_name") if $logger;
                $logger->info($err) if $logger;
            }
        }

        # TODO: __private_save needs to be discarded entirely when upgrade/import is completed
        delete $idmap->{'__private_save'}->{save_policy_names};
    }
}

#----------------------------------------------------------------------

1;
