package F5::CommonUpgrade::BruteForce;

use List::Util qw(max min);

use base 'F5::CommonUpgrade::Base';

our $max_version = '13.1.0';
our $tables_handled = [ qw(PLC.PL_LOGIN_BRUTE_FORCE_OBJECT_ATTRIBUTES) ];

sub row_fixup {
    my($self, %params) = @_;

    my $row = $params{row};

    $row->{max_prevention_time} = max($row->{max_prevention_time},3600) if defined $row->{max_prevention_time};
    $row->{measurement_period} = max($row->{measurement_period},900) if defined $row->{measurement_period};
    $row->{session_expiration_period} = max($row->{session_expiration_period},3600) if defined $row->{session_expiration_period};

    # This field corresponds to the threshold for detecting distributed Brute Force attack (default 100).
    # upgrade logic is:
    #     min(1000, max( (default measurementPeriod / previous measurementPeriod) × previous failedLoginAttemptsRateReached, 100))
    if ($row->{measurement_period} && defined $row->{detection_threshold_abs}) {
        $row->{detection_threshold_abs} = min(1000, max((900/$row->{measurement_period})*$row->{detection_threshold_abs},100));
    }

    if (! exists $row->{flg_detect_distributed_brute_force_attack}) {
        if ($row->{flg_alarm} || $row->{flg_block}) {
            $row->{flg_detect_distributed_brute_force_attack} = 1;
        }
        else {
            $row->{flg_detect_distributed_brute_force_attack} = 0;
        }
    }

    if (! exists $row->{flg_detect_login_attempts_from_the_same_ip}) {
        if ($row->{flg_alarm} || $row->{flg_block}) {
            $row->{flg_detect_login_attempts_from_the_same_ip} = 1;
        }
        else {
            $row->{flg_detect_login_attempts_from_the_same_ip} = 0;
        }
    }

    if (! exists $row->{detection_threshold_action}) {
        if (! $row->{flg_block}) {
            $row->{detection_threshold_action} = 1;
        }
        elsif ($row->{flg_alarm} && $row->{flg_block}) {
            $row->{detection_threshold_action} = 3;
        }
    }

    if (! exists $row->{login_attempts_from_the_same_ip_action}) {
        if (! $row->{flg_block}) {
            $row->{login_attempts_from_the_same_ip_action} = 1;
        }
        elsif ($row->{flg_alarm} && $row->{flg_block}) {
            $row->{login_attempts_from_the_same_ip_action} = 3;
        }
    }

    if (! exists $row->{flg_detect_credentials_stuffing_attack}) {
        $row->{flg_detect_credentials_stuffing_attack} = 0;
    }
}

1;
