package F5::CommonUpgrade::DeleteBotDetectionSuggestion;

use strict;
use warnings;

use base 'F5::CommonUpgrade::Base';

use F5::DbUtils;

our $priority = 1;
our $max_version = '14.1.0';

# these are the tables that have a 'section_type' field
our $tables_handled = [ qw(
                           PLC.PL_CHARSETS
                           PLC.PL_SECTION_DEFAULTS
                           PLC.PL_SECTIONS
                           PLC.PL_SUGGESTIONS
                           PLC.REFINEMENTS
                           PLC.SECTION_PROPERTIES
                           PLC.VIOLATIONS
                          ) ];

my $SECTION_TYPE_BOT_DETECTION = 19;

#------------------------------------------------------------------------------

sub row_fixup {
    my($self, %params) = @_;
    my $row = $params{row};
    my $row_state_rv = $params{row_state_rv};

    if ($row->{section_type} == $SECTION_TYPE_BOT_DETECTION) {
        $$row_state_rv = 'skip';
        return;
    }
}

#------------------------------------------------------------------------------

sub after_all_tables_complete {
    my($self, %params) = @_;

    my $dbh    = $params{dbh};
    my $policy_id = $params{policy_id};
    my $dbname = $params{dbname} || 'PLC';

    my $rec = { section_type => $SECTION_TYPE_BOT_DETECTION };
    if ($policy_id && $policy_id ne "*") {
        $rec->{policy_id} = $policy_id;
    }

    for my $table_name (qw(PL_CHARSETS
                           PL_SECTION_DEFAULTS
                           PL_SECTIONS
                           PL_SUGGESTIONS
                           REFINEMENTS
                           SECTION_PROPERTIES
                           VIOLATIONS
                          )) {

        # this will do the right thing for policies w/o policy_id field
        F5::DbUtils::delete_row(dbh => $dbh,
                                table_name => "$dbname.$table_name",
                                rec => $rec);
    }
}

#----------------------------------------------------------------------

1;
