package F5::CommonUpgrade::DeleteWebScrapingViolation;

use strict;
use warnings;

use base 'F5::CommonUpgrade::Base';

use F5::DbUtils;
use F5::Violation;
use F5::Violation::Utils;

our $priority = 1;
our $max_version = '14.1.0';

# these are the tables that have a 'viol_index' field
our $tables_handled = [ qw(
                           PLC.PL_BLOCKING
                           PLC.PL_MICROSERVICE_BLOCKING_OVERRIDES
                           PLC.PL_SECURITY_LEVEL_DEFAULTS
                           PLC.PL_SESSION_AWARENESS_VIOLATIONS_DEFAULTS
                           PLC.PL_SESSION_AWARENESS_VIOLATIONS
                           PLC.PL_SUGGESTIONS
                           PLC.VIOLATIONS_PROPERTIES_DEFAULTS
                           PLC.VIOLATIONS_PROPERTIES
                           PLC.VIOLATIONS
                          ) ];

my $VIOL_WEB_SCRAPING = 21;

#------------------------------------------------------------------------------

sub row_fixup {
    my($self, %params) = @_;
    my $row = $params{row};
    my $row_state_rv = $params{row_state_rv};

    my $viol_index;
    if (exists $row->{code} && ! exists $row->{viol_index}) {
        $viol_index = F5::Violation::Utils::get_viol_index_from_code($row->{code});
    }
    else {
        $viol_index = $row->{viol_index};
    }

    if ($viol_index && $viol_index == $VIOL_WEB_SCRAPING) {
        $$row_state_rv = 'skip';
        return;
    }
}

#------------------------------------------------------------------------------

sub after_all_tables_complete {
    my($self, %params) = @_;

    my $dbh    = $params{dbh};
    my $policy_id = $params{policy_id};
    my $dbname = $params{dbname} || 'PLC';

    my $rec = { viol_index => $VIOL_WEB_SCRAPING };
    if ($policy_id && $policy_id ne "*") {
        $rec->{policy_id} = $policy_id;
    }

    for my $table_name (qw(PL_BLOCKING
                           PL_MICROSERVICE_BLOCKING_OVERRIDES
                           PL_SECURITY_LEVEL_DEFAULTS
                           PL_SESSION_AWARENESS_VIOLATIONS_DEFAULTS
                           PL_SESSION_AWARENESS_VIOLATIONS
                           PL_SUGGESTIONS
                           VIOLATIONS_PROPERTIES_DEFAULTS
                           VIOLATIONS_PROPERTIES
                           VIOLATIONS
                          )) {

        # this will do the right thing for policies w/o policy_id field
        F5::DbUtils::delete_row(dbh => $dbh,
                                table_name => "$dbname.$table_name",
                                rec => $rec);
    }
}

#----------------------------------------------------------------------

1;
