package F5::CommonUpgrade::FixCsrfUrlsFields;

use base 'F5::CommonUpgrade::Base';
use F5::DbUtils;

our $max_version = '13.1.0';
our $tables_handled = [ qw(PLC.PL_POLICY_CSRF_URLS) ];

#----------------------------------------------------------------------

sub after_all_tables_complete {
  my($self, %params) = @_;
  local $" = ',';

  my $dbh    = $params{dbh};
  my $logger = $params{logger};
  my $dbname = $params{dbname} || 'PLC';

  my $where_fields = {};

  if ($params{policy_id} && $params{policy_id} ne '*') {
    $where_fields->{'policy_id'} = $params{policy_id};
  }

  my $constructed_where_clause = F5::DbUtils::construct_where_clause(where_fields => $where_fields);
  my $where_clause = $constructed_where_clause->{where_clause};
  my @where_values = @{$constructed_where_clause->{where_values}};

  my $csrf_urls = $dbh->selectall_arrayref(<<END_SQL,undef, @where_values);
SELECT csrf_url_id, policy_id
  FROM $dbname.PL_POLICY_CSRF_URLS
  WHERE $where_clause
  ORDER BY length(target_pattern) DESC
END_SQL

  my %policy_max_order;
  if ($csrf_urls && @{$csrf_urls} ) {
      $self->add_info(%params,
                      msg => 'URLs with CSRF Protection in the imported policy are now used for POST requests only.');

      for my $csrf_url (@$csrf_urls) {
          my ($id, $policy_id) = @$csrf_url;
          $policy_max_order{$policy_id}{max}++;
          $dbh->do(<<EndQuery, undef, $policy_max_order{$policy_id}{max}, $id);
UPDATE $dbname.PL_POLICY_CSRF_URLS
   SET pattern_order = ?
 WHERE csrf_url_id = ?
EndQuery
      }
  }

  $where_fields->{'flg_is_enabled'} = 0;

  $constructed_where_clause = F5::DbUtils::construct_where_clause(where_fields => $where_fields);
  $where_clause = $constructed_where_clause->{where_clause};
  @where_values = @{$constructed_where_clause->{where_values}};

  my $policies =  $dbh->selectall_hashref(<<"EndQuery", 'policy_id', undef, @where_values);
SELECT policy_id
  FROM $dbname.PL_POLICY_CSRF
  WHERE $where_clause
EndQuery

  for my $policy_id ( keys %{$policies} ){
      $policy_max_order{$policy_id}{max}++;
      F5::DbUtils::insert_row(dbh => $dbh,
                              ignore => 1,
                              table_name => "$dbname.PL_POLICY_CSRF_URLS",
                                    rec => {
                                            policy_id => $policy_id,
                                            target_pattern => '*',
                                            pattern_order => $policy_max_order{$policy_id}{max},
                                           }
                              );

  }
}
#----------------------------------------------------------------------
1;
