package F5::CommonUpgrade::FixSsrfHostDenyList;

use strict;
use warnings;
use base 'F5::CommonUpgrade::Base';
use F5::DbUtils;

our $max_version = '17.0.0';
our $tables_handled = [ qw(PLC.PL_SSRF_HOSTS) ];

#----------------------------------------------------------------------
sub row_fixup {
    my ($self, %params) = @_;

    my $row = $params{row};

    if ($row->{host} =~ /^\*\.(.*)/) {
        $row->{flg_is_pattern} = 1;
    }
    else {
        $row->{flg_is_pattern} = 0;
    }

}

#----------------------------------------------------------------------

sub after_all_tables_complete {
  my($self, %params) = @_;

  my $dbh    = $params{dbh};
  my $dbname = $params{dbname} || 'PLC';

  my $policies;

  if ($params{policy_id} && $params{policy_id} ne '*') {
    $policies = [$params{policy_id}];
  }
  else {
    $policies = $dbh->selectcol_arrayref("SELECT id FROM $dbname.PL_POLICIES");
  }

  for my $policy_id (@$policies) {
    my $default_ssrf_hosts =  $dbh->selectall_arrayref(<<EndQuery, { Slice => {} });
SELECT host, rest_uuid, action_type, flg_is_pattern
  FROM $dbname.PL_SSRF_HOST_DEFAULTS
EndQuery
    for my $rec (@$default_ssrf_hosts) {
      $rec->{policy_id} = $policy_id;
      F5::DbUtils::insert_row(dbh        => $dbh,
                              ignore     => 1,
                              table_name => "$dbname.PL_SSRF_HOSTS",
                              rec        => $rec
                              );
    }
  }
}

#----------------------------------------------------------------------
1;

