package F5::CommonUpgrade::ForeignKeyMismatch;

use base 'F5::CommonUpgrade::Base';
use F5::DbUtils;
use F5::Logger;
use F5::ASM::COAPI;

our %mcp_methods;

sub row_fixup {
    my($self, %params) = @_;

    if (! %mcp_methods) {
        my $query_result = F5::ASM::COAPI::Query->new('asm_http_method');
        if ($query_result) {
            %mcp_methods = map { $_->{code} => $_ } @$query_result;
        }
    }

    my $row = $params{row};
    if (%mcp_methods && ! exists $mcp_methods{$row->{method_code}}) {
        my $row_state_rv = $params{row_state_rv};
        $$row_state_rv = 'skip';
    }
}

sub after_all_tables_complete {
    my($self, %params) = @_;

    my $dbh    = $params{dbh};
    my $policy_id = $params{policy_id};
    my $dbname = $params{dbname} || 'PLC';
    my $databases = $params{databases};
    my $dcc_dbname = 'DCC';
    if ($databases && $databases->{DCC} && $databases->{DCC}->{ucs_dbname}) {
        $dcc_dbname = $databases->{DCC}->{ucs_dbname};
    }

    my %where_clause_parts;
    if ($policy_id && $policy_id ne '*') {
        $where_clause_parts{where_fields}->{"main_table.policy_id"} = $policy_id;
    }
    my $where_clause = F5::DbUtils::construct_where_clause(%where_clause_parts);

    $dbh->do(<<EndQuery, undef, $where_clause->{where_values});
DELETE FROM $dbname.PL_OBJECT_CONTENT_PROFILES
  WHERE obj_id IN (
      SELECT obj_id FROM (SELECT obj_id FROM $dbname.PL_OBJECT_CONTENT_PROFILES main_table
        JOIN $dbname.PL_OBJECTS o
         ON o.id = main_table.obj_id
         WHERE o.policy_id != main_table.policy_id
          AND $where_clause->{where_clause}
  ) subquery )
EndQuery

}

1;

