package F5::CommonUpgrade::Headers;

use base 'F5::CommonUpgrade::Base';
use F5::DbUtils;

our $max_version = '11.4.0';

our $tables_handled = [ qw(
                           PLC.PL_MANDATORY_HEADERS
                           PLC.PL_NEGSIG_EXCLUDED_HEADERS
                           )
                     ];

our $tables_renamed = { 
                        'PLC.PL_MANDATORY_HEADERS' => '',
                        'PLC.PL_NEGSIG_EXCLUDED_HEADERS'  => '',
                      };


sub row_fixup {
    my($self, %params) = @_;
    my $row = $params{row};
    $row->{header_name} = lc($row->{header_name});
    my $header_key = $row->{header_name}.'--'.$row->{policy_id}; 
    my $orig_table_name = $params{orig_table_name};
    my $idmap = $params{idmap};

    return if $row->{header_name} eq '*'; # explicit * will be deleted in order to add wildcard *
    return if $row->{header_name} eq 'cookie';  # the cookie header will be deleted in order to add the default cookie header
    my %default_headers;
    $default_headers{referer} = 1;
    $default_headers{cookie} = 1;
    $default_headers{authorization} = 1;

    if ( defined $params{policy_id} ){
        $row->{policy_id} = $params{policy_id}
    }

    if ( (!$idmap->{'__private_save'}) || (!$idmap->{'__private_save'}->{save_headers}) ||
           (!exists $idmap->{'__private_save'}->{save_headers}->{$header_key})) {
              $idmap->{'__private_save'}->{save_headers}->{$header_key} = { policy_id => $row->{policy_id},
                                                                            header_name => $row->{header_name},
                                                                            flg_check_signatures => 1,
                                                                            flg_is_mandatory => 0,
                                                                            flg_is_default => 0,
                                                                            flg_enable_normalization_violation => 0,
                                                                            flg_uri_normalization => 0,
                                                                           };
              if ( $row->{header_name} eq 'referer' ){
                  $idmap->{'__private_save'}->{save_headers}->{$header_key}->{flg_enable_normalization_violation} = 1;
                  $idmap->{'__private_save'}->{save_headers}->{$header_key}->{flg_uri_normalization} = 1;
              }
    }

    if ( $orig_table_name eq 'PLC.PL_MANDATORY_HEADERS'){
        $idmap->{'__private_save'}->{save_headers}->{$header_key}->{flg_is_mandatory} = 1;
    }

    if  ( $orig_table_name eq 'PLC.PL_NEGSIG_EXCLUDED_HEADERS'){
         $idmap->{'__private_save'}->{save_headers}->{$header_key}->{flg_check_signatures} = 0;
         $idmap->{'__private_save'}->{save_headers}->{$header_key}->{flg_enable_normalization_violation} = 0;
         $idmap->{'__private_save'}->{save_headers}->{$header_key}->{flg_uri_normalization} = 0;
    }

   if ( $default_headers{$row->{header_name}}) {
        $idmap->{'__private_save'}->{save_headers}->{$header_key}->{flg_is_default} = 1;
    }
}

#----------------------------------------------------------------------

sub after_all_tables_complete {
    my($self, %params) = @_;
    my $idmap = $params{idmap};

    my $dbname = $params{dbname} || 'PLC';
    if ($idmap && $idmap->{'__private_save'} && $idmap->{'__private_save'}->{save_headers}) {
        F5::DbUtils::insert_row(dbh => $params{dbh},
                                table_name => "$dbname.PL_HEADERS",
                                rec => [ values %{$idmap->{'__private_save'}->{save_headers}} ],
                                do_replace => 1,
                               );

        delete $idmap->{'__private_save'}->{save_headers};
    }

}



1;

