package F5::CommonUpgrade::HostsToMicroservices;

use base 'F5::CommonUpgrade::Base';

our $max_version = '14.1.0';
our $tables_handled = [ qw(PLC.PL_VALID_HOST_NAMES) ];

sub row_fixup {
    my($self, %params) = @_;

    my $row = $params{row};

    my $idmap = $params{idmap};
    my $dbh =  $params{dbh};
    my $databases = $params{databases};
    my $policy_id;

    if ( defined $params{policy_id} ){
        $policy_id = $params{policy_id}
    }
    elsif ( defined $row->{policy_id} ){
        $policy_id = $row->{policy_id};
    }

    if ( $row->{transparency_enforcement_mode} ==  F5::ASMConfig::TransparencyEnforcementMode::TRANSPARENCY_ENFORCEMENT_TRANSPARENT() ) {
         $idmap->{'__private_save'}->{convert_hosts_to_microservices}->{$policy_id}->{$row->{host_name}} = 1;
    }

}

#----------------------------------------------------------------------------
sub after_all_tables_complete {
    my($self, %params) = @_;
    my $idmap = $params{idmap};
    my $dbh = $params{dbh};
    my $dbname = $params{dbname} || 'PLC';

    my $policies;

    if ( $params{policy_id} eq "*") {
        $policies =  $dbh->selectall_hashref(<<"EndQuery",'id');
SELECT id, policy_type, name
  FROM $dbname.PL_POLICIES
EndQuery
    }
    else {
        $policies =  $dbh->selectall_hashref(<<"EndQuery",'id', undef, $params{policy_id});
SELECT id, policy_type, name
  FROM $dbname.PL_POLICIES
 WHERE id = ?
EndQuery

    }

    for my $policy_id (keys %{$policies} ) {
        if ( !$idmap ||
             !( exists $idmap->{'__private_save'} ) ||
             !( exists $idmap->{'__private_save'}->{convert_hosts_to_microservices} )||
             !( exists $idmap->{'__private_save'}->{convert_hosts_to_microservices}->{$policy_id} ) ){
            next;
        }

        if ($policies->{$policy_id}->{policy_type} == F5::ASMConfig::PolicyType::POLICY_TYPE_PARENT()){
            $self->add_info(%params,
                      msg => "Skip auto-generated microservices during upgrade from hostname for parent policy $policies->{$policy_id}->{name}.");
            next;
        }

        my $hosts =  $dbh->selectall_arrayref(<<EndQuery, { Slice => {} }, $policy_id);
SELECT *
  FROM $dbname.PL_VALID_HOST_NAMES
 WHERE policy_id = ?
ORDER BY host_name
EndQuery

        my $max_pattern_order = 1;
        my $last_updated = time();

        for my $host ( @$hosts ) {
            my $transparency_enforcement_mode = F5::ASMConfig::TransparencyEnforcementMode::TRANSPARENCY_ENFORCEMENT_POLICY_DEFAULT();
            if( $idmap->{'__private_save'}->{convert_hosts_to_microservices}->{$policy_id}->{$host->{host_name}} ) {
                $transparency_enforcement_mode = F5::ASMConfig::TransparencyEnforcementMode::TRANSPARENCY_ENFORCEMENT_TRANSPARENT();
            }

            my $host_name = lc($host->{host_name});

            F5::DbUtils::insert_row(dbh => $dbh,
                                    table_name => "$dbname.PL_MICROSERVICES",
                                    rec => {
                                        policy_id                           => $policy_id,
                                        microservice_uri                    => '/*',
                                        flg_is_pattern_uri                  => 1,
                                        flg_is_pattern                      => 1,
                                        pattern_order                       => $max_pattern_order++,,
                                        transparency_enforcement_mode       => $transparency_enforcement_mode,
                                        microservice_host_name              => $host_name,
                                        flg_is_pattern_host                 => 0,
                                        description                         => $host->{description}." Auto-generated during upgrade from hostname.",
                                        last_updated                        => $last_updated,
                                       },
                                   );

            if ( $host->{flg_include_subdomains} ) {
                 F5::DbUtils::insert_row(dbh => $dbh,
                                    table_name => "$dbname.PL_MICROSERVICES",
                                    rec => {
                                        policy_id                           => $policy_id,
                                        microservice_uri                    => '/*',
                                        flg_is_pattern_uri                  => 1,
                                        flg_is_pattern                      => 1,
                                        pattern_order                       => $max_pattern_order++,,
                                        transparency_enforcement_mode       => $transparency_enforcement_mode,
                                        microservice_host_name              => "*.$host_name",
                                        flg_is_pattern_host                 => 1,
                                        description                         => $host->{description}." Auto-generated during upgrade from hostname.",
                                        last_updated                        => $last_updated,
                                       },
                                   );
            }
        }
    }

    delete $idmap->{'__private_save'}->{convert_hosts_to_microservices};

}
#----------------------------------------------------------------------------

1;
