package F5::CommonUpgrade::HslFolder;

use base 'F5::CommonUpgrade::Base';

use F5::ASM::COAPI;

our $max_version = '11.0.0';
our $tables_handled = [ qw(DCC.HSL_DATA_PROFILES DCC.HSL_VIRTUAL_SERVERS) ];

# Unfortunately, we still have to be 'hsl http' aware during updrade
my %hsl_profile_type = ( 1 => 'ftp',
                         2 => 'smtp',
                         3 => 'http',
                       );

use F5::Mcp::ProfileType;

sub row_fixup {
    my($self, %params) = @_;

    my $row = $params{row};

    if ($row->{hsl_profile_id}) {
        # skipping rows with hsl http
        if ($hsl_profile_type{$row->{hsl_profile_id}} eq 'http') {
            return;
        }

        my $profile;
        if ($row->{flg_is_enabled}) {
            if ($hsl_profile_type{$row->{hsl_profile_id}} eq 'smtp') {
                # special handling for SMTP
                $profile = F5::ASM::COAPI::Query->new('ltcfg_instance',
                                                      {
                                                       class_name => 'profile_smtp'});
                if ($profile && @$profile) {
                    $profile = [ grep { $_->{instance_leaf_name} eq $row->{data_profile_name} } @$profile ];
                }
            }
            else {
                $profile = F5::ASM::COAPI::Query->new('profile_' . $hsl_profile_type{$row->{hsl_profile_id}},
                                                      {
                                                       leaf_name => $row->{data_profile_name} });
            }
        }

        if ($profile && @$profile) {
            $row->{data_profile_name} = $profile->[0]->{name};
        }
        else {
            $row->{data_profile_name} = '/Common/' . $row->{data_profile_name};
        }
    }
    else {
        my $vs_profile = F5::ASM::COAPI::Query->new('virtual_server_profile',
                                                    {
                                                     leaf_name => $row->{virtual_server_name} });
        my $actual_vs_profile;
        if ($vs_profile && @$vs_profile) {
            ($actual_vs_profile) = grep($_->{profile_type} != F5::Mcp::ProfileType::PROFILE_TCP, @$vs_profile);
        }

        if ($actual_vs_profile) {
            $row->{virtual_server_name} = $actual_vs_profile->{vs_name};
            $row->{data_profile_name} = $actual_vs_profile->{profile_name};
        }
        else {
            $row->{virtual_server_name} = '/Common/' . $row->{virtual_server_name};
            $row->{data_profile_name} = '/Common/' . $row->{data_profile_name};
        }
    }
}

1;
