package F5::CommonUpgrade::IpUnification;

use base 'F5::CommonUpgrade::Base';
use F5::DbUtils;

our $max_version = '11.2.0';

our $tables_handled = [ qw(
                            PLC.LRN_BLACK_LIST_SRC_IPS
                            PLC.PL_ALLOWED_SCRAPER_IPS
                            PLC.PL_BF_IP_WHITE_LIST
                            PLC.PL_POLICY_BUILDER_IP_DATA
                           ) 
                     ];

our $tables_renamed = { 
                        'PLC.LRN_BLACK_LIST_SRC_IPS' => '', # src_ip src_ip_mask 
                        'PLC.PL_ALLOWED_SCRAPER_IPS' => '', # allowed_scraper_ip_address allowed_scraper_ip_mask 
                        'PLC.PL_BF_IP_WHITE_LIST' => '',  # ip_address ip_mask 
                        'PLC.PL_DOS_IP_WHITE_LIST' => '', # from now not in ASM
                        'PLC.PL_POLICY_BUILDER_IP_DATA' => '' #  ip_address netmask
                      };


sub row_fixup {
    my($self, %params) = @_;

    my $row = $params{row};
    my $idmap = $params{idmap};
    my $dbh =  $params{dbh};
    my $databases = $params{databases};
    my $white_ip_rec;
    my $whitelist_ip_type;

    if ( defined $params{policy_id} ){
        $white_ip_rec->{policy_id} = $params{policy_id}
    }
    elsif ( defined $row->{policy_id} ){
        $white_ip_rec->{policy_id} = $row->{policy_id};
    }
    elsif ( defined $row->{account_id} ){
         $white_ip_rec->{policy_id} = F5::DbUtils::get_active_policy_id( account_id => $row->{account_id},
                                                              dbname => $databases->{DCC}->{ucs_dbname},
                                                              dbh => $dbh  );
    }


    if ($row->{src_ip}){ # originally from PLC.LRN_BLACK_LIST_SRC_IPS
      $white_ip_rec->{ip_address} = $row->{src_ip};
      if (exists $row->{src_ip_mask}) {
          $white_ip_rec->{mask} =  $row->{src_ip_mask};
      }
      else {
          $white_ip_rec->{mask} =  '255.255.255.255';
      }
      $whitelist_ip_type = 'flg_never_learn';
    }
    elsif ( $row->{netmask} ){  # originally from PLC.PL_POLICY_BUILDER_IP_DATA
      $white_ip_rec->{ip_address} = $row->{ip_address};
      $white_ip_rec->{mask} = $row->{netmask};
      $whitelist_ip_type = 'flg_pb_trusted';
    }
    elsif( $row->{allowed_scraper_ip_address}) {  # originally from PLC.PL_ALLOWED_SCRAPER_IPS
       $white_ip_rec->{ip_address} = $row->{allowed_scraper_ip_address};
       $white_ip_rec->{mask} = $row->{allowed_scraper_ip_mask};
       $whitelist_ip_type = 'flg_ignore_anomaly';
    }
    elsif( $row->{ip_mask} ){ # originally from  PLC.PL_BF_IP_WHITE_LIST
       $white_ip_rec->{ip_address} = $row->{ip_address};
       $white_ip_rec->{mask} = $row->{ip_mask};
       $whitelist_ip_type = 'flg_ignore_anomaly';
    }

    my $ip_mask_key = $white_ip_rec->{ip_address}.'--'.$white_ip_rec->{mask}.'--'.$white_ip_rec->{policy_id};
    if (! $idmap->{'__private_save'} || ! $idmap->{'__private_save'}->{save_ip_unification} ||
           ! exists $idmap->{'__private_save'}->{save_ip_unification}->{$ip_mask_key}) {
        $idmap->{'__private_save'}->{save_ip_unification}->{$ip_mask_key} = { ip_address => $white_ip_rec->{ip_address},
                                                                              mask => $white_ip_rec->{mask},
                                                                              policy_id => $white_ip_rec->{policy_id},
                                                                              flg_never_learn => 0,
                                                                              flg_pb_trusted => 0,
                                                                              flg_ignore_anomaly => 0,
                                                                              }; 
    }

    $idmap->{'__private_save'}->{save_ip_unification}->{$ip_mask_key}->{$whitelist_ip_type} = 1; 
}

#----------------------------------------------------------------------

sub after_all_tables_complete {
    my($self, %params) = @_;
    my $idmap = $params{idmap};

    my $dbname = $params{dbname} || 'PLC';
    if ($idmap && $idmap->{'__private_save'} && $idmap->{'__private_save'}->{save_ip_unification}) {
        F5::DbUtils::insert_row(dbh => $params{dbh},
                                table_name => "$dbname.PL_IP_WHITELIST",
                                rec => [ values %{$idmap->{'__private_save'}->{save_ip_unification}} ], 
                                do_replace => 1,
                               );

        delete $idmap->{'__private_save'}->{save_ip_unification};
    }

}


1;

