package F5::CommonUpgrade::LayeredSectionTables;

use strict;
use warnings;

use base 'F5::CommonUpgrade::Base';

use F5::ASMConfig::Entity::PBConfigCookie;
use F5::ASMConfig::Entity::PBConfigFiletype;
use F5::ASMConfig::Entity::PBConfigParameter;
use F5::ASMConfig::Entity::PBConfigUrl;
use F5::ASMConfig::Entity::PBConfigHeader;
use F5::ASMConfig::Entity::PBConfigRedirectionProtection;
use F5::ASMConfig::Entity::PBConfigSessionsAndLogins;
use F5::ASMConfig::Entity::PolicyAttributes;
use F5::ASMConfig::Entity::PolicyAntivirus;
use F5::ASMConfig::Entity::PolicyCookieSettings;
use F5::ASMConfig::Entity::PolicyHeaderSettings;

use F5::DbUtils;

our $tables_handled = [ qw(PLC.PL_POLICY_BUILDER_CONFIG PLC.PL_POLICIES PLC.PL_POLICY_NEGSIG_ATTRIBUTES ) ];
our $max_version = '13.0.0';

my %table_mapping = ( 'PLC.PL_POLICIES' => [ 'F5::ASMConfig::Entity::PolicyAttributes',
                                             'F5::ASMConfig::Entity::PolicyAntivirus',
                                             'F5::ASMConfig::Entity::PolicyCookieSettings',
                                             'F5::ASMConfig::Entity::PolicyHeaderSettings' ],
                      'PLC.PL_POLICY_NEGSIG_ATTRIBUTES' => [ 'F5::ASMConfig::Entity::PolicyAttributes' ],
                      'PLC.PL_POLICY_BUILDER_CONFIG' => [ 'F5::ASMConfig::Entity::PBConfigCookie',
                                                          'F5::ASMConfig::Entity::PBConfigFiletype',
                                                          'F5::ASMConfig::Entity::PBConfigParameter',
                                                          'F5::ASMConfig::Entity::PBConfigUrl',
                                                          'F5::ASMConfig::Entity::PBConfigHeader',
                                                          'F5::ASMConfig::Entity::PBConfigRedirectionProtection',
                                                          'F5::ASMConfig::Entity::PBConfigSessionsAndLogins' ]
                    );

sub row_fixup {
    my($self, %params) = @_;

    my $row = $params{row};
    my $idmap = $params{idmap};
    my $common_upgrade = $params{common_upgrade};
    my $table_handled = $params{orig_table_name};
    my $new_entities = $table_mapping{$table_handled};

    my $policy_id = defined $params{policy_id} ? $params{policy_id}
                  : defined $row->{policy_id} ? $row->{policy_id}
                  : $row->{id};


    for my $new_entity (@{$new_entities}) {
        my $db_table = $new_entity->db_table();
            my ($db,$table) = split(/\./, $db_table, 2);
            my $fields = $new_entity->fields();
            for my $field (values %$fields) {
                my $db_field = $field->{db_field_name};
                if( defined($db_field) && defined($row->{$db_field})) {
                    $idmap->{'__private_save'}->{move_table_fields}->{$table}->{$policy_id}->{$db_field} = $row->{$db_field};
                }
            }
            if(exists $idmap->{'__private_save'}->{move_table_fields}->{$table}) {
                $idmap->{'__private_save'}->{move_table_fields}->{$table}->{$policy_id}->{policy_id} = $policy_id;
            }
    }
}

#----------------------------------------------------------------------
sub after_all_tables_complete {
    my($self, %params) = @_;
    my $idmap = $params{idmap};
    my $dbh = $params{dbh};
    my $dbname = $params{dbname} || 'PLC';

    if ($idmap && $idmap->{'__private_save'} && $idmap->{'__private_save'}->{move_table_fields}) {
        my @db_tables = keys %{$idmap->{'__private_save'}->{move_table_fields}};
        for my $db_table (@db_tables) {
            my @policies = keys %{$idmap->{'__private_save'}->{move_table_fields}->{$db_table}};
            for my $policy (@policies) {
               F5::DbUtils::insert_row(dbh => $dbh,
                                        table_name => "$dbname.$db_table",
                                        rec =>  $idmap->{'__private_save'}->{move_table_fields}->{$db_table}->{$policy},
                                        do_replace => 1,
                                       );

            }
        }
        delete $idmap->{'__private_save'}->{move_table_fields};
    }
}

#----------------------------------------------------------------------

1;
