package F5::CommonUpgrade::LoginObjects;

use base 'F5::CommonUpgrade::Base';
use F5::DbUtils;

our $max_version = '11.0.1';
our $tables_handled = [ 
                       qw(
                             PLC.PL_LOGIN_BRUTE_FORCE_OBJECT_ATTRIBUTES
                        )
                      ];

our $tables_renamed = { 'PLC.PL_PREREQUISITE_OBJECTS' => 'PLC.PL_LOGIN_OBJECTS',
                      };

sub row_fixup {
    my($self, %params) = @_;

    my $row = $params{row};
    my $idmap = $params{idmap};

    my $object_id = $row->{object_id};
    if ($idmap && $idmap->{obj_id} && $idmap->{obj_id}->{$object_id}) {
        $object_id = $idmap->{obj_id}->{$object_id};
    }

    if ($object_id) {
        if (! $idmap->{'__private_save'} || ! $idmap->{'__private_save'}->{save_brute_force}) {
            $idmap->{'__private_save'}->{save_brute_force} = [];
        }

        push(@{$idmap->{'__private_save'}->{save_brute_force}},
             { object_id => $object_id,
               row => $row,
             });
    }
}

#----------------------------------------------------------------------

sub after_all_tables_complete {
    my($self, %params) = @_;
    my $idmap = $params{idmap};

    my $dbname = $params{dbname} || 'PLC';
    if ($idmap && $idmap->{'__private_save'} && $idmap->{'__private_save'}->{save_brute_force}) {

        for my $brute_force (@{$idmap->{'__private_save'}->{save_brute_force}}) {
            my $object_id = $brute_force->{object_id};
            if (ref($object_id)) {
                $object_id = $object_id->get_value;
            }

            F5::DbUtils::update_row(dbh => $params{dbh},
                                    table_name => "$dbname.PL_LOGIN_OBJECTS",
                                    where_fields => { object_id => $object_id },
                                    rec => $brute_force->{row},
                                    fields => [ qw(username_param password_param auth_type) ],
                                   );
        }

        delete $idmap->{'__private_save'}->{save_brute_force};
    }
}

1;

