package F5::CommonUpgrade::LoginPageDetection;

use strict;
use warnings;

use base 'F5::CommonUpgrade::Base';
use F5::DbUtils;

our $tables_handled = [ qw(
                             PLC.PL_POLICY_BUILDER_CONFIG
                          ) ];
our $max_version = '12.0.0';
sub row_fixup {
    my($self, %params) = @_;

    my $row = $params{row};
    if ( $row->{flg_learn_from_responses} ) {
        $row->{flg_learn_login_pages} = 1;
    }
    else{
        $row->{flg_learn_login_pages} = 0;
    }
    
}

#----------------------------------------------------------------------

sub after_all_tables_complete {
    my($self, %params) = @_;
    my $idmap = $params{idmap};
    my $dbh    = $params{dbh};

    my $dbname = $params{dbname} || 'PLC';
    local $" = ',';
    my $policies_with_login_objects = $dbh->selectall_hashref(<<"EndQuery", 'policy_id');
SELECT distinct policy_id
  FROM $dbname.PL_LOGIN_OBJECTS
EndQuery

    my $policy_ids = ($params{policy_id} eq "*") ?  $dbh->selectcol_arrayref("SELECT id FROM $dbname.PL_POLICIES") : [$params{policy_id}];

    for my $policy_id ( @{$policy_ids}) {
            my  $default_brute_force_enabled_flag = $policies_with_login_objects->{$policy_id} ? 0 : 1 ;
            F5::DbUtils::insert_row(dbh => $dbh,
                                    table_name => "$dbname.PL_LOGIN_BRUTE_FORCE_OBJECT_ATTRIBUTES",
                                    rec => {
                                        policy_id               => $policy_id,
                                        object_id               => 0,
                                        flg_is_default          => 1,
                                        flg_enabled             => $default_brute_force_enabled_flag,
                                       },
                                    do_replace => 1,
                                   );

            if ( $policies_with_login_objects->{$policy_id} ){
                F5::DbUtils::update_row(dbh => $dbh,
                                        table_name => "$dbname.PL_POLICY_BUILDER_CONFIG_SESSIONS_AND_LOGINS",
                                        rec => { flg_learn_login_pages => 0 },
                                        where_fields => {  policy_id => $policy_id, },
                                        );
            }
    }

}
#------------------------------------------------------------------------
1;

