package F5::CommonUpgrade::NegsigSignatureReferences;

use JSON;

use base 'F5::CommonUpgrade::Base';
use F5::ASMConfig::Entity::Signature;

our $max_version = '14.0.0';
our $tables_handled = [ qw(PLC.NEGSIG_SIGNATURES) ];

my %sig_references_type_map;

sub row_fixup {
    my($self, %params) = @_;
    my $row = $params{row};
    my $dbh = $params{dbh};
    my $dbname = ($params{databases}) ? $params{databases}->{PLC}->{staging_dbname} : "PLC" ;

    if(!%sig_references_type_map) {
        %sig_references_type_map = reverse %{F5::ASMConfig::Entity::Signature->fields->{sig_references}->{fields}->{reference_type}->{display_value_map}};
    }

    if ($row->{sig_references}) {
        my $raw_sig_references = from_json($row->{sig_references});
        for my $raw_sig_reference (@$raw_sig_references) {
            my($reference_type, $reference_val) = split(/,/, $raw_sig_reference, 2);
            F5::DbUtils::insert_row(dbh => $dbh,
                                    table_name => "$dbname.NEGSIG_SIGNATURE_REFERENCES",
                                    rec =>  { sig_id => $row->{sig_id},
                                              reference_type => $sig_references_type_map{$reference_type},
                                              reference_val => $reference_val },
                                   );
        }
    }
}

#----------------------------------------------------------------------

1;

