package F5::CommonUpgrade::OwaFlag;

use base 'F5::CommonUpgrade::Base';

our $max_version = '11.0.0';
our $tables_handled = [ qw(PLC.PL_POLICY_TEMPLATES_SETTINGS) ];

use constant OWA_2003 => 1;
use constant OWA_2003_ACTIVE_SYNC => 12;
use constant OWA_2007 => 2;
use constant OWA_2007_ACTIVE_SYNC => 11;
use constant OWA_2010 => 16;

my @owa_policy_ids;

sub row_fixup {
    my($self, %params) = @_;
    my $row = $params{row};

    my $policy_id = defined $params{policy_id} ? 
                            $params{policy_id} :
                            $row->{policy_id};

    if (grep {$row->{policy_template_system_id} == $_ } (OWA_2003,OWA_2003_ACTIVE_SYNC,OWA_2007,OWA_2007_ACTIVE_SYNC,OWA_2010)) {
        push @owa_policy_ids, $policy_id;
    }
}

sub after_table_complete {
    my($self, %params) = @_;
    return unless @owa_policy_ids;

    local $" = ',';

    my $dbh    = $params{dbh};
    my $dbname = $params{dbname} || 'PLC';
    my @placeholders = ('?') x scalar @owa_policy_ids;

    $dbh->do(<<EndQuery, undef, @owa_policy_ids)
UPDATE $dbname.PL_POLICIES
   SET flg_is_owa = 1
 WHERE id IN (@placeholders)
EndQuery
}

1;
