package F5::CommonUpgrade::PBConfigCombined;

use base 'F5::CommonUpgrade::Base';

use F5::DbUtils;
use F5::Violation;
use F5::ASMConfig::Entity::PBConfig;

our $max_version = '11.7.0';
our $tables_handled = [ qw(PLC.PL_POLICY_BUILDER_CONFIG) ];
our $priority = 11; # must run after F5::CommonUpgrade::Cookies

use constant DEFAULT_COLLAPSE_OCCURRENCE => 10;

sub row_fixup {
    my($self, %params) = @_;

    my $idmap  = $params{idmap};
    my $row = $params{row};
    my $policy_id = defined $params{policy_id} ?
                            $params{policy_id} :
                            $row->{policy_id};


    if (defined $row->{flg_content_profile_classification}) {
        $row->{flg_classify_urls} = $row->{flg_content_profile_classification};
        $row->{flg_classify_parameters} = $row->{flg_content_profile_classification};
        $row->{flg_classify_websocket_urls} = $row->{flg_content_profile_classification};
    }

    my $collapse_occurrences = DEFAULT_COLLAPSE_OCCURRENCE;
    my $collapse_flg = 0;
    if (defined $row->{level_global_threshold}) {
        $collapse_flg = ($row->{level_global_threshold} > 1);
        if ($collapse_flg) {
            $collapse_occurrences = $row->{level_global_threshold};
        }
    }
    $row->{collapse_parameters_occurrences} = $row->{collapse_cookies_occurrences} = $collapse_occurrences;
    $row->{collapse_parameters} = $row->{collapse_cookies} = $collapse_flg;

    if ($row->{flg_enabled}) {
        if (defined($row->{flg_signatures})) {
            $idmap->{'__private_save'}->{save_flg_signatures}->{$policy_id} = $row->{flg_signatures};
        }
        for my $field (keys %F5::ASMConfig::Entity::PBConfig::field_to_viol_mapping) {
            if (defined($row->{$field})) {
                push (@{$idmap->{'__private_save'}->{save_learn_viols}->{$policy_id}->{$row->{$field}}}, @{$F5::ASMConfig::Entity::PBConfig::field_to_viol_mapping{$field}});
            }
        }
    }
}

#------------------------------------------------------------------------------

sub after_all_tables_complete {
    my($self, %params) = @_;

    my $dbh    = $params{dbh};
    my $idmap  = $params{idmap};
    my $dbname = $params{dbname} || 'PLC';
    my $common_upgrade = $params{common_upgrade};

    my $modified_domain_cookie_viol_index = F5::Violation::VIOL_COOKIE_MODIFIED();

    my $policy_ids = ($params{policy_id} eq "*") ? $dbh->selectcol_arrayref("SELECT id FROM $dbname.PL_POLICIES") : [$params{policy_id}] ;
    for my $policy_id (@{$policy_ids}) {
        #------------------------------
        my $modified_domain_cookie_viol =  $dbh->selectrow_hashref(<<EndQuery, undef, $policy_id, $modified_domain_cookie_viol_index) || {};
SELECT flg_learn
  FROM $dbname.PL_BLOCKING
 WHERE policy_id = ?
   AND viol_index = ?
EndQuery
        #------------------------------

        #------------------------------
        my $orig_wild_cookie_enforcement_mode = '';
        if ( $common_upgrade->is_old_version_before('11.4.0') &&
             exists $idmap->{'__private_save'}->{orig_wild_cookie_enforcement_mode}->{$policy_id}
           ) {
            $orig_wild_cookie_enforcement_mode = $idmap->{'__private_save'}->{orig_wild_cookie_enforcement_mode}->{$policy_id} || '';
        }
        else {
            my $orig_wild_cookie =  $dbh->selectrow_hashref(<<EndQuery, undef, $policy_id, '*', 1) || {};
SELECT cookie_enforcement_mode
  FROM $dbname.PL_ALLOWED_COOKIES
 WHERE policy_id = ?
   AND cookie_name = ?
   AND flg_is_pattern = ?
EndQuery
            $orig_wild_cookie_enforcement_mode = $orig_wild_cookie->{cookie_enforcement_mode} || '';
        }
        #------------------------------

        my $flg_enforce_unmodified_cookies = 0;
        if ($modified_domain_cookie_viol->{flg_learn} && $orig_wild_cookie_enforcement_mode eq 'allow') {
            $flg_enforce_unmodified_cookies = 1;
        }

        F5::DbUtils::update_row(dbh => $dbh,
                                table_name => "$dbname.PL_POLICY_BUILDER_CONFIG_COOKIE",
                                where_fields => {policy_id => $policy_id,
                                                },
                                rec => {flg_enforce_unmodified_cookies => $flg_enforce_unmodified_cookies,
                                       },
                               );
    }

    if ($idmap->{'__private_save'}->{save_flg_signatures}) {
        for my $policy_id (keys %{$idmap->{'__private_save'}->{save_flg_signatures}}) {
            F5::DbUtils::update_row(dbh => $dbh,
                                    table_name => "$dbname.PL_POLICY_NEGSIG_SETS",
                                    where_fields => { policy_id => $policy_id, },
                                    rec => { flg_learn => $idmap->{'__private_save'}->{save_flg_signatures}->{$policy_id} },
                                   );
        }
    }

    if ($idmap->{'__private_save'}->{save_learn_viols}) {
        for my $policy_id (keys %{$idmap->{'__private_save'}->{save_learn_viols}}) {
            for my $flg_value (keys %{$idmap->{'__private_save'}->{save_learn_viols}->{$policy_id}}) {
                F5::DbUtils::update_row(dbh => $dbh,
                                        table_name => "$dbname.PL_BLOCKING",
                                        where_fields => { policy_id => $policy_id,
                                                          viol_index => $idmap->{'__private_save'}->{save_learn_viols}->{$policy_id}->{$flg_value} },
                                        rec => {flg_learn => $flg_value},
                                       );
            }
        }
    }

    if ($idmap && $idmap->{'__private_save'}) {
        for my $private_save_key (qw(save_flg_signatures
                                     save_learn_viols
                                     orig_wild_cookie_enforcement_mode
                                    )) {

            delete $idmap->{'__private_save'}->{$private_save_key};
        }
    }
}
1;
