package F5::CommonUpgrade::PolicyEncodingMapping;

use base 'F5::CommonUpgrade::Base';

our $tables_handled = [ qw(PLC.PL_POLICIES DCC.ACCOUNTS) ];

our $lang_set_id_encoding_mapping;

sub row_fixup {
    my($self, %params) = @_;

    my $row = $params{row};
    my $idmap = $params{idmap};

    my $policy_id = $row->{policy_id} || $row->{id};

    if ($row->{encoding}) {
        # PLC.PL_POLICIES encoding stored directly (post-11.1.0, introduced by Single Policy support)
        $idmap->{'__private_save'}->{encoding}->{$policy_id} = $row->{encoding};
        $idmap->{'__private_save'}->{flg_encoding_selected}->{$policy_id} = $row->{flg_encoding_selected};
    }
    elsif ($row->{account_id}) {
        if ($row->{lang_set_id}) {
            # DCC.ACCOUNTS lang_set_id (pre-11.1.0)
            $idmap->{'__private_save'}->{account_encoding}->{$row->{account_id}} = get_encoding_for_lang_set_id(%params,
                                                                                                                lang_set_id => $row->{lang_set_id});
            $idmap->{'__private_save'}->{account_flg_encoding_selected}->{$row->{account_id}} = $row->{flg_encoding_selected};
        }
        else {
            # PLC.PL_POLICIES account_id for policy (pre-11.1.0)
            $idmap->{'__private_save'}->{encoding}->{$policy_id} = $idmap->{'__private_save'}->{account_encoding}->{$row->{account_id}};
            $idmap->{'__private_save'}->{flg_encoding_selected}->{$policy_id} = $idmap->{'__private_save'}->{account_flg_encoding_selected}->{$row->{account_id}};
        }
    }
}

#----------------------------------------------------------------------

sub get_encoding_for_lang_set_id {
    my(%params) = @_;

    if (! $lang_set_id_encoding_mapping) {
        $lang_set_id_encoding_mapping = { @{$params{dbh}->selectcol_arrayref(<<EndQuery, { Columns => [1,2] })} };
SELECT lang_set_id, encoding
  FROM DCC.LANGUAGE_ENCODINGS lang_enc,
       DCC.ENCODINGS enc
 WHERE lang_enc.flg_visible = 1
   AND lang_enc.encoding_id = enc.encoding_id
EndQuery
    }

    return $lang_set_id_encoding_mapping->{$params{lang_set_id}};
}

1;
