package F5::CommonUpgrade::PolicyFolder;

use base 'F5::CommonUpgrade::Base';

use F5::ASM::COAPI;
use F5::CRC;

our $max_version = '11.4.0';
our $tables_handled = [ qw(PLC.PL_POLICIES DCC.ACCOUNTS DCC.ACCOUNT_DOMAINS) ];
our $tables_renamed = { 'DCC.ACCOUNT_DOMAINS' => '', };

sub row_fixup {
    my($self, %params) = @_;

    my $row = $params{row};
    my $idmap = $params{idmap};
    my $policy_name = $row->{name} || $row->{domain_name};
    return unless $policy_name;
    if (! $idmap || ! $idmap->{'__private_save'} ||
        ! $idmap->{'__private_save'}->{hsl_accounts} ||
        ! $idmap->{'__private_save'}->{hsl_accounts}->{$row->{account_id}}) {

        if ($policy_name !~ m|^/|) {
            my $asm_policy = F5::ASM::COAPI::Query->new('asm_policy',
                                                       {
                                                        leaf_name => $policy_name });
            if ($asm_policy && @$asm_policy) {
                $policy_name = $asm_policy->[0]->{name};
            }
            else {
                $policy_name = '/Common/' . $policy_name;
            }
        }
    }
    delete $row->{flg_flowless_model};
    if (exists $row->{name}) {
        $row->{name} = $policy_name;
    }
    if (exists $row->{domain_name}) {
        $row->{policy_name} = $policy_name;
        $row->{policy_name_crc} = F5::CRC::get_crc($policy_name);
    }
}

1;
