package F5::CommonUpgrade::ScannerVuln;

use base 'F5::CommonUpgrade::Base';
use F5::DbUtils;
use F5::ASMConfig::Entity::ScannerVuln;

our $max_version = '11.1.0';
our $tables_handled =[ qw(PLC.PL_SENTINEL_ATTACK_VECTORS)];
our $tables_renamed = { 'PLC.PL_SENTINEL_ATTACK_VECTORS' => 'PLC.PL_SCANNER_VULNS', };

sub row_fixup {
    my($self, %params) = @_;

    my $row = $params{row};

    $row->{vuln_id} = delete $row->{attack_vector_id};
    $row->{scanner_attack_vector_id} = delete $row->{sentinel_attack_vector_id};
    $row->{scanner_vuln_id} = delete $row->{vulnerability_id};
    $row->{attack_vector_status} = delete $row->{sentinel_attack_vector_status};
    $row->{flg_is_resolvable} = F5::ASMConfig::ScannerVulnType::SCANNER_VULN_TYPE_RESOLVABLE();
}


sub after_all_tables_complete {
    my($self, %params) = @_;
    my $idmap = $params{idmap};

    my $dbname = $params{dbname} || 'PLC';
    if ($idmap && $idmap->{'__private_save'} && $idmap->{'__private_save'}->{pl_sentinel_vulnerabilities}) {

        for my $row (@{$idmap->{'__private_save'}->{pl_sentinel_vulnerabilities}}) {

            my $vulnerability_id = delete $row->{vulnerability_id};
            $row->{scanner_vuln_id} = delete $row->{sentinel_vulnerability_id};
            $row->{asm_vuln_status} = delete $row->{asm_vulnerability_status};
            $row->{scanner_vuln_status} = delete $row->{sentinel_vulnerability_status};
            $row->{load_time} = time();
            $row->{scanner_vuln_name} = delete $row->{sentinel_vulnerability_name};
            $row->{scanner_vuln_display_name} = $row->{scanner_vuln_name};


            F5::DbUtils::update_row(dbh => $params{dbh},
                                    table_name => "$dbname.PL_SCANNER_VULNS",
                                    where_fields => { policy_id => $row->{policy_id}, 
                                                      scanner_vuln_id => $vulnerability_id},
                                    rec => $row,
                                   );
        }

        delete $idmap->{'__private_save'}->{pl_sentinel_vulnerabilities};
    }
}



1;
