package F5::CommonUpgrade::SessionAwarenessDelayBlockingViolations;

use strict;
use warnings;

use base 'F5::CommonUpgrade::Base';

use F5::DbUtils;
use F5::Violation;
use F5::Violation::Utils;

our $tables_handled = [ qw(PLC.PL_BLOCKING) ];
our $max_version = '11.6.0';

sub row_fixup {
    my($self, %params) = @_;

    my $row = $params{row};
    my $idmap = $params{idmap};
    my $common_upgrade = $params{common_upgrade};

    my $policy_id = defined $params{policy_id} ?
                            $params{policy_id} :
                            $row->{policy_id};
    my $viol_index;
    if (exists $row->{code} && ! exists $row->{viol_index}) {
        $viol_index = F5::Violation::Utils::get_viol_index_from_code($row->{code});
    }else{
        $viol_index = $row->{viol_index};
    }

    if (defined $row->{flg_session_awareness}){
        $idmap->{'__private_save'}->{session_awareness_feature_exists} = 1;
    }

   my $action;
    if ($row->{flg_session_awareness}) {
        $action = 'add_violations';
    }
    else{
        $action = 'delete_violations';
    }

    push(@{$idmap->{'__private_save'}->{save_delay_blocking_violations}->{$action}}, { policy_id => $policy_id,
                                                                                       viol_index => $viol_index});

}

#----------------------------------------------------------------------
sub after_all_tables_complete {
    my($self, %params) = @_;
    my $idmap = $params{idmap};
    my $dbname = $params{dbname} || 'PLC';
    my $dbh = $params{dbh};

    if ($idmap && $idmap->{'__private_save'} && $idmap->{'__private_save'}->{save_delay_blocking_violations}->{add_violations}) {
        F5::DbUtils::insert_row(dbh => $dbh,
                                table_name => "$dbname.PL_SESSION_AWARENESS_VIOLATIONS",
                                rec =>  $idmap->{'__private_save'}->{save_delay_blocking_violations}->{add_violations},
                                do_replace => 1,
                               );

        delete $idmap->{'__private_save'}->{save_delay_blocking_violations}->{add_violations};
    }

    if ($idmap && $idmap->{'__private_save'} && $idmap->{'__private_save'}->{save_delay_blocking_violations}->{delete_violations}) {
        for my $delete_violation (@{$idmap->{'__private_save'}->{save_delay_blocking_violations}->{delete_violations}}){
            F5::DbUtils::delete_row(dbh => $dbh,
                                    table_name => "$dbname.PL_SESSION_AWARENESS_VIOLATIONS",
                                    rec =>  $delete_violation,
                                   );
        }

        delete $idmap->{'__private_save'}->{save_delay_blocking_violations}->{delete_violations};
    }
}

#----------------------------------------------------------------------

1;

