package F5::CommonUpgrade::SessionAwarenessLoginObjcts;

use strict;
use warnings;

use base 'F5::CommonUpgrade::Base';

use F5::DbUtils;
use F5::Violation;
use F5::Violation::Utils;

our $tables_handled = [ qw(PLC.PL_LOGIN_OBJECTS) ];
our $max_version = '11.6.0';
our $priority = 20; #Must happen before F5::CommonUpgrade::SessionAwarenessSettings

sub row_fixup {
    my($self, %params) = @_;

    my $row = $params{row};
    my $idmap = $params{idmap};
    my $common_upgrade = $params{common_upgrade};

    my $policy_id = defined $params{policy_id} ?
                            $params{policy_id} :
                            $row->{policy_id};

    my $object_id = $row->{object_id};
    if ($idmap && $idmap->{obj_id} && $idmap->{obj_id}->{$object_id}) {
        $object_id = $idmap->{obj_id}->{$object_id};
    }

    if ($object_id) {
        if ($row->{flg_has_session_awareness}) {
            if (! $idmap->{'__private_save'} || ! $idmap->{'__private_save'}->{save_session_awarenes_login_pages}) {
                $idmap->{'__private_save'}->{save_session_awarenes_login_pages} = [];
            }

            push(@{$idmap->{'__private_save'}->{save_session_awarenes_login_pages}}, { policy_id => $policy_id,
                                                                                       object_id => $object_id});
        }

    }
}
#----------------------------------------------------------------------
sub after_all_tables_complete {
    my($self, %params) = @_;
    my $idmap = $params{idmap};
    my $dbname = $params{dbname} || 'PLC';


    if ($idmap && $idmap->{'__private_save'} && $idmap->{'__private_save'}->{save_session_awarenes_login_pages}) {
        for my $login_page (@{$idmap->{'__private_save'}->{save_session_awarenes_login_pages}}) {
            my $object_id = $login_page->{object_id};
            if (ref($object_id)) {
                $login_page->{object_id} = $object_id->get_value;
            }
        }

        F5::DbUtils::insert_row(dbh => $params{dbh},
                                table_name => "$dbname.PL_SESSION_AWARENESS_LOGIN_OBJECTS",
                                rec =>  $idmap->{'__private_save'}->{save_session_awarenes_login_pages} ,
                                do_replace => 1,
                               );

        delete $idmap->{'__private_save'}->{save_session_awarenes_login_pages}
    }

}

#----------------------------------------------------------------------
1;

