package F5::CommonUpgrade::SessionAwarenessSettings;

use strict;
use warnings;

use base 'F5::CommonUpgrade::Base';
use F5::DbUtils;

our $tables_handled = [ qw(PLC.PL_SESSION_AWARENESS_SETTINGS) ];
our $max_version = '11.6.0';

sub row_fixup {
    my($self, %params) = @_;

    my $row = $params{row};
    if ( $row->{flg_use_apm_username} ){
        $row->{application_username_type} = F5::ASMConfig::ApplicationUsernameType::APPLICATION_USERNAME_APM();
    }
    else{
        $row->{application_username_type} = F5::ASMConfig::ApplicationUsernameType::APPLICATION_USERNAME_NONE();
    }

    # if session awareness is turned OFF change the block_all_period
    #    so that it will NOT default to 'Infinite'
    # otherwise, we don't want to affect it
    if (! $row->{flg_session_awareness_on}) {
        $row->{flg_check_block_all_period} = 1;
    }
}
#----------------------------------------------------------------------

sub after_all_tables_complete {
    my($self, %params) = @_;
    my $idmap = $params{idmap};
    my $dbh    = $params{dbh};
    my $policy_id = $params{policy_id};

    my $dbname = $params{dbname} || 'PLC';
    local $" = ',';
    my $policy_ids = $dbh->selectall_hashref(<<"EndQuery", 'policy_id', undef);
SELECT policy_id
  FROM $dbname.PL_SESSION_AWARENESS_LOGIN_OBJECTS
EndQuery

    my $where_clause = '1';

    if ( !(keys %$policy_ids) ){
        return;
    }

    if ($policy_id && $policy_id ne "*") {
        if ( $policy_ids->{$policy_id} ) {
            $where_clause = " policy_id = $policy_id";
        }
        else {
            return;
        }
    }
    else{
        my @ids = keys %$policy_ids;
        $where_clause = " policy_id IN (@ids)";
    }

   $dbh->do(<<EndQuery, undef,  F5::ASMConfig::ApplicationUsernameType::APPLICATION_USERNAME_LOGIN_PAGES() );
UPDATE $dbname.PL_SESSION_AWARENESS_SETTINGS 
   SET application_username_type = ?
 WHERE $where_clause
EndQuery

}
#------------------------------------------------------------------------
1;
