package F5::CommonUpgrade::SessionAwarenessViolations;

use strict;
use warnings;

use base 'F5::CommonUpgrade::Base';

our $tables_handled = [ qw(PLC.PL_SESSION_AWARENESS_VIOLATIONS) ];

my %exclude_session_awareness_viol_ids;

sub row_fixup {
    my($self, %params) = @_;

    my $dbh = $params{dbh};
    my $row = $params{row};
    my $row_state_rv = $params{row_state_rv};

    # fill once on first row
    if (!%exclude_session_awareness_viol_ids) {
        my $h_ref = $dbh->selectall_hashref(<<EndQuery, "viol_index");
SELECT viol_index
  FROM PLC.VIOLATIONS
  WHERE flg_exclude_session_awareness = 1
EndQuery

        %exclude_session_awareness_viol_ids = %$h_ref;
    }

    if ($row->{viol_index} && $exclude_session_awareness_viol_ids{$row->{viol_index}}) {
        $$row_state_rv = 'skip';
        return;
    }
}

#------------------------------------------------------------------------------

sub after_all_tables_complete {
    my($self, %params) = @_;

    my $dbh = $params{dbh};
    my $policy_id = $params{policy_id};
    my $dbname = $params{dbname} || 'PLC';

    if ($policy_id && $policy_id eq "*") {

        $dbh->do(<<EndQuery);
DELETE FROM $dbname.PL_SESSION_AWARENESS_VIOLATIONS
 WHERE policy_id NOT IN (SELECT id FROM $dbname.PL_POLICIES)
EndQuery

    }

}

#----------------------------------------------------------------------

1;
