package F5::CommonUpgrade::SetNegsigSystemId;

use base 'F5::CommonUpgrade::Base';

our $tables_handled = [ qw(PLC.NEGSIG_SET_FILTER_SYSTEMS PLC.NEGSIG_SIGNATURE_SYSTEMS) ];
our $PSM_SYSTEM_ID = 36; # PSM has been removed and this system id is deprecated

sub row_fixup {
    my($self, %params) = @_;

    my $row = $params{row};
    my $idmap = $params{idmap};
    my $row_state_rv = $params{row_state_rv};
    my $common_upgrade = $params{common_upgrade} || {};
    my $logger = $params{logger} || $common_upgrade->{logger};

    my $key_field;
    if (exists $row->{set_id}) {
        $key_field = 'set_id';
    }
    if (exists $row->{sig_id}) {
        $key_field = 'sig_id';
    }

    if (exists $idmap->{system_id}->{$row->{system_id}}) {
        my $new_system_id = $idmap->{system_id}->{$row->{system_id}};
        if ($key_field) {
            # NEGSIG_SYSTEMS does not have a unique key and, due to careless handling in previous versions, a signature
            # or set may reference both instances of the same system. If a duplicate row is found, it must be skipped
            # to avoid an duplicate row error.

            my $unique_system_occurrence = join(',', $key_field, $row->{$key_field}, $new_system_id);
            if ($idmap->{"__private_save"} &&
                $idmap->{"__private_save"}->{negsig_system_id} &&
                $idmap->{"__private_save"}->{negsig_system_id}->{$unique_system_occurrence}) {

                $$row_state_rv = 'skip';
                $logger->info("Duplicate usage of system skipped (system_id = $new_system_id, $key_field = $row->{$key_field})") if $logger;
            }
            $idmap->{"__private_save"}->{negsig_system_id}->{$unique_system_occurrence} = 1;
        }
    }
    else {
        $$row_state_rv = 'skip';
        if ($row->{system_id} != $PSM_SYSTEM_ID) {
            $self->{logger}->warn("Unrecognized signature system id ($row->{system_id})") if $self->{logger};
        }
    }
}

1;
