package F5::CommonUpgrade::SetPolicyTemplateId;

use Memoize;

use base 'F5::CommonUpgrade::Base';
use F5::DbUtils;

our $min_version = '10.1.0';
our $tables_handled = [ qw(PLC.PL_POLICY_TEMPLATES_SETTINGS) ];

memoize(F5::CommonUpgrade::SetPolicyTemplateId::prepare_template_mapping);
sub row_fixup {
    my($self, %params) = @_;

    $self->{template_mapping} = $self->prepare_template_mapping();
    my $row = $params{row};
    $row->{policy_template_id} = $self->guess_template_id($row->{policy_template_system_id}) unless defined $row->{policy_template_id}; 
}

sub guess_template_id {
    my ($self, $system_id) = @_;
    return $self->{template_mapping}->{$system_id};
}

sub prepare_template_mapping {
    my $self = shift;
    my $dbh = F5::DbUtils::get_dbh(); # create a new dbh, but that is not that bad because it is a "read only" db handle

    my $hashref = $dbh->selectall_hashref(<<EndQuery, "policy_template_system_id");
SELECT id, policy_template_system_id 
  FROM PLC.POLICY_TEMPLATES
  WHERE policy_template_system_id IS NOT NULL
EndQuery

    my $template_mapping = {};
    for my $template_record (values %$hashref) {
        $template_mapping->{$template_record->{policy_template_system_id}} = $template_record->{id};
    }
    return $template_mapping;
}


1;
