package F5::CommonUpgrade::SetTightening;

use base 'F5::CommonUpgrade::Base';
use F5::ASMConfig::ASMConfig;

our $max_version = '13.1.0';
our $tables_handled = [ qw(
                           PLC.PL_ALLOWED_COOKIES
                           PLC.PL_FILETYPES
                           PLC.PL_REDIRECTION_DOMAINS
                           PLC.PL_OBJECTS
                           PLC.PL_WEBSOCKET_URLS
                          ),
                      ];
my %table_mapping = (
                        'PLC.PL_ALLOWED_COOKIES' => 'cookie_name',
                        'PLC.PL_FILETYPES' => 'name',
                        'PLC.PL_REDIRECTION_DOMAINS' => 'domain_name',
                        'PLC.PL_OBJECTS' => 'uri',
                        'PLC.PL_WEBSOCKET_URLS' => 'uri',
                    );

sub row_fixup {
    my($self, %params) = @_;
    my $row = $params{row};
    my $orig_table_name = $params{orig_table_name};
    my $db_name_field = $table_mapping{$orig_table_name};

    if ( $row->{$db_name_field} && $row->{$db_name_field} ne '*' && $row->{flg_is_pattern} ) {
        $row->{flg_tightening} = F5::ASMConfig::TighteningMode::TIGHTENING_MODE_NEVER();
    }
}

sub after_all_tables_complete {
    my($self, %params) = @_;
    my $dbname = $params{dbname} || 'PLC';
    my $dbh = $params{dbh};
    my $policy_ids = ($params{policy_id} eq "*") ?  $dbh->selectcol_arrayref("SELECT id FROM $dbname.PL_POLICIES") : [$params{policy_id}];

    for my $policy_id (@{$policy_ids}) {
        $dbh->do(<<EndQuery, undef, F5::ASMConfig::TighteningMode::TIGHTENING_MODE_NEVER(), $policy_id, "*");
UPDATE $dbname.PL_PARAM_ATTRIBUTES 
  SET flg_tightening = ?
  WHERE id IN ( SELECT id FROM (SELECT pa.id FROM $dbname.PL_PARAM_ATTRIBUTES pa
      JOIN $dbname.PL_GLOBAL_PARAMS gp ON pa.id = gp.param_attrib_id
      WHERE gp.flg_is_pattern = 1 AND pa.policy_id = ? AND gp.name != ?
     ) subquery)
EndQuery
    }
}

1;
