package F5::CommonUpgrade::TighteningMode;

use base 'F5::CommonUpgrade::Base';
use F5::DbUtils;
use F5::BigipUtils;

our $max_version = '11.3.0';
our $tables_handled = [ 
                     qw(PLC.PL_OBJECTS
                        PLC.PL_PARAM_ATTRIBUTES
                        PLC.PL_FILETYPES
                        PLC.PL_POLICY_BUILDER_CONFIG
                       ), 
                      ];

sub after_all_tables_complete {
    my($self, %params) = @_;

    my $dbh    = $params{dbh};
    my $policy_id = $params{policy_id};
    my $dbname = $params{dbname} || 'PLC';

    my @mappings = ( {
                       pb_config_field => "flg_urls",
                       name_field      => "uri",
                       entity_table    => "$dbname.PL_OBJECTS",
                       pbconfig_table  => "$dbname.PL_POLICY_BUILDER_CONFIG_URL",
                     },
                     {
                       pb_config_field => "flg_parameters",
                       name_field      => "name",
                       entity_table    => "$dbname.PL_PARAM_ATTRIBUTES",
                       join_table      => ", $dbname.PL_GLOBAL_PARAMS gp",
                       join_clause     => "AND gp.param_attrib_id = entity_table.id",
                       sqlite_where_name_clause => "AND id = (SELECT param_attrib_id FROM $dbname.PL_GLOBAL_PARAMS WHERE name = ? AND $dbname.PL_GLOBAL_PARAMS.policy_id = $dbname.PL_PARAM_ATTRIBUTES.policy_id)",
                       pbconfig_table  => "$dbname.PL_POLICY_BUILDER_CONFIG_PARAMETER",
                     },
                     {
                       pb_config_field => "flg_filetypes",
                       name_field      => "name",
                       entity_table    => "$dbname.PL_FILETYPES",
                       pbconfig_table  => "$dbname.PL_POLICY_BUILDER_CONFIG_FILETYPE",
                     },
                   );

    my $policy_where_clause = ($policy_id eq "*") ? ''
                                                  : "AND pb_config.policy_id = $policy_id";
    my $sqlite_policy_where_clause = ($policy_id eq "*") ? '' : "AND policy_id = $policy_id";

    for my $mapping (@mappings) {
        my $pb_config_field = $mapping->{pb_config_field};
        my $name_field      = $mapping->{name_field};
        my $entity_table    = $mapping->{entity_table};
        my $pbconfig_table  = $mapping->{pbconfig_table};
        my $join_table      = $mapping->{join_table} || "";
        my $join_clause     = $mapping->{join_clause} || "";

        if (F5::DbUtils::is_sqlite($dbh)) {
            my $sqlite_where_name_clause = $mapping->{sqlite_where_name_clause} || "AND $entity_table.$name_field = ?";
            $dbh->do(<<EndQuery, undef, "*");
    UPDATE $pbconfig_table
    SET $pb_config_field = COALESCE((SELECT flg_tightening
                                       FROM $entity_table
                                      WHERE $pbconfig_table.policy_id = $entity_table.policy_id
                                            $sqlite_where_name_clause),
                                    $pb_config_field)
    WHERE
      0 = (SELECT flg_enabled
           FROM $dbname.PL_POLICY_BUILDER_CONFIG pb_config
           WHERE $pbconfig_table.policy_id = pb_config.policy_id)
      $sqlite_policy_where_clause
EndQuery

            $dbh->do(<<EndQuery, undef, "*");
    UPDATE $entity_table
    SET flg_tightening = (SELECT $pb_config_field
                            FROM $pbconfig_table pbconfig_table
                           WHERE pbconfig_table.policy_id = $entity_table.policy_id
                         )
    WHERE
      1 = (SELECT flg_enabled
           FROM $dbname.PL_POLICY_BUILDER_CONFIG pb_config
           WHERE $entity_table.policy_id = pb_config.policy_id)
      $sqlite_where_name_clause
      $sqlite_policy_where_clause
EndQuery
        }
        else {

            $dbh->do(<<EndQuery, undef, "*");
    UPDATE $dbname.PL_POLICY_BUILDER_CONFIG pb_config, $pbconfig_table pbconfig_table, $entity_table entity_table 
           $join_table
    SET $pb_config_field = IF(flg_enabled, $pb_config_field, flg_tightening),
        flg_tightening   = IF(flg_enabled, $pb_config_field, flg_tightening)
    WHERE $name_field = ?
      AND pb_config.policy_id = entity_table.policy_id
      AND pbconfig_table.policy_id = pb_config.policy_id 
      $policy_where_clause
      $join_clause
EndQuery
        }
    }
}

1;

