package F5::CommonUpgrade::UniqueName;

use strict;
use warnings;

use base 'F5::CommonUpgrade::Base';

use F5::DbUtils;

my %tables = (
              'PLC.PL_FILETYPES' => { unique_name_field => 'name' },
              'PLC.PL_GLOBAL_PARAMS' => { unique_name_field => 'name',
                                          trim_spaces => 1, },
              'PLC.PL_OBJECT_PARAMS' => { unique_name_field => 'name',
                                          additional_unique_record_parts => [ 'obj_id' ],
                                          trim_spaces => 1, },
              'PLC.PL_FLOW_PARAMS' => { unique_name_field => 'name',
                                        additional_unique_record_parts => [ 'flow_id' ],
                                        trim_spaces => 1, },
              'PLC.PL_ALLOWED_COOKIES' => { unique_name_field => 'cookie_name' },
             );

our $tables_handled = [ keys %tables ];
our $max_version = '11.5.0';

my $UNLIKELY_DELIMITER = '###UNLIKELY_DELIMITER###';

sub row_fixup {
    my($self, %params) = @_;

    my $row = $params{row};
    my $idmap = $params{idmap};
    my $common_upgrade = $params{common_upgrade} || {};
    my $logger = $params{logger} || $common_upgrade->{logger};
    my $orig_table_name = $params{orig_table_name};

    my $unique_name_field = $tables{$orig_table_name}->{unique_name_field};

    if ($tables{$orig_table_name}->{trim_spaces}) {
        $row->{$unique_name_field} =~ s/^\s+|\s+$//g; #remove leading and trailing spaces
    }

    if (! $idmap->{'__private_save'} ||
        ! $idmap->{'__private_save'}->{$orig_table_name} ||
        ! $idmap->{'__private_save'}->{$orig_table_name}->{$row->{policy_id}}) {

        $idmap->{'__private_save'}->{$orig_table_name}->{$row->{policy_id}} = {};
    }
    my $current_policy_cache = $idmap->{'__private_save'}->{$orig_table_name}->{$row->{policy_id}};

    my $row_key;
    if ($tables{$orig_table_name}->{additional_unique_record_parts}) {
        $row_key = join($UNLIKELY_DELIMITER, $row->{$unique_name_field},
                        map { $row->{$_} } @{$tables{$orig_table_name}->{additional_unique_record_parts}});
    }
    else {
        $row_key = $row->{$unique_name_field};
    }

    if ($current_policy_cache->{$row_key}) {
        my $row_state_rv = $params{row_state_rv};
        $$row_state_rv = 'skip';
        $logger->debug("Duplicate record for $orig_table_name skipped ($unique_name_field = $row->{$unique_name_field})") if $logger;
        return;
    }

    $current_policy_cache->{$row_key} = 1;
}

#------------------------------------------------------------------------------

1;
