package F5::CommonUpgrade::UrlClassification;

use strict;
use warnings;

use base 'F5::CommonUpgrade::Base';

use F5::DbUtils;

our $tables_handled = [ qw( PLC.PL_OBJECT_CONTENT_PROFILES ) ];
our $max_version = '13.1.0';

sub row_fixup {
    my($self, %params) = @_;

    my $row = $params{row};
    my $idmap = $params{idmap};

    my $policy_id = defined $params{policy_id} ? $params{policy_id} : $row->{policy_id};

    if (defined $row->{flg_in_classification} && defined $row->{obj_id}
         && !defined $idmap->{'__private_save'}->{save_flg_classification}->{$policy_id}->{$row->{obj_id}}) {

        my $object_id = $row->{obj_id};
        if (ref $object_id) {
            $object_id = $row->{obj_id}->get_value;
        }
        elsif ($idmap && $idmap->{obj_id} && $idmap->{obj_id}->{$object_id}) {
            $object_id = $idmap->{obj_id}->{$object_id};
        }
        $idmap->{'__private_save'}->{save_flg_classification}->{$policy_id}->{$object_id} = $row->{flg_in_classification};
    }
}

sub after_all_tables_complete {
    my($self, %params) = @_;
    my $idmap = $params{idmap};
    my $dbh = $params{dbh};
    my $dbname = $params{dbname} || 'PLC';

    if ($idmap && $idmap->{'__private_save'} && $idmap->{'__private_save'}->{save_flg_classification}) {
        my @policies = keys %{$idmap->{'__private_save'}->{save_flg_classification}};
        for my $policy (@policies) {
            my @objects = keys %{$idmap->{'__private_save'}->{save_flg_classification}->{$policy}};
            for my $object (@objects) {
                F5::DbUtils::update_row(dbh => $dbh,
                                        table_name => "$dbname.PL_OBJECTS",
                                        where_fields => {policy_id => $policy,
                                                         id => $object},
                                        rec => {flg_in_classification => $idmap->{'__private_save'}->{save_flg_classification}->{$policy}->{$object}},);
            }
        }
        delete $idmap->{'__private_save'}->{save_flg_classification};
    }
}

#----------------------------------------------------------------------

1;
