package F5::CommonUpgrade::UserDefinedViolationsIndexes;

use base 'F5::CommonUpgrade::Base';
use F5::ASMConfig::ASMConfig;


our $max_version = '16.1.0';
our $tables_handled = [ qw(
                             PLC.VIOLATIONS_PROPERTIES
                             PLC.PL_BLOCKING
                             PLC.PL_SECURITY_LEVEL_DEFAULTS
                          ) ];

sub row_fixup {
    my($self, %params) = @_;
    my $row = $params{row};
    my $idmap = $params{idmap};

    if ($idmap && $idmap->{'__private_save'}) {
        if (defined $idmap->{'__private_save'}->{viol_index}
              && defined $idmap->{'__private_save'}->{viol_index}->{$row->{viol_index}}) {
            $row->{viol_index} = $idmap->{'__private_save'}->{viol_index}->{$row->{viol_index}};
        }
        elsif ( defined $idmap->{'__private_save'}->{deleted_viol_index}
              && defined $idmap->{'__private_save'}->{deleted_viol_index}->{$row->{viol_index}} ) {
            my $row_state_rv = $params{row_state_rv};
            $$row_state_rv = 'skip';
        }
    }
}

#----------------------------------------------------------------------

sub after_all_tables_complete {
    my($self, %params) = @_;
    my $dbh    = $params{dbh};
    my $dbname = $params{dbname} || 'PLC';

    my $indexes = $dbh->selectcol_arrayref("SELECT viol_index FROM $dbname.VIOLATIONS_PROPERTIES");
    my $values = join(',', @$indexes);
    $dbh->do("INSERT INTO $dbname.VIOLATIONS_PROPERTIES SELECT * FROM PLC.VIOLATIONS_PROPERTIES_DEFAULTS WHERE viol_index NOT IN ($values)", undef);
}

#----------------------------------------------------------------------

1;
