package F5::CommonUpgrade::WebsocketBlocking;

use strict;
use warnings;

use base 'F5::CommonUpgrade::Base';

use F5::Violation;
use F5::DbUtils;

our $tables_handled = [ qw(PLC.PL_BLOCKING) ];
our $max_version = '12.1.0';

my @websocket_violations = (
    F5::Violation::VIOL_WEBSOCKET_BAD_REQUEST(),
    F5::Violation::VIOL_WEBSOCKET_FRAMING_PROTOCOL(),
    F5::Violation::VIOL_WEBSOCKET_FRAME_MASKING(),
    F5::Violation::VIOL_WEBSOCKET_FRAME_LENGTH(),
    F5::Violation::VIOL_WEBSOCKET_TEXT_NULL_VALUE(),
    F5::Violation::VIOL_CROSS_ORIGIN_REQUEST(),
    F5::Violation::VIOL_WEBSOCKET_TEXT_MESSAGE_NOT_ALLOWED(),
    F5::Violation::VIOL_WEBSOCKET_BINARY_MESSAGE_NOT_ALLOWED(),
    F5::Violation::VIOL_WEBSOCKET_EXTENSION(),
    F5::Violation::VIOL_WEBSOCKET_FRAMES_PER_MESSAGE_COUNT(),
    F5::Violation::VIOL_WEBSOCKET_BINARY_MESSAGE_LENGTH(),
    F5::Violation::VIOL_PLAINTEXT_FORMAT(),
);


#------------------------------------------------------------------------------

sub after_all_tables_complete {
    my($self, %params) = @_;

    my $dbh    = $params{dbh};
    my $dbname = $params{dbname} || 'PLC';
    my $policy_id = $params{policy_id};

    local $" = ',';
    my @placeholders = ('?') x scalar @websocket_violations;
    my $sql =<<EndQuery;
UPDATE $dbname.PL_BLOCKING
   SET flg_learn = 0, flg_alarm = 0, flg_reject = 0
 WHERE viol_index IN (@placeholders)
EndQuery

    if ($policy_id ne '*') {
        $sql .= ' and policy_id = ?';
        $dbh->do($sql, undef, @websocket_violations, $policy_id);
    }
    else {
        $dbh->do($sql, undef, @websocket_violations);
    }
}

#------------------------------------------------------------------------------


1;
