package F5::CommonUpgrade::WhitehatFlag;

use base 'F5::CommonUpgrade::Base';
use F5::DbUtils;

our $max_version = '12.0.0';

our $tables_handled = [ qw(
                           PLC.PL_PARAM_ATTRIBUTE_ATTACKS
                           PLC.PL_POLICIES
                           
                           )
                     ];

our $tables_renamed = { 
                        'PLC.PL_PARAM_ATTRIBUTE_ATTACKS' => '',
                      };


sub row_fixup {
    my($self, %params) = @_;
    my $row = $params{row};
    my $idmap = $params{idmap};

    my $policy_id = $params{policy_id} || $row->{policy_id} || $row->{id};

    if ($row->{param_id} || $row->{flg_is_whitehat}) {
        #Came from PL_PARAM_ATTRIBUTE_ATTACKS so there can't be any scanner config yet, or is a whitehat policy
        if ( (!$idmap->{'__private_save'}) || (!$idmap->{'__private_save'}->{save_create_sentinel_config}) ||
               (!exists $idmap->{'__private_save'}->{save_create_sentinel_config}->{$policy_id})) {
            $idmap->{'__private_save'}->{save_create_sentinel_config}->{$policy_id} = 1;
        }
    }
}

#----------------------------------------------------------------------

sub after_all_tables_complete {
    my($self, %params) = @_;
    my $idmap = $params{idmap};

    my $dbname = $params{dbname} || 'PLC';
    my $dbh = $params{dbh};
    my $policies_with_scanner_config = $dbh->selectall_hashref(<<"EndQuery", 'policy_id');
SELECT distinct policy_id
  FROM $dbname.PL_SCANNER_CONFIG
EndQuery

    my @policy_ids;
    if ($idmap && $idmap->{'__private_save'}->{force_whitehat} && $params{policy_id}) {
        @policy_ids = ($params{policy_id});

        delete $idmap->{'__private_save'}->{force_whitehat};
    }
    elsif ($idmap && $idmap->{'__private_save'} && $idmap->{'__private_save'}->{save_create_sentinel_config}) {
        @policy_ids = keys %{$idmap->{'__private_save'}->{save_create_sentinel_config}};

        delete $idmap->{'__private_save'}->{save_create_sentinel_config};
    }

    for my $policy_id (@policy_ids) {
         if (!$policies_with_scanner_config->{$policy_id}) {
             F5::DbUtils::insert_row(dbh => $params{dbh},
                                     table_name => "$dbname.PL_SCANNER_CONFIG",
                                     rec => {policy_id => $policy_id, scanner_type => 'sentinel'},
                                    );
         }
         elsif ($policies_with_scanner_config->{$policy_id} && $policies_with_scanner_config->{$policy_id}->{scanner_type} eq '') {
             F5::DbUtils::update_row(dbh => $params{dbh},
                                     table_name => "$dbname.PL_SCANNER_CONFIG",
                                     rec => {scanner_type => 'sentinel'},
                                     where_fields => {policy_id => $policy_id},
                                    );
         }
    }
}


1;

