package F5::CommonUpgrade::WhitelistIp;

use base 'F5::CommonUpgrade::Base';

use F5::DbUtils;
use F5::IpAddr;
use F5::ASMConfig::Entity::WhitelistIp;

our $max_version = '13.0.0';
our $tables_handled = [ qw(PLC.PL_IP_WHITELIST) ];


sub row_fixup {
    my($self, %params) = @_;

    my $row = $params{row};
    my $idmap = $params{idmap};
    my $policy_id = defined $params{policy_id} ? $params{policy_id} : $row->{policy_id};

    $row->{block_ip} = $row->{flg_never_block} ?
                                                 F5::ASMConfig::BlockIpType::BLOCK_IP_NEVER() :
                                                 F5::ASMConfig::BlockIpType::BLOCK_IP_POLICY_DEFAULT();

    if (! F5::ASMConfig::Entity::WhitelistIp->is_ip_mask_consistent(ip_address => $row->{ip_address}, mask => $row->{mask})) {
        $row->{rest_uuid} = '';
        my $ip_address = $row->{ip_address};
        my $route_domain;
        if ($ip_address =~ s/(\%.*)//) { #remove route domain
            $route_domain = $1;
        }
        $ip_address = convert_ip_mask($ip_address, $row->{mask});
        $row->{ip_address} = $ip_address. $route_domain;
    }

    $idmap->{__private_save}->{whitelist_ips}->{$policy_id} ||= {};
    my $entry_map = $idmap->{__private_save}->{whitelist_ips}->{$policy_id};
    my $entry_key = $row->{ip_address}. '/'. $row->{mask};

    if (exists $entry_map->{$entry_key}) {
        my $exist_row = $entry_map->{$entry_key}->{row};
        for my $flg_entry_name ( grep {$_ =~ /^flg_/} keys %$exist_row) {
            if ($exist_row->{$flg_entry_name} != $row->{$flg_entry_name}) {
                $entry_map->{$entry_key}->{updated} = 1;
                $exist_row->{$flg_entry_name} ||= $row->{$flg_entry_name};
            }
        }

        my $row_state_rv = $params{row_state_rv};
        $$row_state_rv = 'skip';
    } else {
        $entry_map->{$entry_key} = {updated => 0, row => $row};
    }

}

#----------------------------------------------------------------------

sub convert_ip_mask {
    my ($ip, $mask) = @_;

    my $ip_bin = F5::IpAddr::ip_pton($ip);
    my $mask_bin = F5::IpAddr::ip_pton($mask);

    my $res_ip_bin = $ip_bin & $mask_bin;
    my $res_ip = F5::IpAddr::ip_ntop($res_ip_bin);

    return $res_ip;
}

#----------------------------------------------------------------------

sub after_all_tables_complete {
    my($self, %params) = @_;

    my $idmap = $params{idmap};
    my $dbname = $params{dbname} || 'PLC';
    my $dbh = $params{dbh};

    for my $policy_id (keys %{$idmap->{__private_save}->{whitelist_ips}}) {
        my $entry_map = $idmap->{__private_save}->{whitelist_ips}->{$policy_id};
        for my $entry_key (keys %$entry_map) {
            my $entry = $entry_map->{$entry_map};
            if ($entry->{updated}) {
                my $row = $entry->{row};

                $row->{block_ip} = $row->{flg_never_block} ?
                                                 F5::ASMConfig::BlockIpType::BLOCK_IP_NEVER() :
                                                 F5::ASMConfig::BlockIpType::BLOCK_IP_POLICY_DEFAULT();

                F5::DbUtils::update_row(dbh => $dbh,
                                        table_name => "$dbname.PL_IP_WHITELIST",
                                        rec => $row,
                                        where_fields => {
                                                         policy_id => $policy_id,
                                                         ip_address => $row->{ip_address},
                                                         mask => $row->{mask},
                                                        },
                                       );
            }

            delete $entry_map->{$entry_key};
        }
    }
}

#----------------------------------------------------------------------

1;

