package F5::CommonUpgrade::XmlProfileValidationFiles;

use base 'F5::CommonUpgrade::Base';
use F5::ASMConfig::Entity::XmlValidationFile;

our $max_version = '11.6.0';
our $tables_handled = [ qw(PLC.PL_XML_PROFILE_VALIDATION_FILES) ];

my %FILE_TYPE_MAPPING = ( 'xsd' => F5::ASMConfig::XmlValidationFileType::XML_VALIDATION_FILE_TYPE_XSD(),
                          'xsd_soap' => F5::ASMConfig::XmlValidationFileType::XML_VALIDATION_FILE_TYPE_XSD_SOAP(),
                          'wsdl' => F5::ASMConfig::XmlValidationFileType::XML_VALIDATION_FILE_TYPE_WSDL(),);

sub row_fixup {
    my($self, %params) = @_;

    my $row = $params{row};

    # Avoid creating a 'NULL' 'file_contents' on 'PL_XML_VALIDATION_FILES' when importing a problematic policy
    if(not defined $row->{validation_file_contents}) {
        my $row_state_rv = $params{row_state_rv};
        $$row_state_rv = 'skip';
        return;
    }

    my $dbh = $params{dbh};
    my $dbname = ($params{databases}) ? $params{databases}->{PLC}->{staging_dbname} : "PLC" ;
    my $policy_id = $params{policy_id} || $row->{policy_id};

    my $xml_validation_file_id;
    my $validation_file = F5::ASMConfig::Entity::XmlValidationFile->new( object => { file_name =>$row->{validation_file_name}, 
                                                                                     file_contents => $row->{validation_file_contents},
                                                                                   }                                           
                                                                        );
    my $rest_uuid = $validation_file->get_rest_uuid();
    F5::DbUtils::insert_row(dbh => $dbh,
                            table_name => "$dbname.PL_XML_VALIDATION_FILES",
                            rec => {
                                    policy_id               => $policy_id,
                                    file_contents           => $row->{validation_file_contents},
                                    file_name               => $row->{validation_file_name},
                                    file_type               => $FILE_TYPE_MAPPING{$row->{validation_file_type}},
                                    rest_uuid               => $rest_uuid,
                                    last_updated            => time(),
                                   },
                            ignore => 1,
                            last_insert_id => \$xml_validation_file_id,
                           );

   if (!$xml_validation_file_id) {
       ($xml_validation_file_id) =  $dbh->selectrow_array(<<EndQuery, undef, $policy_id, $rest_uuid);
SELECT xml_validation_file_id
  FROM $dbname.PL_XML_VALIDATION_FILES
 WHERE policy_id = ?
  AND rest_uuid = ?
EndQuery
   }

   $row->{xml_validation_file_id} = $xml_validation_file_id;
}

#----------------------------------------------------------------------

1;
