package F5::ContentProfile;

use strict;
use warnings;

use XML::Twig;

use File::Path;
use File::Copy;
use File::stat;
use File::Basename;
use Text::Iconv;
use Digest::MD5;
use F5::ParseUri;
use F5::DbUtils;

#------------------------------------------------------------------------------
sub clean_old_files {
  my(%params) = @_;

  my $dir = $params{dir};
  my $max_size = $params{max_size};

  my @files;
  opendir(my $dir_h, $dir);
  while (my $filename  = readdir($dir_h)) {
    next if ($filename =~ /^\.+$/);
    $filename = "$dir/$filename";

    my $stat_obj = stat($filename);
    if (! $stat_obj) {
        next;
    }
    push(@files,
         { filename => $filename,
           'stat' => $stat_obj });
  }
  closedir($dir_h);

  @files = sort { $b->{'stat'}->mtime <=> $a->{'stat'}->mtime } @files;
  my $running_total_size = 0;
  # Remove most recent files (up to max_size total) from list of files to be deleted
  while (@files) {
    $running_total_size += $files[0]->{'stat'}->size;
    last if ($running_total_size > $max_size);
    shift(@files);
  }
  # Delete all other (older) files
  for my $file (@files) {
    unlink $file->{filename};
  }
}
#----------------------------------------------------------------------
sub convert_to_utf8{

  my (%params) = @_ ;

  my $string  = $params{string};
  my $profile_id = $params{profile_id};
  my $dbh = $params{dbh};

  my ($encoding) = $dbh->selectrow_array(<<EndQuery, undef ,$profile_id);
SELECT encoding
  FROM PLC.PL_POLICIES AS policies,
       PLC.PL_CONTENT_PROFILES AS profiles
 WHERE policies.id = profiles.policy_id
   AND profiles.id = ?
EndQuery

  my $converter = Text::Iconv->new($encoding, "utf8");
  my $converted;

  eval { $converted = $converter->convert($string);};
  die "Can't convert $string at F5::ContentProfile::convert_to_utf8" if ($@);

  return $converted;
}
#---------------------------------------------------------------------
sub add_node_with_value {
  my($twig, $path, $value) = @_;
  add_or_find_node($twig, $path)->set_text($value);
}
#----------------------------------------------------------------------
sub add_or_find_node {
  my($root, $path) = @_;

  if (my @found = $root->get_xpath($path)) {
    return $found[0];
  }
  else {
    my $new_node = XML::Twig::Elt->new(basename($path));
    $new_node->paste(last_child => add_or_find_node($root, dirname($path)));
    return $new_node;
  }
}
1;
