package F5::Db::Partition;

use strict;
use warnings;

#----------------------------------------------------------------------

sub retrieve_partitions_info {
    my(%params) = @_;

    my $dbh = $params{dbh};
    my $table_name = $params{table_name};
    my $logger = $params{logger};

    my ($db_name_only,$table_name_only) = split('\.',$table_name);
    my $rv = $dbh->selectall_arrayref(<<EndQuery, { Slice => {} }, $table_name_only, $db_name_only);
SELECT *
  FROM INFORMATION_SCHEMA.PARTITIONS
 WHERE table_name = ?
   AND table_schema = ?
EndQuery

    return $rv;
}

#----------------------------------------------------------------------

sub add_partition {
    my(%params) = @_;

    my $dbh = $params{dbh};
    my $table_name = $params{table_name};
    my $next_partition_first_id = $params{next_partition_first_id};
    my $logger = $params{logger};

    my $partition_name = "p$next_partition_first_id";

    my $sql = "ALTER TABLE $table_name ADD PARTITION (PARTITION $partition_name VALUES LESS THAN ($next_partition_first_id))";
    $logger->trace($sql) if $logger;
    if(! $dbh->do($sql)) {
        $logger->error("Failed to add partition - '$sql': ".DBI::errstr()) if $logger;
    }

    return $partition_name;
}

#----------------------------------------------------------------------

sub delete_partition {
    my(%params) = @_;

    my $dbh = $params{dbh};
    my $table_name = $params{table_name};
    my $partition_name = $params{partition_name};
    my $logger = $params{logger};

    my $sql = "ALTER TABLE $table_name DROP PARTITION $partition_name";
    $logger->trace($sql) if $logger;
    if(! $dbh->do($sql)) {
        $logger->error("Failed to delete partition - '$sql': ".DBI::errstr()) if $logger;
    }
}

#----------------------------------------------------------------------

sub remove_partitioning {
    my(%params) = @_;

    my $dbh = $params{dbh};
    my $table_name = $params{table_name};
    my $logger = $params{logger};

    my $sql = "ALTER TABLE $table_name REMOVE PARTITIONING";
    $logger->trace($sql) if $logger;
    $dbh->do($sql);
}

#----------------------------------------------------------------------

sub setup_partitioning {
    my(%params) = @_;

    my $dbh = $params{dbh};
    my $table_name = $params{table_name};
    my $next_partition_first_id = $params{next_partition_first_id};
    my $logger = $params{logger};

    my $partition_name = "p$next_partition_first_id";
    my $sql = "ALTER TABLE $table_name PARTITION BY RANGE(id) (PARTITION $partition_name VALUES LESS THAN ($next_partition_first_id))";
    $logger->trace($sql) if $logger;
    $dbh->do($sql);

    return $partition_name;
}

#----------------------------------------------------------------------

1;
