package F5::DbFixup::CharsetDefaults;

use base 'F5::DbFixup::Default';

# Package globals, handled by AUTOLOAD
our $fixup_db = 'PLC';

sub after_fixup {
  my($self, $dbh, $old_dbname, $new_dbname, $logger, $params) = @_;
  my $databases = $params->{databases};

  $logger->debug("Begin Charset defaults fixup");
  # Load the default charset for each account for each of: header, URI, parameter name, parameter value

  my %charset_types;
  my $account_charset_query = <<"EndQuery";
SELECT ACC.account_id as account_id, CS.charset_type as charset_type, CS.charset as charset, CS.section_type as section_type, CS.rest_uuid as rest_uuid
    FROM $databases->{DCC}->{staging_dbname}.DEFAULT_CHARSETS AS CS,
         $databases->{DCC}->{staging_dbname}.ACCOUNTS AS ACC,
         $databases->{DCC}->{staging_dbname}.LANGUAGE_ENCODINGS AS LANG_ENC
    WHERE ACC.lang_set_id = LANG_ENC.lang_set_id
      AND LANG_ENC.encoding_id = CS.encoding_id
EndQuery

  my %accounts;
  my $sth = $dbh->prepare($account_charset_query);
  $sth->execute;
  while(my $row = $sth->fetchrow_hashref) {
    $accounts{$row->{account_id}}->{$row->{charset_type}} = $row;
    $charset_types{$row->{charset_type}} = 1;
  }

  # Find all policies that have no charsets and add the default charset.
  
  my $missing_charset_policy_query = <<"EndQuery";
SELECT id, account_id
FROM 
$new_dbname.PL_POLICIES
LEFT JOIN (SELECT policy_id, count(*) as cnt FROM $new_dbname.PL_CHARSETS GROUP BY policy_id) AS A
ON id = A.policy_id
WHERE A.cnt < ? OR A.cnt IS NULL
EndQuery
  
  my $policies = $dbh->selectall_arrayref($missing_charset_policy_query, { Slice => {} }, scalar keys %charset_types);

  my $ignore = F5::DbUtils::get_ignore_string($dbh);
  my $insert_charset_sth = $dbh->prepare("INSERT $ignore INTO $new_dbname.PL_CHARSETS (policy_id, charset_type, charset, section_type, rest_uuid) VALUES (?,?,?,?,?)");

  for my $policy (@$policies) {
    my $policy_id = $policy->{id};
    my $account_id = $policy->{account_id};
    $logger->debug("Found default charsets for policy $policy_id");
    for my $charset_type (keys %{$accounts{$account_id}}) {
      my $row = $accounts{$account_id}->{$charset_type};
      $insert_charset_sth->execute($policy_id, $charset_type, $row->{charset}, $row->{section_type}, $row->{rest_uuid});
    }
  }

  $logger->debug("End Charset defaults fixup");
  return 1;
}

1;
