package F5::DbFixup::CharsetRemapping;

use base 'F5::DbFixup::Default';

our $HEADER_CHARSET_TYPE_CODE = 1;
our $URI_CHARSET_TYPE_CODE = 2;
our $PARAM_NAME_CHARSET_TYPE_CODE = 3;
our $PARAM_VALUE_CHARSET_TYPE_CODE = 4;
our %CHARSET_TYPE_UPGRADE_MAP = (0 => $PARAM_VALUE_CHARSET_TYPE_CODE, 
                                1 => undef,
                                2 => undef,
                                100 => $URI_CHARSET_TYPE_CODE,
                                101 => $HEADER_CHARSET_TYPE_CODE,
                                102 => $PARAM_NAME_CHARSET_TYPE_CODE);

sub charset_remapping {
  my($self, $dbh, $logger, $table, $id_field) = @_;
  # $table must include explicit database name

  $logger->info("Remapping charsets for table $table");
  my $rightpad_charset_query = "UPDATE $table SET charset = RPAD(charset, 256, '0')";
  $dbh->do($rightpad_charset_query);
  
  # Check if charset_type should be updated
  my $row = $dbh->selectall_arrayref("SELECT $id_field FROM $table WHERE charset_type = $PARAM_NAME_CHARSET_TYPE_CODE LIMIT 1");
  if (@$row) {
    $logger->info("Table $table has already been converted -- no action taken");
  }
  else {
    # Remove deleted codes
    for my $from_val (grep(!defined($CHARSET_TYPE_UPGRADE_MAP{$_}), keys %CHARSET_TYPE_UPGRADE_MAP)) {
      $dbh->do("DELETE FROM $table WHERE charset_type = $from_val");
    }

    # Change all others
    for my $from_val (grep(defined($CHARSET_TYPE_UPGRADE_MAP{$_}), keys %CHARSET_TYPE_UPGRADE_MAP)) {
      my $to_val = $CHARSET_TYPE_UPGRADE_MAP{$from_val};
      $dbh->do("UPDATE $table SET charset_type = $to_val WHERE charset_type = $from_val");
    }

    $logger->info("Remapping charsets (table $table) done");
  }
}

1;
