package F5::DbFixup::CrcRecalculate;

use strict;
use warnings;
use Carp;

our @ISA = qw(F5::DbFixup::Default);

# Package globals, handled by AUTOLOAD
our $max_version = '0009.0004.0002.0000.0000';
our $fixup_db = 'PLC';

use constant MAX_BATCH_SIZE => 5000;


use F5::CRC;

sub after_fixup {
  my($self,$dbh,$old_dbname,$new_dbname, $logger, $params) = @_;
  my $databases = $params->{databases};
  
  my %policy2account;
  my @crc_fields = (
		    {
		     table => "$databases->{DCC}->{staging_dbname}.ACCOUNTS",
		     field => 'policy_name_crc',
		     src_fields => [ 'policy_name' ],
		     id_fields => [ 'account_id' ],
		     crc_recalc_handler => sub {
		       my($row) = @_;
		       return F5::CRC::get_crc_any('get_crc', $row->{policy_name});
		     }
		    },
		   );

  $logger->debug("Begin CRC Recalculate fixup");

  my $policy2account_sth = $dbh->prepare("SELECT id, account_id FROM $new_dbname.PL_POLICIES");
  $policy2account_sth->execute;
  while(my $row = $policy2account_sth->fetchrow_hashref) {
    $policy2account{$row->{id}} = $row->{account_id};
  }

  for my $crc_field (@crc_fields) {
    my @id_fields = (defined($crc_field->{id_fields}) ? @{$crc_field->{id_fields}} :  ('id'));
    my $sth = $dbh->prepare("SELECT " . join(',', @{$crc_field->{src_fields}}, @id_fields) . " FROM $crc_field->{table}");
    $sth->execute;
    while(my $rows = $sth->fetchall_arrayref({}, MAX_BATCH_SIZE)) {
      last unless @$rows;

      for my $row (@$rows) {
	my $recalculated_field = $crc_field->{crc_recalc_handler}->($row);
	my $id_where_clause = join(' AND ', map("$_ = ?", @id_fields));
	$dbh->do("UPDATE $crc_field->{table} SET $crc_field->{field} = ? WHERE $id_where_clause",
		 undef, $recalculated_field, @$row{@id_fields});
      }

      last unless $sth->{Active};
    }
  }

  $logger->debug("End CRC Recalculate fixup");
}

1;
