package F5::DbFixup::DeceptionResponsePages;

use strict;
use warnings;
use base 'F5::DbFixup::Default';

our $fixup_db = 'PLC';
our $tables = [ 'PLC.DECEPTION_RESPONSES', 'PLC.DECEPTION_RESPONSES_SIGNATURES' ];

sub after_fixup {
    my ($self, $dbh, $old_dbname, $new_dbname, $logger, $params) = @_;

    my $databases = $params->{databases};
    $logger->debug("Begin Deception Response Pages fixup");

    my ($last_updated) = $dbh->selectrow_array(<<EndQuery);
SELECT IFNULL(MAX(deception_response_id), 0)
  FROM $databases->{$fixup_db}->{bigip_dbname}.DECEPTION_RESPONSES
EndQuery

    my ($staging_last_updated) = $dbh->selectrow_array(<<EndQuery);
SELECT IFNULL(MAX(deception_response_id), 0)
  FROM $databases->{$fixup_db}->{staging_dbname}.DECEPTION_RESPONSES
EndQuery

    return if $staging_last_updated >= $last_updated;

    for my $full_table (@$tables) {
        my($db_type, $table_name) = split(/\./, $full_table);

        $dbh->do(<<EndQuery, undef);
DELETE FROM $databases->{$db_type}->{staging_dbname}.$table_name
EndQuery

        $self->copy_table(dbh => $dbh,
                          old_dbname => $databases->{$db_type}->{bigip_dbname},
                          new_dbname => $databases->{$db_type}->{staging_dbname},
                          table_name => $table_name,
                          id_field_name => '',
                          logger => $logger,);
    }

    $logger->debug("End Deception Response Pages fixup");
}

#---------------------------------------------------------
1;
