package F5::DbFixup::DeceptionTraps;

use strict;
use warnings;
use base 'F5::DbFixup::Default';

our $fixup_db = 'PLC';
our @tables = qw (
                  PLC.DECEPTION_TRAPS
                  PLC.DECEPTION_TRAP_INJECTIONS
                  PLC.DECEPTION_TRAP_INJECTION_HEADERS
                  PLC.DECEPTION_TRAP_CONDITIONS
                 );

sub after_fixup {
    my ($self, $dbh, $old_dbname, $new_dbname, $logger, $params) = @_;

    my $databases = $params->{databases};
    $logger->debug("Begin Deception Traps fixup");

    my ($last_id) = $dbh->selectrow_array(<<EndQuery);
SELECT IFNULL(MAX(deception_trap_id), 0)
  FROM $databases->{$fixup_db}->{bigip_dbname}.DECEPTION_TRAPS
EndQuery

    my ($staging_last_id) = $dbh->selectrow_array(<<EndQuery);
SELECT IFNULL(MAX(deception_trap_id), 0)
  FROM $databases->{$fixup_db}->{staging_dbname}.DECEPTION_TRAPS
EndQuery

    return if $staging_last_id >= $last_id;

    for my $full_table (@tables) {
        my($db_type, $table_name) = split(/\./, $full_table);

        $dbh->do(<<EndQuery, undef);
DELETE FROM $databases->{$db_type}->{staging_dbname}.$table_name
EndQuery

        $self->copy_table(dbh => $dbh,
                          old_dbname => $databases->{$db_type}->{bigip_dbname},
                          new_dbname => $databases->{$db_type}->{staging_dbname},
                          table_name => $table_name,
                          id_field_name => '',
                          logger => $logger,);
    }

    $logger->debug("End Deception Traps fixup");
}

#---------------------------------------------------------

1;
