package F5::DbFixup::GlobalParametersPlc;

use strict;
use warnings;

use F5::GlobalParametersUpgrade qw(params_upgrade_import);
use F5::DbUtils qw(insert_data_to_table foreign_key_scrub);

use constant MAX_BATCH_SIZE => 5000;

our @ISA = qw(F5::DbFixup::Default);

# Package globals, handled by AUTOLOAD
our $max_version = '0009.0004.0000.0000.0000';
our $fixup_db = 'PLC';
our @param_attrib_tables = qw(PLC.PL_STATIC_PARAMS);

our $excluded_tables = [
			qw(PLC.PL_PARAM_ATTRIBUTES
			   PLC.PL_GLOBAL_PARAMS
			   PLC.PL_OBJECT_PARAMS
			   PLC.PL_FLOW_PARAMS
			   PLC.PL_EXTRACT_ATTRIBUTES
			   PLC.PL_GLOBAL_EXTRACTS
			   PLC.PL_GLOBAL_EXTRACT_OBJECT_TYPES
			   PLC.PL_GLOBAL_EXTRACT_OBJECTS
			   PLC.PL_OBJECT_EXTRACTS
			   PLC.PL_DYNAMIC_PARAM_EXTRACTS
			  ),
			@param_attrib_tables
		       ];


sub before_fixup {
  my($self,$dbh,$old_dbname,$new_dbname, $logger) = @_;
  $logger->debug("Begin before fixup for global parameters - foreign key scrub");
  # Foreign keys - this should be in a configuration file
  my @foreign_keys =  ( 'PL_FLOWS.to_id=>PL_OBJECTS.id',
			'PL_FLOWS.from_id=>PL_OBJECTS.id',
			'PL_DYNAMIC_FLOWS.obj_id=>PL_OBJECTS.id',
			'PL_PARAMS.extract_from_object_id=>PL_OBJECTS.id',
			'PL_PARAMS.flow_id=>PL_FLOWS.id',
			'PL_STATIC_PARAMS.param_id=>PL_PARAMS.id',
			'PL_PARAMS_META_CHARS.param_id=>PL_PARAMS.id',
			'PL_PARAMS_META_CHARS.meta_char_id=>PL_ILLEGAL_META_CHARS_POOL.id',
		      );

  if (!grep($_ eq 'extract_from_object_id', F5::DbUtils::get_db_fields($dbh, "$old_dbname.PL_PARAMS"))) {
    @foreign_keys = grep(!/extract_from_object_id/, @foreign_keys);
  }

  foreign_key_scrub($dbh, $old_dbname, $logger, \@foreign_keys);

  $logger->debug("End before fixup for global parameters - foreign key scrub");
}

sub after_fixup {
  my($self,$dbh,$old_dbname,$new_dbname, $logger, $params) = @_;
  $logger->debug("Begin global parameters (PLC) fixup");

  my $databases = $params->{databases};

  my $idmap = {};
  my $sth = $dbh->prepare("SELECT * FROM $old_dbname.PL_PARAMS");
  $sth->execute;
  while(my $rows = $sth->fetchall_arrayref({}, MAX_BATCH_SIZE)) {
    last unless @$rows;
    my $data = { map(($_->{id},$_), @$rows) };
    params_upgrade_import($dbh, $logger, $data, undef, undef, $idmap,
			  { databases => $databases });
    last unless $sth->{Active};
  }
	
  for my $table (@param_attrib_tables) {
    $table =~ s/^.*?\.//;
    my $sth = $dbh->prepare("SELECT * FROM $old_dbname.$table");
    $sth->execute;
    while(my $rows = $sth->fetchall_arrayref({}, MAX_BATCH_SIZE)) {
      last unless @$rows;
      for my $row (@$rows) {
	$row->{param_attrib_id} = $row->{param_id};
      }
      my $data = { map(($_->{id},$_), @$rows) };

      insert_data_to_table(dbh => $dbh,
			   logger => $logger,
			   data => $data,
			   table_name => "$new_dbname.$table",
			   idmap => $idmap,
                           ignore => 1,
                          );
      last unless $sth->{Active};
    }
  }
 
  # set the exported idmap to be used by other fixups
  F5::GlobalParametersUpgrade::set_idmap($idmap);
  $logger->debug("End global parameters (PLC) fixup");
}

#----------------------------------------------------------------------

1;
