package F5::DbFixup::InternalConfigItems;

use strict;
use warnings;
use base 'F5::DbFixup::Default';

use F5::DbUtils;
use F5::Cluster::Utils;

# Package globals, handled by AUTOLOAD
our $fixup_db = 'DCC';
our $excluded_tables = [ 'DCC.INTERNAL_CONFIG_ITEMS' ];

# These INTERNAL_CONFIG_ITEMS are not copied to the new_dbname -- either because
# they should be reset to their factory values (due to bugs like 210771) or
# because they have been removed from the product.
our %non_copied_keys = map { ($_ => 1) } qw(
                                              max_filtered_html_length
                                              PRXRateLimit
                                              MaxJobs
                                              dos_min_detection_ip_threshold
                                              dos_min_detection_object_threshold
                                              cookie_digest_key
                                              max_concurrent_long_request
                                              cs_excluded_headers
                                              cs_excluded_urls
                                         );

sub after_fixup {
    my($self,$dbh,$old_dbname,$new_dbname, $logger, $params) = @_;
    my $idmap = $params->{idmap};

    $logger->debug("Begin Internal Config Items fixup");

    my $new_keys_ref = $dbh->selectcol_arrayref(<<EndQuery);
SELECT field_name
  FROM $new_dbname.INTERNAL_CONFIG_ITEMS
EndQuery

    my %new_keys = map { ($_ => 1) } @$new_keys_ref;

    my $old_sth = $dbh->prepare(<<EndQuery);
SELECT *
  FROM $old_dbname.INTERNAL_CONFIG_ITEMS
EndQuery

    $old_sth->execute;
    while(my $rec = $old_sth->fetchrow_hashref) {

        if ($rec->{field_name} eq "cs_excluded_headers") {
            my $obj = F5::Cluster::Utils::set_mcp_val('dosl7.asm_cs_excluded_headers', $rec->{current_value});
        }
        elsif ($rec->{field_name} eq "cs_excluded_urls") {
            my $obj = F5::Cluster::Utils::set_mcp_val('dosl7.asm_cs_excluded_urls', $rec->{current_value});
        }

        if ($non_copied_keys{$rec->{field_name}}) {
            $idmap->{'__private_save'}->{deleted_internal_config_items}->{$rec->{field_name}} = $rec->{current_value};
            next;
        }

        if ($new_keys{$rec->{field_name}}) {
            F5::DbUtils::update_row(dbh => $dbh,
                                    table_name => "$new_dbname.INTERNAL_CONFIG_ITEMS",
                                    rec => $rec,
                                    fields => [ qw(current_value) ],
                                    where_fields => { field_name => $rec->{field_name} },
                                   );
        }
        else {
            delete $rec->{item_id};
            F5::DbUtils::insert_row(dbh => $dbh,
                                    table_name => "$new_dbname.INTERNAL_CONFIG_ITEMS",
                                    rec => $rec,
                                   );
        }
    }

    $logger->debug("End Internal Config Items fixup");
}

1;
