package F5::DbFixup::NegsigRemoveOldFactorySignatures;

use base 'F5::DbFixup::Default';

# Package globals, handled by AUTOLOAD
our $min_version = '0009.0004.0002.0000.0000';
our $max_version = '0011.0002.0000.0000.0000';
our $fixup_db = 'PLC';

#----------------------------------------------------------------------

sub before_fixup {
    my($self,$dbh,$old_dbname,$new_dbname, $logger, $params) = @_;
    my $databases = $params->{databases};
    $logger->debug("Begin NegsigRemoveOldFactorySignatures before fixup");

    # Delete factory signatures from ucs to force upgrade to new versions
    $dbh->do(<<EndQuery);
DELETE 
  FROM $old_dbname.NEGSIG_SIGNATURE_SYSTEMS
 WHERE sig_id IN (
    SELECT sig_id
      FROM $old_dbname.NEGSIG_SIGNATURES
     WHERE flg_is_user_defined = 0
   )
EndQuery

    $dbh->do(<<EndQuery);
DELETE FROM $old_dbname.NEGSIG_SIGNATURES
      WHERE flg_is_user_defined = 0
EndQuery

    $logger->debug("End NegsigRemoveOldFactorySignatures before fixup");
}

#----------------------------------------------------------------------

sub after_fixup {
    my($self,$dbh,$old_dbname,$new_dbname, $logger, $params) = @_;
    $logger->debug("Begin NegsigRemoveOldFactorySignatures after fixup");

    my $negative_signatures = F5::NegativeSignatures->new($dbh);
    $negative_signatures->add_all_signatures_to_policy(policy_id => '*',
                                                       dbname => $new_dbname);

    $logger->debug("Begin NegsigRemoveOldFactorySignatures after fixup");
}

#----------------------------------------------------------------------

1;
