package F5::DbFixup::NewDccDefaultRows;

use strict;
use warnings;
use base 'F5::DbFixup::Default';

# Package globals, handled by AUTOLOAD
our $fixup_db = 'PLC';
our $max_version = '0013.0001.0000.0000.0000';
our $excluded_tables = [ 'DCC.DATABASE_PROTECTION_SETTINGS', 'DCC.VIRUS_DETECTION_SERVER'];

#Handle "merge" manually as a new index was introduced, and old versions may or may not have had a row
sub after_fixup {   
    my ($self, $dbh, $old_dbname, $new_dbname, $logger, $params) = @_;

    my $databases = $params->{databases};
    $logger->debug("Begin Database Protection / Virus Detection Server fixup");

    for my $full_table (@$excluded_tables) {
        my($db_type, $table_name) = split(/\./, $full_table);
 
        if (F5::DbUtils::check_table_exists(dbh => $dbh, dbname => $databases->{DCC}->{ucs_dbname},  table_name => $table_name)) {

            my ($config_exists) = $dbh->selectrow_array(<<EndQuery, undef);
SELECT COUNT(*) FROM $databases->{DCC}->{ucs_dbname}.$table_name
EndQuery

            next unless $config_exists;

            $dbh->do(<<EndQuery, undef);
DELETE FROM $databases->{PLC}->{staging_dbname}.$table_name
EndQuery

            $self->copy_table(dbh => $dbh,
                          old_dbname => $databases->{DCC}->{ucs_dbname},
                          new_dbname => $databases->{PLC}->{staging_dbname},
                          table_name => $table_name,
                          id_field_name => '',
                          logger => $logger,
                          fixup_callback => sub {
                              my ($dbh, $logger, $data, $new_table_name, $foo, $idmap) = @_;
                              for my $key (keys %$data) {
                                  my $row = $data->{$key};
                                  $row->{hostname} = $row->{server_ip_addr} || $row->{icap_server_ip_addr} || $row->{server_host_name} || $row->{icap_server_host_name} || '';
                              }
                          },
                        );
        }
    }

    $logger->debug("End Database Protection / Virus Detection Server fixup");
}

1;
