package F5::DbFixup::NormalizationDefaults;

use base 'F5::DbFixup::Default';

use F5::DbUtils;
use F5::Violation;

use constant NORMALIZATION_CODE_BAD_UNESCAPING => 0x1000;
use constant NON_RFC_BITMASK_BAD_UNESCAPING => 0x20;

# Package globals, handled by AUTOLOAD
our $fixup_db = 'PLC'; 
our $max_version = '0009.0004.0002.0000.0000';
our $priority = 99;

sub after_fixup {
    my($self,$dbh,$old_dbname,$new_dbname, $logger, $params) = @_;

    $logger->debug("Begin NormalizationDefaults fixup");

    F5::DbUtils::copy_defaults_table(dbh => $dbh,
                                 dbname => $new_dbname,
                                 table_name => 'PL_NORMALIZATION',
                                 policy_id => '*');

    # Extract bad unescaping from non-rfc bitmask in INTERNAL_CONFIG_ITEMS
    my $databases = $params->{databases};
    my($non_rfc_bitmask) = $dbh->selectrow_array(<<EndQuery);
SELECT current_value
  FROM $databases->{DCC}->{staging_dbname}.INTERNAL_CONFIG_ITEMS
 WHERE field_name = 'non_rfc_bitmask'
EndQuery

    my $bad_unescaping = ($non_rfc_bitmask & NON_RFC_BITMASK_BAD_UNESCAPING ? 1 : 0);
    $dbh->do(<<EndQuery, undef, NORMALIZATION_CODE_BAD_UNESCAPING, $bad_unescaping, F5::Violation::VIOL_HTTP_PROTOCOL());
REPLACE INTO $new_dbname.PL_NORMALIZATION
  (policy_id, normalization_code, flg_is_enabled)
SELECT policy_id, ?, ? && (flg_reject || flg_alarm || flg_learn)
  FROM $new_dbname.PL_BLOCKING
 WHERE viol_index = ?
EndQuery

    $logger->debug("End Normalization fixup");
}

1;
