package F5::DbFixup::ServerTechnologies;

use strict;
use warnings;
use base 'F5::DbFixup::Default';

our $fixup_db = 'PLC';
our $tables = [ 'PLC.SERVER_TECHNOLOGIES', 'PLC.SERVER_TECHNOLOGIES_POLICY_ACTIONS', 'PLC.SERVER_TECHNOLOGIES_IMPLIED', 'PLC.SERVER_TECHNOLOGIES_FINGERPRINTS'];

#Retain the latest Server Technologies
sub after_fixup {
    my ($self, $dbh, $old_dbname, $new_dbname, $logger, $params) = @_;

    my $databases = $params->{databases};
    $logger->debug("Begin Server Technologies fixup");

    my ($last_updated) = $dbh->selectrow_array(<<EndQuery);
SELECT IFNULL(MAX(last_updated), 0)
  FROM $databases->{$fixup_db}->{bigip_dbname}.SERVER_TECHNOLOGIES
EndQuery

    my ($staging_last_updated) = $dbh->selectrow_array(<<EndQuery);
SELECT IFNULL(MAX(last_updated), 0)
  FROM $databases->{$fixup_db}->{staging_dbname}.SERVER_TECHNOLOGIES
EndQuery

    if ($staging_last_updated >= $last_updated) {
        my $server_technology_ids = F5::DbUtils::get_column_values(
                                                dbh => $dbh,
                                                field_name => 'server_technology_id',
                                                table_name => $databases->{$fixup_db}->{bigip_dbname}. '.SERVER_TECHNOLOGIES',
                                                where_fields => {'flg_has_deception_responses' => 1 });
        F5::DbUtils::update_row(dbh => $dbh,
                                table_name => $databases->{$fixup_db}->{staging_dbname}. '.SERVER_TECHNOLOGIES',
                                where_fields => { server_technology_id => $server_technology_ids },
                                rec => { flg_has_deception_responses => 1 },);

        return;
    }

    for my $full_table (@$tables) {
        my($db_type, $table_name) = split(/\./, $full_table);

        $dbh->do(<<EndQuery, undef);
DELETE FROM $databases->{$db_type}->{staging_dbname}.$table_name
EndQuery

        $self->copy_table(dbh => $dbh,
                          old_dbname => $databases->{$db_type}->{bigip_dbname},
                          new_dbname => $databases->{$db_type}->{staging_dbname},
                          table_name => $table_name,
                          id_field_name => '',
                          logger => $logger,);
    }

    $logger->debug("End Server Technologies fixup");
}

#---------------------------------------------------------
1;
