package F5::DbFixup::SetAttackTypeId;

use base 'F5::DbFixup::Default';

# Package globals, handled by AUTOLOAD
our $max_version = '0010.0001.0000.0000.0000';
our $min_version = '0009.0004.0002.0000.0000';
our $fixup_db = 'PLC';
our $priority = 99;

sub after_fixup {
  my($self, $dbh, $old_dbname, $new_dbname, $logger) = @_;
  local $" = ',';

  $logger->debug("Begin set attack type id fixup");

  my $query = <<"ENDQUERY";
SELECT sig_id 
  FROM $old_dbname.NEGSIG_SIGNATURES sigs, $old_dbname.NEGSIG_ATTACK_TYPES atypes
 WHERE sigs.attack_type_id=atypes.attack_type_id AND atypes.attack_type_name LIKE '(WH)%'
ENDQUERY

  my $wh_sigs = $dbh->selectcol_arrayref($query);

  $query = <<"ENDQUERY";
SELECT old.attack_type_id as old_id, new.attack_type_id as new_id
  FROM $old_dbname.NEGSIG_ATTACK_TYPES as old, $new_dbname.NEGSIG_ATTACK_TYPES as new
 WHERE old.attack_type_name=new.attack_type_name
ENDQUERY
  my $attack_types_mapping = $dbh->selectall_hashref($query, "old_id");

  my %sigs_by_attack_types = ();
  
  $query = <<"ENDQUERY";
SELECT set_id,attack_type_id
  FROM $new_dbname.NEGSIG_SET_FILTERS
ENDQUERY

  my %set_filters_by_attack_types = ();
  my $set_filters = $dbh->selectall_hashref($query,"set_id");
  for my $set_id (keys %$set_filters) {
     my $attack_type_id = $set_filters->{$set_id}->{attack_type_id};
     push @{$set_filters_by_attack_types{$attack_type_id}}, $set_id;
  }

  for my $old_attack_type_id (keys %sigs_by_attack_types) {
    my $new_attack_type_id = $attack_types_mapping->{$old_attack_type_id}->{new_id};
    my @placeholders = ('?') x scalar @{$sigs_by_attack_types{$old_attack_type_id}}; 
    $dbh->do("UPDATE $new_dbname.NEGSIG_SIGNATURES set attack_type_id=? WHERE sig_id IN (@placeholders)",
             undef, 
             $new_attack_type_id,
             @{$sigs_by_attack_types{$old_attack_type_id}});
  } 
  
  for my $old_attack_type_id (keys %set_filters_by_attack_types) {
    my $new_attack_type_id = $attack_types_mapping->{$old_attack_type_id}->{new_id};
    my @placeholders = ('?') x scalar @{$set_filters_by_attack_types{$old_attack_type_id}};
    $dbh->do("UPDATE $new_dbname.NEGSIG_SET_FILTERS set attack_type_id=? WHERE set_id IN (@placeholders)", 
             undef,
             $new_attack_type_id,
             @{$set_filters_by_attack_types{$old_attack_type_id}});
  } 

  # delete all signatures of whitehat because they don't have the relevant attack type anymore
  # note that using the delete operation of the negsig system won't work because foreign keys are disabled
  if(scalar @$wh_sigs) {

      my @placeholders = ('?') x scalar @$wh_sigs;

      for my $sig_id_table ("NEGSIG_SET_SIGNATURES", "NEGSIG_SIGNATURE_SYSTEMS", "PL_PARAM_NEGSIG_SIGNATURES", "PL_POLICY_NEGSIG_SIGNATURES", "PL_CONTENT_PROFILE_NEGSIG_SIGNATURES", "NEGSIG_SIGNATURES" ) {
          my $affected = $dbh->do("DELETE FROM $new_dbname.$sig_id_table WHERE sig_id IN (@placeholders)",
                         undef,
                         @$wh_sigs);
          if(defined $affected) {
            $logger->debug("deleted $affected whitehat signatures from $sig_id_table");
          } else {
            $logger->info("failed to delete whitehat sigs from $sig_id_table- err $dbh->errstr");
          }
      }
  }
  $logger->debug("End set attack type id fixup");
  
  return 1;
}

1;
