package F5::DbFixup::SetNegsigSystemId;

use strict;
use warnings;

use base 'F5::DbFixup::Default';
use F5::NegativeSignatures::Utils;
use F5::DbUtils;

# Package globals, handled by AUTOLOAD
our $fixup_db = 'PLC';
our $priority = 99;
our $excluded_tables = [ 'PLC.NEGSIG_SYSTEMS' ];

our $PSM_SYSTEM_ID = 36; # PSM has been removed and this system id is deprecated

sub before_fixup {
    my($self, $dbh, $old_dbname, $new_dbname, $logger, $fixup_params) = @_;

    $logger->debug("Begin negsig system fixup");

    return unless F5::DbUtils::check_table_exists(dbh => $dbh,
                                                  dbname => $old_dbname,
                                                  table_name => 'NEGSIG_SYSTEMS');

    my $old_systems = $dbh->selectall_arrayref(<<EndQuery, { Slice => {} });
SELECT system_id, system_name
  FROM $old_dbname.NEGSIG_SYSTEMS
 ORDER BY system_id
EndQuery

    my $systems_idmap = {};

    for my $old_system (@$old_systems) {
        # Skip handling the PSM systems
        next unless $old_system->{system_id} != $PSM_SYSTEM_ID;
        my $new_system_id = F5::NegativeSignatures::Utils::find_or_add_system(dbh => $dbh,
                                                                              dbname => $new_dbname,
                                                                              system_name => $old_system->{system_name});
        $systems_idmap->{$old_system->{system_id}} = $new_system_id;
    }

    $fixup_params->{idmap}->{system_id} = $systems_idmap;

    $logger->debug("End negsig system fixup");

    return 1;
}


1;
