package F5::DbFixup::SigfileAutoUpdate;

use strict;
use warnings;

use F5::BigipVersionUtils;

our @ISA = qw(F5::DbFixup::Default);

# Package globals, handled by AUTOLOAD
our $min_version = '0009.0004.0002.0000.0000';
our $fixup_db = 'PLC';

our $excluded_tables = [ 'PLC.SIGFILE_AUTO_UPDATE_HISTORY',
                         'PLC.SIGFILE_STATUS',
                       ];

sub after_fixup {
  my($self,$dbh,$old_dbname,$new_dbname, $logger, $extra_params) = @_;

  $logger->debug("Begin sigfile auto update");

  my $installed_factory_sigfile_status = $dbh->selectrow_hashref("SELECT * FROM $new_dbname.SIGFILE_STATUS");

  $dbh->do(<<EndQuery);
DELETE FROM $new_dbname.SIGFILE_STATUS
EndQuery

  $self->copy_table(dbh => $dbh,
                    logger => $logger,
                    old_dbname => $old_dbname,
                    new_dbname => $new_dbname,
                    table_name => 'SIGFILE_STATUS',
                    id_field_name => '',
                    fixup_callback => sub {
                      my($dbh, $logger, $data) = @_;
                      for my $k (keys %$data) {
                        $data->{$k}->{sigfile_status_id} = $k + 1;
                        $data->{$k}->{bigip_version} = F5::BigipVersionUtils::zero_unpad_segments($extra_params->{old_version})
                            unless defined $data->{$k}->{bigip_version};
                      }
                    }
                   );

  if ($extra_params->{old_version} lt '0011.0002.0000.0000.0000') {
      # Due to RE2 changes in 11.2.0, factory signatures are retained (rather than using factory signatures from UCS)
      delete $installed_factory_sigfile_status->{sigfile_status_id};

      F5::DbUtils::insert_row(dbh => $dbh,
                              table_name => "$new_dbname.SIGFILE_STATUS",
                              rec => $installed_factory_sigfile_status);
  }


  $self->copy_table(dbh => $dbh,
                    logger => $logger,
                    old_dbname => $old_dbname,
                    new_dbname => $new_dbname,
                    table_name => 'SIGFILE_AUTO_UPDATE_HISTORY',
                    id_field_name => '',
                    fixup_callback => sub {
                      my($dbh, $logger, $data) = @_;
                      for my $k (keys %$data) {
                        $data->{$k}->{sigfile_auto_update_id} = $k + 1;
                      }
                    }
                   );

  $logger->debug("End sigfile auto update");
}

1;
