package F5::DbFixup::UserDefinedViolations;

use strict;
use warnings;

use base 'F5::DbFixup::Default';
use F5::DbUtils;

our $max_version = '0016.0001.0000.0000.0000';
our $fixup_db = 'PLC';

sub before_fixup {
    my($self, $dbh, $old_dbname, $new_dbname, $logger, $fixup_params) = @_;

    $logger->debug("Begin user defined violations fixup");

    return unless F5::DbUtils::check_table_exists(dbh => $dbh,
                                                  dbname => $old_dbname,
                                                  table_name => 'VIOLATIONS');

    my $violations = $dbh->selectall_arrayref(<<EndQuery, { Slice => {} }, 1, $F5::ASMConfig::Entity::Violation::MAX_USER_VIOL_INDEX, 0);
SELECT viol_index, flg_is_deleted, viol_name
  FROM $old_dbname.VIOLATIONS
  WHERE flg_is_user_defined = ? AND viol_index > ? AND flg_is_deleted = ?
  ORDER BY viol_index ASC
EndQuery

    if ($violations) {
        my $violations_idmap = {};

        my $deleted_viols = $dbh->selectall_arrayref(<<EndQuery, undef, 1, $F5::ASMConfig::Entity::Violation::MAX_USER_VIOL_INDEX, 1);
SELECT viol_index, viol_name
  FROM $old_dbname.VIOLATIONS
  WHERE flg_is_user_defined = ? AND viol_index <= ? AND flg_is_deleted = ?
  ORDER BY viol_index ASC
EndQuery

        for my $deleted_viol (@$deleted_viols) {
            my ($new_viol_index, $viol_name) = @$deleted_viol;
            my $viol = shift(@$violations);

            $logger->info("Deleting violation $viol_name (viol_index = $new_viol_index)") if $logger;
            $fixup_params->{idmap}->{'__private_save'}->{deleted_viol_index}->{$new_viol_index} = 1;

            $logger->info("Setting viol_index = $new_viol_index for violation $viol->{viol_name} (old viol_index = $viol->{viol_index})") if $logger;
            $fixup_params->{idmap}->{'__private_save'}->{viol_index}->{$viol->{viol_index}} = $new_viol_index;

            for my $table ( qw(VIOLATIONS PL_SECURITY_LEVEL_DEFAULTS) ) {
                F5::DbUtils::delete_row(dbh => $dbh,
                                        table_name => "$old_dbname.$table",
                                        rec => { viol_index => $new_viol_index },
                                       );


                F5::DbUtils::update_row(dbh => $dbh,
                                        table_name => "$old_dbname.$table",
                                        where_fields => { viol_index => $viol->{viol_index}, },
                                        rec => { viol_index => $new_viol_index },
                                       );
            }
        }

        $violations = $dbh->selectall_arrayref(<<EndQuery, { Slice => {} }, $F5::ASMConfig::Entity::Violation::MAX_USER_VIOL_INDEX);
SELECT viol_index, viol_name, flg_is_deleted FROM $old_dbname.VIOLATIONS
 WHERE viol_index > ?
EndQuery

        for my $viol (@$violations) {
            $fixup_params->{idmap}->{'__private_save'}->{deleted_viol_index}->{$viol->{viol_index}} = 1;
            if (!$viol->{flg_is_deleted}) {
                $logger->info("Deleting violation $viol->{viol_name} (viol_index = $viol->{viol_index}) -- no available index for user defined violation") if $logger;
            }
            F5::DbUtils::delete_row(dbh => $dbh,
                                    table_name => "$old_dbname.VIOLATIONS",
                                    rec => { viol_index => $viol->{viol_index} } );
            F5::DbUtils::delete_row(dbh => $dbh,
                                    table_name => "$old_dbname.PL_SECURITY_LEVEL_DEFAULTS",
                                    rec => { viol_index => $viol->{viol_index} } );
        }
    }

    $logger->debug("End user defined violations fixup");

    return 1;
}

1;
