package F5::DbFixup::ViolationsSeverities;

use strict;
use warnings;
use base 'F5::DbFixup::Default';
use F5::Violation::Utils;
use constant MAX_BATCH_SIZE => 5000;
use F5::DbUtils;

our $fixup_db = 'PLC';
our $max_version = '0011.0005.0000.0000.0000';

our $excluded_tables = [ 'DCC.PARSER_STATUS_CODE_SEVERITIES',
                         'PLC.VIOLATIONS_PROPERTIES',
                       ];

my $DEBUG_SEVERITY = 7;

sub after_fixup {
  my($self,$dbh,$old_dbname,$new_dbname, $logger,$fixup_param_hashref) = @_;
  $logger->debug("Begin Violation Properties fixup");

  $old_dbname = $fixup_param_hashref->{databases}->{DCC}->{ucs_dbname};
  $new_dbname = $fixup_param_hashref->{databases}->{PLC}->{staging_dbname};
  my $old_table_name = 'PARSER_STATUS_CODE_SEVERITIES';
  my $new_table_name = 'VIOLATIONS_PROPERTIES';

  return unless  F5::DbUtils::check_table_exists(dbh => $dbh,
                                                 dbname => $old_dbname,
                                                 table_name => $old_table_name);



  my  $sql  = "INSERT INTO $new_dbname.$new_table_name SELECT * FROM PLC.VIOLATIONS_PROPERTIES_DEFAULTS";
  $dbh->do($sql);

  my $sth = $dbh->prepare("SELECT * FROM $old_dbname.$old_table_name");
  $sth->execute;
  while (my $row = $sth->fetchrow_hashref) {
      if (defined $row->{code}) {
          $row->{viol_index} = F5::Violation::Utils::get_viol_index_from_code($row->{code});
          delete $row->{code};
      }

      if ( $row->{severity_id} == $DEBUG_SEVERITY ){
           $row->{severity_id} = F5::ASMConfig::ViolationSeverity::VIOLATION_SEVERITY_INFO() ;
      }

      if ($row->{viol_index}) {
          F5::DbUtils::update_row(dbh => $dbh,
                                  table_name => "$new_dbname.$new_table_name",
                                  where_fields => { viol_index => $row->{viol_index} },
                                  rec => { severity_id => $row->{severity_id} },
                                  ignore => 1,
                                 );

      }
  }

  $logger->debug("End Violation Properties fixup");

}

1;
