package F5::DbFixup::WebScrapingMigration;

use F5::DbUtils;
use F5::ASM::COAPI;
use F5::Mcp::PolicyTarget;
use F5::Mcp::PolicyAction;
use F5::Mcp::BotDefenseDeviceIdMode;
use F5::ASMConfig::ASMConfig;

our @ISA = qw(F5::DbFixup::Default);

our $min_version = '0012.0001.0000.0000.0000';
our $max_version = '0014.0001.0000.0000.0000';

our $fixup_db = 'PLC';
our $priority = 1000;

my $FACTORY_BOT_DEFENSE_PROFILE = '/Common/bot-defense-device-id-generate-before-access';

sub after_fixup {
    my ($self,$dbh,$old_dbname,$new_dbname,$logger,$params) = @_;

    my $databases = $params->{databases};

    $logger->debug("Begin Web Scraping fixup");

    attach_default_bot_defense_profile(logger => $logger, dbh => $dbh, databases => $databases);

    $logger->debug("End Web Scraping fixup");
}

sub calc_mask{
  my %params = @_;
  my $mask = 0;

  for my $flg_index (1 .. scalar keys %params ){
     if( $params{$flg_index} ){
       $mask |= (2**($flg_index-1));
     }
  }
  return $mask;
}

sub session_awareness_attributes_handler {
    my (%params) = @_;

    my $dbh = $params{dbh};
    my $row = $params{row};
    my $policy_id = $params{policy_id};
    my $logger = $params{logger};
    my $databases = $params{databases};

    $row->{flg_use_apm_username} = ( $row->{application_username_type} == F5::ASMConfig::ApplicationUsernameType::APPLICATION_USERNAME_APM() ) ? 1 : 0;
    my $sa_login_page_table = $row->{application_username_type} == F5::ASMConfig::ApplicationUsernameType::APPLICATION_USERNAME_ALL_LOGIN_PAGES() ? 'PL_LOGIN_OBJECTS' : 'PL_SESSION_AWARENESS_LOGIN_OBJECTS' ;

    $sa_login_page_table = $databases->{PLC}->{ucs_dbname} . '.' . $sa_login_page_table;

    my ($login_page_count) = $dbh->selectrow_array(<<EndQuery, undef, $policy_id);
SELECT count(*)
  FROM $sa_login_page_table
 WHERE policy_id = ?
EndQuery

    for my $field (keys %$row) {
        if ($field =~ /^flg_(.*)_enabled$/) {
            my $threshold_field = $1;

            if ($threshold_field =~ /block_all/ && $row->{flg_block_only_authenticated} && ! $row->{flg_use_apm_username} &&
                $login_page_count == 0) {

                $row->{$field} = 0;
            }

            if ($row->{flg_violations_actions_enabled} == 0 || $row->{$field} == 0) {
                $row->{$threshold_field} = 0;
            }
        }
    }
}

sub get_js_fingerprinting_by_policy_id {
    my(%params) = @_;

    my $logger = $params{logger};
    my $databases = $params{databases};
    my $dbh = $params{dbh};

    # only for policies that are active (account_id != 0)
    my $attributes_by_policy_id = $dbh->selectall_hashref(<<EndQuery, 'policy_id');
SELECT sas.device_block_all_threshold,
       sas.device_block_illegal_threshold,
       sas.device_track_all_threshold,
       sas.flg_session_hijacking,
       sas.application_username_type,
       sas.flg_device_track_all_threshold_enabled,
       sas.flg_device_block_illegal_threshold_enabled,
       sas.flg_device_block_all_threshold_enabled,
       sas.flg_block_only_authenticated,
       sas.flg_violations_actions_enabled,
       plc.name AS policy_name,
       plc.id AS policy_id,
       plc.flg_disable_rejects AS is_policy_transparent,
       wsc.flg_web_scraping_alarm, wsc.flg_session_opening_anomaly_alarm, wsc.flg_suspicious_clients_alarm,
       wsc.flg_web_scraping_block, wsc.flg_session_opening_anomaly_block, wsc.flg_suspicious_clients_block,
       wsc.flg_fingerprinting_enable AS flg_wsc_fingerprint,
       bf.flg_identify_device AS flg_bf_fingerprint
  FROM $databases->{PLC}->{ucs_dbname}.PL_POLICIES plc
  JOIN $databases->{PLC}->{ucs_dbname}.PL_SESSION_AWARENESS_SETTINGS sas ON sas.policy_id = plc.id
  JOIN $databases->{PLC}->{ucs_dbname}.PL_WEB_SCRAPING_SETTINGS wsc ON wsc.policy_id = plc.id
  JOIN $databases->{PLC}->{ucs_dbname}.PL_LOGIN_BRUTE_FORCE_OBJECT_ATTRIBUTES bf ON bf.policy_id = plc.id
 WHERE plc.account_id != 0
EndQuery

    my $viol_web_scraping_by_policy_id = $dbh->selectall_hashref(<<EndQuery, 'policy_id');
SELECT (b.flg_alarm || b.flg_reject) AS viol_web_scraping_enabled,
        b.policy_id
  FROM $databases->{PLC}->{ucs_dbname}.VIOLATIONS a
  JOIN $databases->{PLC}->{ucs_dbname}.PL_BLOCKING b
    ON a.viol_index = b.viol_index
 WHERE viol_name = 'VIOL_WEB_SCRAPING'
EndQuery

    my $js_fingerprinting_by_policy_id;

    for my $policy_id (keys %$attributes_by_policy_id) {

        my $is_web_scraping_viol_enabled = $viol_web_scraping_by_policy_id->{$policy_id}->{viol_web_scraping_enabled};
        my $attributes = $attributes_by_policy_id->{$policy_id};

        session_awareness_attributes_handler(%params, policy_id => $policy_id, row => $attributes);

        $attributes->{flg_sas_fingerprint} = $attributes->{device_block_all_threshold} ||
                                             $attributes->{device_block_illegal_threshold} ||
                                             $attributes->{device_track_all_threshold} ||
                                             $attributes->{flg_session_hijacking};

        # defaults, as defined in relevant DCC tables:
        #   DCC.ACCOUNT_WEB_SCRAPING_DATA.sql:  web_scraping_blocking_mask
        #   DCC.ACCOUNT_EFOXY_SESSION_RESET.sql:  ws_session_open_persistent_mask
        #   DCC.ACCOUNT_SUSPICIOUS_CLIENTS.sql:  ws_session_suspicious_clients_mask
        my $masks = {web_scraping_blocking_mask => 1,
                     ws_session_open_persistent_mask => 1,
                     ws_session_suspicious_clients_mask => 0,
                    };

        if ( $is_web_scraping_viol_enabled ||
             $attributes->{flg_web_scraping_alarm} || $attributes->{flg_session_opening_anomaly_alarm} || $attributes->{flg_suspicious_clients_alarm} ||

            (  ! $attributes->{is_policy_transparent} &&
                 ( $attributes->{flg_web_scraping_block} || $attributes->{flg_session_opening_anomaly_block} || $attributes->{flg_suspicious_clients_block} ) ) ) {

            $masks->{web_scraping_blocking_mask} = calc_mask( 1 => $attributes->{flg_web_scraping_alarm},
                                                            2 => $attributes->{is_policy_transparent} ? 0 : $attributes->{flg_web_scraping_block} );

            $masks->{ws_session_open_persistent_mask} = calc_mask( 1 => $attributes->{flg_session_opening_anomaly_alarm},
                                                                 2 => $attributes->{is_policy_transparent} ? 0 : $attributes->{flg_session_opening_anomaly_block} );

            if ( $attributes->{flg_wsc_fingerprint} ){
                $masks->{ws_session_suspicious_clients_mask} = calc_mask( 1 => $attributes->{flg_suspicious_clients_alarm},
                                                                        2 => $attributes->{is_policy_transparent} ? 0 : $attributes->{flg_suspicious_clients_block} );
            }
        }
        else {
            $masks->{web_scraping_blocking_mask} = 0;
            $masks->{ws_session_open_persistent_mask} = 0;
            $masks->{ws_session_suspicious_clients_mask} = 0;
        }


        if ($masks->{web_scraping_blocking_mask}         == 0 &&
            $masks->{ws_session_open_persistent_mask}    == 0 &&
            $masks->{ws_session_suspicious_clients_mask} == 0) {

            $attributes->{flg_wsc_fingerprint}  = 0;
        }

        if ($attributes->{flg_wsc_fingerprint} || $attributes->{flg_sas_fingerprint} || $attributes->{flg_bf_fingerprint}) {
            $logger->trace("Value of 'flg_wsc_fingerprint' is $attributes->{flg_wsc_fingerprint} for policy id $policy_id") if $logger;
            $logger->trace("Value of 'flg_sas_fingerprint' is $attributes->{flg_sas_fingerprint} for policy id $policy_id") if $logger;
            $logger->trace("Value of 'flg_bf_fingerprint' is $attributes->{flg_bf_fingerprint} for policy id $policy_id") if $logger;
            $js_fingerprinting_by_policy_id->{$policy_id} = {
                                                          enable_js_fingerprint => 1,
                                                          policy_name => $attributes->{policy_name},
                                                        };
        }
    }

    return $js_fingerprinting_by_policy_id;
}

sub attach_default_bot_defense_profile {
    my(%params) = @_;

    my $logger = $params{logger};

    my $js_fingerprinting_by_policy_id = get_js_fingerprinting_by_policy_id(%params);

    my $profile_bot_defense_query = F5::ASM::COAPI::Query->new('profile_bot_defense');
    my %bot_defense_profiles_by_name = map { $_->{name} => $_ } @$profile_bot_defense_query;

    my %seen_virtual_servers = ();
    my %seen_l7_policies = ();

    # The next section implements the 'Device ID DoS Hidden Profiles' section logic
    # of 'Unified Bot Defense: Backward Compatibility' doc, of 'Unified Bot Defense' FS

    $logger->info("Begin attach default bot defense profile to LTM Virtual Servers") if $logger;

    # for every account
    for my $policy_id (keys %$js_fingerprinting_by_policy_id) {
        # only if these specific features are enabled
        next unless $js_fingerprinting_by_policy_id->{$policy_id}->{enable_js_fingerprint};

        my $asm_policy_name = $js_fingerprinting_by_policy_id->{$policy_id}->{policy_name};

        # for every l7 policy attached to asm policy
        $logger->info("ASM Policy $asm_policy_name has DID feature enabled") if $logger;

        for my $l7_policy_action (@{F5::ASM::COAPI::Query->new("policy_action", {target => F5::Mcp::PolicyTarget::L7_POLICY_TARGET_ASM,
                                                                                 action => F5::Mcp::PolicyAction::L7_POLICY_ACTION_ENABLE,
                                                                                 policy => $asm_policy_name})}) {
            my $l7_policy_name = $l7_policy_action->{policy_name};

            # evaluate each L7 policy only once (we can reach same L7 policy from different ASM policies)
            next if exists $seen_l7_policies{$l7_policy_name};
            $seen_l7_policies{$l7_policy_name} = 1;

            $logger->info("L7 Policy $l7_policy_name is referencing ASM Policy $asm_policy_name") if $logger;

            # for every virtual server attached to l7 policy
            for my $virtual_server_policy (@{F5::ASM::COAPI::Query->new("virtual_server_policy", {policy_name => $l7_policy_name})}) {
                my $virtual_server_name = $virtual_server_policy->{vs_name};

                # evaluate each VS only once (we can reach same VS from different L7 policies)
                next if exists $seen_virtual_servers{$virtual_server_name};
                $seen_virtual_servers{$virtual_server_name} = 1;

                $logger->info("LTM Virtual Server $virtual_server_name is referencing L7 Policy $l7_policy_name") if $logger;

                my $skip_vs = 0;

                # for every profile attached to virtual server
                for my $virtual_server_profile (@{F5::ASM::COAPI::Query->new("virtual_server_profile", {vs_name => $virtual_server_name})}) {
                    # this is a full profile name - with the folder
                    my $profile_name = $virtual_server_profile->{profile_name};
                    $logger->info("Evaluating $profile_name profile, that is attached to LTM Virtual Server $virtual_server_name") if $logger;

                    # if the profile is a bot_defense profile, make sure it has correct device_id_mode and skip this VS
                    if (my $bot_defense_profile = $bot_defense_profiles_by_name{$profile_name}) {
                        $logger->info("Profile $profile_name is a bot defense profile") if $logger;

                        my $required_did_mode = F5::Mcp::BotDefenseDeviceIdMode::BOT_DEFENSE_DEVICE_ID_MODE_GENERATE_BEFORE_ACCESS;
                        if ($bot_defense_profile->{device_id_mode} != $required_did_mode) {
                            $logger->info("Enabling 'GENERATE BEFORE ACCESS' mode for profile $profile_name") if $logger;
                            $bot_defense_profile->{device_id_mode} = $required_did_mode;
                            eval {
                                $bot_defense_profile->save();
                            };
                            if ($@) {
                                $logger->error("Cannot enable required DID mode for bot defense profile $profile_name: $@ $F5::COAPI::error") if $logger;
                            }
                        }
                        $skip_vs = 1;
                        last;
                    }
                }

                next if $skip_vs;

                $logger->info("LTM Virtual Server $virtual_server_name has no Bot Defense profiles attached") if $logger;

                # attach the factory Bot Defense profile 'bot-defense-device-id-generate-before-access' to the VS
                my $server_profile = F5::ASM::COAPI::Object->new('virtual_server_profile');
                $server_profile->{vs_name} = $virtual_server_name;
                $server_profile->{profile_name} = $FACTORY_BOT_DEFENSE_PROFILE;
                eval {
                    $server_profile->save();
                };
                if ($@) {
                    $logger->error("Cannot attach profile $FACTORY_BOT_DEFENSE_PROFILE to virtual server $virtual_server_name: $@ $F5::COAPI::error") if $logger;
                }

                $logger->info("Successfully assigned the factory Bot Defense profile $FACTORY_BOT_DEFENSE_PROFILE to the LTM Virtual Server $virtual_server_name") if $logger;
            }
        }
    }

    $logger->info("End attach default bot defense profile to LTM Virtual Servers") if $logger;
}

#----------------------------------------------------------------------

1;
