package F5::DbFixup::WildcardEntity;

use base 'F5::DbFixup::Default';
use F5::WildcardEntityUpgrade;
use F5::GlobalParametersUpgrade qw(get_idmap set_idmap);

# Package globals, handled by AUTOLOAD
our $fixup_db = 'PLC';
our $max_version = '0009.0004.0002.0000.0000';
our $priority = 100;

sub after_fixup {
  my($self,$dbh,$old_dbname,$new_dbname, $logger) = @_;
  $logger->debug("Begin Wildcard Entity fixup");

  # Convert METACHAR tables
  my $idmap = {};
  my $global_params_idmap = F5::GlobalParametersUpgrade::get_idmap();
  $idmap->{param_attrib_id} = $global_params_idmap->{param_attrib_id} if ($global_params_idmap->{param_attrib_id});
  $self->copy_table(dbh => $dbh,
		    old_dbname => $old_dbname,
		    logger => $logger,
		    old_table_name => 'PL_ILLEGAL_META_CHARS_POOL',
		    # new_table_name deliberately omitted -- only reading old values to store in idmap
		    idmap => $idmap,
		    fixup_callback => sub {
		      my($dbh, $logger, $import_table, $import_table_name, $import_config, $idmap) = @_;
		      my $metachar_idmap = F5::WildcardEntityUpgrade::get_metachar_ids($dbh, $logger, $import_table, $import_table_name, $import_config, $idmap);
		      $idmap->{metachar} = $metachar_idmap;
		    },
		   );

  $self->copy_table(dbh => $dbh,
		    old_dbname => $old_dbname,
		    new_dbname => $new_dbname,
		    logger => $logger,
		    old_table_name => 'PL_PARAMS_META_CHARS',
		    new_table_name => 'PL_PARAM_VALUE_METACHARS',
		    idmap => $idmap,
		    ignore => 1,
		    fixup_callback => sub {
		      my($dbh, $logger, $data, $table_name, $config, $idmap) = @_;
		      for my $row (values %$data) {
			$row->{metachar} = $row->{meta_char_id};
			$row->{param_attrib_id} = $row->{param_id} if ($row->{param_id});
			$row->{flg_is_allowed} = 1;
		      }
		    }
		   );

  # Convert charset N (C) => NG, Y (R) => YG
  F5::WildcardEntityUpgrade::globalize_charset_definitions(dbh => $dbh,
							   dbname => $new_dbname,
							   policy_id => '*',
							   logger => $logger);

  # Convert allowed_object_regexp
  $self->copy_table(dbh => $dbh,
		    old_dbname => $old_dbname,
		    new_dbname => $new_dbname,
		    logger => $logger,
		    old_table_name => 'PL_ALLOWED_OBJECTS_REGEXP',
		    new_table_name => 'PL_OBJECTS',
		    ignore => 1,
		    fixup_callback => sub {
                        my($dbh, $logger, $import_table, $import_table_name, $import_config, $idmap) = @_;                        
                        F5::WildcardEntityUpgrade::convert_allowed_objects_regexp_to_wildcard($dbh, $logger, $import_table, $import_table_name, $import_config, $new_dbname);
                    }
		   );

  # Add Wildcard if needed based on violation
  F5::WildcardEntityUpgrade::add_wildcard_entities(dbh => $dbh,
						   dbname => $new_dbname,
						   policy_id => '*',
						   logger => $logger);
  $logger->debug("End Wildcard Entity fixup");
}

1;
