package F5::DbFixup::XMLSecurity;

use base 'F5::DbFixup::Default';
use F5::XMLSecurityUpgrade;

# Package globals, handled by AUTOLOAD
our $fixup_db = 'PLC';
our $min_version = '0009.0004.0002.0000.0000';
our $max_version = '0010.0000.0000.0000.0000';
our $excluded_tables = [ 'PLC.PL_XML_PROFILE_DEFENSE_ATTRIBUTES' ];

sub after_fixup {
  my($self,$dbh,$old_dbname,$new_dbname, $logger) = @_;
  $logger->debug("Begin XML Security fixup");

  $self->copy_table(
                    dbh => $dbh,
                    old_dbname => $old_dbname,
                    new_dbname => $new_dbname,
                    logger => $logger,
                    table_name => 'PL_XML_PROFILE_DEFENSE_ATTRIBUTES',
                    id_field_name => undef,
                    fixup_callback => sub {
                      my($dbh, $logger, $table_data) = @_;
                      for my $key (keys %$table_data) {
                          my $row = $table_data->{$key};
                          $row->{profile_id} = $row->{xml_profile_id};
                      }
                      F5::XMLSecurityUpgrade::convert_xml_defense_attributes(dbh => $dbh,
                                                                             table_data => $table_data);
                    },
                   );


  my $xml_profile_defense_levels;
  $self->copy_table(
                    dbh => $dbh,
                    old_dbname => $old_dbname,
                    logger => $logger,
                    old_table_name => 'PL_XML_PROFILES',
                    new_table_name => 'PL_CONTENT_PROFILES',
                    id_field_name => undef,
                    fixup_callback => sub {
                      my($dbh, $logger, $table_data) = @_;
                      $xml_profile_defense_levels = F5::XMLSecurityUpgrade::save_xml_defense_levels( table_data => $table_data);
                    },
                   );


  F5::XMLSecurityUpgrade::populate_xml_defense_attributes(
                                                          dbh => $dbh,
                                                          dbname => $new_dbname,
                                                          xml_profile_defense_levels => $xml_profile_defense_levels,
                                                         );

  F5::XMLSecurityUpgrade::set_primary_enforced_validation_file(
                                                               dbh => $dbh,
                                                               dbname => $new_dbname,
                                                              );

  $logger->debug("End XML Security fixup");
}

1;
