package F5::ExportPolicy::LegacyBinary;

use strict;
use warnings;

use F5::ASMConfig::Entity::Violation;
use F5::Violation;

$YAML::Numify = 1;
$YAML::XS::Numify = 1;
$YAML::XS::QuoteNumericStrings = 0;

#-----------------------------------------------------------

my @timestamp_keys = qw(
                        last_updated
                        last_tightening_time
                        staged_since
                        createdtime
                       );

my %delete_tables = (
                     '15.1.0' => {
                                  'PL_BOT_SIGNATURES' => 1,
                                  'PL_BOT_DEFENSE' => 1,
                                  'PL_BOT_CLASSES' => 1,
                                  'PL_REQUEST_LOGGERS' => 1,
                                  'PL_REQUEST_LOGGERS_FILTERS' => 1,
	                          'PL_WEBHOOKS' => 1,
                                  }
                     );

my %min_viol = ( '15.1.0' => F5::Violation::VIOL_HEADER_REPEATED() );

my %table_modifications = (
                           '15.1.0' => {
                               'NEGSIG_SIGNATURES' => {
                                                       delete_fields => { sig_tag => 1,
                                                                          sig_tag_policy => 1, },
                                                      },
                               'PL_HEADERS' => {
                                                delete_fields => { flg_allow_repeated => 1, },
                                                delete_objects => { header_name => 'transfer-encoding', },
                                               },
                               'PL_OBJECTS' => {
                                                delete_fields => { flg_disallow_file_upload_of_executables => 1, },
                                               },
                               'PL_HTTP_CHECKS' => {
                                                    delete_objects => { http_check_code => 2097152 },
                                                   },
                               'PL_ALTERNATE_RESPONSES' => {
                                                            delete_objects => { cause => 'grpc' },
                                                           },
                           },
                          );

sub convert_for_version {
    my(%params) = @_;
    my $tables_file = $params{policy_file};
    my $version = $params{version};

    my (@tables) = YAML::XS::LoadFile($tables_file);
    open(my $fh, '>', $tables_file);

    for my $table_doc (@tables) {
        my @doc_keys = keys %$table_doc;
        my $table_name = $doc_keys[0];

        if ( $delete_tables{$table_name} ) {
            next;
        }

        if ($table_name eq 'NEGSIG_SETS') {
            my($prefix, @sets) = split(/\n(?=\s*<set |\s*<\/sets>)/, $table_doc->{NEGSIG_SETS});
            $table_doc->{NEGSIG_SETS} = join("\n", $prefix, sort @sets);
        }

        if (scalar @doc_keys == 1 && ref($table_doc->{$table_name}) eq 'HASH') {
            my $modifications = $table_modifications{$version}->{$table_name};

            for my $key (keys %{$table_doc->{$table_name}}) {
                my $rec = $table_doc->{$table_name}->{$key};
                if ($ENV{DEBUG_STANDARDIZE_OUTPUT}) {
                    for my $k (@timestamp_keys) {
	                if ($rec->{$k}) {
                            $rec->{$k} =~ s/^16\d{8,}/1600000000/;
                        }
                    }
                }

                if ($table_name eq 'NEGSIG_SIGNATURES') {
                    $rec->{last_update} =~ s|(20\d\d)-(\d\d)-(\d\d)|$1/$2/$3|;
                }

                if ($table_name eq 'PL_BLOCKING' && ($rec->{viol_index} >= $min_viol{$version}) ) {
                    if ( !(($rec->{viol_index} > $F5::ASMConfig::Entity::Violation::MIN_USER_VIOL_INDEX)
                              && ($rec->{viol_index} < $F5::ASMConfig::Entity::Violation::MAX_USER_VIOL_INDEX)) ) {
                        delete $table_doc->{PL_BLOCKING}->{$key};
                    }
                }

                next if !defined $modifications;

                if ($modifications->{delete_fields}) {
                    map { delete $rec->{$_} } keys %{$modifications->{delete_fields}};
                }

                if ($modifications->{delete_objects}) {
                    for my $field (keys %{$modifications->{delete_objects}}) {
                        if ($rec->{$field} eq $modifications->{delete_objects}->{$field}) {
                            delete $table_doc->{$table_name}->{$key};
                        }
                    }
                }
            }
        }

        print $fh YAML::XS::Dump($table_doc);
    }

    close($fh);
}

#-----------------------------------------------------------

1;
