package F5::Hsl::Populate;

use strict;
use warnings;
use F5::DbUtils;
use F5::SetActive;
use F5::ASMConfig::Entity::Policy;
use F5::Utils::Account;

sub populate_defaults {
    my(%params) = @_;

    my $dbh = $params{dbh};
    my $logger = $params{logger};

    my ($missing_profiles_types) = $dbh->selectall_hashref(<<"EndQuery", 'hsl_profile_type_id');
SELECT PROFILE_TYPES.hsl_profile_type_name, PROFILE_TYPES.hsl_profile_type_id 
  FROM $params{plc_dbname}.HSL_PROFILE_TYPES AS PROFILE_TYPES
       LEFT OUTER JOIN $params{plc_dbname}.HSL_PROFILES AS PROFILES 
       ON PROFILES.hsl_profile_type_id = PROFILE_TYPES.hsl_profile_type_id 
       AND PROFILES.flg_default = 1 
 WHERE PROFILES.flg_default IS NULL;
EndQuery

    foreach my $type_id (keys(%{$missing_profiles_types})) {

        $missing_profiles_types->{$type_id}->{hsl_profile_type_name} =~ tr/[A-Z]/[a-z]/;
        my $profile_name = $missing_profiles_types->{$type_id}->{hsl_profile_type_name}. "_security";

        $dbh->do(<<"EndQuery",undef,$profile_name, $type_id);
INSERT INTO $params{plc_dbname}.HSL_PROFILES 
(hsl_profile_name,hsl_profile_type_id,flg_is_enabled,flg_default) 
VALUES (?,?,1,1)
EndQuery
        my ($profile_id) = $dbh->last_insert_id(undef, undef, undef, undef);

        $dbh->do(<<"EndQuery",undef,$profile_id,$type_id);
INSERT INTO $params{plc_dbname}.HSL_PROFILE_VIOLATION_CONFIG 
SELECT ?,
       A.violation_code,
       A.flg_alarm,
       A.flg_reject 
  FROM $params{plc_dbname}.HSL_PROFILE_VIOLATION_CONFIG_DEFAULTS AS A, 
       $params{plc_dbname}.HSL_VIOLATION_CODES AS B 
 WHERE A.violation_code = B.violation_code 
   AND B.hsl_profile_type_id = ? 
   AND B.hsl_profile_type_id = A.hsl_profile_type_id
EndQuery


    }

  populate_command_codes(%params);
}

#----------------------------------------------------------------------

sub populate_command_codes(){

  my(%params) = @_;
  my $dbh = $params{dbh};

 my($profiles) = $dbh->selectall_arrayref(<<"EndQuery");
SELECT hsl_profile_id, hsl_profile_type_id
         FROM $params{plc_dbname}.HSL_PROFILES
EndQuery


  foreach my $profile_type  (@$profiles){
       my $hsl_profile_id =  $profile_type->[0];
       my $hsl_profile_type_id = $profile_type->[1];

       my ($count) = $dbh->selectrow_array(<<"EndQuery", undef, $hsl_profile_id);
SELECT COUNT(*)
  FROM $params{plc_dbname}.HSL_PROFILE_COMMAND_CONFIG
 WHERE hsl_profile_id = ?
EndQuery

       $dbh->do(<<"EndQuery",undef, $hsl_profile_id, $hsl_profile_type_id) unless $count;
INSERT INTO $params{plc_dbname}.HSL_PROFILE_COMMAND_CONFIG
(hsl_profile_id, command_code, flg_is_enabled)
SELECT ?, command_code, flg_is_enabled
 FROM $params{plc_dbname}.HSL_COMMAND_CODES
 WHERE hsl_profile_type_id=?
EndQuery
       
  }
}

#----------------------------------------------------------------------

1;
