package F5::HttpChecksUpgrade;

use strict;
use warnings;

use F5::DbUtils;
use F5::BigipVersionUtils;

# Old violation codes (64-bit) that were subsequently re-used - hard-coded for upgrade only
use constant PARSER_NOT_SUPPORTED => "100";
use constant PARSER_NULL_IN_REQ_DATA => "1000000000000000";

use constant NULL_IN_REQUEST => "16384";
use constant UNPARSEABLE_REQUEST => "65536";

my %viol_to_http_check = (PARSER_NOT_SUPPORTED() => UNPARSEABLE_REQUEST(),
                          PARSER_NULL_IN_REQ_DATA() => NULL_IN_REQUEST(),
                         );

sub populate_http_checks_defaults {
    my(%params) = @_;

    my $dbh = $params{dbh};
    my $logger = $params{logger};

    my $policy_id = $params{policy_id};
    my $dbname = $params{dbname} || "PLC";

    F5::DbUtils::copy_defaults_table(dbh => $dbh,
                                     dbname => $dbname,
                                     table_name => 'PL_HTTP_CHECKS',
                                     ignore   => 1,
                                     policy_id => $policy_id);

    $logger->info("Importing blocking flags") if $logger;
    my $where_clause = "policy_id = ? AND  http_check_code = ?";

    for my $blocking_rec (@{$params{saved_blocking}}) {
        if (exists $blocking_rec->{code} && $viol_to_http_check{$blocking_rec->{code}}) {
            my $any_blocking_flag = $blocking_rec->{flg_reject} || $blocking_rec->{flg_alarm} || $blocking_rec->{flg_learn};

            $dbh->do(<<EndQuery, undef, $any_blocking_flag, $blocking_rec->{policy_id}, $viol_to_http_check{$blocking_rec->{code}});
UPDATE $dbname.PL_HTTP_CHECKS
   SET flg_is_enabled = ?
 WHERE policy_id = ?
   AND http_check_code = ?
EndQuery
        }
    }
}

1;
