package F5::IO::ASMConfig::EventListener;

use strict;
use warnings;

use base qw(F5::IO::Handle);

use Scalar::Util;
use IO::Handle;
use F5::ASMConfig::Client;

#----------------------------------------------------------------------

sub new {
    my($class, %params) = @_;
    if (ref($class)) {
        %params = (%$class,
                   %params);
        $class = ref($class);
    }
    my $self = bless {}, $class;
    my $rv = $self->init(%params);
    if (! $rv) {
        return;
    }
    return $self;
}

#----------------------------------------------------------------------

sub init {
    my($self, %params) = @_;
    my $username = $params{username} || 'System';

    eval {
        $self->{client} = F5::ASMConfig::Client->new(username => $username);
        $self->{client}->init_event_queue;

        my @subscriptions = $params{event_subscriptions} ? @{$params{event_subscriptions}}
                                                         : ($params{event_subscription});
        $self->{subscriptions} = \@subscriptions;
        for my $subscription (@subscriptions) {
            $self->{client}->add_event_subscription($self->{client}->{session},
                                                    $subscription);
        }

        $self->{socket} = $self->{client}->{event_socket};
    };
    if ($@) {
        $params{logger}->info("Could not open event listener") if $params{logger};
        return;
    }

    return $self->SUPER::init(%params);
}

#----------------------------------------------------------------------

sub reinit_subscriptions {
    my ($self, %params) = @_;
    $self->{client}->delete_all_event_subscriptions($self->{client}->{session});
    my @subscriptions = @{$self->{subscriptions}};
    for my $subscription (@subscriptions) {
        $self->{client}->add_event_subscription($self->{client}->{session},
                                                $subscription);
    }
}

#----------------------------------------------------------------------

sub get_client {
    my($self) = @_;
    return $self->{client};
}

#----------------------------------------------------------------------

sub handle_read {
    my($self, %params) = @_;

    eval {
        my $event = $self->{client}->get_next_event;
        $self->{read_message_handler}->($self, $event);
    };
    if ($@) {
        # We have to assume that this is a regular socket close error, since the
        # read is handled (badly) deep within Thrift::Socket, and the true IO error
        # cannot be retrieved cleanly.
        my $err = $@;
        if (ref($err) && $err->{message}) {
            $err = $err->{message};
        }
        $self->{socket_close_handler}->($self, $err);
    }
    return;
}

#----------------------------------------------------------------------

sub close {
    my($self) = @_;

    eval {
        if ($self->{socket}) {
            $self->{socket}->close;
        }
    };
}

#----------------------------------------------------------------------

1;
