package F5::IO::Mtcl;

use strict;
use warnings;

use base 'F5::IO::Handle';
use F5::Mtcl;

#----------------------------------------------------------------------

sub read_message {
    my($self) = @_;

    my $buf = $self->read(100);

    if (! $buf) {
        return;
    }

    # take No.1
    my $message = F5::Mtcl->new(buf => $buf);

    if ($message) { # got something back to work with

        # more data is required
        if (my $required_len = $message->{buf_needed_length}) {

            # place back the previously read buff and try to read required num of bytes
            $self->unshift_to_read_buffer($buf);
            $buf = $self->read($required_len);

            # could not read the required num of bytes, place back the buffer and give up
            if ($buf && length($buf) < $required_len) {
                $self->unshift_to_read_buffer($buf);
                return;
            }

            # got the required num of bytes, take No.2
            $message = F5::Mtcl->new(buf => $buf);
        }

        # after either take No.1 or take No.2 above: 
        #     if all is okay - place back the residual buffer and return the msg object
        if ($message && ! $message->{buf_needed_length}) {
            if (length($message->{buf_remainder})) {
                my $buf_remainder = delete $message->{buf_remainder};
                $self->unshift_to_read_buffer($buf_remainder);
            }
            return $message;
        }
        #    if NOT okay - place back the buffer and give up
        else {
            $self->unshift_to_read_buffer($buf);
            return;
        }

    }
    else { # got nothing back to work with, place back the buffer and give up
        $self->unshift_to_read_buffer($buf);
        return;
    }

    # should only get here if F5::Mtcl object failed to instantiate
    return;
}

#----------------------------------------------------------------------

sub write_message {
    my($self, $message) = @_;

    $self->write($message->as_string);
}

#----------------------------------------------------------------------

1;
