package F5::LockFactory::File;

use base qw(F5::LockFactory::Base);
use Fcntl qw/:flock/; # import LOCK_* constants

sub init {
    my ($self, %params) = @_;
    if ($params{lock_filename}) {
        $self->{lock_filename} = $params{lock_filename};
    }
}

sub init_lock {
    my ($self) = @_;
    if (!open ($self->{locks}->{$self->{lock_filename}}->{fh},
                   ">",
                   $self->{lock_filename})) {

        my $err = $!;
        die "Could not open $self->{lock_filename}: $err\n";
    }

    # permissions to lockfile to make sure that permissions' masks don't cause problems
    chmod(0664, $self->{locks}->{$self->{lock_filename}}->{fh});
}

sub add_lock {
    my ($self, $flags) = @_;
    flock($self->{locks}->{$self->{lock_filename}}->{fh}, $flags)
        || return; # Return on failure

    return 1;
}

sub _cleanup {
    my ($self, %params) = @_;
    my $fh = $self->{locks}->{$self->{lock_filename}}->{fh};
    if ($fh) {
        flock( $fh, LOCK_UN );
        close $fh;
    }
    return;
}

1;

