#******************************************************************************
#copyright 2000-2002 Magnifire Networks Ltd. All Right Reserved.
#
#PACKAGE: Log  
#
#USES: 
#
#CREATED: 17 May 2002 By Edi Pinhasov
#
#MODIFIED:
#
#COMMENTS: Writes to logfiles/consule logs in predefined format

#*******************************************************************************


#$Header$#

package F5::Log;

use Exporter ();
@ISA = qw(Exporter);

@EXPORT = qw(logprint $INF $DBG $ERR $F_ERR $TRACE $PRINT_LOG $PRINT_TERM $PRINT_BOTH);

use strict;

use vars qw{$PRINT_LOG 
	    $PRINT_TERM
	    $PRINT_BOTH
	    $INF
	    $DBG
	    $ERR
	    $F_ERR
            $TRACE
           };



use POSIX qw/strftime/;
use FindBin;
use Time::HiRes;
use Memoize;

memoize('get_timezone');
$PRINT_LOG =  1;
$PRINT_TERM = 2;
$PRINT_BOTH = 3;

# log/debug levels  INFO, DBG, ERR, FATAL_ERR 
$INF = 'INFO';
$DBG = 'DEBUG';
$ERR = 'ERR';
$F_ERR = 'CRIT';
$TRACE = 'TRACE';


#--------------------------------------------------------------------------------
# logprint - write logs to file and/or consule 
#gets:
#   <level> - one of:  INFO, DEBUG, ERR, CRIT 
#   <action> - the action on which err happened 
#   <msg> - the err message (free format) - as convention with "\n" at the end.
#   <file> - the log_file name
#   <target> - 1/2/3 - 1 print to log_file, 2 - print to consule, 3 - both 
#prints the next format:
#   [<level>,<date_time>]<pid>,<executable>,<module>::<function>,<action>,<msg>
sub logprint
{
    my($level, $action, $msg, $file, $target, $log_time, $params) = @_;
    
    my $save_umask = umask;
    my $umask = 0077;
    if (defined($params) && ref($params) eq 'HASH' && defined($params->{umask})) {
        $umask = $params->{umask};
    }
    
    if($save_umask !=  $umask) {
        umask($umask);
    }
    
    my $sub = (caller(2))[3];
    $sub = "" unless defined $sub;
    if ($sub =~ /^F5::Logger::/) {
        $sub = (caller(3))[3] || '';
        if ($sub =~ /logprint/) {
            $sub = (caller(4))[3] || '';
        }
    }

    $action = "" unless defined $action;
    my $date = get_date($log_time);
    chomp $msg;
    $msg.="\n";
    my $component = $FindBin::Script;
    $component = 'asm' if ($component eq '-e');
    my $log = "$component|$level|$date|$$|$sub,$action,$msg";

    if (($target & $PRINT_LOG) && open(LOG, ">>$file"))
    {
        print LOG $log;
        close(LOG);
    }
    
    print $log if($target & $PRINT_TERM);
    require F5::BigIpLog;
    if ( $level eq $ERR || $level eq $F_ERR )
    {
        F5::BigIpLog::bigip_ts_ts_subsystem_error_log( $FindBin::Script, $sub, $msg );
    }
    elsif ( $file =~ /\Qts.log\E$/ )
    {
        F5::BigIpLog::bigip_ts_ts_subsystem_info_log( $FindBin::Script, $sub, $msg );
    }
    
    if($save_umask !=  $umask) {
        umask($save_umask);
    }
}

#--------------------------------------------------------------------------------

sub get_date
{
    my($log_time) = @_;
    my($sec, $frac_sec);

    if ($log_time) {
        $sec = $log_time;
    }
    else {
        ($sec, $frac_sec) = Time::HiRes::gettimeofday();
    }
    my $str_time = POSIX::strftime("%b %d %H:%M:%S", localtime($sec));

    if ($frac_sec) {
        $str_time .= sprintf(".%03d", $frac_sec/1000);
    }
    return $str_time;
}

#--------------------------------------------------------------------------------

sub get_timezone {
    return POSIX::strftime( "%Z", localtime() );
}

1;
