package F5::MessagePack;

use strict;
use warnings;

use Data::MessagePack;
use Data::MessagePack::Stream;

#======================================================================

package F5::MessagePack::Iterator;


sub new {
    my($class, %params) = @_;

    my $self = bless {}, $class;
    $self->init(%params);
    return $self;
}

#--------------------------------------------------------------------------------

sub init {
    my($self, %params) = @_;
    $self->{filename} = $params{filename};
    $self->{buffer_read_size} = $params{buffer_read_size} || (1024 * 500);
    return $self;
}

#--------------------------------------------------------------------------------

sub get_next {
    my($self, %params) = @_;

    if (! $self->{fh}) {
        open($self->{fh}, '<', $self->{filename});
        $self->{buffer} = '';
        $self->{unpacker} = Data::MessagePack::Stream->new;
    }

    if ($self->{unpacker}->next) {
        my $data = $self->{unpacker}->data;
        return $data;
    }

    if (! read($self->{fh}, $self->{buffer}, $self->{buffer_read_size})) {
        return;
    }

    $self->{unpacker}->feed($self->{buffer});
    if ($self->{unpacker}->next) {
        my $data = $self->{unpacker}->data;
        return $data;
    }
    else {
        die "Buffer is not empty, but a record cannot be unpacked\n";
    }
}

#--------------------------------------------------------------------------------

1;
