package F5::NavigationParamsUpgrade;

use strict;
use warnings;

require Exporter;
our @ISA = qw(Exporter);
our @EXPORT_OK = qw(upgrade_navigation_params);

sub upgrade_navigation_params {
  my(%params) = @_;

  my $dbh = $params{dbh};
  my $logger = $params{logger};
  my $policy_id = $params{policy_id};
  my $dbname = $params{dbname};

  my $policy_param_where_clause = '1';
  if ($policy_id ne '*') {
    $policy_param_where_clause = "policy_id in ($policy_id)";
  }

  my $get_navigation_params_sql = <<"EndQuery";
SELECT policy_id, param_name 
  FROM $dbname.PL_CRAWLER_NAVIGATION_PARAMS
 WHERE $policy_param_where_clause
EndQuery

  my $get_nav_param_objects_sql = <<"EndQuery";
SELECT id, uri
  FROM $dbname.PL_OBJECTS
 WHERE policy_id = ?
   AND uri like ?
EndQuery

  my $fix_nav_param_objects_sql = <<"EndQuery";
UPDATE $dbname.PL_OBJECTS
   SET uri = ?
 WHERE id = ?
EndQuery

  my $get_nav_param_objects_sth = $dbh->prepare($get_nav_param_objects_sql);
  my $fix_nav_param_objects_sth = $dbh->prepare($fix_nav_param_objects_sql);

  my $get_navigation_params_sth = $dbh->prepare($get_navigation_params_sql);
  $get_navigation_params_sth->execute;
  while(my $navigation_param = $get_navigation_params_sth->fetchrow_hashref) {
    $get_nav_param_objects_sth->execute($navigation_param->{policy_id},
					'% <' . $navigation_param->{param_name} . ' = %');

    while(my $nav_param_object = $get_nav_param_objects_sth->fetchrow_hashref) {
      my $fixed_uri = $nav_param_object->{uri};
      $fixed_uri =~ s{ (\<\S+) = }{$1=};

      $logger->info("Correcting PL_OBJECTS '$nav_param_object->{uri}' to '$fixed_uri'") if $logger;
      $fix_nav_param_objects_sth->execute($fixed_uri, $nav_param_object->{id});
    }
  }
}

1;
