package F5::ObjectTypeWildcardUpgrade;

use strict;
use warnings;

use F5::CRC;

sub add_object_type_wildcards {
  my(%params) = @_;
  local $" = ',';

  my $dbh = $params{dbh};
  my $logger = $params{logger};
  my $table_data = $params{table_data};
  my $dbname = $params{dbname} || 'PLC';

  my $policy_where_clause = '';
  if ($params{policy_id}) {
    $policy_where_clause = "WHERE id in ($params{policy_id})";
  }
  my $policies = $dbh->selectall_hashref("SELECT id, account_id, flg_disable_rejects FROM $dbname.PL_POLICIES $policy_where_clause",
                                         'id');
  my $ignore = F5::DbUtils::get_ignore_string($dbh);
  for my $row (values %$table_data) {
    next if ($row->{name} eq 'no_ext');
    next if ($row->{flg_access_check}); # Skip unless this is a "Don't Check" filetype

    my %wildcard_object;
    $wildcard_object{policy_id} = $params{policy_id} || $row->{policy_id};

    my($max_pattern_order) = $dbh->selectrow_array(<<EndQuery, undef, $wildcard_object{policy_id});
SELECT MAX(pattern_order)
  FROM $dbname.PL_OBJECTS
 WHERE policy_id = ?
EndQuery
    $max_pattern_order ||= 0;
    $wildcard_object{pattern_order} = $max_pattern_order + 1;

    $wildcard_object{uri} = '*.' . $row->{name};
    $wildcard_object{flg_is_pattern} = 1;

    for my $app_id ((0,1)) {
      $wildcard_object{app_id} = $app_id;

      my @field_names = keys %wildcard_object;
      my @placeholders = ('?') x scalar @field_names;
      $dbh->do("INSERT $ignore INTO $dbname.PL_OBJECTS (@field_names) VALUES (@placeholders)",
               undef, @wildcard_object{@field_names});
      $wildcard_object{pattern_order}++;
    }
  }
}

1;
