package F5::ParamExtractAttribCleanup;

use strict;
use warnings;

require Exporter;
our @ISA = qw(Exporter);
our @EXPORT_OK = qw(cleanup_param_extract_attrib);

my %dependent_tables = (
			'PL_PARAM_ATTRIBUTES' => [ 'PL_STATIC_PARAMS',
						   'PL_PARAM_VALUE_METACHARS',
						   'PL_DYNAMIC_PARAM_EXTRACTS' ],
			'PL_EXTRACT_ATTRIBUTES' => ['PL_DYNAMIC_PARAM_EXTRACTS' ]
		       );

sub cleanup_param_extract_attrib {
  my(%params) = @_;

  my $dbh = $params{dbh};
  my $logger = $params{logger};
  my $policy_id = $params{policy_id};
  my $dbname = $params{dbname};

  my $policy_param_where_clause = '';
  if ($policy_id ne '*') {
    $policy_param_where_clause = "AND param_attrib.policy_id in ($policy_id)";
  }

  my $find_unmatched_sql = <<"EndQuery";
SELECT param_attrib.id
  FROM $dbname.PL_PARAM_ATTRIBUTES AS param_attrib
    LEFT JOIN $dbname.PL_FLOW_PARAMS AS flow_param
           ON param_attrib.id = flow_param.param_attrib_id
          AND param_attrib.policy_id = flow_param.policy_id
    LEFT JOIN $dbname.PL_OBJECT_PARAMS AS object_param
           ON param_attrib.id = object_param.param_attrib_id
          AND param_attrib.policy_id = object_param.policy_id
    LEFT JOIN $dbname.PL_GLOBAL_PARAMS AS global_param
           ON param_attrib.id = global_param.param_attrib_id
          AND param_attrib.policy_id = global_param.policy_id
 WHERE flow_param.id IS NULL
   AND object_param.id IS NULL
   AND global_param.id IS NULL
$policy_param_where_clause
EndQuery

  my $param_sth = $dbh->prepare($find_unmatched_sql);
  $param_sth->execute;
  while(my $param_attrib_row = $param_sth->fetchrow_hashref) {
    $logger->debug("PL_PARAM_ATTRIBUTES id = $param_attrib_row->{id} is not referenced by flow, object, or global params") if $logger;
    for my $dependent_table (@{$dependent_tables{'PL_PARAM_ATTRIBUTES'}}) {
      $dbh->do("DELETE FROM $dbname.$dependent_table WHERE param_attrib_id = ?", undef,
	       $param_attrib_row->{id});
    }
    $dbh->do("DELETE FROM $dbname.PL_PARAM_ATTRIBUTES WHERE id = ?", undef,
	     $param_attrib_row->{id});
  }

  my $policy_extract_where_clause = '';
  if ($policy_id ne '*') {
    $policy_extract_where_clause = "AND extract_attrib.policy_id in ($policy_id)";
  }

  $find_unmatched_sql = <<"EndQuery";
SELECT extract_attrib.id
  FROM $dbname.PL_EXTRACT_ATTRIBUTES AS extract_attrib
    LEFT JOIN $dbname.PL_OBJECT_EXTRACTS AS object_extract
           ON extract_attrib.id = object_extract.extract_attrib_id
          AND extract_attrib.policy_id = object_extract.policy_id
    LEFT JOIN $dbname.PL_GLOBAL_EXTRACTS AS global_extract
           ON extract_attrib.id = global_extract.extract_attrib_id
          AND extract_attrib.policy_id = global_extract.policy_id
 WHERE object_extract.id IS NULL
   AND global_extract.id IS NULL
$policy_extract_where_clause
EndQuery

  my $extract_sth = $dbh->prepare($find_unmatched_sql);
  $extract_sth->execute;
  while(my $extract_attrib_row = $extract_sth->fetchrow_hashref) {
    $logger->debug("PL_EXTRACT_ATTRIBUTES id = $extract_attrib_row->{id} is not referenced by object, or global extracts") if $logger;
    for my $dependent_table (@{$dependent_tables{'PL_EXTRACT_ATTRIBUTES'}}) {
      $dbh->do("DELETE FROM $dbname.$dependent_table WHERE extract_attrib_id = ?", undef,
	       $extract_attrib_row->{id});
    }
    $dbh->do("DELETE FROM $dbname.PL_EXTRACT_ATTRIBUTES WHERE id = ?", undef,
	     $extract_attrib_row->{id});
  }

  return 1;
}

1;
