package F5::ParseUri;


use constant APP_ID_HTTP             => 0;
use constant APP_ID_HTTPS            => 1;

#--------------------------------------------------------------------------------
# takes: full uri, and optional query_separators (uses '?' by default)
# returns: app_id, domain, object, query
sub parse_uri
{
    my ($uri, $query_seps, $uri_session_pattern) = @_;

    $query_seps = '?' if (not defined $query_seps);

    # if instructed, the uri session pattern must be removed
    
    if(defined $uri_session_pattern)
    {
       $uri =~ s/$uri_session_pattern//;
    }

    my ($url, $query) = split /[$query_seps]/, $uri, 2;
    my ($https, $domain, $object) = $url =~ m#^http(s)?://(?:[^@]*@)?((?:\[[^/]+\])|[^\[\]:/]+)(?::\d+)?(/\S*)?#; 
    $object =~ s/#.*$//;
    my $app_id = ($https) ? APP_ID_HTTPS : APP_ID_HTTP;

    return ($app_id, $domain, $object, $query);
}

#--------------------------------------------------------------------------------
# takes: full uri, and optional query_separators (uses '?' by default)
# returns: app_id, domain, object, query
# supports spaces in the object

sub parse_unescaped_uri
{
   my ($uri, $query_seps) = @_;

   $query_seps = '?' if (not defined $query_seps);

   my ($url, $query) = split /[$query_seps]/, $uri, 2;
   my ($https, $domain, $object) = $url =~ m#^http(s)?://((?:\[[^/]+\])|[^\[\]:/]+)(?::\d+)?(/[^\t\r\n]*)?#;
   my $app_id = ($https) ? APP_ID_HTTPS : APP_ID_HTTP;

   return ($app_id, $domain, $object, $query);
}
1;
