package F5::PeriodicTask::AbandonedTaskSweep;

use strict;
use warnings;
use base qw(F5::PeriodicTask::Base);

use F5::Db::Single;
use F5::ASMConfig::ASMConfig;
use F5::ProcessTable;

#----------------------------------------------------------------------

sub init {
    my ($self, %params) = @_;
    $self->SUPER::init(%params);
}

#----------------------------------------------------------------------

sub module_run {
    my ($self, %params) = @_;
    my $dbh = F5::Db::Single->instance();

    my $proc_table = F5::ProcessTable::get_process_table();
    my %entity_by_table_map = F5::ASMConfig::Utils::entity_by_table_map();
    my @task_tables = F5::DbUtils::get_task_tables('PLC');

    for my $task_table (@task_tables) {
        my $running_tasks = $dbh->selectall_hashref(<<EndQuery, 'task_id', undef, F5::ASMConfig::TaskStatus::TASK_STATUS_STARTED());
SELECT task_id, pid
  FROM $task_table
 WHERE task_status = ?
EndQuery
        my @abandoned_tasks = grep { ( $running_tasks->{$_}->{pid} != 0 ) && !( exists $proc_table->{$running_tasks->{$_}->{pid}} ) } keys %$running_tasks;
        my $entity_class = $entity_by_table_map{$task_table}[0];
        my $entity_display_name;
        if (@abandoned_tasks) {
            eval {
                eval "require $entity_class";
                $entity_display_name = $entity_class->get_element_type_display_name();
            };
            if ($@) {
                $entity_display_name = $task_table;
            }
        }
        for my $abandoned_task (@abandoned_tasks) {
            my $error_message = $entity_display_name . " [task_id = " . $abandoned_task . "] failed. Task execution timeout.";
            $self->{logger}->error($error_message) if $self->{logger};
            F5::DbUtils::update_row(dbh => $dbh,
                                    table_name => $task_table,
                                    where_fields => { task_id => $abandoned_task },
                                    rec => { task_status => F5::ASMConfig::TaskStatus::TASK_STATUS_FAILURE(),
                                             message => 'Task execution timeout. Please retry the operation.'},
                                   );
        }
    }

    return;
}

#----------------------------------------------------------------------

1;
