package F5::PeriodicTask::ApplyClusterSync;

use strict;
use warnings;
use base qw(F5::PeriodicTask::Base);
use F5::Db::Single;
use F5::ASMConfig::ASMConfig;
use F5::ASMConfig::EasyClient;

#----------------------------------------------------------------------

sub get_subscriptions {
    return [F5::ASMConfig::EventSubscription->new({element_event_types => [F5::ASMConfig::ElementEventType->new({ event_type => F5::ASMConfig::EventType::ALL_TYPES,
                                                                                                                 element_type => F5::ASMConfig::ElementType::APPLY_CLUSTER_SYNC})],
                                                   send_topology_suppressed_events => 1,
                                                  })
           ];
}

#----------------------------------------------------------------------

sub handle_event {
    my ($self, %params) = @_;
    my $event = $params{event};
    my $element = $event->{element};
    if ($element &&
        $element->{element_type} == F5::ASMConfig::ElementType::APPLY_CLUSTER_SYNC) {

        # apply_sync is intentionally called in a new ASM-config client (even though asmcrond
        # already has an ASM-config client) to prevent a potential memory leak. Also, apply_sync
        # is called synchronously to ensure that only a single apply_sync RPC process will be
        # created. All other asmcrond tasks are blocked while this sync file is loaded, since
        # whatever they would be handling is likely subject to change (based on the newly loaded config).
        my $client = F5::ASMConfig::EasyClient->new(username => 'apply_cluster_sync');
        my $session = $client->{session};
        my $call_flags = F5::ASMConfig::CallFlags->new();

        $self->{logger}->debug("Calling apply_sync via ASM-config") if $self->{logger};
        my $retval = $client->apply_sync($session, $call_flags,
                                      F5::ASMConfig::ApplySyncFlags->new(max_sync_file_handled_count => 1));
        if ($retval->{sync_files_handled_count}) {
            $self->{logger}->info("Sync file successfully handled ($retval->{last_sync_file_handled})") if $self->{logger};
        }
    }
}

#----------------------------------------------------------------------

1;
