package F5::PeriodicTask::Base;
use strict;
use warnings;

use F5::Cfg;
use F5::DateTimeUtils;

#----------------------------------------------------------------------

sub new {
  my($class, %params) = @_;
  my $self = bless({}, $class);
  $self->init(%params);
  return $self;
}

#----------------------------------------------------------------------

sub init {
    my ($self, %params) = @_;
    $self->{logger} = $params{logger};
    $self->{config_file} = $params{config_file};
    my $name = ref $self;
    $name =~ s/.*:://;
    $self->{name} = $name;
}

#----------------------------------------------------------------------

sub run {
    my ($self, %params) = @_;
    $self->{logger}->info("Running Task: $self->{name}");
    $self->module_run(%params);
    $self->{last_run_time} = time();
    return;
}

#----------------------------------------------------------------------

sub module_run {
    return;
}

#----------------------------------------------------------------------

sub next_run_interval {
    my ($self, %params) = @_;

    if (! exists($self->{minute_on_every_hour})) {
        $self->{minute_on_every_hour} = F5::Cfg::cfg_get_config_item($self->{config_file}, $self->{name}, 'MinuteOnEveryHour');
    }
    my $minute_on_every_hour = $self->{minute_on_every_hour};
    if (defined($minute_on_every_hour)) {
        return F5::DateTimeUtils::get_time_until_hour_minute(minute_on_every_hour => $minute_on_every_hour,
                                                             start_time => $self->{last_run_time});
    }
    else {
        if (!$self->{interval}) {
            $self->{interval} = F5::Cfg::cfg_get_config_item($self->{config_file}, $self->{name}, 'Interval') || (60*60*24); #default daily
        }
        return F5::DateTimeUtils::get_time_until_maximum_interval(maximum_interval => $self->{interval},
                                                                  start_time => $self->{last_run_time});
    }
}

#----------------------------------------------------------------------

sub get_subscriptions {
    return [];
}

#----------------------------------------------------------------------

sub handle_event {
    return;
}

#----------------------------------------------------------------------

1;
