package F5::PeriodicTask::CleanFiles;

use strict;
use warnings;
use base qw(F5::PeriodicTask::Base);

use F5::Deploy;
use F5::Utils::File;
use F5::Logger;

my $SYNC_DIR = F5::Deploy::map_path("/ts/var/sync");
my $REST_DIR = F5::Deploy::map_path("/ts/var/rest");
my $SITEMAP_DIR = F5::Deploy::map_path("/var/ts/var/sitemap");
my $TMP_DIR = F5::Deploy::map_path("/var/ts/var/tmp");
my $ASMDATA_TMP_DIR = F5::Deploy::map_path("/var/asmdata1/tmp");
my @files_to_clean_list = ({ glob_pattern => "$SYNC_DIR/*",
                             age          => 60 * 60 * 2,  #2 hours
                             min_retained_files => 2,
                             max_size => 500_000_000, # 500 MB
                           },
                           { glob_pattern => "$REST_DIR/*",
                             age          => 60 * 60 * 24,  #24 hours
                           },
                           { glob_pattern => "$SITEMAP_DIR/tmp/*.json",
                             age          => 60 * 60 * 24,  #24 hours
                           },
                           { glob_pattern => "$SITEMAP_DIR/new/*.json",
                             age          => 60 * 60 * 24,  #24 hours
                           }, 
                           { glob_pattern => "$TMP_DIR/*",
                             age          => 60 * 60 * 24,  #24 hours
                           },
                           { glob_pattern => "$TMP_DIR/sigfile.tmp.*",
                             age          => 60 * 60 * 2,   #2 hours
                             clean_dirs    => 1
                           },
                           { glob_pattern => "$ASMDATA_TMP_DIR/*",
                             age          => 60 * 60 * 24, #24 hours
                           },
                          );

#----------------------------------------------------------------------

sub module_run {
    my($self, %params) = @_;

    for my $files_to_clean (@files_to_clean_list) {
        F5::Utils::File::cleanup_files(%$files_to_clean);
    }
}
