package F5::PeriodicTask::CleanUnreferencedFiles;

use strict;
use warnings;
use base qw(F5::PeriodicTask::Base);

use F5::Deploy;
use F5::Logger;
use F5::Db::Single;
use File::Find;

my $POLICY_VERSION_DIR = F5::Deploy::map_path("/ts/dms/policy/policy_versions");

sub module_run {
  my ($self, %params) = @_;

  my $dbh = F5::Db::Single->instance();
  my $policy_history = $dbh->selectall_arrayref(<<EndQuery, { Slice => {} });
SELECT revision, policy_id
FROM PLC.PL_POLICY_HISTORY
EndQuery

  my %known_history_files = map { (qq{$_->{policy_id}/$_->{revision}.plc} => 1) } @$policy_history;
  find(
    {
      wanted => sub {
        if (-f $_) {
          $_ =~ m|/policy_versions/(.*)$|;
          if (!$known_history_files{$1}) {
            my $file_to_delete = $_;
            $self->{logger}->trace("Deleting history file: $file_to_delete") if $self->{logger};
            unlink($file_to_delete);
          }
        }
      },
      no_chdir => 1,
    },
    $POLICY_VERSION_DIR
  );
}

1;
