package F5::PeriodicTask::HandleStalledAsyncCalls;

use strict;
use warnings;
use base qw(F5::PeriodicTask::Base);

use File::Find;

use F5::Db::Single;
use F5::ASMConfig::ASMConfig;
use F5::ProcessTable;

our $ASM_DATA_PARTITION = "/var/asmdata1";
our $TEMP_FILE_DIR = "$ASM_DATA_PARTITION/tmp";

#----------------------------------------------------------------------

sub init {
    my ($self, %params) = @_;
    $self->SUPER::init(%params);
}

#----------------------------------------------------------------------

sub module_run {
    my ($self, %params) = @_;
    my $client = $params{client};
    my $session = $client->{session};

    my $async_call_filename;
    find({
          wanted => sub {
              if ($File::Find::name =~ /saved_async_calls_/) {
                  $async_call_filename = $File::Find::name;
              }
          },
         },
         $TEMP_FILE_DIR);

    if ($async_call_filename) {
        $self->{logger}->info("Found async call file $async_call_filename") if $self->{logger};
        eval {
            $client->handle_stalled_async_calls($session,
                                                F5::ASMConfig::CallFlags->new(is_asynchronous => 1),
                                               );
        };
        if (my $err = $@) {
            if (ref($err) && (ref($err) eq 'F5::ASMConfig::ASMConfigException')) {
                $err = $err->{display_message} || $err->{error_message};
            }
            $self->{logger}->info("Could not handle async calls ($err)") if $self->{logger};
        }
    }
    return;
}

#----------------------------------------------------------------------

1;
