package F5::PeriodicTask::PolicyStabilityGraphs;

use strict;
use warnings;
use base qw(F5::PeriodicTask::Base);

use F5::Deploy;
use F5::PolicyStabilityGraphs;

#----------------------------------------------------------------------

sub module_run {
    my($self, %params) = @_;

    if (! $self->{last_run_time}) {
        #asmcrond startup call
        #tmstat lose tables on reboot
        #reinitiate stability tables on asm start
        system(F5::Deploy::map_path('/usr/share/ts/bin/stability_graphs_tables_initialize.py'));
        return;
    }

    my $dbh = F5::Db::Single->instance;
    my $logger = $self->{logger} || F5::Logger->new;

    my $current_time = time();
    my $seconds_since_last_run = $self->{task_last_run_time} ? $current_time - $self->{last_run_time} : 3600;
    my $policy_stability_graphs = F5::PolicyStabilityGraphs->new(logger => $logger);
    eval {
        $policy_stability_graphs->update(seconds_since_last_run => $seconds_since_last_run);
    };

    if ($@) {
        my $error_msg = "Policy stability graphs process ended unexpectedly: $@";
        $logger->error($error_msg);
    }

    $self->{task_last_run_time} = $current_time;

    return;
}

#----------------------------------------------------------------------


1;


