package F5::PeriodicTask::ScannerRetest;

use strict;
use warnings;
use base qw(F5::PeriodicTask::Base);
use F5::Db::Single;
use F5::ASMConfig::ASMConfig;
use F5::ASMConfig::Entity::ScannerConfig;

#----------------------------------------------------------------------

sub init {
    my ($self, %params) = @_;
    $self->SUPER::init(%params);
    $self->{policy_scanners} = {};
}

#----------------------------------------------------------------------

sub module_run {
    my ($self, %params) = @_;
    my $dbh = F5::Db::Single->instance;   
    for my $policy_id (keys %{$self->{policy_scanners}}) {
        my $scanner_config = F5::ASMConfig::Entity::ScannerConfig->new(
                                                                       dbh => $dbh,
                                                                       policy => F5::ASMConfig::Policy->new({ policy_id => $policy_id }),
                                                                       get_criteria => {},);

        if ($scanner_config && $scanner_config->{scanner_type}) {
            eval {
                $self->{policy_scanners}->{$policy_id}->reinit(client => $params{client}, dbh => $dbh);
                $self->{policy_scanners}->{$policy_id}->policy_check_retest_requests;
                $self->{policy_scanners}->{$policy_id}->policy_check_scan_results;
                $self->{policy_scanners}->{$policy_id}->get_and_store_api_key;
                $self->{policy_scanners}->{$policy_id}->get_free_scan_results;
            };
            if ($@) {
                $self->{logger}->error("Retrieving scanner results failed. ($@)");
                delete $self->{policy_scanners}->{$policy_id};
                if (ref ($@) && $@->error_code != F5::ASMConfig::ErrorCode::NONEXISTENT_RECORD()) {
                    require F5::Scanner;
                    $self->{policy_scanners}->{$policy_id} =
                      F5::Scanner->new(
                                       dbh => $dbh,
                                       client => $params{client},
                                       call_flags => F5::ASMConfig::CallFlags->new(outbound_call_timeout => 60),
                                       policy => F5::ASMConfig::Policy->new({ policy_id => $policy_id }),);
                }
            }
        }
        else {
            delete $self->{policy_scanners}->{$policy_id};
        }
    }
    return;
}

#----------------------------------------------------------------------

sub get_subscriptions {
    return [F5::ASMConfig::EventSubscription->new({element_event_types => [F5::ASMConfig::ElementEventType->new({ event_type => F5::ASMConfig::EventType::ALL_TYPES,
                                                                                                                 element_type => F5::ASMConfig::ElementType::SCANNER_CONFIG})],
                                                   send_topology_suppressed_events => 1,})
           ];
}

#----------------------------------------------------------------------

sub handle_event {
    my ($self, %params) = @_;
    my $event = $params{event};
    my $element = $event->{element};
    if ($element &&
        $element->{element_type} == F5::ASMConfig::ElementType::SCANNER_CONFIG) {

        require F5::Scanner;
        my $dbh = F5::Db::Single->instance;
        my $policy = $event->{policy};
        my $policy_id = $policy->{policy_id};
        my $scanner_config = F5::ASMConfig::Entity::ScannerConfig->new(
                                                                       dbh => $dbh,
                                                                       policy => $policy,
                                                                       get_criteria => {},);

        if ($scanner_config && $scanner_config->{scanner_type}) {
            $self->{policy_scanners}->{$policy_id} =
              F5::Scanner->new(
                               dbh => $dbh,
                               client => $params{client},
                               call_flags => F5::ASMConfig::CallFlags->new(outbound_call_timeout => 60),
                               policy => F5::ASMConfig::Policy->new({
                                                                     policy_id => $policy_id
                                                                    }),
                              );;
        }
    }
}

#----------------------------------------------------------------------

1;
