package F5::PeriodicTask::SessionAwareness;

use strict;
use warnings;
use base qw(F5::PeriodicTask::Base);
use F5::Db::Single;
use F5::ASMConfig::ASMConfig;
use F5::ASMConfig::Entity::SessionAwarenessDataPoint;

#----------------------------------------------------------------------

sub init {
    my ($self, %params) = @_;
    $self->SUPER::init(%params);
    my $dbh = F5::Db::Single->instance;   
    $self->{session_awareness} = $dbh->selectall_hashref(<<EndQuery, 'policy_name_crc');
SELECT policy.policy_name_crc, id as active_policy_id, flg_session_awareness_on
  FROM PLC.PL_POLICIES policy,
       DCC.ACCOUNT_SESSION_AWARENESS_SETTINGS sa
 WHERE policy.policy_name_crc = sa.policy_name_crc
EndQuery
}

#----------------------------------------------------------------------

sub get_subscriptions {
    return [F5::ASMConfig::EventSubscription->new({element_event_types => [F5::ASMConfig::ElementEventType->new({ event_type => F5::ASMConfig::EventType::ALL_TYPES,
                                                                                                                 element_type => F5::ASMConfig::ElementType::SET_ACTIVE})],
                                                   send_topology_suppressed_events => 1})
           ];
}

#----------------------------------------------------------------------

sub handle_event {
    my ($self, %params) = @_;
    my $event = $params{event};
    my $element = $event->{element};
    if ($element &&
        $element->{element_type} == F5::ASMConfig::ElementType::SET_ACTIVE) {

        my $dbh = F5::Db::Single->instance;
        my $policy = $event->{policy};
        my $policy_id = $policy->{policy_id};
        my $policy_name_crc = $policy->{policy_name_crc};
        my ($flg_session_awareness) = $dbh->selectrow_array(<<EndQuery, undef, $policy_name_crc);
SELECT flg_session_awareness_on
 FROM DCC.ACCOUNT_SESSION_AWARENESS_SETTINGS
 WHERE policy_name_crc = ?
EndQuery

        my $sa_was_disabled = ($self->{session_awareness}->{$policy_name_crc}->{flg_session_awareness_on} && ! $flg_session_awareness);
        my $active_policy_was_changed = $element->{set_active}->{active_policy_was_changed};

        if ($sa_was_disabled) {
            eval {
                F5::ASMConfig::Entity::SessionAwarenessDataPoint->delete_many(dbh => $dbh,
                                                                              policy => $policy,
                                                                              search_criteria => {},
                                                                              master_keys => 1,
                                                                              logger => $self->{logger},
                                                                             );
            };
            if ($@) {
                my $error_msg = $@;
                chomp($error_msg);
                $self->{logger}->error("Session Awarenesss data could not be cleared ($error_msg)");
            }
        }
        elsif ($active_policy_was_changed) {
            eval {
                F5::ASMConfig::Entity::SessionAwarenessDataPoint->reload_session_db(dbh => $dbh,
                                                                                    policy => $policy,
                                                                                    logger => $self->{logger},
                                                                                    delete_existing => 1,
                                                                                   );
            };
            if ($@) {
                my $error_msg = $@;
                chomp($error_msg);
                $self->{logger}->error("Session Awarenesss data could not be loaded ($error_msg)");
            }
        }
        $self->{session_awareness}->{$policy_name_crc}->{flg_session_awareness_on} = $flg_session_awareness;
        $self->{session_awareness}->{$policy_name_crc}->{active_policy_id} = $policy->{policy_id};
    }
}

#----------------------------------------------------------------------

1;
