package F5::PeriodicTask::SessionAwarenessDataPointSweep;

use strict;
use warnings;
use base qw(F5::PeriodicTask::Base);

use F5::ASMReady;
use F5::DbUtils;
use F5::Db::Single;
use F5::ASMConfig::ASMConfig;
use F5::ASMConfig::Entity::SessionAwarenessDataPoint;
use F5::ASMConfig::Entity::Policy;

my $BATCH_SIZE = 100;
my $GET_FROM_SESSION_DB_TIMEOUT = 5; # seconds

#----------------------------------------------------------------------

sub module_run {
    my ($self, %params) = @_;

    return if F5::ASMReady::exists_asm_load_complete_end_lock_file();

    my $dbh = F5::Db::Single->instance;   

    my @search_filters = (
            F5::ASMConfig::SearchFilter->new({field_name => qw{is_active_policy},
                                              field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_EQ(),
                                              field_value => 1,
                                             }),
        );

    my $policy_iter = F5::ASMConfig::Entity::Policy->retrieve_all(dbh => $dbh,
                                                                  search_filters => \@search_filters,
                                                                 );

    while (my $policy = $policy_iter->get_next()) {
        my $policy_id = $policy->{policy_id};
        my $policy_name = $policy->{policy_name};

        # data points from db only
        @search_filters = (F5::ASMConfig::SearchFilter->new({field_name => qw{is_persistent},
                                                                field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_EQ(),
                                                                field_value => 1 })
                             );
        my $db_objs_iter = F5::ASMConfig::Entity::SessionAwarenessDataPoint->retrieve_all(policy => { policy_id => $policy_id },
                                                                                          dbh => $dbh,
                                                                                          search_filters => \@search_filters,
                                                                                         );

        # nothing to do
        next unless $db_objs_iter->{total_record_count};

        # data points from session db only
        my @session_db_objs_ary;
        eval {
            local $SIG{ALRM} = sub { die "Timeout triggered\n" };
            alarm $GET_FROM_SESSION_DB_TIMEOUT;
            @session_db_objs_ary = F5::ASMConfig::Entity::SessionAwarenessDataPoint->get_from_session_db(policy => { policy_id => $policy_id },
                                                                                                         dbh => $dbh,
                                                                                                        );
            alarm 0;
        };
        if (my $err = $@) {
            if ($err eq "Timeout triggered\n") {
                $self->{logger}->info("Session Awareness Data Point Sweeper: timeout triggered while trying to retrieve from SessionDB for policy $policy_name") if $self->{logger};
            }
            else {
                $self->{logger}->error("Session Awareness Data Point Sweeper: error encountered while trying to retrieve from SessionDB for policy $policy_name") if $self->{logger};
                $self->{logger}->error($err) if $self->{logger};
            }
            next;
        }

        my %session_db_objs = map {($_->unique_record_parts_str => 1)} @session_db_objs_ary;

        my @ids_to_delete = ();
        my $id_field_name = 'data_point_id';
        my $table_name = 'PLC.PL_SESSION_AWARENESS_DATA_POINT';
        while(my $db_obj = $db_objs_iter->get_next) {
            if (! exists $session_db_objs{$db_obj->unique_record_parts_str}) {
                push(@ids_to_delete,$db_obj->{$id_field_name});
                if (scalar(@ids_to_delete) >= $BATCH_SIZE) {
                    F5::DbUtils::delete_rows(dbh => $dbh,
                                             where_fields => { $id_field_name => \@ids_to_delete },
                                             table_name => $table_name,
                                            );
                    @ids_to_delete = ();
                }
            }
        }
        # there might be less than $BATCH_SIZE ids left in the batch, delete those as well
        if (@ids_to_delete) {
            F5::DbUtils::delete_rows(dbh => $dbh,
                                     where_fields => { $id_field_name => \@ids_to_delete },
                                     table_name => $table_name,
                                    );
        }
    }

    return;
}

1;
