package F5::PolicyValidator::AllBlockingOff;

use strict;
use warnings;

use base 'F5::PolicyValidator::Default';
use F5::BigipUtils;

sub validate {
    my ($self, %params) = @_;
    my $dbh = $params{dbh};
    my $logger = $params{logger};
    my $validator = $params{validator};
    my $policy = $params{policy};
    my $policy_id = $policy->{policy_id};
    my $violations = $params{violations};
    my $is_policy_transparent = $params{is_policy_transparent};

    return if $is_policy_transparent;

    
    my ($number_of_blocked_violations) = grep { $_->{flg_reject} } values %$violations;
    return if ($number_of_blocked_violations);

    my ($number_of_blocking_signature_sets) = $dbh->selectrow_array(<<EndQuery, undef, $policy_id);
SELECT COUNT(*) 
  FROM PLC.PL_POLICY_NEGSIG_SETS
 WHERE policy_id = ?
   AND flg_reject = 1
EndQuery

    return if ($number_of_blocking_signature_sets);

    if (F5::BigipUtils::is_feature_enabled('bot_defense')) {
        my ($is_bot_defense_enabled) = $dbh->selectrow_array(<<EndQuery, undef, $policy_id);
SELECT flg_enabled 
  FROM PLC.PL_BOT_DEFENSE 
 WHERE policy_id = ?
EndQuery

        return if ($is_bot_defense_enabled);
    }

    $validator->report_error(
            error_type => $F5::PolicyValidator::ERROR_TYPE_LOGIC,
	    error_msg => 'All blocking flags are unset while policy is in blocking enforcement mode.');
}


1;
