package F5::PolicyValidator::CheckReferrerObjects;

use strict;
use warnings;

use base 'F5::PolicyValidator::Default';

use F5::ASMConfig::Entity::Flow;

sub validate {
  my($self, %params) = @_;
  my $dbh = $params{dbh};
  my $logger = $params{logger};
  my $validator = $params{validator};
  my $policy = $params{policy};
  my $action = $params{action};
  my $overlay = $params{overlay};

  if ($action eq 'import_policy' && $overlay) {
      return unless $overlay->is_supported(path => 'flows');
  }

  my $sth = $dbh->prepare(<<"EndQuery");
SELECT flow.id, flow.from_id
  FROM PLC.PL_FLOWS AS flow, PLC.PL_OBJECTS AS obj
 WHERE flow.from_id = obj.id
   AND flow.from_id != 0
   AND obj.flg_is_referrer = 0
   AND flow.policy_id = ?
EndQuery

  $sth->execute($policy->{policy_id});
  while(my $row = $sth->fetchrow_hashref) {
    $validator->report_error(error_type => $F5::PolicyValidator::ERROR_TYPE_LOGIC,
			     error_msg => 'The following flows have a non-referrer URLs as the "from" URL',
                             entity => F5::ASMConfig::Entity::Flow->new(dbh => $dbh,
                                                                        id => $row->{id}),
                            );
  }
}

1;
