package F5::PolicyValidator::IgnoredParamLearning;

use strict;
use warnings;

use base 'F5::PolicyValidator::Default';
use F5::ASMConfig::Entity::Parameter;
use F5::BigipUtils;

sub validate {
    my ($self, %params) = @_;
    my $dbh = $params{dbh};
    my $logger = $params{logger};
    my $validator = $params{validator};
    my $policy = $params{policy};
    my $policy_id = $policy->{policy_id};

    return unless F5::BigipUtils::is_feature_enabled('allow_staging');

    my ($param) = F5::ASMConfig::Entity::Parameter->retrieve_all(dbh => $params{dbh},
                                                                 logger => $params{logger},
                                                                 page_size => 1,
                                                                 policy => $params{policy},
                                                                 get_criteria => { is_wildcard => 1,
                                                                                   param_name => "*",
                                                                                   param_level => F5::ASMConfig::ParamLevel::PARAM_LEVEL_GLOBAL() },
                                                                );
    if ( $param
        && ( $param->{param_value_type} == F5::ASMConfig::ParamValueType::PARAM_VALUE_IGNORE_VALUE() )
        && ( $param->{has_tightening} == F5::ASMConfig::TighteningMode::TIGHTENING_MODE_EXCEPTION() ) 
       ) {
        $validator->report_error(error_type => $F5::PolicyValidator::ERROR_TYPE_LOGIC,
				 error_msg => 'Selective learning mode is enabled on ignored Parameter',
                                 entity  => $param,
                                );
    }
}


1;
