package F5::PolicyValidator::MissingBase64Violation;

use strict;
use warnings;

use base 'F5::PolicyValidator::Default';

sub validate {
    my($self, %params) = @_;
    my $dbh = $params{dbh};
    my $logger = $params{logger};
    my $validator = $params{validator};
    my $policy = $params{policy};
    my $is_policy_transparent = $params{is_policy_transparent};
    my $flags = $params{violations}->{F5::Violation::VIOL_PARAMETER_VALUE_BASE64()};

    return unless (! $flags->{flg_learn} && ! $flags->{flg_alarm} && ! ($flags->{flg_reject} && ! $is_policy_transparent));

    my @entity_tables = qw(
                              PLC.PL_PARAM_ATTRIBUTES
                              PLC.PL_ALLOWED_COOKIES
                              PLC.PL_HEADERS
                         );

    for my $entity_table (@entity_tables) {
        my $exclude_default_headers = ($entity_table eq 'PLC.PL_HEADERS') ? 'AND flg_is_default = 0' : '';
        my($base64_entity_count) = $dbh->selectrow_array(<<EndQuery, undef, $policy->{policy_id});
SELECT COUNT(*) 
  FROM $entity_table
 WHERE policy_id = ?
   AND flg_is_base64 = 1
   $exclude_default_headers
EndQuery

        if ($base64_entity_count) {
            $validator->report_error(
                                     error_type => $F5::PolicyValidator::ERROR_TYPE_LOGIC,
                                     error_msg => 'Entities are configured for Base64 decoding while "Illegal Base64 value" violation is disabled');
            last;
        }
    }
}

1;

