package F5::PolicyValidator::MissingCustomResponsePage;

use strict;
use warnings;

use base 'F5::PolicyValidator::Default';


sub validate {
    my($self, %params) = @_;
    my $dbh = $params{dbh};
    my $logger = $params{logger};
    my $validator = $params{validator};
    my $policy = $params{policy};
    my $policy_id = $policy->{policy_id};
    my $action = $params{action};
    my $overlay = $params{overlay};

    if ($action eq 'import_policy' && $overlay) {
        return unless $overlay->is_supported(path => 'deception-settings');
    }
    my $validation_msg = 'Selected Server Responses is enabled, although no custom deception response pages exist in the policy',

    my $deception_settings = $dbh->selectrow_hashref(<<EndQuery, undef, $policy_id);
SELECT *
  FROM PLC.PL_DECEPTION_SETTINGS
 WHERE policy_id = ?
EndQuery
    

    if ($deception_settings->{flg_user_defined_responses}) {
 
       my ($number_of_custom_response_pages) = $dbh->selectrow_array(<<EndQuery, undef, $policy_id);
SELECT COUNT(*)
  FROM PLC.PL_DECEPTION_RESPONSES
 WHERE policy_id = ?
EndQuery



       if (! $number_of_custom_response_pages) {
               $validator->report_error(error_type => $F5::PolicyValidator::ERROR_TYPE_LOGIC,
                                        error_msg => $validation_msg);
           }
     }
}

1;

