package F5::PolicyValidator::MissingEntities;

use strict;
use warnings;

use base 'F5::PolicyValidator::Default';
use constant POLICY_BLOCKING => 0;

use F5::Violation;

my @undef_entities = (
                      {
                       viol_index => F5::Violation::VIOL_URL(),
                       has_flg_is_allowed => 1,
                       table_name => 'PLC.PL_OBJECTS',
                       validation_msg => 'Illegal URL violation is enforced, although no URLs exist in the policy',
                      },
                      {
                       viol_index => F5::Violation::VIOL_FILETYPE(),
                       has_flg_is_allowed => 1,
                       table_name => 'PLC.PL_FILETYPES',
                       validation_msg => 'Illegal file type violation is enforced, although no file types exist in the policy',
                      },
                      {
                       viol_index => F5::Violation::VIOL_PARAMETER(),
                       table_name => 'PLC.PL_PARAM_ATTRIBUTES',
                       validation_msg => 'Illegal parameter violation is enforced, although no parameters exist in the policy',
                      },
                     );

sub validate {
    my ($self, %params) = @_;
    my $dbh = $params{dbh};
    my $logger = $params{logger};
    my $validator = $params{validator};
    my $policy = $params{policy};
    my $policy_id = $policy->{policy_id};
    my $violations = $params{s};
    my $is_policy_transparent = $params{is_policy_transparent};

    return if($is_policy_transparent);


    for my $undef_entity (@undef_entities) {

        if (! $violations->{$undef_entity->{viol_index}}->{flg_reject} ) {
            next;
        }

        my @bind_params = ($policy_id);
        my $where_clause = '';
        if ($undef_entity->{has_flg_is_allowed}) {
            $where_clause = ' AND flg_is_allowed = ?';
            push(@bind_params, 1);
        }

        my ($number_of_entities) = $dbh->selectrow_array(<<EndQuery, undef, @bind_params);
SELECT COUNT(*)
  FROM $undef_entity->{table_name}
 WHERE policy_id = ? $where_clause
EndQuery

        if (! $number_of_entities) {
            $validator->report_error(error_type => $F5::PolicyValidator::ERROR_TYPE_LOGIC,
                                     error_msg => $undef_entity->{validation_msg});
        }
    }
}


1;
