package F5::PolicyValidator::MissingIdlFile;

use strict;
use warnings;

use base 'F5::PolicyValidator::Default';

my $MAX_QUERY_COST_ID = 7;
my $ENABLED_DEFENSE_ATTRIBUTE = 1;

sub validate {
    my($self, %params) = @_;
    my $dbh = $params{dbh};
    my $validator = $params{validator};
    my $policy = $params{policy};
    my $policy_id = $policy->{policy_id};

    my $action = $params{action};
    my $overlay = $params{overlay};

    if ($action eq 'import_policy' && $overlay) {
        return unless $overlay->is_supported(path => 'graphql-profiles');
    }

    my $graphql_profiles = $dbh->selectall_hashref(<<EndQuery, 'profile_id', undef, $policy_id, $MAX_QUERY_COST_ID, $ENABLED_DEFENSE_ATTRIBUTE);
SELECT DISTINCT def.profile_id profile_id, prof.profile_name profile_name, idl.id idl_id
  FROM PLC.PL_GRAPHQL_PROFILE_DEFENSE_ATTRIBUTES def
 LEFT JOIN PLC.PL_CONTENT_PROFILES prof ON def.profile_id = prof.id
 LEFT JOIN PLC.PL_GRAPHQL_PROFILE_IDL_FILES idl ON def.profile_id = idl.profile_id
 WHERE def.policy_id = ? AND def.def_attrib_id = ? AND def.flg_check_attrib = ? AND idl.id IS NULL
EndQuery

    for my $profile (values %$graphql_profiles) {
        my $validation_msg = "Maximum Query Cost is enabled for '$profile->{profile_name}' GraphQL Profile, although no GraphQL Schema File is set";
        $validator->report_error(error_type => $F5::PolicyValidator::ERROR_TYPE_LOGIC,
                                 error_msg => $validation_msg);
    }
}

1;
