package F5::PolicyValidator::MissingVirusDetectionServer;

use strict;
use warnings;

use base 'F5::PolicyValidator::Default';

sub validate {
    my ($self, %params) = @_;
    my $dbh = $params{dbh};
    my $logger = $params{logger};
    my $validator = $params{validator};
    my $policy = $params{policy};
    my $action = $params{action};
    my $overlay = $params{overlay};

    if ($action eq 'import_policy' && $overlay) {
        return unless $overlay->is_supported(path => 'antivirus');
    }

    my ($flg_inspect_http_uploads) = $dbh->selectrow_array(<<EndQuery, undef, $policy->{policy_id});
SELECT flg_inspect_http_uploads
  FROM PLC.PL_POLICY_ANTIVIRUS
 WHERE policy_id = ?
EndQuery

    return unless defined $flg_inspect_http_uploads;
    return if $flg_inspect_http_uploads == 0;

    my ($flg_virus_detection_server_configured) = $dbh->selectrow_array(<<EndQuery, undef);
SELECT COUNT(*) 
  FROM DCC.VIRUS_DETECTION_SERVER
EndQuery

    if ($flg_virus_detection_server_configured == 0) {
        $validator->report_error(
            error_type => $F5::PolicyValidator::ERROR_TYPE_LOGIC,
	    error_msg => 'Anti-Virus Protection Configuration is not defined while Anti-Virus Protection functionality is enabled');
    }
}
1;
