package F5::PolicyValidator::ParamDataType;

use strict;
use warnings;

use base 'F5::PolicyValidator::Default';
use F5::Violation;

sub validate {
    my($self, %params) = @_;
    my $dbh = $params{dbh};
    my $logger = $params{logger};
    my $validator = $params{validator};
    my $policy = $params{policy};

    my $flags = $params{violations}->{F5::Violation::VIOL_PARAMETER_DATA_TYPE()};
    return if ( $flags->{flg_learn} || $flags->{flg_alarm} || $flags->{flg_reject} );

    my @bind_params = ($policy->{policy_id}, 
                       F5::ASMConfig::ParamDataType::PARAM_DATA_INTEGER,
                       F5::ASMConfig::ParamDataType::PARAM_DATA_DECIMAL,
                       F5::ASMConfig::ParamDataType::PARAM_DATA_EMAIL,
                       F5::ASMConfig::ParamDataType::PARAM_DATA_PHONE,
                       F5::ASMConfig::ParamDataType::PARAM_DATA_BOOLEAN,
                       F5::ASMConfig::ParamDataType::PARAM_DATA_URI,
                       F5::ASMConfig::ParamValueType::PARAM_VALUE_USER_INPUT,
                      );
    my($has_data_type_parameters) = $dbh->selectrow_array(<<EndQuery, undef, @bind_params);
SELECT COUNT(*)
  FROM PLC.PL_PARAM_ATTRIBUTES
 WHERE policy_id = ?
   AND key_ui_type IN (?,?,?,?,?,?)
   AND key_input_type = ?
EndQuery

    if ($has_data_type_parameters) {
        $validator->report_error(
            error_type => $F5::PolicyValidator::ERROR_TYPE_LOGIC,
            error_msg => 'Parameters with specific data types are defined, but \'Illegal parameter data type\' violation is not enforced.');
    }
}
1;

