package F5::PolicyValidator::ParamExtractNameMatch;

use strict;
use warnings;

use base 'F5::PolicyValidator::Default';
use F5::ASMConfig::Entity::Parameter;
use F5::ASMConfig::Entity::Extraction;

my $DYNAMIC_CONTENT_VALUE = 2;
my $DYNAMIC_PARAM_NAME = 16;

sub validate {
    my ($self, %params) = @_;
    my $dbh = $params{dbh};
    my $logger = $params{logger};
    my $validator = $params{validator};
    my $policy = $params{policy};
    my $policy_id = $policy->{policy_id};
    my $action = $params{action};
    my $overlay = $params{overlay};

    if ($action eq 'import_policy' && $overlay) {
        return unless $overlay->is_supported(path => 'extractions');
    }

  my %scopes = ( param => [ qw(global object flow) ],
		 extract => [ qw(global object) ] );

  for my $scope_type (keys %scopes) {
    my $join_scope_type = ($scope_type eq 'param' ? 'extract': 'param');
    my @join_tables;
    my @where_clauses;
    for my $join_scope (@{$scopes{$join_scope_type}}) {
      my $join_table = uc("PL_${join_scope}_${join_scope_type}S");
      my $join_alias = "${join_scope}_${join_scope_type}";
      if ($join_scope_type eq 'param') {
	push(@join_tables, <<"EndQuery");
LEFT JOIN (
           PLC.$join_table AS $join_alias 
           INNER JOIN PLC.PL_PARAM_ATTRIBUTES AS ${join_alias}_attrib
                   ON $join_alias.param_attrib_id = ${join_alias}_attrib.id
                  AND ${join_alias}_attrib.key_input_type IN ($DYNAMIC_CONTENT_VALUE, $DYNAMIC_PARAM_NAME)
          )
  ON $join_alias.name = current_search.name 
 AND $join_alias.policy_id = current_search.policy_id
EndQuery
        push(@where_clauses, "$join_alias"."_attrib.id IS NULL");
      }
      else {
	push(@join_tables, <<"EndQuery");
LEFT JOIN PLC.$join_table AS $join_alias 
  ON $join_alias.name = current_search.name 
 AND $join_alias.policy_id = current_search.policy_id
EndQuery
      
        push(@where_clauses, "$join_alias.id IS NULL");
      }
    }

    push(@where_clauses, 'current_search.policy_id = ?');

    if ($scope_type eq 'param') {
      push(@join_tables, <<"EndQuery");
LEFT JOIN PLC.PL_PARAM_ATTRIBUTES AS current_attrib
  ON current_search.param_attrib_id = current_attrib.id
EndQuery
      push(@where_clauses, "current_attrib.key_input_type IN ($DYNAMIC_CONTENT_VALUE, $DYNAMIC_PARAM_NAME)");
    }

    for my $scope (@{$scopes{$scope_type}}) {
      my $error_msg;
      my $entity_type;
      if ($scope_type eq 'param') {
	$error_msg = 'The following parameters do not have a matching extract';
        $entity_type = 'F5::ASMConfig::Entity::Parameter';
      }
      else {
	$error_msg = 'The following extracts do not have a matching parameter';
        $entity_type = 'F5::ASMConfig::Entity::Extraction';
      }

      my $current_table = uc("PL_${scope}_${scope_type}S");
      my $sql = "SELECT current_search.id, current_search.name, current_search.${scope_type}_attrib_id FROM PLC.$current_table AS current_search ";
      $sql .= join('', @join_tables) . ' WHERE ' . join(' AND ', @where_clauses);

      my $sth = $dbh->prepare($sql);
      $sth->execute($policy_id);
      while(my $row = $sth->fetchrow_hashref) {
	$validator->report_error(error_type => $F5::PolicyValidator::ERROR_TYPE_LOGIC,
				 error_msg => $error_msg,
                                 entity  => $entity_type->new(dbh => $dbh,
                                                              id => $row->{"${scope_type}_attrib_id"}),
                                 table => $current_table,
                                 row_id => $row->{id},
                                );
      }
    }
  }
}


1;
