package F5::PolicyValidator::ValidateDataGuardProperties;

use strict;
use warnings;

use base 'F5::PolicyValidator::Default';
use F5::Violation;

sub validate {
    my($self, %params) = @_;
    my $dbh = $params{dbh};
    my $logger = $params{logger};
    my $validator = $params{validator};
    my $policy = $params{policy};

    my $flags = $params{violations}->{F5::Violation::VIOL_DATA_GUARD()};
    return unless ( $flags->{flg_learn} || $flags->{flg_alarm} || $flags->{flg_reject} );

    my $data_guard_properties = $dbh->selectrow_hashref(<<EndQuery, { Slice => {} } , $policy->{policy_id});
SELECT flg_is_enabled,
       flg_check_for_credit_card,
       flg_check_for_ssn,
       flg_check_custom_patterns,
       flg_check_exception_patterns,
       flg_check_file_content
  FROM PLC.PL_RESPONSE_SCRUBBING
 WHERE policy_id = ?
EndQuery

    if ($data_guard_properties->{flg_is_enabled} && 
                                        !( 
                                         $data_guard_properties->{flg_check_for_credit_card} ||
                                         $data_guard_properties->{flg_check_for_ssn} ||
                                         $data_guard_properties->{flg_check_custom_patterns} ||
                                         $data_guard_properties->{flg_check_exception_patterns} ||
                                         $data_guard_properties->{flg_check_file_content} 
                                        ) ) {
        $validator->report_error(
            error_type => $F5::PolicyValidator::ERROR_TYPE_LOGIC,
            error_msg => 'Data Guard Configuration is not defined while Data Guard functionality is enabled');
    }
}
1;

