package F5::PolicyValidator::WildcardIncludeSlashes;

use strict;
use warnings;

use base 'F5::PolicyValidator::Default';

sub validate {
    my($self, %params) = @_;
    my $dbh = $params{dbh};
    my $logger = $params{logger};
    my $validator = $params{validator};
    my $policy = $params{policy};
    my $action = $params{action};
    my $overlay = $params{overlay};

    my $check_urls = 1;

    if ($action eq 'import_policy' && $overlay &&
        ( !($overlay->is_supported(path => 'urls')) || $overlay->get_overlay(path => 'urls')->{wildcardIncludesSlash})) {
        $check_urls = 0;
    }

    if ($check_urls) {
        my @urls = F5::ASMConfig::Entity::URL->retrieve_all(  dbh => $params{dbh},
                                                              logger => $params{logger},
                                                              policy => $params{policy},
                                                              get_criteria => { is_wildcard => 1,
                                                                                flg_wildcard_includes_slash => 0,
                                                                              },
                                                              search_filters => [ F5::ASMConfig::SearchFilter->new( field_name => 'url_name',
                                                                                                                    field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_LIKE(),
                                                                                                                    field_value => '*%',
                                                                                                                   ) ],
                                                           );

        if (@urls) {
            for my $url( @urls ){
                $validator->report_error(
                                     error_type => $F5::PolicyValidator::ERROR_TYPE_LOGIC,
                                     error_msg => "The following Wildcard URLs starting with '*' will have no matches because the wildcard will reject the leading slash appearing in every URL. Either change the URL name or enable 'Wildcard Match Includes Slashes':",
                                     entity  => $url);
            }
       }
    }

    if ($action eq 'import_policy' && $overlay &&
         ( !($overlay->is_supported(path => 'websocket-urls')) || $overlay->get_overlay(path => 'websocket-urls')->{wildcardIncludesSlash})) {
         return;
    }


    my @websocket_urls = F5::ASMConfig::Entity::WebsocketUrl->retrieve_all(  dbh => $params{dbh},
                                                                             logger => $params{logger},
                                                                             policy => $params{policy},
                                                                             get_criteria => { is_wildcard => 1,
                                                                                               flg_wildcard_includes_slash => 0,
                                                                                             },
                                                                             search_filters => [ F5::ASMConfig::SearchFilter->new( field_name => 'websocket_url_name',
                                                                                                                                   field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_LIKE(),
                                                                                                                                   field_value => '*%',
                                                                                                                                  ) ],
                                                                        );


    if (@websocket_urls) {
        for my $websocket_url( @websocket_urls ){
            $validator->report_error(
                                 error_type => $F5::PolicyValidator::ERROR_TYPE_LOGIC,
                                 error_msg => "The following Wildcard WebSocket URLs starting with '*' will have no matches because the wildcard will reject the leading slash appearing in every URL. Either change the URL name or enable 'Wildcard Match Includes Slashes':",
                                 entity  => $websocket_url);
        }
   }

}

1;

