package F5::ProcessTable;

use strict;
use warnings;

use File::Basename;
use List::Util;

sub get_process_table {
    my %procs;

    opendir(my $dirh, '/proc');
    while (my $d = readdir($dirh)) {

        next unless $d =~ /\d/;

        open(my $fh, '<', "/proc/$d/cmdline") or next;
        my $full_cmdline = join(' ', <$fh>);
        close $fh;

        my @cmd_parts = split(/\000/, $full_cmdline);

        $full_cmdline = join(' ', @cmd_parts);
        my $cmdline = $cmd_parts[0];
        if ($cmdline && $cmdline eq '/usr/bin/perl') {
            $cmdline = $cmd_parts[1];
        }

        my $basename = '';
        if ($cmdline) {
            $basename = basename($cmdline);
        }

        $procs{$d} = {
                      pid => $d,
                      full_cmdline => $full_cmdline,
                      cmdline => $cmdline,
                      basename => $basename,
                     };
    }
    closedir($dirh);
    return \%procs;
}

#---------------------------------------------------

sub is_proc_up
{
    my $proc_name = shift; # single process name

    my $proc_table = get_process_table();

    # search for the process in the process table
    if ( List::Util::first { $proc_table->{$_}->{basename} eq $proc_name } keys %$proc_table )
    {
        # the process is running
        return 1;
    }

    # not running
    return 0;
}

1;
