package F5::Provisioning;

use strict;
use warnings;

use F5::BigipUtils;
use F5::ASM::COAPI;
use F5::Cfg;

my $ASM_ENABLED = "ASM_ENABLED";
my $NOT_ENABLED = "ASM_DISABLED";

F5::ASM::COAPI::Login('root');

#----------------------------------------------------------------------

sub is_module_enabled {
    my ($module_name_lc) = @_;
    my $module_name_uc = uc($module_name_lc);
    return 1 if F5::BigipUtils::is_docker();

    my $query;

    eval {
        $query = F5::ASM::COAPI::Query->new("db_variable", { name => "provision.cpu.$module_name_lc"});
    };

    if ($@) {
        if (F5::ASM::COAPI::Login('root')) {
            $query = F5::ASM::COAPI::Query->new("db_variable", { name => "provision.cpu.$module_name_lc"});
        }
    }

    my @objs = @$query;
    my $provision_var = $objs[0];
    my $module_on;

    if ((defined $provision_var->{value}) && ($provision_var->{value} ne "" )) {
        $module_on = $provision_var->{value};
    } elsif ((defined $provision_var->{default}) &&($provision_var->{default} ne "" )) {
        $module_on = $provision_var->{default};
    }

    if (defined $module_on) {
        return ($module_on > 0) ? 1 : 0;
     }

    $module_on = F5::Cfg::cfg_get_config_item("/config/BigDB.dat","Provision.CPU.$module_name_uc","value");
    if (defined $module_on){
        return 1 if $module_on > 0;
    }   
   return 0;

}
#----------------------------------------------------------------------
sub get_provisioning_str {
    my $provisioning_state;

    if( is_module_enabled("asm") ){
        $provisioning_state = $ASM_ENABLED;
    }else{
        $provisioning_state =  $NOT_ENABLED;
    }
 
    return $provisioning_state;


}
#----------------------------------------------------------------------
sub is_asm_on {
    if(F5::BigipUtils::is_install_mode()) {
        return 1;
    }

    if (is_module_enabled("asm")) {
        return 1;
    }

  return;
}

sub was_tsconfig_run {
    return (! -e qw(/ts/var/install/run_tsconfig));
}

1;
