package F5::ResetReusedViolations;

use strict;
use warnings;

use F5::Violation;
use F5::BigipVersionUtils;
use Memoize;

memoize('get_reused_violations_by_bigip_version');

my %REUSED_VIOLATIONS_BY_VERSION = (
                                      "0009.0004.0002" => [ F5::Violation::VIOL_EVASION() ],
                                      "0010.0001.0000" => [ F5::Violation::VIOL_XML_WEB_SERVICES_SECURITY() ],
                                      "0010.0002.0000" => [
                                                           F5::Violation::VIOL_VIRUS(),
                                                           F5::Violation::VIOL_PARAMETER_REPEATED(),
                                                          ],
                                      "0011.0000.0000" => [
                                                           F5::Violation::VIOL_URL_CONTENT_TYPE(),
                                                           F5::Violation::VIOL_JSON_MALFORMED(),
                                                           F5::Violation::VIOL_JSON_FORMAT(),
                                                          ],
                                      "0011.0000.0001" => [
                                                           F5::Violation::VIOL_GEOLOCATION(),
                                                           F5::Violation::VIOL_FILE_UPLOAD(),
                                                          ],
                                      # next versions come here
                                   );

sub get_reused_violations_by_bigip_version
{
   my ($import_bigip_version) = @_;
   $import_bigip_version = F5::BigipVersionUtils::zero_pad_segments($import_bigip_version, 3);
   my %output = ();
   foreach my $version (keys %REUSED_VIOLATIONS_BY_VERSION)
   {
      if($import_bigip_version lt $version)
      {
         %output = (%output, map {$_ => 1} @{$REUSED_VIOLATIONS_BY_VERSION{$version}});
      }
   }
   
   return keys %output;
}

1;
