package F5::ResetStaging;

use strict;
use warnings;

use F5::DbUtils;
use F5::BigipUtils;

sub reset_staging_and_tightening {
    my(%params) = @_;

    my $dbh = $params{dbh};
    my $logger = $params{logger};
    my $policy_id = $params{policy_id};
    my $dbname = $params{dbname} || "PLC";
    my $table_name = $params{table_name} || [qw/PL_FILETYPES PL_OBJECTS PL_PARAM_ATTRIBUTES PL_ALLOWED_COOKIES PL_WEBSOCKET_URLS PL_REDIRECTION_DOMAINS PL_POLICY_NEGSIG_SIGNATURES/];
    my @bind_values = ();

    my $last_tightening_time = time;
    for my $current_table (@$table_name) {
        my $where_clause_and = "";
        my $where_clause_where = "";

        if ($policy_id ne '*') {
            $where_clause_and = " AND $current_table.policy_id IN (?)";
            $where_clause_where = " WHERE $current_table.policy_id IN (?)";
            push @bind_values, $policy_id;
        }
    
        if ($current_table eq 'PL_PARAM_ATTRIBUTES') {
            if (F5::DbUtils::is_sqlite($dbh)) {
                $dbh->do(<<EndQuery, undef, $last_tightening_time, @bind_values);
   UPDATE $dbname.PL_PARAM_ATTRIBUTES
      SET last_tightening_time = ?
    WHERE id IN (
   SELECT PL_PARAM_ATTRIBUTES.id
     FROM $dbname.PL_PARAM_ATTRIBUTES
LEFT JOIN $dbname.PL_GLOBAL_PARAMS 
       ON PL_PARAM_ATTRIBUTES.id = PL_GLOBAL_PARAMS.param_attrib_id 
LEFT JOIN $dbname.PL_FLOW_PARAMS 
       ON PL_PARAM_ATTRIBUTES.id = PL_FLOW_PARAMS.param_attrib_id 
LEFT JOIN $dbname.PL_OBJECT_PARAMS 
       ON PL_PARAM_ATTRIBUTES.id = PL_OBJECT_PARAMS.param_attrib_id 
    WHERE (PL_GLOBAL_PARAMS.flg_is_pattern = 1 
       OR PL_FLOW_PARAMS.flg_is_pattern = 1 
       OR PL_OBJECT_PARAMS.flg_is_pattern = 1) 
          $where_clause_and
)
EndQuery
            }
            else {
                $dbh->do(<<EndQuery, undef, $last_tightening_time, @bind_values);
   UPDATE $dbname.PL_PARAM_ATTRIBUTES 
LEFT JOIN $dbname.PL_GLOBAL_PARAMS 
       ON PL_PARAM_ATTRIBUTES.id = PL_GLOBAL_PARAMS.param_attrib_id 
LEFT JOIN $dbname.PL_FLOW_PARAMS 
       ON PL_PARAM_ATTRIBUTES.id = PL_FLOW_PARAMS.param_attrib_id 
LEFT JOIN $dbname.PL_OBJECT_PARAMS 
       ON PL_PARAM_ATTRIBUTES.id = PL_OBJECT_PARAMS.param_attrib_id 
      SET last_tightening_time = ?
    WHERE (PL_GLOBAL_PARAMS.flg_is_pattern = 1 
       OR PL_FLOW_PARAMS.flg_is_pattern = 1 
       OR PL_OBJECT_PARAMS.flg_is_pattern = 1) 
          $where_clause_and
EndQuery
            }
        } elsif ($current_table ne 'PL_POLICY_NEGSIG_SIGNATURES') {
            $dbh->do(<<EndQuery, undef, $last_tightening_time, @bind_values);
UPDATE $dbname.$current_table
   SET last_tightening_time = ?
 WHERE flg_is_pattern = 1
       $where_clause_and
EndQuery
       }

       my $date_fields_to_now_unix = ($current_table eq 'PL_POLICY_NEGSIG_SIGNATURES' || $current_table eq 'PL_REDIRECTION_DOMAINS') ? ['last_updated']
                                                                                                                                     : ['last_updated', 'staged_since'];
       my $where_fields = ($policy_id ne '*') ? { policy_id => $policy_id } : {};
       F5::DbUtils::update_row(dbh => $dbh,
                               table_name => "$dbname.$current_table",
                               rec => {},
                               set_date_fields_to_now_unix => $date_fields_to_now_unix,
                               where_fields => $where_fields);
    }
}

1;
