package F5::Scanner::Cenzic;

use strict;
use warnings;

use JSON;

use F5::ASMConfig::ASMConfig;
use F5::CenzicKey;

use base qw/F5::Scanner/;

sub policy_check_scan_results {
    my ($self, %params) = @_;

    my $policy = $self->{policy};
    my $config = $self->{config}->{api};
    my $client = $self->{client};

    my $scanner_config = $self->{client}->get_scanner_config(
                             $self->{client}->{session}, undef, $self->{policy},);

    return unless $scanner_config->{scanner_auth_data_2};

    my $response = $self->do_scanner_service_request(url => $config->{poll_scanner_scan_status_url});

    die "Cannot read response from scanner server\n" unless $response;
    my $poll_result = from_json($response->content);

    if ($poll_result->{ScanInfo}->{Status} eq 'Complete') {
        my $xml_data = $self->get_xml_data(scan_id => $scanner_config->{scanner_auth_data_2});
        $self->parse_xml(scanner_actions_fields => F5::ASMConfig::ScannerActionsFields->new({scanner_vuln_data_xml => $xml_data->{xml_data}}));
        $client->update_scanner_config(
                 $client->{session},
                 undef,
                 $policy,
                 F5::ASMConfig::ScannerConfig->new({
                     scan_requested_time => 0,
                     scanner_auth_data_1 => '',
                     scanner_auth_data_2 => ''}));
    }
}

#----------------------------------------------------------------------

sub get_xml_data {
    my($self, %params) = @_;

    my $config = $self->{config}->{api};

    my $response = $self->do_scanner_service_request(url => $config->{get_scanner_scan_results_url},
                                                     scan_id => $params{scan_id});

    die "Cannot read response from scanner server\n" unless $response;


    my $retval;
    $retval->{is_success} = $response->is_success;

    if ($response->is_success) {
        my $scan_result = from_json($response->content);
        $retval->{xml_data} = $scan_result->{ScanInfo}->{ResultDetail};
        $self->get_logger()->debug("Success. Response content: $retval->{xml_data}");
    }

    return $retval;
}

#----------------------------------------------------------------------

sub get_secret {
    my($self) = @_;

    require PassCrypt;
    return PassCrypt::decrypt_password(F5::CenzicKey::get_cenzic_key());

}

#----------------------------------------------------------------------

sub get_scan_server_address {
    my($self, %params) = @_;

    my $config = $self->{config}->{api};
    my $client = $self->{client};
    my $cenzic_server_address = $client->get_gui_settings($self->{client}->{session},
                                                          undef,
                                                          F5::ASMConfig::GuiSettings->new(gui_setting_name => 'cenzic_server_address'));

    if ($cenzic_server_address->{value}) {
        return $cenzic_server_address->{value};
    }
    else {
        if ($params{server_type} eq 'key') {
            return $config->{default_key_server};
        }
        if ($params{server_type} eq 'api') {
            return $config->{default_api_server};
        }
    }
    return '';
}

#----------------------------------------------------------------------

sub get_scanner_more_info_url {
    my($self, %params) = @_;

    my $config = $self->{config}->{api};
    my $client = $self->{client};

    my $scanner_config = $self->{client}->get_scanner_config(
                             $self->{client}->{session}, undef, $self->{policy},);

    my $response = $self->do_scanner_service_request(url => $config->{more_info_url});
    die "Cannot read response from scanner server\n" unless $response;


    my $result = from_json($response->content);
    my $retval;

    if ($result->{ReturnCode}) {
        $retval = F5::ASMConfig::ScannerMoreInfoWindow->new();
    }
    else {
        $retval = F5::ASMConfig::ScannerMoreInfoWindow->new(
            url => $result->{Url},
            height => $result->{Height},
            width => $result->{Width},);
    }

    return $retval;
}

#----------------------------------------------------------------------

sub get_global_scanner_more_info_url {
    my($self, %params) = @_;

    my $config = $self->{config}->{api};
    my $client = $self->{client};

    my $scanner_config = $self->{client}->get_global_scanner_config($self->{client}->{session});

    my $response = $self->do_scanner_service_request(url => $config->{more_info_url});
    die "Cannot read response from scanner server\n" unless $response;


    my $result = from_json($response->content);
    my $retval;

    if ($result->{ReturnCode}) {
        $retval = F5::ASMConfig::ScannerMoreInfoWindow->new();
    }
    else {
        $retval = F5::ASMConfig::ScannerMoreInfoWindow->new(
            url => $result->{Url},
            height => $result->{Height},
            width => $result->{Width},);
    }

    return $retval;
}

#----------------------------------------------------------------------

sub get_scanner_more_info_url_with_subscription {
    my($self, %params) = @_;

    my $config = $self->{config}->{api};
    my $client = $self->{client};

    my $scanner_config = $self->{client}->get_scanner_config(
                             $self->{client}->{session}, undef, $self->{policy},);

    my $response = $self->do_scanner_service_request(url => $config->{more_info_url_with_subscription},
                                                     subscription_id => $scanner_config->{scanner_auth_data_1});
    die "Cannot read response from scanner server\n" unless $response;

    my $result = from_json($response->content);
    my $retval;

    if ($result->{ReturnCode}) {
        $retval = F5::ASMConfig::ScannerMoreInfoWindow->new();
    }
    else {
        $retval = F5::ASMConfig::ScannerMoreInfoWindow->new(
            url => $result->{Url},
            height => $result->{Height},
            width => $result->{Width},);
    }

    return $retval;
}

1;
