package F5::ServerLookup;

use strict;
use warnings;

use Socket qw(:addrinfo SOCK_RAW);
use F5::SystemUtils;

sub get_ip_address {
  my($hostname,$timeout) = @_;

  my $ret_hostname;
  my $err;
  my $res = undef;
  eval {
      local $SIG{ALRM} = sub { die "Timeout triggered\n" };
      if ($timeout) {
          alarm $timeout;
      }

      ($err, $ret_hostname) = getaddrinfo($hostname, "", {socktype => SOCK_RAW});

      if ($timeout) {
          alarm 0;
      }

      if (defined $ret_hostname) {
          ($err, $ret_hostname) = getnameinfo($ret_hostname->{addr}, NI_NUMERICHOST, NIx_NOSERV);
          die "Cannot getnameinfo - $err" if $err;
      }
      elsif (-e '/usr/local/bin/find-activate.pl') {
          my @params;
          push ( @params, '/usr/local/bin/find-activate.pl','-l',$hostname);
          $res =  F5::SystemUtils::run_system_tool( command_line_params => \@params,
                                                    timeout => $timeout,
                                                  );
          if ($res && $res->{completed_successfully}) {
              $ret_hostname = $res->{standard_output};
              chomp($ret_hostname);
          }
      }
      else {
          $ret_hostname = $hostname;
      }

  };

  my $ex = $@;
  if ($timeout) {
      alarm 0;
  }
  if ($ex) {
      die "Can't get hostname IP address\n";
  }

  if ($res && ! $res->{completed_successfully}) {
      die "Can't get hostname IP address ($res->{standard_error})\n";
  }

  return $ret_hostname;

}

1;
