package F5::SetDbSize;

use strict;
use warnings;

use Math::BigInt;
# f5 packges

use F5::Syslog;
use F5::Logger;
use F5::Cfg;
use F5::Logger;

use constant DISK_PATH => '/var/lib/mysql';
use constant IBDATA1_PATH => '/var/lib/mysql/ibdata1';
use constant MY_CNF => '/var/lib/mysql/my.cnf';
use constant CLEAN_DB => '/etc/ts/tools/clean_db.cfg';
use constant GIGA => 1073741824;
use constant MEGA => 1048576;

use constant PRATITION_LINE => 1;
use constant PRATITION_COLUMN => 1;
use constant VOLUME_LINE => 2;
use constant VOLUME_COLUMN => 0;
use constant VOLUME_SIGN_PLACE => 1;


#-------------------------------------------------------------------------------
sub new {
   my ($class, $logger) = @_;

   my $self = bless({}, $class);

   $self->{logger} = $logger;

   return $self;
}
#-------------------------------------------------------------------------------

sub set_ibdata1_size{
   my ($self) = @_;
   my $logger =  $self->{logger};

   $logger->info("Starting set ibdata1 size ...\n");

   my $partition_size =  $self->get_relevant_partition_size(DISK_PATH);
   $logger->info("Partition size: $partition_size\n");

   my $current_file_size =  $self->get_file_size(IBDATA1_PATH);
   $logger->info("Current ibdata1 size: $current_file_size\n");

   my $desired_size = $self->get_desired_size($partition_size); 
   $logger->info("Desired size: $desired_size");

   if($desired_size < $current_file_size){
      $logger->info("The ibdata1 <$current_file_size> size is larger than the internal allowed limit <$desired_size>.");  
      F5::Syslog::ts_syslog('err', "The ibdata1 <$current_file_size> size is larger than the internal allowed limit <$desired_size>.");
      exit;
   }
   my $size  = $desired_size / MEGA; 
   $self->set_value_in_file( file_name => MY_CNF,   
                      section => "mysqld",
                      item => "innodb_data_file_path",
                      value => "ibdata1:10M:autoextend:max:".$size."M");
   $logger->info("Setting ibdata1 size finished successfully, a new size is: $size"."M");
   F5::Syslog::ts_syslog('info', "Setting ibdata1 size finished successfully, a new size is: ".$size."M");
}
#--------------------------------------------------------------------------
sub get_desired_size{
   my ($self, $partition_size) = @_;

   my $ibdata1_limit = Math::BigInt->new(15 * GIGA);
   $partition_size = ($partition_size * 7) / 10;
   return $self->min($partition_size, $ibdata1_limit);
}
#--------------------------------------------------------------------------
sub set_value_in_file{
   my ($self, %params) = @_;
   
   F5::Cfg::cfg_set_config_item($params{file_name}, $params{section}, $params{item}, $params{value});
}
#-------------------------------------------------------------------------
sub min{
   my ($self, $a, $b) = @_;

   return ($a < $b) ? $a : $b;
}
#-------------------------------------------------------------------------
sub get_relevant_partition_size{
    my ($self, $diskpath) = @_;
    my $s=0;
    my @r;
    my $vol_col = VOLUME_COLUMN;
    my $vol_line = VOLUME_LINE;
    my $prat_col = PRATITION_COLUMN;

# Check for Centos 7 df which has different output format
    my $returnCode = system( "df --output=size / > /dev/null 2>&1");
    if ( $returnCode != 0 ) { 
      @r = `df $diskpath`;
    } else {
      @r = `df --output=size $diskpath`;
      $vol_line = 1;
      $vol_col = 0;
      $prat_col = 0;
    }

    my $l=$r[VOLUME_SIGN_PLACE];

    chomp $l;
    if ( -l $diskpath ) {
      # Discovery
      $s = Math::BigInt->new(((split(' ', $r[$vol_line]))[$vol_col]) );
      # if partition size than 20GB, limit to 20GB, that will put max ~9.2GB with above calculations
      if ($s > (1024*1024*20)) {
        $s =  (1024*1024*20);
      }
      $s *=1024;
    } elsif ($l =~ /ASWADB/){
      $s = Math::BigInt->new(((split(' ', $r[$vol_line]))[$vol_col]) * 1024);
    }else{
      $s = Math::BigInt->new((split(' ', $r[PRATITION_LINE]))[$prat_col]);
      $s *=1024;
    }
    return $s;
}
#-------------------------------------------------------------------------
sub get_file_size{
   my ($self, $filename) = @_;

   return  Math::BigInt->new((stat($filename))[7]);
}
1;
