package F5::SpecialCharactersUnmap;

use strict;
use warnings;

sub unmap_all {
    my (%params) = @_;
    
    my @convert_tables = (
        {
            table_name  => 'PL_DYNAMIC_FLOWS',
            fields      => [qw{regexp_prefix regexp_body regexp_suffix}],
            db_type     => 'PLC',
        },
        {
            table_name  => 'PL_EXTRACT_ATTRIBUTES',
            fields      => ['regexp_body'],
            db_type     => 'PLC',
        },
        {
            table_name  => 'PL_PARAM_ATTRIBUTES',
            fields      => ['regexpr_values'],
            db_type     => 'PLC',
        },
        {
            table_name  => 'PL_GLOBAL_EXTRACTS',
            fields      => ['applies_to_regexp'],
            db_type     => 'PLC',
        },
        {
            table_name  => 'PL_ALTERNATE_RESPONSES',
            fields      => [qw{alternate_response_content alternate_response_header}],
            db_type     => 'PLC',
        },
        {
            table_name  => 'ACCOUNTS',
            db_type     => 'DCC',
            fields      => ['dynamic_session_pattern'],
            upgrade_only => 1,
        },
    );

    my %replace_special_chars = (
        "\n" => "\x04",
        "\r" => "\x03",
        ","  => "\x02",
    );

    my $dbh = $params{dbh};
    my $logger = $params{logger};
    my $policy_id = $params{policy_id};
    my $databases = $params{databases};
    
    my $where_clause = '';
    if ($policy_id ne '*') {
        $where_clause = " WHERE policy_id in ($policy_id)";
    }

    for my $convert_table (@convert_tables) {
        if ($convert_table->{upgrade_only} && $policy_id ne '*') {
            next;
        }

        my $convert_dbname = $databases->{$convert_table->{db_type}}->{staging_dbname} || $convert_table->{db_type};

        for my $field (@{$convert_table->{fields}}){
            while (my($dest,$src) = each(%replace_special_chars)) {
                my $sql = <<"EndQuery";
                UPDATE $convert_dbname.$convert_table->{table_name}
                SET $field = REPLACE ( $field, ?, ? )
                $where_clause
EndQuery
                $dbh->do($sql, undef, $src, $dest);
            }
        }
    }

  return 1;
}

1;
