package F5::Syslog;

use strict;
use warnings;

use F5::BigipUtils;
use File::Basename;
use Sys::Syslog qw/:DEFAULT setlogsock/;

my $SYSLOG_FACILITY = 'local3';
my $LTM_SYSLOG_FACILITY = 'local0';

#--------------------------------------------------------------------------------
# arguments:
#    component (use constant from Constants)
#    code      (integer to indicate status/error code)
#    event     (use constant from Constants)
#    format    (printf format for description, with addition of '%m' for $!)
#    args      (args for printf)
sub write_syslog {
  my ($component, $code, $event, $format, @args) = @_;

  return simple_syslog(
                       "(ts_proc)$component",
                       $SYSLOG_FACILITY,
                       'notice',
                       "($code)($event): $format",
                       @args);
}

#--------------------------------------------------------------------------------
## See: man logger for valid facility names and valid level names
## and man syslog(3) for the order of the levels

sub simple_syslog {
  my ( $ident, $facility, $priority, $format, @args ) = @_;


  return if F5::BigipUtils::is_docker();
  # these routines throw a lot of dies (croaks)
  eval {
    setlogsock('unix');
    openlog($ident, 'cons,ndelay,pid', $facility);
    syslog($priority, $format, @args);
    closelog();
  };

  if ($@) {
    warn "simple_syslog failed ($@)";
    return;                     # FAILURE
  }

  # SUCCESS
  return 1;
}

#--------------------------------------------------------------------------------

sub ts_syslog {
  my ( $priority, $format, @args ) = @_;

  if (F5::BigipUtils::is_install_mode()) {
    print STDERR sprintf($format, @args) if ($priority eq 'err');
    return 1;
  }

  my ( $ident, $facility ) = ( get_proc_name(), $SYSLOG_FACILITY );
  simple_syslog( $ident, $facility, $priority, $format, @args );
}

#--------------------------------------------------------------------------------

sub ltm_syslog {
  my ( $priority, $format, @args ) = @_;
  my ( $ident, $facility ) = ( get_proc_name(), $LTM_SYSLOG_FACILITY );
  simple_syslog( $ident, $facility, $priority, $format, @args );
}

#--------------------------------------------------------------------------------

sub get_proc_name
{
    return File::Basename::basename( $0 );
}

1;
