package F5::TmshParser;

sub parse_config {
    my (%params) = @_;
    my $text = $params{text};
    my $filter_type = $params{filter_type}; # for top type only
    my %struct;
    my @context = (\%struct);
    my $depth = 0;
    my $skip = 0;
    for my $line (split("\n", $text)) {
        $line =~ s/\#.*$//; # remove comments (in cfg file)
        if ($line =~ /(.*)\s([\S]+)\s\{/xms) { #Take the last token before the "{" as the key
            my $type = $1;
            my $key = $2;
            if ($depth == 0 && $filter_type) {
                $skip = ($type eq $filter_type) ? 0 : 1;
            }
            if (!$skip) {
                $context[-1]->{$key} = {};
                if ($depth == 0) {
                    $context[-1]->{$key}->{__type} = $type;
                }
            }
            if ($line !~ /}/ ) {
                push @context, $context[-1]->{$key} if !$skip;
                $depth++;
            }
        }
        elsif ($line =~ /}/ ) {
            pop @context if !$skip;
            $depth--;
        }
        elsif (!$skip) {
            my ($key, $value) = split(" ", $line, 2); # preserve spaces inside value
            if ($key ne '') { # can be empty after removing comments
                $context[-1]->{$key} = $value;
            }
        }
    }
    return \%struct;
}

package F5::TmshParser::CfgFile;

sub new {
    my ($class, %params) = @_;
    my $filename = $params{filename};
    my $logger = $params{logger};
    return if !$filename;
    my $fh;

    if (!open ($fh, "<", $filename)) {
        if ($logger) {
            $logger->error("can't open file $filename");
        }
        return { completed_successfully => 0 };
    }

    my $rv = { completed_successfully => 1 };
    $rv->{content} = join('', <$fh>);
    close $fh;

    $rv->{parsed} = F5::TmshParser::parse_config(text => $rv->{content},
                                                 filter_type => $params{filter_type});
    return $rv;
}

1;
