package F5::UrlContentUpgrade;

use strict;
use warnings;
use F5::BigipUtils;
#----------------------------------------------------------------

sub populate_url_content_defaults {
    my(%params) = @_;

    my $dbh = $params{dbh};
    my $dbname = $params{dbname} || 'PLC';
    my $policy_id =  $params{policy_id};
    my $idmap = $params{idmap};

    # Add missing defaults for all URLs
    my $policy_where_clause = '';
    my @bind_values;
    if ($policy_id ne '*') {
        $policy_where_clause = 'AND url.policy_id = ?';
        push(@bind_values, $policy_id);
    }

    $dbh->do(<<EndQuery, undef, @bind_values);
INSERT INTO $dbname.PL_OBJECT_CONTENT_PROFILES (policy_id, obj_id, header_name, header_value, header_order, enforcement_type)
 SELECT url.policy_id, url.id, '*', '*', 0, 'http'
    FROM $dbname.PL_OBJECTS url
     LEFT JOIN $dbname.PL_OBJECT_CONTENT_PROFILES url_assoc
            ON (    url.id = url_assoc.obj_id
                AND url_assoc.header_order = 0)
   WHERE url_assoc.header_name IS NULL
         $policy_where_clause
EndQuery

    # Add URL content profile association for URLs previously marked as check AMF
    if ($idmap->{'__private_save'} && $idmap->{'__private_save'}->{add_amf_association}) {


        for my $amf_url (@{$idmap->{'__private_save'}->{add_amf_association}}) {
            if ($policy_id eq '*') {
                $policy_where_clause = 'AND url.policy_id = ?';
                @bind_values = ($amf_url->{policy_id});
            }

            my $db_url = $dbh->selectrow_hashref(<<EndQuery, undef, $amf_url->{uri}, $amf_url->{flg_is_pattern}, @bind_values);
  SELECT url.id AS obj_id, MAX(header_order) AS max_header_order, url.policy_id AS policy_id
    FROM $dbname.PL_OBJECTS url,
         $dbname.PL_OBJECT_CONTENT_PROFILES url_assoc
   WHERE url.id = url_assoc.obj_id
     AND url.uri = ?
     AND url.flg_is_pattern = ?
     $policy_where_clause
GROUP BY url.id
EndQuery

            F5::DbUtils::insert_row(dbh => $dbh,
                                    table_name => "$dbname.PL_OBJECT_CONTENT_PROFILES",
                                    rec => {
                                            policy_id => $db_url->{policy_id},
                                            obj_id => $db_url->{obj_id},
                                            header_name => 'Content-Type',
                                            header_value => '*[aA][mM][fF]*',
                                            header_order => $db_url->{max_header_order} + 1,
                                            enforcement_type => 'apply_value_signatures',
                                           }
                                   );
        }
    }
}

#----------------------------------------------------------------------

1;
