package F5::Utils::Policy;
use strict;
use warnings;

use F5::DbUtils;
use F5::ASMConfig::Entity::Policy;
use F5::ASMConfig::Entity::PolicySignatureRequirement;
use F5::DateTimeUtils;

#----------------------------------------------------------------------

sub get_policy_signature_requirements {
    my (%params) = @_;
    my $dbh = $params{dbh};
    my $datetime = $params{datetime};
    my $timestamp = $params{timestamp};
    my $sig_tags = $params{sig_tags};
    if (!defined($timestamp) && defined($datetime)) {
        $timestamp = F5::DateTimeUtils::convert_any_datetime_to_epoch($datetime);
    }
    my $sig_tag_filter = F5::ASMConfig::SearchFilter->new(
                                         field_name       => 'sig_tag',
                                         field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_IN(),
                                         field_value_list  => $sig_tags,
                                     );

    my @sig_reqs;
    if (defined($timestamp)) {
        my @policy_sig_req_max = F5::ASMConfig::Entity::PolicySignatureRequirement->retrieve_all(
                                  dbh                  => $dbh,
                                  full_policy_populate => 1,
                                  search_filters       => [
                                     F5::ASMConfig::SearchFilter->new(
                                         field_name       => 'max_revision_datetime',
                                         field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_LT(),
                                         field_value      => $timestamp,
                                     ),
                                     $sig_tag_filter
                                  ],
                                  get_criteria => {},
        );
        my @policy_sig_req_min = F5::ASMConfig::Entity::PolicySignatureRequirement->retrieve_all(
                                  dbh                  => $dbh,
                                  full_policy_populate => 1,
                                  search_filters       => [
                                     F5::ASMConfig::SearchFilter->new(
                                         field_name       => 'min_revision_datetime',
                                         field_comparison => F5::ASMConfig::FieldComparison::FIELD_COMPARISON_GT(),
                                         field_value      => $timestamp,
                                     ),
                                     $sig_tag_filter
                                  ],
                                  get_criteria => {},
                              );
    
        @sig_reqs = (@policy_sig_req_max, @policy_sig_req_min);
    }
    else {
         @sig_reqs = F5::ASMConfig::Entity::PolicySignatureRequirement->retrieve_all(
                                  dbh                  => $dbh,
                                  full_policy_populate => 1,
                                  search_filters       => [ $sig_tag_filter ],
                                  get_criteria => {},
                              );
    }
    return @sig_reqs;
}

#----------------------------------------------------------------------


1;
