package F5::Utils::Storable;

use strict;
use warnings;
use YAML::XS;
use Storable qw(fd_retrieve store_fd);


$YAML::Numify = 1;
$YAML::XS::Numify = 1;
$YAML::XS::QuoteNumericStrings = 0;


#----------------------------------------------------------------------

sub write_file {
    my(%params) = @_;
    my $input_file = $params{input_file};
    my $output_file = $params{output_file};

    my $fh;
    eval {
        no warnings;
        open OLD_STDOUT, ">&STDOUT";
        open NEW_STDOUT, ">", $output_file;
        open STDOUT, ">&NEW_STDOUT";
        open $fh, "| /usr/bin/gzip";
        open STDOUT, ">&OLD_STDOUT";
    };

    my(@documents) = YAML::XS::LoadFile($input_file);
    for my $doc (@documents) {
        my $result;
        eval {
            $result = store_fd($doc, $fh);
        };
        if (! defined $result || $@){
            my $err = $@ || 'unknown';
            close($fh);
            die("Failed to write record to exported policy file (store_fd failed - $err).");
            
        }
    }

    close($fh);
}

#----------------------------------------------------------------------

sub read_file {
    my(%params) = @_;
    my $in_filename = $params{in_filename};
    my $out_filename = $params{out_filename};

    open(my $in_fh, "/bin/gzip -q -d -c '$in_filename' |");
    open(my $out_fh, '>', $out_filename);

    while(!eof($in_fh)) {
        my $import_data = fd_retrieve($in_fh);
        print $out_fh YAML::XS::Dump($import_data);
    }

    close($in_fh);
    close($out_fh);
}

#----------------------------------------------------------------------

1;
