package F5::XMLSecurityUpgrade;

use strict;
use warnings;

my %WSENGINE_DEFENSE_ATTRIBS = (
                                'xml.parser.maximumDocumentSize' => 1,
                                'xml.parser.maximumElements' => 2,
                                'xml.parser.maximumNameLength' => 3,
                                'xml.parser.maximumAttributeValueLength' => 4,
                                'xml.parser.maximumDocumentDepth' => 5,
                                'xml.parser.maximumChildrenPerElement' => 6,
                                'xml.parser.maximumAttributesPerElement' => 7,
                                'xml.parser.maximumNSDecls' => 8,
                                'xml.parser.allowDTDs' => 9,
                                'xml.parser.allowExternalReferences' => 13,
                                'xml.parser.tolerateLeadingWhitespace' => 16,
                                'xml.parser.tolerateCloseShorthand' => 17,
                                'xml.parser.tolerateNumericNames' => 18,
                               );

#----------------------------------------------------------------------

sub save_xml_defense_levels {
  my(%params) = @_;

  my $table_data = $params{table_data};
  my $id_map = $params{id_map};
  my $xml_defense_levels = {};

  for my $row (values %$table_data) {
    my $new_profile_id = $row->{id};
    if ($id_map) {
      $new_profile_id = $id_map->{profile_id}->{$row->{id}};
    }

    $xml_defense_levels->{$new_profile_id} = {
                                                  profile_id => $new_profile_id,
                                                  defense_level => $row->{defense_level},
                                                 };

  }

  return $xml_defense_levels;
}

#----------------------------------------------------------------------

sub convert_xml_defense_attributes {
  my(%params) = @_;

  my $dbh = $params{dbh};
  my $table_data = $params{table_data};

  # Override defaults with converted pre-existing defense attribs
  for my $key (keys %$table_data) {
    my $row = $table_data->{$key};

    if ($WSENGINE_DEFENSE_ATTRIBS{$row->{def_attrib_name}}) {
      $row->{def_attrib_id} = $WSENGINE_DEFENSE_ATTRIBS{$row->{def_attrib_name}};
    }
    else {
      delete $table_data->{$key};
    }
  }
}

#----------------------------------------------------------------------

sub populate_xml_defense_attributes {
  my(%params) = @_;

  my $dbh = $params{dbh};
  my $dbname = $params{dbname} || 'PLC';
  my $xml_profile_defense_levels = $params{xml_profile_defense_levels};

  for my $xml_profile (values %$xml_profile_defense_levels) {
    # Add missing defaults
    $dbh->do(<<EndQuery, undef, $xml_profile->{profile_id}, $xml_profile->{defense_level});
INSERT INTO $dbname.PL_XML_PROFILE_DEFENSE_ATTRIBUTES (policy_id, profile_id, def_attrib_id, def_attrib_value)
SELECT prof.policy_id,
       prof.id,
       def_attrib_defaults.def_attrib_id,
       def_attrib_defaults.def_attrib_value
  FROM           ($dbname.PL_CONTENT_PROFILES prof,
                  $dbname.XML_DEFENSE_ATTRIBUTES_DEFAULTS def_attrib_defaults)
       LEFT JOIN $dbname.PL_XML_PROFILE_DEFENSE_ATTRIBUTES def_attrib
              ON (prof.id = def_attrib.profile_id
             AND def_attrib.def_attrib_id = def_attrib_defaults.def_attrib_id)
WHERE def_attrib.def_attrib_id IS NULL
  AND prof.id = ?
  AND def_attrib_defaults.defense_level = ?
EndQuery
  }
}

#----------------------------------------------------------------------

sub set_primary_enforced_validation_file {
  my(%params) = @_;

  my $dbh = $params{dbh};
  my $dbname = $params{dbname} || 'PLC';

  $dbh->do(<<EndQuery);
UPDATE $dbname.PL_XML_PROFILE_VALIDATION_FILES
   SET flg_is_primary_enforced = 1, import_url=''
 WHERE import_url IS NULL
EndQuery
}

#----------------------------------------------------------------------

1;
