#!/usr/bin/perl

use strict;
use warnings;

use XML::Twig;
use YAML::XS;

my $twig = XML::Twig->new(pretty_print => 'indented',
                          comments => 'process',
                          twig_handlers => {
                                            '/xs:schema/xs:element[@name="attack_type"]/xs:simpleType/xs:restriction' => \&add_attack_types,
                                           },
                         );

$twig->safe_parsefile('common_template.xsd');
$twig->print();

sub add_attack_types {
    my $attack_types_file = "$ENV{DEVFS}/var/ts/install/attack_types.yaml";
    my $attack_types = YAML::XS::LoadFile($attack_types_file);
    for my $attack_type (@$attack_types) {
        my $element = {
                       'value'  => $attack_type->{attack_type_name},
                       'asm:internalID' => $attack_type->{attack_type_id},
                      };
        if ($attack_type->{attack_type_display_name}) {
            $element->{'asm:displayName'} = $attack_type->{attack_type_display_name};
        }

        XML::Twig::Elt->new('xs:enumeration' => $element)->paste( last_child => $_ );
    }
}
