package F5::ASMConfig::Entity::ImportVulnerabilitiesTask;

use strict;
use warnings;

use base qw/F5::ASMConfig::TaskBase F5::ASMConfig::ImportVulnerabilitiesTask/;

#----------------------------------------------------------------------

sub before_store {
    my($self, %params) = @_;

    my $db_rec = $params{db_rec};
    my $store_mode = $params{store_mode};
    my $store_obj = $params{store_obj};
    my $existing_obj = $params{existing_obj};
    my $call_flags = $params{call_flags};
    
    if ($store_mode eq "add") {
        my $policy = $store_obj->{target_policy};
        if (!$policy) {
            die {
                  error_message => 'Policy not found',
                  error_code => F5::ASMConfig::ErrorCode::MISSING_POLICY(),
                  error_level => 'info',
                };
        }
        my $scanner_config = F5::ASMConfig::Entity::ScannerConfig->new(policy => $policy,
                                                                       dbh => $self->{dbh},
                                                                       get_criteria => {});
        my $error_message;
        
        if (!$scanner_config || !$scanner_config->{scanner_type}) {
            $error_message = 'A Vulnerability Assessment Tool must be configured to import Vulnerabilities';
        }
        #for hailstorm there must be either file/file_content or scan_id/subscription
        elsif ($scanner_config->{scanner_type} eq 'hailstorm') {
            if ( !$store_obj->{filename} && !$store_obj->{file_contents} && 
                (!$store_obj->{scan_id} || !$store_obj->{subscription_id})) {
                $error_message = 'You must provide either an import file, or a subscriptionId and scanId (for download)';
            }
            elsif(($store_obj->{scan_id} || $store_obj->{subscription_id}) &&
                  ($store_obj->{filename} || $store_obj->{file_contents})) {
                $error_message = 'You must provide either an import file, or a subscriptionId and scanId (for download)';
            }
        }
        #for whitehat sentitel either api_key and scanner_site_name are required or file/file_content 
        elsif ($scanner_config->{scanner_type} =~ /sentinel/) {
            if (!$store_obj->{filename} && !$store_obj->{file_contents} &&
                !($scanner_config->{api_key} && $scanner_config->{scanner_site_name}) ) {
                    $error_message = 'API Key and Site Name must be selected to download verified vulnerabilities directly from the WhiteHat Sentinel service, or import file must be specified';    
            }
        }
        #for other scanners - only file/file_content required
        elsif (!$store_obj->{filename} && !$store_obj->{file_contents}) {
            $error_message = 'Import file must be specified';
        }

        if ($error_message) {
             die {
              error_message => $error_message,
              error_code => F5::ASMConfig::ErrorCode::ACTION_NOT_ALLOWED(),
              error_level => 'info',
            };
        }
    }
    return $self->SUPER::before_store(%params);
}

#----------------------------------------------------------------------

1;
