package F5::ASMConfig::Entity::ResolveVulnerabilitiesTask;

use base qw/F5::ASMConfig::TaskBase F5::ASMConfig::ResolveVulnerabilitiesTask/;

#----------------------------------------------------------------------

sub before_store {
    my($self, %params) = @_;

    if ($params{store_mode} eq 'add') {
        my $dbh = $self->{dbh};
        my $policy =  $self->{policy} || $params{policy};
        my $policy_id = $policy->{policy_id};

        if (!$policy_id || $policy_id == -1) {
            die "Valid policy required.\n";
        }

        my $vulnerabilities = $self->get_new_value(%params, field_name => 'vulnerabilities') || [];
        if (!@$vulnerabilities) {
            die "Vulnerabilities are required\n";
        }

        my $stage_vulnerabilities = $self->get_new_value(%params, field_name => 'stage_vulnerabilities');
        my %policy_ids = ($policy_id => 1);
        for my $vuln (@$vulnerabilities) {
            if (!$vuln->{policy}) {
                $vuln = F5::ASMConfig::Entity::ScannerVuln->new(dbh => $self->{dbh}, get_criteria => $vuln, policy => $policy);
            }
            $policy_ids{$vuln->{policy}->{policy_id}}++;
            if ($vuln->{flg_is_resolvable} == F5::ASMConfig::ScannerVulnType::SCANNER_VULN_TYPE_NOT_RESOLVABLE() ||
                ($vuln->{flg_is_resolvable} == F5::ASMConfig::ScannerVulnType::SCANNER_VULN_TYPE_RESOLVABLE_MANUALLY() && $stage_vulnerabilities)) {
                $vuln = F5::ASMConfig::Entity::ScannerVuln->new(dbh => $self->{dbh}, get_criteria => $vuln, policy => $policy);
                my $field_def = $vuln->fields->{flg_is_resolvable};
                my $action = $stage_vulnerabilities ? " and stage " : "";
                my $field_display_name = $vuln->get_field_display_name_string(%params, 
                                                                              field_def => $field_def,
                                                                              field_name => 'flg_is_resolvable');
                my $field_value = $vuln->{flg_is_resolvable};
                if($params{call_flags}->{rest_call}) {
                    $field_value = $vuln->map_rest_field_value( %params,
                                                                field_def => $field_def,
                                                                field_value => $field_value,
                                                                field_name => 'flg_is_resolvable',
                                                                direction => 'to_rest',
                                                               );
                }

                my $error = "Can not resolve" . ($stage_vulnerabilities ? " and stage" : "") . " Vulnerabilities with $field_display_name '$field_value'\n";
                die $error;
            }
        }
  
        if (keys %policy_ids > 1) {
            die "All Vulnerabilities must be from the same Security Policy ($policy->{policy_name})\n";
        }
    }
    return $self->SUPER::before_store(%params);
}

#----------------------------------------------------------------------

sub self_link { #Ensure policy is not used as part of the self link path
    my ($self, %params) = @_;
    my $link = $params{base_link} || '';
    $link .= '/' . $self->element_rest_path_name() . "/" . $self->{rest_uuid};
    $link = $self->add_ver_to_self_link(%params,
                                        'link' => $link);

    return $link;
}

#----------------------------------------------------------------------


1;
