package F5::ASMConfig::Entity::TerminateVulnerabilityAssessmentTask;

use strict;
use warnings;

use base qw/F5::ASMConfig::TaskBase F5::ASMConfig::TerminateVulnerabilityAssessmentTask/;

#----------------------------------------------------------------------

sub before_store {
    my($self, %params) = @_;

    my $db_rec = $params{db_rec};
    my $store_mode = $params{store_mode};
    my $store_obj = $params{store_obj};
    my $existing_obj = $params{existing_obj};
    my $call_flags = $params{call_flags};
    
    if ($store_mode eq "add") {
        my $policy = $store_obj->{target_policy};
        if (!$policy) {
            die {
                  error_message => 'Policy not found',
                  error_code => F5::ASMConfig::ErrorCode::MISSING_POLICY(),
                  error_level => 'info',
                };
        }
        my $scanner_config = F5::ASMConfig::Entity::ScannerConfig->new(policy => $policy,
                                                                       dbh => $self->{dbh},
                                                                       get_criteria => {});
        my $error_message;
        if (!$scanner_config || !$scanner_config->{scanner_type}) {
            $error_message = 'A Vulnerability Assessment Tool must be configured to terminate Vulnerability Assessment';
        }
        elsif ($scanner_config->{scanner_type} ne 'hailstorm') {
            $error_message = 'Configured Vulnerability Assessment Tool does not support automated Vulnerability Assessment';
        }
        elsif (!$scanner_config->{scanner_auth_data_2}) {
            $error_message = 'No Vulnerability Assessment in progress to terminate';
        }

        if ($error_message) {
             die {
              error_message => $error_message,
              error_code => F5::ASMConfig::ErrorCode::ACTION_NOT_ALLOWED(),
              error_level => 'info',
            };
        }
    }
    return $self->SUPER::before_store(%params);
}

#----------------------------------------------------------------------

1;
