package F5::ASMConfig::Handler::GetVulnerabilityAssessmentSubscriptionsTask;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::TaskBase/;
use F5::ASMConfig::Entity::GetVulnerabilityAssessmentSubscriptionsTask;

our @handled_functions = (
                            {
                             function_name => 'add_get_vulnerability_assessment_subscriptions_task',
                             criteria_index => 3,
                             store_index    => 4,
                            },
                            {
                              function_name => 'update_get_vulnerability_assessment_subscriptions_task',
                              criteria_index => 3,
                              store_index => 4
                            },
                            { 
                             function_name => 'execute_get_vulnerability_assessment_subscriptions_task',
                            },
                            {
                             function_name => 'delete_get_vulnerability_assessment_subscriptions_task',
                            },
                            {
                             function_name => 'delete_many_get_vulnerability_assessment_subscriptions_tasks',
                            },

                         );

our $entity_class = 'F5::ASMConfig::Entity::GetVulnerabilityAssessmentSubscriptionsTask';

#----------------------------------------------------------------------

sub add_get_vulnerability_assessment_subscriptions_task {
  my($self, @rpc_params) = @_;
  return F5::ASMConfig::Handler::TaskBase::add_task(self => $self,
                                                    rpc_params => \@rpc_params,
                                                    entity_class => $entity_class,
                                                    task_type => 'get_vulnerability_assessment_subscriptions',);
}

#----------------------------------------------------------------------

sub update_get_vulnerability_assessment_subscriptions_task {
    my ($self, @rpc_params) = @_;   
    return F5::ASMConfig::Handler::TaskBase::update_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                         task_type => 'get_vulnerability_assessment_subscriptions',);
}

#----------------------------------------------------------------------

sub get_vulnerability_assessment_subscriptions {
    my (%params) = @_;
    my $self       = $params{self};
    my $session    = $params{session};
    my $call_flags = $params{call_flags};
    my $task       = $params{task};


    my $policy = $task->{target_policy};
    if (!$policy) {
        return $self->handle_error(action => 'execute',
                                   error_thrown => 'Policy not found',
                                   error_code => F5::ASMConfig::ErrorCode::MISSING_POLICY(),
                                   call_flags => $call_flags,
                                   entity_class => $entity_class,
                                   thrift_obj => $task,
                                  );
    }


    my $rv;
    $policy = F5::ASMConfig::Cache::get_cached_policy(policy => $policy, dbh => $self->{dbh});

    eval {
        $rv = $self->get_scanner_subscriptions($session, $call_flags, $policy);
    };

    if ($@) {
        $task->{message} = $@->{error_message};
        $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_FAILURE();
    }
    else {
        $task->{subscriptions} = $rv;
        $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_COMPLETED();
    }

}


#----------------------------------------------------------------------
sub execute_get_vulnerability_assessment_subscriptions_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::execute_task(self => $self,
                                                          rpc_params => \@rpc_params,
                                                          entity_class => $entity_class,
                                                          task_type => 'get_vulnerability_assessment_subscriptions',
                                                          execute_func => sub { get_vulnerability_assessment_subscriptions(@_); },
                                                          );
}

#----------------------------------------------------------------------

sub delete_get_vulnerability_assessment_subscriptions_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                        );
}

#----------------------------------------------------------------------

sub delete_many_get_vulnerability_assessment_subscriptions_tasks {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_many_tasks(self => $self,
                                                               rpc_params => \@rpc_params,
                                                               entity_class => $entity_class,
                                                              );
}

#----------------------------------------------------------------------

1;
