package F5::ASMConfig::Handler::ResolveVulnerabilitiesTask;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::TaskBase/;
use F5::ASMConfig::Entity::ResolveVulnerabilitiesTask;

our @handled_functions = (
                            {
                             function_name => 'add_resolve_vulnerabilities_task',
                             criteria_index => 3,
                             store_index    => 4,
                            },
                            {
                              function_name => 'update_resolve_vulnerabilities_task',
                              criteria_index => 3,
                              store_index => 4
                            },
                            { 
                             function_name => 'execute_resolve_vulnerabilities_task',
                            },
                            {
                             function_name => 'delete_resolve_vulnerabilities_task',
                            },
                            {
                             function_name => 'delete_many_resolve_vulnerabilities_tasks',
                            },
                         );

our $entity_class = 'F5::ASMConfig::Entity::ResolveVulnerabilitiesTask';

#----------------------------------------------------------------------

sub add_resolve_vulnerabilities_task {
  my($self, @rpc_params) = @_;
  return F5::ASMConfig::Handler::TaskBase::add_task(self => $self,
                                                    rpc_params => \@rpc_params,
                                                    entity_class => $entity_class,
                                                    task_type => 'resolve_vulnerabilities',);
}

#----------------------------------------------------------------------

sub update_resolve_vulnerabilities_task {
    my ($self, @rpc_params) = @_;   
    return F5::ASMConfig::Handler::TaskBase::update_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                         task_type => 'resolve_vulnerabilities',);
}

#----------------------------------------------------------------------

sub resolve_vulnerabilities {
    my (%params) = @_;
    my $self       = $params{self};
    my $session    = $params{session};
    my $call_flags = $params{call_flags};
    my $task       = $params{task};

    my $vulnerabilities = $task->{vulnerabilities};
    my $policy = $params{policy};

    my $rv;
    my $pre_resolve_messages;
    my @messages;
    eval {
        $self->begin_work($self->{session});
        for my $vuln (@$vulnerabilities) {
            if ($task->{get_pre_resolve_messages}) {
                my $warnings = $self->get_pre_resolve_messages($session, $call_flags, $policy, $vuln, $task->{stage_vulnerabilities});
                for my $warning (@$warnings) {
                    my $old_values = $pre_resolve_messages->{$warning->{message_text}} || {};
                    $pre_resolve_messages->{$warning->{message_text}} = {%$old_values, (map { ( $_ => 1 ) } @{$warning->{bind_values}})};
                }
            }
            else {
                my $warnings = $self->resolve_vuln($session, $call_flags, $policy, $vuln, $task->{stage_vulnerabilities});
                
                push (@messages, map { $_->{msg} } @{$warnings->{msgs}});
            }
        }
        $self->commit($self->{session});
    };
    
    if ($@) {
        $task->{message} = $@->{error_message};
        $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_FAILURE();
    }
    else {
        $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_COMPLETED();
        if ($pre_resolve_messages) {
            for my $message_text (keys %$pre_resolve_messages) {
                push (@messages, ("$message_text " . join(", ", keys %{$pre_resolve_messages->{$message_text}})));
            }
        }
        $task->{message} = join("\n", @messages);
    }

}


#----------------------------------------------------------------------
sub execute_resolve_vulnerabilities_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::execute_task(self => $self,
                                                          rpc_params => \@rpc_params,
                                                          entity_class => $entity_class,
                                                          task_type => 'resolve_vulnerabilities',
                                                          execute_func => sub { resolve_vulnerabilities(@_); },
                                                          );
}

#----------------------------------------------------------------------

sub delete_resolve_vulnerabilities_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                        );
}

#----------------------------------------------------------------------

sub delete_many_resolve_vulnerabilities_tasks {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_many_tasks(self => $self,
                                                               rpc_params => \@rpc_params,
                                                               entity_class => $entity_class,
                                                              );
}

#----------------------------------------------------------------------

1;
