package F5::ASMConfig::Handler::TerminateVulnerabilityAssessmentTask;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::TaskBase/;
use F5::ASMConfig::Entity::TerminateVulnerabilityAssessmentTask;

our @handled_functions = (
                            {
                             function_name => 'add_terminate_vulnerability_assessment_task',
                             criteria_index => 3,
                             store_index    => 4,
                            },
                            {
                              function_name => 'update_terminate_vulnerability_assessment_task',
                              criteria_index => 3,
                              store_index => 4
                            },
                            { 
                             function_name => 'execute_terminate_vulnerability_assessment_task',
                            },
                            {
                             function_name => 'delete_terminate_vulnerability_assessment_task',
                            },
                            {
                             function_name => 'delete_many_terminate_vulnerability_assessment_tasks',
                            },

                         );

our $entity_class = 'F5::ASMConfig::Entity::TerminateVulnerabilityAssessmentTask';

#----------------------------------------------------------------------

sub add_terminate_vulnerability_assessment_task {
  my($self, @rpc_params) = @_;
  return F5::ASMConfig::Handler::TaskBase::add_task(self => $self,
                                                    rpc_params => \@rpc_params,
                                                    entity_class => $entity_class,
                                                    task_type => 'terminate_vulnerability_assessment',);
}

#----------------------------------------------------------------------

sub update_terminate_vulnerability_assessment_task {
    my ($self, @rpc_params) = @_;   
    return F5::ASMConfig::Handler::TaskBase::update_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                         task_type => 'terminate_vulnerability_assessment',);
}

#----------------------------------------------------------------------

sub terminate_vulnerability_assessment {
    my (%params) = @_;
    my $self       = $params{self};
    my $session    = $params{session};
    my $call_flags = $params{call_flags};
    my $task       = $params{task};


    my $policy = $task->{target_policy};
    if (!$policy) {
        return $self->handle_error(action => 'execute',
                                   error_thrown => 'Policy not found',
                                   error_code => F5::ASMConfig::ErrorCode::MISSING_POLICY(),
                                   call_flags => $call_flags,
                                   entity_class => $entity_class,
                                   thrift_obj => $task,
                                  );
    }


    my $rv;
    eval {
        $rv = $self->terminate_scanner_scan($session, $call_flags, $policy);
    };

    if ($@) {
        $task->{message} = $@->{error_message};
        $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_FAILURE();
    }
    else {
        $task->{scan_id} = $rv;       
        $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_COMPLETED();
    }
}


#----------------------------------------------------------------------
sub execute_terminate_vulnerability_assessment_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::execute_task(self => $self,
                                                          rpc_params => \@rpc_params,
                                                          entity_class => $entity_class,
                                                          task_type => 'terminate_vulnerability_assessment',
                                                          execute_func => sub { terminate_vulnerability_assessment(@_); },
                                                          );
}

#----------------------------------------------------------------------

sub delete_terminate_vulnerability_assessment_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                        );
}

#----------------------------------------------------------------------

sub delete_many_terminate_vulnerability_assessment_tasks {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_many_tasks(self => $self,
                                                               rpc_params => \@rpc_params,
                                                               entity_class => $entity_class,
                                                              );
}

#----------------------------------------------------------------------

1;
