package F5::CommonUpgrade::UserDefinedFilters;

use base 'F5::CommonUpgrade::Base';
use F5::DbUtils;
use F5::Violation;
use F5::Violation::Utils;
use Bit::Vector;

our $max_version = '11.6.0';
our $tables_handled = [ qw( DCC.FILTER_CUSTOM_DATA ) ];

my $ip_reputation_name_mapping = { 1 => 'spam-sources',
                                   2 => 'windows-exploits',
                                   4 => 'web-attacks',
                                   8 => 'bot-nets',
                                   16 => 'scanners',
                                   32 => 'denial-of-service',
                                   64 => 'infected-sources',
                                   128 => 'phishing-proxies',
                                   256 => 'anonymous-proxies',
                                   512 => 'networks',
                                   1024 => 'illegal-websites',
                                   2048 => 'cloud-services',
                                 };

my $app_id_mapping = { 1 => 'http', 2 => 'https' };

sub row_fixup {
    my($self, %params) = @_;

    my $dbh    = $params{dbh};
    my $row = $params{row};
    my $common_upgrade = $params{common_upgrade};
    my $databases = $params{databases};

    if ($row->{filter_item} eq 'filter_app_id' &&  $row->{item_value} && $app_id_mapping->{$row->{item_value}} ){
           $row->{item_value} = $app_id_mapping->{$row->{item_value}};
    }

    if ( $row->{filter_item} eq 'filter_ip_reputation' ){
        if ( $ip_reputation_name_mapping->{$row->{item_value}} ){
            $row->{item_value} = $ip_reputation_name_mapping->{$row->{item_value}};
        }
    }


    if ( $row->{filter_item} eq 'filter_violation' || $row->{filter_item} eq 'filter_violation_staging' ){
        if ( $row->{item_value} && ( $row->{item_value} > 128 || $common_upgrade->is_old_version_before('11.3.0') ) ) {
             # GUI used to store the code in base 10, we need to convert it back in order to get the violation code
            my $code = Bit::Vector->new_Dec(64, $row->{item_value})->to_Hex();
            $row->{item_value} = F5::Violation::Utils::get_viol_index_from_code($code);
        }
    }

    if ( $row->{filter_item} eq 'filter_method' && $row->{item_value} ){
        ($row->{item_value}) = $dbh->selectrow_array(<<EndQuery, undef, $row->{item_value});
SELECT method_name 
  FROM $databases->{DCC}->{ucs_dbname}.METHOD_POOL 
 WHERE method_code  = ?
EndQuery
    } 
}

1;

